-- ========================================================
-- tugboat.GTD PostgreSQL ver0.8.6 and 0.9.0-RC1,RC2 fix.
-- ========================================================
--
-- create tables.
--
CREATE TABLE task_groups (
    ID                       integer       NOT NULL
   ,NAME                     text          NOT NULL
   ,DESCRIPTION              text
   ,START_DATE               char(8)                  default NULL
   ,DEADLINE                 char(8)                  default NULL
   ,REGISTERED               char(14)      NOT NULL
   ,MODIFIED                 char(14)      NOT NULL
   ,USER_ID                  integer       NOT NULL
   ,DEL_FLAG                 char(1)       NOT NULL   default '0'
);

CREATE TABLE task_group_activities (
    ID                       serial        NOT NULL
   ,TASK_GROUP_ID            integer       NOT NULL
   ,ACTIVITY_ID              integer       NOT NULL
);

CREATE TABLE task_group_relations (
    ID                       serial        NOT NULL
   ,TASK_GROUP_ID            integer       NOT NULL
   ,TASK_ID                  integer       NOT NULL
);

CREATE TABLE task_group_comment_bodies (
    ID                       serial        NOT NULL
   ,TASK_GROUP_ID            integer       NOT NULL
   ,REV                      integer       NOT NULL   default '1'
   ,BODY                     text          NOT NULL
);

CREATE TABLE task_reviews (
    ID                       serial        NOT NULL
   ,REVIEW_DATE              char(14)      NOT NULL
   ,USER_ID                  integer       NOT NULL
);

CREATE TABLE task_rsslet_tokens (
    ID                       serial        NOT NULL
   ,TASK_ID                  integer       NOT NULL
   ,TOKEN                    char(20)      NOT NULL
);

CREATE TABLE versions (
    ID                       serial        NOT NULL
   ,MAJOUR_NUM               integer       NOT NULL
   ,MINOR_NUM                integer       NOT NULL
   ,MAINTENANCE_NUM          integer       NOT NULL
   ,MAINTENANCE_SUB          integer       NOT NULL
   ,REGISTERED               char(14)      NOT NULL
);

--
-- primary key.
--
ALTER TABLE task_groups 
  ADD CONSTRAINT task_groups_pk 
  PRIMARY KEY (ID);

ALTER TABLE task_group_activities 
  ADD CONSTRAINT task_group_activities_pk 
  PRIMARY KEY (ID);

ALTER TABLE task_group_relations 
  ADD CONSTRAINT task_group_relations_pk 
  PRIMARY KEY (ID);
  
ALTER TABLE task_group_comment_bodies 
  ADD CONSTRAINT task_group_comment_bodies_pk 
  PRIMARY KEY (ID);
  
ALTER TABLE task_reviews 
  ADD CONSTRAINT task_reviews_pk 
  PRIMARY KEY (ID);

ALTER TABLE task_rsslet_tokens 
  ADD CONSTRAINT task_rsslet_tokens_pk 
  PRIMARY KEY (ID);

ALTER TABLE versions 
  ADD CONSTRAINT versions_pk 
  PRIMARY KEY (ID);

--
-- foreign key.
--
ALTER TABLE task_groups 
  ADD FOREIGN KEY (USER_ID) 
  REFERENCES users(ID);

ALTER TABLE task_group_activities 
  ADD FOREIGN KEY (TASK_GROUP_ID) 
  REFERENCES task_groups(ID);
  
ALTER TABLE task_group_activities 
  ADD FOREIGN KEY (ACTIVITY_ID) 
  REFERENCES activities(ID);

ALTER TABLE task_group_relations 
  ADD FOREIGN KEY (TASK_GROUP_ID) 
  REFERENCES task_groups(ID);
  
ALTER TABLE task_group_relations 
  ADD FOREIGN KEY (TASK_ID) 
  REFERENCES tasks(ID);
  
ALTER TABLE task_group_comment_bodies 
  ADD FOREIGN KEY (TASK_GROUP_ID) 
  REFERENCES task_groups(ID);

ALTER TABLE task_reviews 
  ADD FOREIGN KEY (USER_ID) 
  REFERENCES users(ID);

ALTER TABLE task_rsslet_tokens 
  ADD FOREIGN KEY (TASK_ID)
  REFERENCES tasks(ID);

-- grant.
GRANT ALL ON task_groups                                      TO tangerine;
GRANT ALL ON task_group_activities                            TO tangerine;
GRANT ALL ON task_group_relations                             TO tangerine;
GRANT ALL ON task_group_comment_bodies                        TO tangerine;
GRANT ALL ON task_reviews                                     TO tangerine;
GRANT ALL ON task_rsslet_tokens                               TO tangerine;
GRANT ALL ON versions                                         TO tangerine;

GRANT ALL ON TABLE task_group_activities_id_seq            TO tangerine;
GRANT ALL ON TABLE task_group_relations_id_seq             TO tangerine;
GRANT ALL ON TABLE task_group_comment_bodies_id_seq        TO tangerine;
GRANT ALL ON TABLE task_reviews_id_seq                     TO tangerine;
GRANT ALL ON TABLE task_rsslet_tokens_id_seq               TO tangerine;
GRANT ALL ON TABLE versions_id_seq                         TO tangerine;

-- comments.
COMMENT ON TABLE  task_groups                                     IS 'Task Group';
COMMENT ON COLUMN task_groups.NAME                                IS 'Name';
COMMENT ON COLUMN task_groups.DESCRIPTION                         IS 'Description';
COMMENT ON COLUMN task_groups.START_DATE                          IS 'Start Date';
COMMENT ON COLUMN task_groups.DEADLINE                            IS 'Deadline';
COMMENT ON COLUMN task_groups.REGISTERED                          IS 'Registered';
COMMENT ON COLUMN task_groups.MODIFIED                            IS 'Modified';
COMMENT ON COLUMN task_groups.USER_ID                             IS 'User Id';
COMMENT ON COLUMN task_groups.DEL_FLAG                            IS 'Delete flag';

COMMENT ON TABLE  task_group_activities                           IS 'Task Group Activities';
COMMENT ON COLUMN task_group_activities.ID                        IS 'Id';
COMMENT ON COLUMN task_group_activities.TASK_GROUP_ID             IS 'Task Group Id';
COMMENT ON COLUMN task_group_activities.ACTIVITY_ID               IS 'Activity Id';

COMMENT ON TABLE  task_group_relations                            IS 'Task Group Relation';
COMMENT ON COLUMN task_group_relations.ID                         IS 'Id';
COMMENT ON COLUMN task_group_relations.TASK_GROUP_ID              IS 'Task Group Id';
COMMENT ON COLUMN task_group_relations.TASK_ID                    IS 'Task Id';

COMMENT ON TABLE  task_group_comment_bodies                       IS 'Task Group Comment Body';
COMMENT ON COLUMN task_group_comment_bodies.ID                    IS 'Id';
COMMENT ON COLUMN task_group_comment_bodies.TASK_GROUP_ID         IS 'Task Group Id';
COMMENT ON COLUMN task_group_comment_bodies.REV                   IS 'Revision';
COMMENT ON COLUMN task_group_comment_bodies.BODY                  IS 'Body';

COMMENT ON TABLE  task_reviews                                    IS 'Task Review';
COMMENT ON COLUMN task_reviews.ID                                 IS 'Id';
COMMENT ON COLUMN task_reviews.REVIEW_DATE                        IS 'Review Date';
COMMENT ON COLUMN task_reviews.USER_ID                            IS 'User Id';

COMMENT ON TABLE task_rsslet_tokens                               IS 'Task RSS-let Tokens';
COMMENT ON COLUMN task_rsslet_tokens.ID                           IS 'Id';
COMMENT ON COLUMN task_rsslet_tokens.TASK_ID                      IS 'Task Id';
COMMENT ON COLUMN task_rsslet_tokens.TOKEN                        IS 'Token';

COMMENT ON TABLE versions                                         IS 'versions';
COMMENT ON COLUMN versions.ID                                     IS 'Id';
COMMENT ON COLUMN versions.MAJOUR_NUM                             IS 'majour number';
COMMENT ON COLUMN versions.MINOR_NUM                              IS 'minor number';
COMMENT ON COLUMN versions.MAINTENANCE_NUM                        IS 'maintenance number';
COMMENT ON COLUMN versions.MAINTENANCE_SUB                        IS 'maintenance sub number';
COMMENT ON COLUMN versions.REGISTERED                             IS 'registered';

--
-- update tables.
--
DROP table web_service_securities;

CREATE TABLE web_service_securities (
    NONCE                   char(20)      NOT NULL
   ,CREATED_AT              char(20)      NOT NULL
   ,CLIENT_ADDRESS          varchar(256)  NOT NULL
);
ALTER TABLE web_service_securities
  ADD CONSTRAINT web_service_securities_pk
  PRIMARY KEY (NONCE,CREATED_AT);
  
COMMENT ON TABLE  web_service_securities                      IS 'Web service securities ';
COMMENT ON COLUMN web_service_securities.CLIENT_ADDRESS       IS 'Client address';
COMMENT ON COLUMN web_service_securities.NONCE                IS 'Nonce';
COMMENT ON COLUMN web_service_securities.CREATED_AT           IS 'Created at';

GRANT ALL ON web_service_securities                           TO tangerine;

ALTER TABLE meeting_agendas DROP COLUMN "priority";
ALTER TABLE meeting_agendas ADD COLUMN STATUS char(1) NOT NULL DEFAULT '0';

COMMENT ON COLUMN meeting_agendas.STATUS IS 'Status of Agenda  [0:active, 1:pending, 2:done]';

--
-- version
--
INSERT INTO versions (
    majour_num
   ,minor_num
   ,maintenance_num
   ,maintenance_sub
   ,registered
) VALUES (
  0,9,3,0,'20080612190000'
);

--
-- feed tokens
--
DROP TABLE feed_tokens;
CREATE TABLE feed_tokens (
    ID                      serial        NOT NULL
   ,TOKEN                   char(36)      NOT NULL
   ,ACTIVITY_ID             integer       
   ,USER_ID                 integer       NOT NULL
   ,DEL_FLAG                char(1)       NOT NULL
   ,CREATED_AT              char(14)      NOT NULL
);
ALTER TABLE feed_tokens
  ADD CONSTRAINT feed_tokens_pk
  PRIMARY KEY (ID);
  
ALTER TABLE feed_tokens 
  ADD FOREIGN KEY (ACTIVITY_ID) 
  REFERENCES activities(ID);
  
ALTER TABLE feed_tokens 
  ADD FOREIGN KEY (USER_ID) 
  REFERENCES users(ID);
  
COMMENT ON TABLE  feed_tokens              IS 'Feed tokens';
COMMENT ON COLUMN feed_tokens.ID           IS 'Id';
COMMENT ON COLUMN feed_tokens.TOKEN        IS 'Token';
COMMENT ON COLUMN feed_tokens.ACTIVITY_ID  IS 'User id';
COMMENT ON COLUMN feed_tokens.USER_ID      IS 'User id';
COMMENT ON COLUMN feed_tokens.DEL_FLAG     IS 'Role id';
COMMENT ON COLUMN feed_tokens.CREATED_AT   IS 'Created at';
  
GRANT ALL ON feed_tokens                   TO tangerine;
GRANT ALL ON TABLE feed_tokens_id_seq   TO tangerine;

--
-- task_collection_of_records
--
CREATE TABLE task_collection_of_records (
    ID                      serial        NOT NULL
   ,COLLECT_TYPE            char(1)       NOT NULL
   ,FULL_DATE               char(8)       NOT NULL
   ,TASK_ID                 integer       NOT NULL
   ,ACTIVITY_ID             integer       NOT NULL
   ,STATUS_ID               integer       NOT NULL
   ,CLASSIFICATION_ID       integer       NOT NULL
   ,PERSON_IN_CHARGE_ID     integer       NOT NULL
   ,CREATED_AT              char(14)      NOT NULL
);
ALTER TABLE task_collection_of_records
  ADD CONSTRAINT task_collection_of_records_pk
  PRIMARY KEY (ID);
ALTER TABLE task_collection_of_records 
  ADD FOREIGN KEY (TASK_ID) 
  REFERENCES tasks(ID);
ALTER TABLE task_collection_of_records 
  ADD FOREIGN KEY (ACTIVITY_ID) 
  REFERENCES activities(ID);
ALTER TABLE task_collection_of_records 
  ADD FOREIGN KEY (STATUS_ID) 
  REFERENCES task_statuses(ID);
ALTER TABLE task_collection_of_records 
  ADD FOREIGN KEY (CLASSIFICATION_ID) 
  REFERENCES classifications(ID);
ALTER TABLE task_collection_of_records 
  ADD FOREIGN KEY (PERSON_IN_CHARGE_ID) 
  REFERENCES users(ID);

COMMENT ON TABLE  task_collection_of_records                      IS 'task collection of records';
COMMENT ON COLUMN task_collection_of_records.ID                   IS 'Id';
COMMENT ON COLUMN task_collection_of_records.COLLECT_TYPE         IS 'collect type';
COMMENT ON COLUMN task_collection_of_records.FULL_DATE            IS 'full date';
COMMENT ON COLUMN task_collection_of_records.TASK_ID              IS 'task id';
COMMENT ON COLUMN task_collection_of_records.ACTIVITY_ID          IS 'activity id';
COMMENT ON COLUMN task_collection_of_records.STATUS_ID            IS 'status id';
COMMENT ON COLUMN task_collection_of_records.CLASSIFICATION_ID    IS 'classification id';
COMMENT ON COLUMN task_collection_of_records.PERSON_IN_CHARGE_ID  IS 'person in charge id';
COMMENT ON COLUMN task_collection_of_records.CREATED_AT           IS 'created at';

GRANT ALL ON task_collection_of_records                   TO tangerine;
GRANT ALL ON TABLE task_collection_of_records_id_seq   TO tangerine;


-- update meeting agenda.status
UPDATE meeting_agendas 
SET status = 3
WHERE
    id IN (
        SELECT agenda_id
        FROM meeting_agenda_discussions
        WHERE
            type = '2'
        GROUP BY agenda_id
    );

