/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.commons.util;

import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.factory.SingletonS2ContainerFactory;
import org.seasar.tugboat.gtd.container.dto.BrowseInfoDto;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * Session Utility.
 * 
 * 
 */
public class SessionUtil {
    private SessionUtil() {
    }

    static S2Container container = SingletonS2ContainerFactory.getContainer();

    /**
     * get login user info.
     * 
     * @return
     */
    public static UserDto getLoginUser() {
        return (UserDto) container.getComponent(UserDto.LOGIN_USER_SESSION_ATTRIBUTE_NAME);

    }

    /**
     * get login user info for admini tools.
     * 
     * @return
     */
    public static UserDto getLoginAdminUser() {
        return (UserDto) container.getComponent(UserDto.LOGIN_ADMIN_USER_SESSION_ATTRIBUTE_NAME);
    }

    /**
     * get browse info dto.(request from mail-client and rss-reader)
     * 
     * @return
     */
    public static BrowseInfoDto getBrowseInfo() {
        return (BrowseInfoDto) container.getComponent("browseInfo");
    }

    /**
     * clear browse info
     * 
     */
    public static void clearBrowseInfo() {
        BrowseInfoDto browseInfo = getBrowseInfo();
        browseInfo.setPid(null);
        browseInfo.setType(null);
        browseInfo.setUrl(null);
    }
}
