/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.commons.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.seasar.framework.log.Logger;

/**
 * System Configuration
 * 
 * @author yousuke
 * 
 */
public class SystemConfiguration {
    /* LOGGER */
    Logger logger_ = Logger.getLogger(SystemConfiguration.class);

    static List<Map<String, String>> systemConfigList_;

    /* configuration file path */
    String configurationName_ = "";

    public void setConfigurationName(String configurationName) {
        this.configurationName_ = configurationName;
    }

    public static final String SEN_HOME_DIR                 = "tugboat.gtd.search.sen.home_dir";
    public static final String SEARCH_INDEX_HOME            = "tugboat.gtd.search.searchIndexFolder";
    public static final String CONTEXT_PATH                 = "tugboat.gtd.context_path";
    public static final String SEARCH_RESULT_SUMMARY_LENGTH = "tugboat.gtd.search.sen.summary_byte_length";
    public static final String MAIL_SMTP_ADDRESS            = "tugboat.gtd.mail.smtp";
    public static final String MAIL_FROM_ADDRESS            = "tugboat.gtd.mail.from";
    public static final String MAIL_RETURN_PATH             = "tugboat.gtd.mail.return_path";
    public static final String RSS_LANG                     = "tugboat.gtd.rss.lang";
    public static final String RSS_PUB_ROWS                 = "tugboat.gtd.rss.pub_rows";
    public static final String RSS_RETROACTIVE_DATE         = "tugboat.gtd.rss.retroactive_date";
    public static final String BATCH_SCHEDULE_HOUR          = "tugboat.gtd.batch.schedule_hour";
    public static final String BATCH_SCHEDULE_MIN           = "tugboat.gtd.batch.schedule_min";

    /**
     * initialize
     * 
     */
    public synchronized void initialize() {
        systemConfigList_ = new ArrayList<Map<String, String>>();

        try {
            Configuration config = new PropertiesConfiguration(this.configurationName_);
            String mode = config.getString("tugboat.gtd.mode") + ".";

            Iterator<?> i = config.getKeys();
            String key, value;

            while (i.hasNext()) {
                key = (String) i.next();
                value = config.getString(key);

                if (key.indexOf(mode) == 0 || key.indexOf("default.") == 0) {
                    String storeKey = key.indexOf(mode) > -1 ? key.substring(mode.length()) : key;

                    Map<String, String> map = new HashMap<String, String>();
                    map.put(storeKey, value);

                    systemConfigList_.add(map);
                }
            }
        } catch (ConfigurationException e) {
            e.printStackTrace();
        }
    }

    /**
     * Get configuration value
     * 
     * @param key
     * @return
     */
    public static String getValue(String key) {
        if (systemConfigList_ == null)
            return null;

        String result = null;
        for (Map<String, String> config : systemConfigList_) {
            if (config.containsKey(key)) {
                result = config.get(key);
                break;
            }
        }
        if (result == null && SEN_HOME_DIR.equals(key)) {
            String path = SystemConfiguration.class.getResource("").getPath();
            path = path.substring(0, path.indexOf("WEB-INF")) + "WEB-INF/sen";
        }
        if (result == null && SEARCH_INDEX_HOME.equals(key)) {
            String path = SystemConfiguration.class.getResource("").getPath();
            path = path.substring(0, path.indexOf("WEB-INF")) + "WEB-INF/searchIndex";
        }
        return result;
    }

    /**
     * Get configuration value with string.
     * 
     * @param key
     * @param string
     * @return
     */
    public static String getValueWithString(String key, String string) {
        return getValue(key) + (string == null ? "" : string);
    }
}
