/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.container.dto;

import java.io.Serializable;
import java.util.Calendar;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.tugboat.search.annotations.Index;
import org.seasar.tugboat.search.annotations.Searchable;
import org.seasar.tugboat.search.annotations.SearchableField;
import org.seasar.tugboat.search.annotations.SearchableId;
import org.seasar.tugboat.search.annotations.Store;

/**
 * Search result dto.
 */
@Searchable(keyName = "org.seasar.tugboat.gtd.container.dto.SearchDto")
public class SearchDto implements Serializable {
    private static final long serialVersionUID = -8053160420437871370L;
    
    /* unique id */
    @SearchableId(name = "uniqueId", store = Store.YES, index = Index.UN_TOKENIZED)
    private String uniqueId;

    public String getUniqueId() {
        return uniqueId;
    }

    public void setUniqueId(String uniqueId) {
        this.uniqueId = uniqueId;
    }

    /* id */
    @SearchableField(name = "id", store = Store.YES, index = Index.UN_TOKENIZED)
    private long id;

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    /* class id */
    @SearchableField(name = "class", store = Store.YES, index = Index.UN_TOKENIZED)
    private int classId;

    public int getClassId() {
        return classId;
    }

    public void setClassId(int classId) {
        this.classId = classId;
    }

    /* user id */
    @SearchableField(name = "userId", store = Store.YES, index = Index.UN_TOKENIZED)
    private long userId = -1;

    public long getUserId() {
        return userId;
    }

    public void setUserId(long userId) {
        this.userId = userId;
    }

    /* activity id */
    @SearchableField(name = "activityId", store = Store.YES, index = Index.UN_TOKENIZED)
    private long activityId;

    public long getActivityId() {
        return activityId;
    }

    public void setActivityId(long activityId) {
        this.activityId = activityId;
    }

    /* title */
    @SearchableField(name = "title", store = Store.YES, index = Index.TOKENIZED)
    private String title;

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    /* body */
    @SearchableField(name = "body", store = Store.YES, index = Index.TOKENIZED)
    private String body;

    public String getBody() {
        return body;
    }

    public void setBody(String body) {
        this.body = body;
    }

    /* summary */
    private String summary;

    public String getSummary() {
        return summary;
    }

    public void setSummary(String summary) {
        this.summary = summary;
    }

    /* classification id */
    @SearchableField(name = "classificationId", store = Store.YES, index = Index.UN_TOKENIZED)
    private int classificationId;

    public int getClassificationId() {
        return classificationId;
    }

    public void setClassificationId(int classificationId) {
        this.classificationId = classificationId;
    }

    /* sub project id */
    @SearchableField(name = "subProjectId", store = Store.YES, index = Index.UN_TOKENIZED)
    private int subProjectId = -1;

    public int getSubProjectId() {
        return subProjectId;
    }

    public void setSubProjectId(int subProjectId) {
        this.subProjectId = subProjectId;
    }

    /* context id */
    @SearchableField(name = "contextId", store = Store.YES, index = Index.UN_TOKENIZED)
    private int contextId = -1;

    public int getContextId() {
        return contextId;
    }

    public void setContextId(int contextId) {
        this.contextId = contextId;
    }

    /* privacy */
    @SearchableField(name = "privacy", store = Store.YES, index = Index.UN_TOKENIZED)
    private int privacy = -1;

    public int getPrivacy() {
        return privacy;
    }

    public void setPrivacy(int privacy) {
        this.privacy = privacy;
    }

    /* share */
    @SearchableField(name = "share", store = Store.YES, index = Index.UN_TOKENIZED)
    private int share = -1;

    public int getShare() {
        return share;
    }

    public void setShare(int share) {
        this.share = share;
    }

    /* modified */
    @SearchableField(name = "modified", store = Store.YES, index = Index.UN_TOKENIZED)
    private long modified;

    public long getModified() {
        return modified;
    }

    public void setModified(long modified) {
        this.modified = modified;
    }

    public Calendar getCalendarModified() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(modified);
        return calendar;
    }

    public void setCalendarModified(Calendar modified) {
        this.modified = modified.getTimeInMillis();
    }

    /* xml */
    private String xml;

    public String getXml() {
        return xml;
    }

    public void setXml(String xml) {
        this.xml = xml;
    }

    /* modified */
    @SearchableField(name = "userName", store = Store.YES, index = Index.UN_TOKENIZED)
    private String userName;

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    /* data level-1 */
    @SearchableField(name = "dataLevel1", store = Store.YES, index = Index.UN_TOKENIZED)
    private long dataLevel1;

    public long getDataLevel1() {
        return dataLevel1;
    }

    public void setDataLevel1(long dataLevel1) {
        this.dataLevel1 = dataLevel1;
    }

    /* data level-2 */
    @SearchableField(name = "dataLevel2", store = Store.YES, index = Index.UN_TOKENIZED)
    private long dataLevel2;

    public long getDataLevel2() {
        return dataLevel2;
    }

    public void setDataLevel2(long dataLevel2) {
        this.dataLevel2 = dataLevel2;
    }

    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this, ToStringStyle.MULTI_LINE_STYLE);
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + (int) (activityId ^ (activityId >>> 32));
        result = prime * result + ((body == null) ? 0 : body.hashCode());
        result = prime * result + classId;
        result = prime * result + classificationId;
        result = prime * result + contextId;
        result = prime * result + (int) (dataLevel1 ^ (dataLevel1 >>> 32));
        result = prime * result + (int) (dataLevel2 ^ (dataLevel2 >>> 32));
        result = prime * result + (int) (id ^ (id >>> 32));
        result = prime * result + (int) (modified ^ (modified >>> 32));
        result = prime * result + privacy;
        result = prime * result + share;
        result = prime * result + subProjectId;
        result = prime * result + ((summary == null) ? 0 : summary.hashCode());
        result = prime * result + ((title == null) ? 0 : title.hashCode());
        result = prime * result
                + ((uniqueId == null) ? 0 : uniqueId.hashCode());
        result = prime * result + (int) (userId ^ (userId >>> 32));
        result = prime * result
                + ((userName == null) ? 0 : userName.hashCode());
        result = prime * result + ((xml == null) ? 0 : xml.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final SearchDto other = (SearchDto) obj;
        if (activityId != other.activityId)
            return false;
        if (body == null) {
            if (other.body != null)
                return false;
        } else if (!body.equals(other.body))
            return false;
        if (classId != other.classId)
            return false;
        if (classificationId != other.classificationId)
            return false;
        if (contextId != other.contextId)
            return false;
        if (dataLevel1 != other.dataLevel1)
            return false;
        if (dataLevel2 != other.dataLevel2)
            return false;
        if (id != other.id)
            return false;
        if (modified != other.modified)
            return false;
        if (privacy != other.privacy)
            return false;
        if (share != other.share)
            return false;
        if (subProjectId != other.subProjectId)
            return false;
        if (summary == null) {
            if (other.summary != null)
                return false;
        } else if (!summary.equals(other.summary))
            return false;
        if (title == null) {
            if (other.title != null)
                return false;
        } else if (!title.equals(other.title))
            return false;
        if (uniqueId == null) {
            if (other.uniqueId != null)
                return false;
        } else if (!uniqueId.equals(other.uniqueId))
            return false;
        if (userId != other.userId)
            return false;
        if (userName == null) {
            if (other.userName != null)
                return false;
        } else if (!userName.equals(other.userName))
            return false;
        if (xml == null) {
            if (other.xml != null)
                return false;
        } else if (!xml.equals(other.xml))
            return false;
        return true;
    }
    
}
