/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.container.interceptor;

import javax.faces.context.FacesContext;

import org.aopalliance.intercept.MethodInvocation;
import org.seasar.framework.log.Logger;
import org.seasar.jsf.util.MessageUtil;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.dto.BrowseInfoDto;
import org.seasar.tugboat.gtd.container.dto.SecurityDto;
import org.seasar.tugboat.gtd.container.exception.AppRuntimeException;

/**
 * Action Method Intercepter
 * 
 * @author yousuke.hara
 * 
 */
public class ActionMethodInterceptor extends AbstractTugboatInterceptor {
    private static final long serialVersionUID = 9067610505717899996L;

    final int METHOD_NONE = -1;

    final int METHOD_EVENT = 0;

    final int METHOD_STATE = 1;

    final String LOGIN_ACTION_NAME = "LoginAction";

    final String MSG_ID_SYSTEM_ERROR = "system.error.otherException";

    /* LOGGER */
    Logger logger_ = Logger.getLogger(ActionMethodInterceptor.class);

    public ActionMethodInterceptor() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.aopalliance.intercept.MethodInterceptor#invoke(
     *      org.aopalliance.intercept.MethodInvocation)
     */
    public Object invoke(MethodInvocation invocation) throws Throwable {
        long start, end;
        Object result = null;
        start = System.currentTimeMillis();

        // -- Get Form Annotations. => Control.
        SecurityDto securityDto = this.getSecurity(invocation);
        logger_.debug("[\n" + securityDto.toString() + "\n]");

        BrowseInfoDto browseInfo = SessionUtil.getBrowseInfo();

        if (!this.isExistsReferer() && securityDto.isDirectAccessCheck()
                && browseInfo.getPid() == null) {

            this.addErrorMessageToHttpRequest("Direct access is not admitted."); // TASK
            return securityDto.getErrorPage();
        }
        if (!this.isExistsSession(securityDto.getIsAdminTool())
                && securityDto.isAuthorizationCheck()) {
            this.addErrorMessageToHttpRequest("There is not a session. Please log in."); // TASK
            return securityDto.getErrorPage();
        }

        boolean isExistError = false;
        try {
            result = invocation.proceed();
        } catch (Exception ex) {
            logger_.error(ex.getMessage());
            FacesContext context = FacesContext.getCurrentInstance();

            if (ex instanceof AppRuntimeException) {
                context.addMessage(null, MessageUtil.getErrorMessage(((AppRuntimeException) ex)
                        .getMessageId(), ((AppRuntimeException) ex).getArgs()));
                this.addErrorMessageToHttpRequest(MessageUtil.getErrorMessage(
                        ((AppRuntimeException) ex).getMessageId(),
                        ((AppRuntimeException) ex).getArgs()).getDetail());
            } else {
                context.addMessage(null, MessageUtil.getErrorMessage(MSG_ID_SYSTEM_ERROR,
                        new Object[] { ex.getMessage() }));

                this.addErrorMessageToHttpRequest(MessageUtil.getErrorMessage(MSG_ID_SYSTEM_ERROR,
                        new Object[] { ex.getMessage() }).getDetail());
            }
            isExistError = true;
        } finally {
            if (invocation.getClass().getName().indexOf(LOGIN_ACTION_NAME) == -1) {
                browseInfo.setPid(null);
                browseInfo.setType(null);
                browseInfo.setUrl(null);
            }
            if (isExistError) {
                result = securityDto.getErrorPage() == null
                        || "".equals(securityDto.getErrorPage().trim()) ? null : securityDto
                        .getErrorPage();
            }
        }
        end = System.currentTimeMillis();

        logger_.debug("\n::::: processing time:[" + (end - start) + "ms] :::::\n");
        return result;
    }
}
