/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.container.interceptor;

import org.aopalliance.intercept.MethodInvocation;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.container.dto.SecurityDto;
import org.seasar.tugboat.gtd.container.exception.SystemAuthorizationRuntimeException;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;

/**
 * Ajax Intercepter
 * 
 * @author yousuke.hara
 * 
 */
public class AjaxMethodInterceptor extends AbstractTugboatInterceptor {
    private static final long serialVersionUID = 6562033476693957366L;
    
    private static final Logger LOG = Logger.getLogger(AjaxMethodInterceptor.class);

    public AjaxMethodInterceptor() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.aopalliance.intercept.MethodInterceptor#invoke(org.aopalliance.intercept.MethodInvocation)
     */
    public Object invoke(MethodInvocation invocation) throws Throwable {
        long start = System.currentTimeMillis();

        SecurityDto securityDto = this.getSecurity(invocation);
        LOG.debug("[\n" + securityDto.toString() + "\n]");

        if (!this.isExistsSession(securityDto.getIsAdminTool())
                && securityDto.isAuthorizationCheck()) {

            throw new SystemAuthorizationRuntimeException("Not Exists session");
        }

        try {
            Object result = invocation.proceed();
            long end = System.currentTimeMillis();

            LOG.debug("\n::::: processing time:[" + (end - start) + "ms] :::::\n");
            return result;

        } catch (Exception e) {
            LOG.error(e.getMessage());
            throw new SystemUnexpectedRuntimeException(e);
        }
    }
}
