/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.container.service.impl;

import java.util.StringTokenizer;

import org.apache.commons.lang.StringUtils;
import org.seasar.tugboat.gtd.container.service.TextSearchResultFormatter;
import org.seasar.framework.log.Logger;

/**
 * text search result formatter implement.
 * 
 * @author yosuke
 */
public class HtmlTextSearchResultFormatterImpl implements TextSearchResultFormatter {

    /* start tag */
    String preTag_;

    /* end tag */
    String postTag_;

    /* Logger */
    Logger logger_ = Logger.getLogger(HtmlTextSearchResultFormatterImpl.class);

    public HtmlTextSearchResultFormatterImpl() {
        super();

        this.preTag_ = "<b style=\"background-color:#999; color:#FFFF66;\">";
        this.postTag_ = "</b>";
    }

    /**
     * 
     * @param preTag
     * @param postTag
     */
    public HtmlTextSearchResultFormatterImpl(String preTag, String postTag) {
        super();

        this.preTag_ = this.htmlUnescape(preTag);
        this.postTag_ = this.htmlUnescape(postTag);
    }

    /**
     * HTML-Unescape.
     * 
     * @param string
     * @return
     */
    private String htmlUnescape(String string) {
        String result = "";
        result = StringUtils.replace(string, "&lt;", "<");
        result = StringUtils.replace(string, "&gt;", ">");
        return result;
    }

    /**
     * @see jp.co.tenartni.tangerine.system.logic.TextSearchResultFormatter#highlightTerm(
     *      java.lang.String, java.lang.String)
     */
    public String highlightTerm(String document, String searchString) {
        String result = StringUtils.replace(document, searchString,
                this.preTag_ + searchString + this.postTag_);

        return result;
    }

    /**
     * @see jp.co.tenartni.tangerine.system.logic.TextSearchResultFormatter#summaryDocument(
     *      java.lang.String, java.lang.String, int)
     */
    public String summaryDocument(String document, String searchString,
            int length) {
        final String[] token = new String[] { ".", "、", "。", "\n\r", "\n", "\r" };
        final String TOKEN_SPACE = " ";
        final String STR_PLUS = "+";
        final String STR_MINUS = "-";
        final String STR_MORE = "...";

        if (document.getBytes().length < length) {
            return document;
        }

        StringTokenizer searchStringToken = new StringTokenizer(searchString,
                TOKEN_SPACE);
        String[] searchStringList = new String[searchStringToken.countTokens()];

        int contentIndex = 0;
        int counter = 0;

        while (searchStringToken.hasMoreTokens()) {
            counter++;
            searchStringList[counter - 1] = searchStringToken.nextToken();

            // ---[-]
            if (searchStringList[counter - 1].startsWith(STR_MINUS)) continue;
            // ---[+]
            if (searchStringList[counter - 1].startsWith(STR_PLUS)) {
                searchStringList[counter - 1] = searchStringList[counter - 1].substring(1);
            }
            int pos = document.indexOf(searchStringList[counter - 1]);
            contentIndex = pos > contentIndex ? pos : contentIndex;
        }
        logger_.debug("::::: contentIndex :[" + contentIndex + "]");

        String result = document;
        int startPosition = document.getBytes().length;
        boolean isHit = false;

        for (int i = contentIndex; i > 0; i--) {
            for (int j = 0; j < token.length; j++) {
                if (token[j].equals(document.substring(i, i + 1))) {
                    startPosition = i + token[j].length();
                    isHit = true;
                    break;
                }
            }
            if (isHit)
                break;
        }
        if (startPosition == -1 
         || startPosition == document.getBytes().length
         || contentIndex == 0) {

            startPosition = 0;
        }
        logger_.debug("::::: startPosition:[" + startPosition + "]");

        int dstlen = 0;
        boolean isExistsBefore = (startPosition == 0) ? false : true;
        boolean isExistsAfter = false;

        for (int i = startPosition; i < document.length(); i++) {
            dstlen += (document.charAt(i) <= 0xff ? 1 : 2);

            if (dstlen > length || i == document.length() - 1) {
                result = document.substring(startPosition, i);
                isExistsAfter = (i == document.length() - 1) ? false : true;
                break;
            }
        }
        logger_.debug("::::: summaryLength:[" + result.getBytes().length + "]");

        if (result.length() == document.getBytes().length) {
        } else {
            if (isExistsBefore) result = STR_MORE + result;
            if (isExistsAfter)  result += STR_MORE;
        }
        return result;
    }
}
