/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.container.servlet;

import java.io.IOException;
import java.util.StringTokenizer;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.factory.SingletonS2ContainerFactory;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.dto.BrowseInfoDto;
import org.seasar.tugboat.gtd.contents.meeting.service.MeetingService;
import org.seasar.tugboat.gtd.contents.meeting.service.impl.MeetingServiceImpl;
import org.seasar.tugboat.gtd.contents.task.service.TaskService;
import org.seasar.tugboat.gtd.contents.task.service.impl.TaskServiceImpl;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * tugboat browse servlet (request from mail client and rss reader).
 * 
 * @author yosuke
 * 
 */
public class TugboatBrowseServlet extends HttpServlet {
    private static final long serialVersionUID = 2041049878189978319L;
    
    //private static final Logger LOG = Logger.getLogger(TugboatBrowseServlet.class);

    @Override
    public void init(ServletConfig config) throws ServletException {
    }

    @Override
    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        this.doDispatch(request, response);
    }

    @Override
    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        this.doDispatch(request, response);
    }

    /**
     * do dispatch view.
     * 
     * @param request
     * @param response
     */
    protected void doDispatch(HttpServletRequest request, HttpServletResponse response) {
        final String PATH_TASKS = "tasks";
        final String PATH_MEETINGS = "meetings";
        final String URL_INDEX = "/index.html";
        final String URL_ERROR = "/view/invalidError.html";
        final String URL_TASK_DETAIL = "/view/gtdTaskDetail.html?id=";
        final String URL_MEETING_DETAIL = "/view/gtdMeetingDetail.html?id=";

        if (request.getPathInfo().indexOf("/" + PATH_TASKS + "/") == -1
                && request.getPathInfo().indexOf("/" + PATH_MEETINGS + "/") == -1)
            return;

        StringTokenizer token = new StringTokenizer(request.getPathInfo(), "/");
        String type = "";
        String pid = "";

        int i = 0;
        while (token.hasMoreTokens()) {
            if (i == 0)
                type = token.nextToken();
            else if (i == 1)
                pid = token.nextToken();
            i++;
        }
        try {
            if (!StringUtils.isNumeric(pid)) {
                request.setAttribute("ERROR_MESSAGE", "Invalid Access");
                response.sendRedirect(request.getContextPath() + URL_ERROR);
                return;
            }
            String url = PATH_TASKS.equals(type) ? request.getContextPath() + URL_TASK_DETAIL + pid
                    : request.getContextPath() + URL_MEETING_DETAIL + pid;

            BrowseInfoDto browseInfo = SessionUtil.getBrowseInfo();
            browseInfo.setPid(pid);
            browseInfo.setType(type);
            browseInfo.setUrl(url);

            UserDto user = SessionUtil.getLoginUser();

            if (user == null || user.getUserId() == 0) {
                // no session
                response.sendRedirect(request.getContextPath() + URL_INDEX);

            } else {
                // session already exist
                S2Container container = SingletonS2ContainerFactory.getContainer();

                boolean isBelongActivityInfo = false;
                if (PATH_TASKS.equals(type)) {
                    TaskService taskService = (TaskServiceImpl) container
                            .getComponent(TaskServiceImpl.class);

                    isBelongActivityInfo = taskService.checkBelongActivityTask(Long.parseLong(pid),
                            user.getUserId());

                } else {
                    MeetingService meetingService = (MeetingServiceImpl) container
                            .getComponent(MeetingServiceImpl.class);

                    isBelongActivityInfo = meetingService.checkBelongActivityMeeting(Long
                            .parseLong(pid), user.getUserId());
                }
                if (!isBelongActivityInfo) {
                    response.sendRedirect(request.getContextPath() + URL_ERROR);
                    SessionUtil.clearBrowseInfo();
                } else {
                    response.sendRedirect(url);
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
