/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.container.servlet;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.seasar.framework.beans.BeanDesc;
import org.seasar.framework.beans.factory.BeanDescFactory;
import org.seasar.framework.container.ComponentDef;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.servlet.S2ContainerServlet;
import org.seasar.tugboat.gtd.commons.util.SystemConfiguration;
import org.seasar.tugboat.gtd.container.constant.HttpStatus;
import org.seasar.tugboat.gtd.webservices.entity.FeedToken;

/**
 * icalendar publish servlet.
 * 
 * @author yosuke
 * 
 */
public class TugboatICalendarPubServlet extends HttpServlet {
    private static final long serialVersionUID = -5994600454572839566L;
    private static final int TOKEN_LENGTH = 36;
    
    //private static final Logger LOG = Logger.getLogger(TugboatRSSServlet.class);

    S2Container container;

    @Override
    public void init(ServletConfig config) throws ServletException {
        this.container = S2ContainerServlet.getContainer();
    }

    @Override
    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        this.invokeMethod(request, response);
    }

    @Override
    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        this.invokeMethod(request, response);
    }

    /**
     * invoke mothod.
     * 
     * @param request
     * @param response
     * @throws ServletException
     * @throws IOException
     */
    protected void invokeMethod(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        String[] pathInfo = StringUtils.split(request.getPathInfo(), '/');

        if (pathInfo == null || pathInfo.length < 5) {
            this.onFailure(response, "", HttpStatus.NOT_ACCEPTABLE);
            return;
        }

        //#1
        // ical/activities/1/tasks/PSsBUyXvMnighdAyXuvHGlbjzyPESLIrUwZkXydkoRqstOzGDytBJtSWLejPWciI
        // => [activities, 1, tasks, $token]
        //
        for (int i = 0; i < pathInfo.length; i++) {
            if (i == 0 && !"activities".equals(pathInfo[i])) {
                this.onFailure(response, "", HttpStatus.NOT_ACCEPTABLE);
                return;
            }
            if (i == 1 && !StringUtils.isNumeric(pathInfo[i])) {
                this.onFailure(response, "", HttpStatus.NOT_ACCEPTABLE);
                return;
            }
            if (i == 2 && !"tasks".equals(pathInfo[i])) {
                this.onFailure(response, "", HttpStatus.NOT_ACCEPTABLE);
                return;
            }
            if (i == 3 && pathInfo[i].length() != TOKEN_LENGTH) {
                this.onFailure(response, "", HttpStatus.NOT_ACCEPTABLE);
                return;
            }
        }
        this.publishByActivity(response, pathInfo);
    }

    /**
     * publish by activity.
     * 
     * @param response
     * @param pathInfo
     */
    private void publishByActivity(HttpServletResponse response, String[] pathInfo) throws ServletException,IOException{
        long activityId = Long.parseLong(pathInfo[1]);
        String token = pathInfo[3];
        ComponentDef feedTokenComponent = this.getComponentDefNoException("feedTokenService");
        Object feedTokenServiceObj = feedTokenComponent.getComponent();
        Object target = null;
        try {
            BeanDesc beanDesc = BeanDescFactory.getBeanDesc(feedTokenComponent.getConcreteClass());
            target = beanDesc.invoke(
                feedTokenServiceObj, "findByActivityIdAndToken", new Object[] {activityId, token});

        } catch (Exception e) {
            e.printStackTrace();
            throw new ServletException("The error occurred while create Rest Web Service response."
                    + e.getMessage(), e);
        }

        FeedToken feedToken = (FeedToken) target;
        boolean isInvalid = feedToken != null;
        if (!isInvalid) {
            this.onFailure(response, "invalid access", HttpStatus.NOT_ACCEPTABLE);
            return;
        }

        // publish.
        ComponentDef component = this.getComponentDefNoException("ical.tasks");
        Object obj = component.getComponent();
        target = null;
        try {
            BeanDesc beanDesc = BeanDescFactory.getBeanDesc(component.getConcreteClass());
            String confRssPublishRows = SystemConfiguration.getValue("tugboat.gtd.rss.pub_rows");
            
            int limit = StringUtils.isNumeric(confRssPublishRows) ? Integer.parseInt(confRssPublishRows) : 50;
            target = beanDesc.invoke(
                obj, "publish", new Object[] { activityId,feedToken.getUserId(), limit });

        } catch (Exception e) {
            e.printStackTrace();
            throw new ServletException("The error occurred while create Rest Web Service response."
                    + e.getMessage(), e);
        }
        this.onSuccess(response, target);
    }
    
    /**
     * get component def.
     * 
     * @param componentName
     * @return
     */
    protected ComponentDef getComponentDefNoException(String componentName) {
        if (!this.container.hasComponentDef(componentName)) {
            return null;
        }
        return this.container.getComponentDef(componentName);
    }

    /**
     * on success.
     * 
     * @param response
     * @param target
     * @throws IOException
     */
    protected void onSuccess(HttpServletResponse response, Object target) throws IOException {
        response.setHeader("Cache-Control", "no-cache");
        response.setHeader("Pragma", "no-cache");
        response.setContentType("text/calendar; charset=utf-8");
        response.setCharacterEncoding("UTF-8");

        PrintWriter pw = response.getWriter();
        pw.write(new String(String.valueOf(target).getBytes("UTF-8")));
        pw.close();
    }

    /**
     * on failure.
     * 
     * @param response
     * @param message
     * @param statusCode
     * @throws IOException
     */
    protected void onFailure(HttpServletResponse response, String message, int statusCode)
            throws IOException {

        response.setHeader("Cache-Control", "no-cache");
        response.setHeader("Pragma", "no-cache");
        response.setStatus(statusCode);

        PrintWriter pw = response.getWriter();
        pw.write(message);
        pw.close();
    }
}
