/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.container.servlet;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.seasar.framework.container.ComponentDef;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.servlet.S2ContainerServlet;

/**
 * rss publish servlet.
 * 
 * @author yosuke
 * 
 */
public class TugboatTaskCollectOfRecordsServlet extends HttpServlet {
    private static final long serialVersionUID = 6992197382926596092L;
    
    private S2Container container;

    @Override
    public void init(ServletConfig config) throws ServletException {
        this.container = S2ContainerServlet.getContainer();
    }

    @Override
    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        this.invokeMethod(request, response);
    }

    @Override
    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        this.invokeMethod(request, response);
    }

    /**
     * invoke mothod.
     * 
     * @param request
     * @param response
     * @throws ServletException
     * @throws IOException
     */
    protected void invokeMethod(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException {
        // Pending... ver0.9.4
        //String[] pathInfo = StringUtils.split(request.getPathInfo(), '/');
        response.sendRedirect(request.getContextPath() + "/index.html");
    }
    
    /**
     * get component def.
     * 
     * @param componentName
     * @return
     */
    protected ComponentDef getComponentDefNoException(String componentName) {
        if (!this.container.hasComponentDef(componentName)) {
            return null;
        }
        return this.container.getComponentDef(componentName);
    }

    /**
     * on success.
     * 
     * @param response
     * @param target
     * @throws IOException
     */
    protected void onSuccess(HttpServletResponse response, Object target) throws IOException {
        response.setHeader("Cache-Control", "no-cache");
        response.setHeader("Pragma", "no-cache");
        response.setContentType("application/xml; charset=utf-8");
        response.setCharacterEncoding("UTF-8");

        PrintWriter pw = response.getWriter();
        pw.write(new String(String.valueOf(target).getBytes("UTF-8")));
        pw.close();
    }

    /**
     * on failure.
     * 
     * @param response
     * @param message
     * @param statusCode
     * @throws IOException
     */
    protected void onFailure(HttpServletResponse response, String message, int statusCode)
            throws IOException {

        response.setHeader("Cache-Control", "no-cache");
        response.setHeader("Pragma", "no-cache");
        response.setStatus(statusCode);

        PrintWriter pw = response.getWriter();
        pw.write(message);
        pw.close();
    }
}
