/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.activity.dao;

import org.seasar.dao.annotation.tiger.Arguments;
import org.seasar.dao.annotation.tiger.S2Dao;
import org.seasar.tugboat.gtd.contents.activity.entity.Activity;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityPageCondition;

/**
 * Activibities DAO
 * 
 * @author yousuke.hara
 * 
 */
@S2Dao(bean = Activity.class)
public interface ActivitiesDao {

    /**
     * Select Activities - Set offset and rows.
     * 
     * @param offset
     * @param rows
     * @return
     */
    @Arguments("condition")
    Activity[] selectActivitiesLimit(ActivityPageCondition condition);

    /**
     * Select Activity - Keword Search.
     * 
     * @param keyword
     * @return
     */
    @Arguments("keyword")
    Activity[] selectActivitiesSearch(String keyword);

    /**
     * Select new activity Id
     * 
     * @return
     */
    Activity selectNewId();

    /**
     * Select activity by Id
     * 
     * @param id
     * @return
     */
    @Arguments("id")
    Activity selectActivityById(long id);

    /**
     * Select own activities
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    Activity[] selectOwnActivities(long userId);

    /**
     * insert.
     * 
     * @param dto
     * @return
     */
    @Arguments("activity")
    int insert(Activity activity);

    /**
     * update.
     * 
     * @param id
     * @return
     */
    @Arguments("activity")
    int update(Activity activity);

    /**
     * delete.
     * 
     * @param activity
     * @return
     */
    @Arguments("id")
    int updateDeleteFlagById(long id);

}
