/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.activity.dao;

import org.seasar.tugboat.gtd.contents.activity.entity.ActivityMember;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityPageCondition;
import org.seasar.dao.annotation.tiger.Arguments;
import org.seasar.dao.annotation.tiger.S2Dao;

/**
 * activity members DAO.
 * 
 * @author naoya.saito
 * 
 */
@S2Dao(bean = ActivityMember.class)
public interface ActivityMembersDao {

    /**
     * find all.
     * 
     * @return
     */
    ActivityMember[] findAll();

    /**
     * select activity members by activity id
     * 
     * @param activityId
     * @return
     */
    @Arguments("activityId")
    ActivityMember[] selectByActivityId(long activityId);

    /**
     * select activity members by activity id / page.
     * 
     * @param activityId
     * @param offset
     * @param limit
     * @return
     */
    @Arguments("condition")
    ActivityMember[] selectByActivityIdPerPage(ActivityPageCondition condition);

    /**
     * find by user id.
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    ActivityMember[] findByUserId(long userId);

    /**
     * find by activity id and user id.
     * 
     * @param activityId
     * @param userId
     * @return
     */
    @Arguments( { "activityId", "userId" })
    ActivityMember findByActivityIdAndUserId(long activityId, long userId);

    /**
     * find activity members.
     * 
     * @param keyword
     * @return
     */
    @Arguments("keyword")
    ActivityMember[] findActivityMembersSearch(String keyword);

    /**
     * find activity members by activity id.
     * 
     * @param activityId
     * @return
     */
    @Arguments("activityId")
    ActivityMember[] findActivityMembersByActivityId(long activityId);

    /**
     * find activity members by activity ids.
     * 
     * @param activityIds
     * @return
     */
    @Arguments("activityIds")
    ActivityMember[] findActivityMembersByActivityIds(Long[] activityIds);

    /**
     * find activity members by activity id and keyword.
     * 
     * @return
     */
    @Arguments( { "id", "keyword" })
    ActivityMember[] findActivityMembersByIdAndKeyword(long id, String keyword);

    /**
     * insert.
     * 
     * @param activityMembers
     * @return
     */
    @Arguments("activityMembers")
    int insert(ActivityMember activityMembers);

    /**
     * update.
     * 
     * @param activityMembers
     * @return
     */
    @Arguments("activityMembers")
    int update(ActivityMember activityMembers);

    /**
     * get activity record count by user id.
     * 
     * @param userId
     * @return
     */
    @Arguments( { "activityId", "offset", "limit" })
    int getActivityRecordCountByUserId(long activityId);

    /**
     * update delete flag by activity id and user id.
     * 
     * @param activityId
     * @param userId
     * @return
     */
    @Arguments( { "activityId", "userId" })
    int updateDeleteFlagByActivityIdAndUserId(long activityId, long userId);
}
