/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.activity.helper.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.beanutils.BeanUtils;
import org.seasar.tugboat.gtd.commons.util.SystemConfiguration;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.activity.helper.ActivityHelper;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * Activity helper impl.
 * 
 * @author yousuke
 * 
 */
public class ActivityHelperImpl implements ActivityHelper {

    public ActivityHelperImpl() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.helper.ActivityHelper#createOwnActivitiesJSON(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto[])
     */
    public String createOwnActivitiesJSON(ActivityDto[] activities) {
        if (activities == null || activities.length == 0) return null;
        
        try {
            List<Map<?,?>> listJson = new ArrayList<Map<?,?>>();
            for (ActivityDto activity : activities) {
                Map<?,?> map = BeanUtils.describe(activity);
                listJson.add(map);
            }
            JSONArray jsonArray = 
                JSONArray.fromObject(listJson.toArray(new Map[listJson.size()]));

            String smtp = SystemConfiguration.getValue("tugboat.gtd.mail.smtp");
            if (smtp == null) smtp = "";
            
            return "({activities:" + jsonArray.toString() + ",smtp:\"" + smtp + "\"})";
            
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.helper.ActivityHelper#createOwnActivitiesForSelectMenuJSON(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto[])
     */
    public String createOwnActivitiesForSelectMenuJSON(ActivityDto[] activities) {
        if (activities == null || activities.length == 0) return null;
        
        try {
            List<Map<?,?>> listJson = new ArrayList<Map<?,?>>();
            for (ActivityDto activity : activities) {
                Map<String, String> map = new HashMap<String, String>();
                map.put("id", String.valueOf(activity.getId()));
                map.put("value", activity.getName());
                map.put("color", activity.getColorValue());
                map.put("icon", "");

                listJson.add(map);
            }
            JSONArray jsonArray = 
                JSONArray.fromObject(listJson.toArray(new Map[listJson.size()]));
            return jsonArray.toString();
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.helper.ActivityHelper#createMembersJSON(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityMembersDto[],
     *      int, int, int, long)
     */
    public String createMembersJSON(ActivityMemberDto[] members,
            int totalRecords, int currentPageNumber, int recordPerPage,
            long userId) {

        if (members == null || members.length == 0) return null;
        try {
            // members info.
            int myRoleId = 0;

            List<Map<?,?>> listJson = new ArrayList<Map<?,?>>();
            for (ActivityMemberDto member : members) {
                Map<?,?> map = BeanUtils.describe(member);
                listJson.add(map);

                if (member.getUserId() == userId) {
                    myRoleId = member.getRoleId();
                }
            }
            JSONArray jsonArray = 
                JSONArray.fromObject(listJson.toArray(new Map[listJson.size()]));

            // page info.
            Map<String, Integer> page = new HashMap<String, Integer>();
            page.put("results", new Integer(totalRecords));
            page.put("totalPages", 
              new Integer(totalRecords / recordPerPage + (totalRecords % recordPerPage == 0 ? 0 : 1)));
            page.put("currentPageNumber", new Integer(currentPageNumber));
            JSONObject pageJson = JSONObject.fromObject(page);

            // role info
            Map<String, Integer> role = new HashMap<String, Integer>();
            role.put("roleId", new Integer(myRoleId));
            JSONObject roleJson = JSONObject.fromObject(role);

            return "({members:" + jsonArray.toString() + ",page:"
                    + pageJson.toString() + ",role:" + roleJson.toString()
                    + "})";
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.helper.ActivityHelper#createMemberJson(
     *   org.seasar.tugboat.gtd.contents.activity.dto.ActivityMembersDto)
     */
    public String createMemberJson(ActivityMemberDto member) {
        if (member == null) return null;
        
        try {
            JSONObject jsonObjcet = JSONObject.fromObject(member);
            return jsonObjcet.toString();
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.helper.ActivityHelper#createUsersForAutoCompleterJSON(
     *      org.seasar.tugboat.gtd.contents.user.dto.UserDto[])
     */
    public String createUsersForAutoCompleterHTML(UserDto[] users) {
        if (users == null || users.length == 0) return "";
        try {
            StringBuilder buf = new StringBuilder();
            buf.append(
              "<ul class=\"list_for_auto_completer\" xmlns:tgb=\"http://tugboat.seasar.org/ns/compatibility/annotations/1.0\">");

            for (UserDto user : users) {
                buf.append("<li ").append("tgb:value=\"").append(
                        user.getUserId()).append("\">").append(
                        user.getLastName()).append(",").append(
                        user.getFirstName()).append("</li>");
            }
            buf.append("</ul>");
            return buf.toString();
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.helper.ActivityHelper#createPersonInChargeJSON(int,
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityMembersDto[])
     */
    public String createPersonInChargeJSON(long userId, ActivityMemberDto[] members) {
        if (members == null) return null;
        List<Map<?,?>> listJson = new ArrayList<Map<?,?>>();

        // -- ALL.
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("personId", "0");
        map.put("personLastName", "All");
        map.put("personFirstName", "");
        map.put("isMe", false);
        listJson.add(map);

        // -- Me.
        for (ActivityMemberDto dto : members) {
            if (userId == dto.getUserId()) {
                map = new HashMap<String, Object>();
                map.put("personId", String.valueOf(dto.getUserId()));
                map.put("personFirstName", dto.getFirstName());
                map.put("personLastName", "[Me] " + dto.getLastName());
                map.put("isMe", true);
                listJson.add(map);
                break;
            }
        }

        // -- Other.
        for (ActivityMemberDto dto : members) {
            if (userId == dto.getUserId()) {
            } else if (0 == dto.getUserId()) {
            } else {
                map = new HashMap<String, Object>();
                map.put("personId", String.valueOf(dto.getUserId()));
                map.put("personFirstName", dto.getFirstName());
                map.put("personLastName", dto.getLastName());
                map.put("isMe", false);
                listJson.add(map);
            }
        }
        JSONArray jsonArray = JSONArray.fromObject(
            listJson.toArray(new Map[listJson.size()]));
        return jsonArray.toString();
    }
}
