/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.activity.service;

import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityRelationDto;

/**
 * activity service.
 * 
 * @author yousuke.hara
 * 
 */
public interface ActivityService {

    /**
     * get activities by keyword.
     * 
     * @param keyword
     * @return
     */
    ActivityDto[] getActivitiesByKeyword(String keyword);

    /**
     * get activity by Id.
     * 
     * @param dto
     * @return
     */
    ActivityDto getActivityById(long id);

    /**
     * get own activity.
     * 
     * @param userId
     * @return
     */
    ActivityDto[] getOwnActivities(long userId);

    /**
     * Add Activity.
     * 
     * @param dto
     * @param notifications
     */
    void add(ActivityDto dto);

    /**
     * add activity with member.
     * 
     * @param dto
     * @param relationsDto
     * @param memberDto
     * @return
     */
    void addWithMember(ActivityDto dto, ActivityRelationDto relationsDto,
            ActivityMemberDto memberDto);

    /**
     * Update Activity.
     * 
     * @param dto
     */
    void update(ActivityDto dto);

    /**
     * update activity color.
     * 
     * @param dto
     * @return
     */
    void updateActivityColor(ActivityRelationDto dto);

    /**
     * update with relations.
     * 
     * @param dto
     * @param dto
     */
    void updateWithRelations(ActivityDto activity, ActivityRelationDto relations);

    /**
     * get activity record count by user id.
     * 
     * @param activityId
     * @return
     */
    int getActivityCountByUserId(long activityId);

    /**
     * remove activity.
     * 
     * @param dto
     * @return
     */
    int remove(ActivityDto dto);
}
