/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.activity.service.impl;

import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.contents.activity.dao.ActivitiesDao;
import org.seasar.tugboat.gtd.contents.activity.dao.ActivityMembersDao;
import org.seasar.tugboat.gtd.contents.activity.dao.ActivityRelationsDao;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityRelationDto;
import org.seasar.tugboat.gtd.contents.activity.dxo.ActivityDxo;
import org.seasar.tugboat.gtd.contents.activity.entity.Activity;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityMember;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityRelation;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityService;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingBasicsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskRelationsDao;
import org.seasar.tugboat.gtd.webservices.service.FeedTokenService;

/**
 * activity Service Implements.
 * 
 * @author yousuke.hara
 * 
 */
public class ActivityServiceImpl implements ActivityService {
    /* LOGGER */
    Logger logger_ = Logger.getLogger(ActivityServiceImpl.class);

    public ActivityServiceImpl() {
        super();
    }

    /* Activity DAO */
    ActivitiesDao activitiesDao_;

    public void setActivitiesDao(ActivitiesDao activitiesDao) {
        this.activitiesDao_ = activitiesDao;
    }

    /* Activity Members Dao */
    ActivityMembersDao activityMembersDao_;

    public void setActivityMembersDao(ActivityMembersDao activityMembersDao) {
        this.activityMembersDao_ = activityMembersDao;
    }

    /* Activity relations dao */
    ActivityRelationsDao activityRelationsDao_;

    public void setActivityRelationsDao(
            ActivityRelationsDao activityRelationsDao) {
        this.activityRelationsDao_ = activityRelationsDao;
    }

    /* Activity DXO */
    ActivityDxo activityDxo_;

    public void setActivityDxo(ActivityDxo activityDxo) {
        this.activityDxo_ = activityDxo;
    }
    
    /* task relations dao */
    TaskRelationsDao taskRelationsDao_;
    
    public void setTaskRelationsDao(TaskRelationsDao taskRelationsDao) {
        this.taskRelationsDao_ = taskRelationsDao;
    }
    
    /* meeting basics dao */
    MeetingBasicsDao meetingBasicDao_;
    
    public void setMeetingBasicsDao(MeetingBasicsDao meetingBasicDao) {
        this.meetingBasicDao_ = meetingBasicDao;
    }
    
    /* feed token service */
    FeedTokenService feedTokenService_;
    
    public void setFeedTokenService(FeedTokenService feedTokenService) {
        this.feedTokenService_ = feedTokenService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.relation.logic.ActivityService#getActivitiesList(java.lang.String)
     */
    public ActivityDto[] getActivitiesByKeyword(String keyword) {
        Activity[] activities = 
            this.activitiesDao_.selectActivitiesSearch("%" + keyword + "%");
        ActivityMember[] members = 
            this.activityMembersDao_.findActivityMembersSearch("%" + keyword + "%");

        return this.activityDxo_.convertActivityDtos(activities, members);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.relation.logic.ActivityService#getActivityById(int)
     */
    public ActivityDto getActivityById(long id) {
        Activity activity = this.activitiesDao_.selectActivityById(id);
        ActivityMember[] members = 
            this.activityMembersDao_.findActivityMembersByActivityId(id);

        return this.activityDxo_.convertActivityDto(activity, members);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityService#getOwnActivities(int)
     */
    public ActivityDto[] getOwnActivities(long userId) {
        Activity[] activities = this.activitiesDao_.selectOwnActivities(userId);

        for (Activity a : activities) {
        	if (a.getFeedToken() == null || "".equals(a.getFeedToken())) {
        		String token = this.feedTokenService_.create(a.getId(), userId);
        		a.setFeedToken(token);
        	}
        }
        return this.activityDxo_.convertActivityDtos(activities);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityService#add(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto)
     */
    public void add(ActivityDto dto) {
        dto.setId(this.activitiesDao_.selectNewId().getId());
        Activity activity = this.activityDxo_.convertActivity(dto);
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        activity.setStartDate(nowString.substring(0, 8));
        activity.setRegistered(nowString);
        activity.setModified(nowString);
        
        this.activitiesDao_.insert(activity);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityService#addWithMember(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto,
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityRelationsDto,
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityMembersDto)
     */
    public void addWithMember(
		ActivityDto dto, ActivityRelationDto relationsDto, ActivityMemberDto memberDto) {

        long activityId = this.activitiesDao_.selectNewId().getId();

        Activity activity = this.activityDxo_.convertActivity(dto);
        activity.setId(activityId);
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        activity.setStartDate(nowString.substring(0, 8));
        activity.setRegistered(nowString);
        activity.setModified(nowString);

        ActivityRelation relations = 
            this.activityDxo_.convertActivityRelations(relationsDto);
        relations.setActivityId(activityId);

        ActivityMember member = this.activityDxo_.convertActivityMember(memberDto);
        member.setActivityId(activityId);
        member.setStartDate(nowString.substring(0, 8));
        member.setRegistered(nowString);
        member.setModified(nowString);

        this.activitiesDao_.insert(activity);
        this.activityRelationsDao_.insert(relations);
        this.activityMembersDao_.insert(member);
        
        // create feed token.
        this.feedTokenService_.create(activityId, member.getUserId());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityService#update(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto)
     */
    public void update(ActivityDto dto) {
        Activity activity = this.activityDxo_.convertActivity(dto);
        activity.setModified(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
        
        this.activitiesDao_.update(activity);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityService#updateActivityColor(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityRelationsDto)
     */
    public void updateActivityColor(ActivityRelationDto dto) {
        ActivityRelation relations = 
            this.activityDxo_.convertActivityRelations(dto);
        this.activityRelationsDao_.update(relations);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityService#updateWithRelations(
     *   org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto,
     *   org.seasar.tugboat.gtd.contents.activity.dto.ActivityRelationDto)
     */
    public void updateWithRelations(ActivityDto activity, ActivityRelationDto relation) {
    	this.update(activity);
    	this.updateActivityColor(relation);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityService#getActivityCountByUserId(int)
     */
    public int getActivityCountByUserId(long activityId) {
        return this.activityMembersDao_.getActivityRecordCountByUserId(activityId);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityService#remove(
     *   org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto)
     */
    public int remove(ActivityDto dto) {
        int tasks = this.taskRelationsDao_.getCountByActivityId(dto.getId());
        int meetings = this.meetingBasicDao_.getCountByActivityId(dto.getId());
        
        int result = 0;
        if (tasks == 0 && meetings == 0) {
            result = this.activitiesDao_.updateDeleteFlagById(dto.getId());
            
            // delete feed token.
            this.feedTokenService_.deleteByActivityId(dto.getId());
        }
        return result;
    }
}
