/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.common.entity;

import java.io.Serializable;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.dao.annotation.tiger.Bean;
import org.seasar.dao.annotation.tiger.Column;

/**
 * Calendar classification.
 * 
 * @author yousuke.hara
 * 
 */
@Bean(table = "classifications")
public class Classification implements Serializable {
    private static final long serialVersionUID = -3883387322386754877L;

    /** tasks */
    public static int TASKS = 0;

    /** inbox. */
    public static int INBOX = 9;

    /** today. */
    public static int TODAY = 1;

    /** week */
    public static int WEEK = 2;

    /** month */
    public static int MONTH = 3;

    /** hold */
    public static int HOLD = 4;

    /** someday */
    public static int SOMEDAY = 5;

    /** archives */
    public static int ARCHIVES = 21;

    /** trash */
    public static int TRASH = 99;
    

    public Classification() {
    }

    public Classification(int id, String value, int numberOfCount) {
        this.id_ = id;
        this.value_ = value;
        this.numberOfCount_ = numberOfCount;
    }

    /* id */
    private int id_;

    @Column("ID")
    public int getId() {
        return id_;
    }

    public void setId(int id) {
        this.id_ = id;
    }

    /* type */
    private String type_;

    @Column("TYPE")
    public String getType() {
        return type_;
    }

    public void setType(String type) {
        this.type_ = type;
    }

    /* value */
    private String value_;

    @Column("VALUE")
    public String getValue() {
        return value_;
    }

    public void setValue(String value) {
        this.value_ = value;
    }

    /* number of count */
    private int numberOfCount_;

    public int getNumberOfCount() {
        return numberOfCount_;
    }

    public void setNumberOfCount(int numberOfCount) {
        this.numberOfCount_ = numberOfCount;
    }

    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this, ToStringStyle.MULTI_LINE_STYLE);
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + id_;
        result = prime * result + numberOfCount_;
        result = prime * result + ((type_ == null) ? 0 : type_.hashCode());
        result = prime * result + ((value_ == null) ? 0 : value_.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final Classification other = (Classification) obj;
        if (id_ != other.id_)
            return false;
        if (numberOfCount_ != other.numberOfCount_)
            return false;
        if (type_ == null) {
            if (other.type_ != null)
                return false;
        } else if (!type_.equals(other.type_))
            return false;
        if (value_ == null) {
            if (other.value_ != null)
                return false;
        } else if (!value_.equals(other.value_))
            return false;
        return true;
    }
    
}
