/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.common.entity;

import java.io.Serializable;
import java.util.Calendar;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.dao.annotation.tiger.Bean;
import org.seasar.dao.annotation.tiger.Column;

/**
 * Calendar preference.
 * 
 * @author yousuke.hara
 * 
 */
@Bean(table = "calendar_preference_relations")
public class Preference implements Serializable {
    private static final long serialVersionUID = 3974731922482149063L;
    
    /* id */
    private int id_;

    @Column("ID")
    public int getId() {
        return id_;
    }

    public void setId(int id) {
        this.id_ = id;
    }

    /* time zone */
    private String timeZone_;

    @Column("TIMEZONE")
    public String getTimeZone() {
        return timeZone_;
    }

    public void setTimeZone(String timeZone) {
        this.timeZone_ = timeZone;
    }

    /* date format */
    private String dateFormat_;

    @Column("DATE_FORMAT")
    public String getDateFormat() {
        return dateFormat_;
    }

    public void setDateFormat(String dateFormat) {
        this.dateFormat_ = dateFormat;
    }

    /* time format */
    private String timeFormat_;

    @Column("TIME_FORMAT")
    public String getTimeFormat() {
        return timeFormat_;
    }

    public void setTimeFormat(String timeFormat) {
        this.timeFormat_ = timeFormat;
    }

    /* user id */
    private long userId_;

    @Column("USER_ID")
    public long getUserId() {
        return userId_;
    }

    public void setUserId(long userId) {
        this.userId_ = userId;
    }

    /* first day of week */
    private int firstDayOfWeek_ = Calendar.SUNDAY;

    @Column("FIRST_DAY_OF_WEEK")
    public int getFirstDayOfWeek() {
        return firstDayOfWeek_;
    }

    public void setFirstDayOfWeek(int firstDayOfWeek) {
        this.firstDayOfWeek_ = firstDayOfWeek;
    }

    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this, ToStringStyle.MULTI_LINE_STYLE);
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result
                + ((dateFormat_ == null) ? 0 : dateFormat_.hashCode());
        result = prime * result + firstDayOfWeek_;
        result = prime * result + id_;
        result = prime * result
                + ((timeFormat_ == null) ? 0 : timeFormat_.hashCode());
        result = prime * result
                + ((timeZone_ == null) ? 0 : timeZone_.hashCode());
        result = prime * result + (int) (userId_ ^ (userId_ >>> 32));
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final Preference other = (Preference) obj;
        if (dateFormat_ == null) {
            if (other.dateFormat_ != null)
                return false;
        } else if (!dateFormat_.equals(other.dateFormat_))
            return false;
        if (firstDayOfWeek_ != other.firstDayOfWeek_)
            return false;
        if (id_ != other.id_)
            return false;
        if (timeFormat_ == null) {
            if (other.timeFormat_ != null)
                return false;
        } else if (!timeFormat_.equals(other.timeFormat_))
            return false;
        if (timeZone_ == null) {
            if (other.timeZone_ != null)
                return false;
        } else if (!timeZone_.equals(other.timeZone_))
            return false;
        if (userId_ != other.userId_)
            return false;
        return true;
    }
    
}
