/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.dashboard.helper.impl;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.json.JSONObject;

import org.seasar.tugboat.gtd.container.dto.SearchConditionDto;
import org.seasar.tugboat.gtd.container.dto.SearchDto;
import org.seasar.tugboat.gtd.container.service.SearchService;
import org.seasar.tugboat.gtd.contents.dashboard.helper.DashboardHelper;

/**
 * dashboard helper implements.
 * 
 * @author yousuke.hara
 * 
 */
public class DashboardHelperImpl implements DashboardHelper {
    static DateFormat DATE_SHORT_FORMAT = 
        new SimpleDateFormat("yyyy-MM-dd");

    /* title length */
    int titleLength_;

    public void setTitleLength(int titleLength) {
        this.titleLength_ = titleLength;
    }

    /* body length */
    int bodyLength_;

    public void setBodyLength(int bodyLength) {
        this.bodyLength_ = bodyLength;
    }

    /* highlight start tag */
    String highlightStartTag_ = "<span>";

    public void setHighlightStartTag(String highlightStartTag) {
        this.highlightStartTag_ = highlightStartTag;
    }

    /* highlight end tag */
    String highlightEndTag_ = "</span>";

    public void setHighlightEndTag(String highlightEndTag) {
        this.highlightEndTag_ = highlightEndTag;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.dashboard.helper.DashboardHelper#createKeywordSearchResultContents(
     *      org.seasar.tugboat.gtd.container.dto.SearchDto[])
     */
    public String createKeywordSearchResultContents(
            SearchDto[] searchResults, SearchConditionDto searchCondition) {

        Map<String, Object> returnMap = new HashMap<String, Object>();
        returnMap.put("search_count", searchCondition.getCount());
        returnMap.put(
            "page_offset",
            searchCondition.getCount() == 0 ? "0" : searchCondition.getOffset());
        
        returnMap.put("total_page", searchCondition.getTotalPage());
        returnMap.put("next_page", searchCondition.hasNext());
        returnMap.put("previous_page", searchCondition.hasPrevious());

        List<SearchDto> taskList = new ArrayList<SearchDto>();
        List<SearchDto> meetingList = new ArrayList<SearchDto>();
        List<SearchDto> activityList = new ArrayList<SearchDto>();

        for (SearchDto searchResult : searchResults) {
            if (searchResult.getClassId() == SearchService.CLASS_TASK
             || searchResult.getClassId() == SearchService.CLASS_TASK_COMMENT) {
                taskList.add(searchResult);
            } else if (searchResult.getClassId() == SearchService.CLASS_MEETING
                    || searchResult.getClassId() == SearchService.CLASS_MEETING_AGENDA
                    || searchResult.getClassId() == SearchService.CLASS_MEETING_COMMENT) {
                meetingList.add(searchResult);
            } else if (searchResult.getClassId() == SearchService.CLASS_ACTIVETY) {
                activityList.add(searchResult);
            }
        }
        returnMap.put("task", returnJsonList(taskList, searchCondition));
        returnMap.put("meeting", returnJsonList(meetingList, searchCondition));
        returnMap.put("activity", returnJsonList(activityList, searchCondition));

        JSONObject jsonObject = JSONObject.fromObject(returnMap);
        String json = jsonObject.toString();
        return json;
    }

    /**
     * return json list.
     * 
     * @param searchResults
     * @param searchCondition
     * @return
     */
    private List<Map<?,?>> returnJsonList(List<SearchDto> searchResults, SearchConditionDto searchCondition) {
        List<Map<?,?>> dtoList = new ArrayList<Map<?,?>>();
        for (SearchDto searchResult : searchResults) {
            Map<String, Object> result = new HashMap<String, Object>();
            if (searchResult.getTitle().length() > this.titleLength_) {
                result.put(
                    "title", 
                    searchResult.getTitle().substring(0,this.titleLength_) + "...");
            } else {
                result.put("title", searchResult.getTitle());
            }

            String body = searchResult.getBody();
            String keyword = searchCondition.getKeyword();
            result.put("body", createBody(body, keyword));
            result.put("modified", DATE_SHORT_FORMAT.format(new Date(searchResult.getModified())));
            result.put("userName", searchResult.getUserName());
            result.put("dataID", searchResult.getId());
            result.put(
                "dataLevel", searchResult.getDataLevel1() > 0 
                ? String.valueOf(searchResult.getDataLevel1())
                : "");
            dtoList.add(result);
        }
        return dtoList;
    }

    /**
     * create body.
     * 
     * @param body
     * @param keywords
     * @return
     */
    private String createBody(String body, String keywords) {
        String returnValue = body;
        StringTokenizer sk = new StringTokenizer(keywords, " ");
        List<String> keywordList = new ArrayList<String>();

        while (sk.hasMoreElements()) {
            keywordList.add(sk.nextToken());
        }
        return replaceTag(returnValue, keywordList);
    }

    /**
     * replace tag.
     * 
     * @param document
     * @param args
     * @return
     */
    private String replaceTag(String document, List<String> args) {
        if (document == null || "".equals(document) || args == null || args.size() == 0) return document;

        String patterns = "";
        for (String arg : args) {
            patterns = patterns + Pattern.quote(arg) + "|";
        }
        patterns = patterns.substring(0, patterns.length() - 1);
        Pattern pattern = Pattern.compile(patterns, Pattern.CASE_INSENSITIVE);

        Matcher matcher2 = pattern.matcher(document);

        String separatDoc;
        if (matcher2.find()) {

            int wordIndex = matcher2.start();
            int returnCodeIndex = document.substring(0, wordIndex).lastIndexOf("\n");
            if (returnCodeIndex == -1) {
                separatDoc = document.substring(0, document.length());
            } else {
                separatDoc = document.substring(returnCodeIndex, document.length());
            }
            if (separatDoc.length() >= bodyLength_) {
                separatDoc = separatDoc.substring(0, bodyLength_) + "...";
            }
        } else {
            if (document.length() >= bodyLength_) {
                return document.substring(0, bodyLength_) + "...";
            }
            return document;
        }
        Matcher matcher = pattern.matcher(separatDoc);
        StringBuffer sb = new StringBuffer();
        while (matcher.find()) {
            for (int i = 0; i <= matcher.groupCount(); i++) {
                String mgroup = matcher.group(i);
                matcher.appendReplacement(
                    sb, this.highlightStartTag_ + mgroup + this.highlightEndTag_);
            }
        }
        matcher.appendTail(sb);
        return sb.toString();
    }
}
