/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.dashboard.page.impl;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.container.dto.SearchConditionDto;
import org.seasar.tugboat.gtd.container.dto.SearchDto;
import org.seasar.tugboat.gtd.container.service.SearchService;
import org.seasar.tugboat.gtd.contents.dashboard.helper.DashboardHelper;
import org.seasar.tugboat.gtd.contents.dashboard.page.SearcherAjax;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * searcher ajax implement.
 * 
 * @author yousuke.hara
 * 
 */
@FormAnnotation(
    formId = "searchAjax", description = "searchAjax", 
    isAuthorizationCheck = true, isDirectAccessCheck = true, isAdminTool= false, errorPage = "")
public class SearcherAjaxImpl implements SearcherAjax {

    /* search service */
    SearchService searchService_;

    public void setSearchService(SearchService searchService) {
        this.searchService_ = searchService;
    }

    /* dashboard helper */
    DashboardHelper dashboardHelper_;

    public void setDashboardHelper(DashboardHelper dashboardHelper) {
        this.dashboardHelper_ = dashboardHelper;
    }

    /* offset */
    int offset;

    public void setOffset(int offset) {
        this.offset = offset;
    }

    /* keyword */
    String keyword_;

    public void setKeyword(String keyword) {
        this.keyword_ = keyword;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.dashboard.ajax.SearcherAjax#keywordSearch()
     */
    public String keywordSearch() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        SearchConditionDto searchCondition = new SearchConditionDto();
        searchCondition.setKeyword(this.keyword_);
        searchCondition.setOffset(offset);
        SearchDto[] searchDto = 
            this.searchService_.search(searchCondition, sessionLoginUser.getUserId());

        return this.dashboardHelper_.createKeywordSearchResultContents(searchDto, searchCondition);
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }
}
