/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.login.page.impl;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.factory.SingletonS2ContainerFactory;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.login.page.LoginAdminAjax;
import org.seasar.tugboat.gtd.contents.login.service.LoginAdminService;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * login ajax impl.
 * 
 * @author yosuke
 * 
 */
@FormAnnotation(
    formId = "logiAdminnAjax", isAuthorizationCheck = false, isDirectAccessCheck = true, isAdminTool = true)
public class LoginAdminAjaxImpl implements LoginAdminAjax {
    /* LOGGER */
    Logger logger_ = Logger.getLogger(LoginAdminAjaxImpl.class);

    /* Login Service */
    LoginAdminService loginAdminService_;

    public void setLoginAdminService(LoginAdminService loginAdminService) {
        this.loginAdminService_ = loginAdminService;
    }

    /* user id */
    String userId_;

    public String getUserId() {
        return userId_;
    }

    public void setUserId(String userId) {
        this.userId_ = userId;
    }

    /* password */
    String password_;

    public String getPassword() {
        return password_;
    }

    public void setPassWord(String password) {
        this.password_ = password;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.login.page.LoginAjax#login()
     */
    public String login() {
        UserDto loginAdminUser = this.loginAdminService_.login(userId_, password_);

        if (loginAdminUser != null) {
            S2Container container = SingletonS2ContainerFactory.getContainer();
            UserDto sessionLoginAdminUser = 
                (UserDto) container.getComponent(UserDto.LOGIN_ADMIN_USER_SESSION_ATTRIBUTE_NAME);

            try {
                BeanUtils.copyProperties(sessionLoginAdminUser, loginAdminUser);
                logger_.debug(sessionLoginAdminUser.toString());
            } catch (Exception e) {
                throw new SystemUnexpectedRuntimeException(e);
            }
        }
        return loginAdminUser == null ? "[ERROR]" : "";
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.login.page.LoginAjax#logout()
     */
    public String logout() {
        return null;
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.login.page.LoginAjax#continuation()
     */
    public String continuation() {
        S2Container container = SingletonS2ContainerFactory.getContainer();
        UserDto sessionLoginUser = 
            (UserDto) container.getComponent(UserDto.LOGIN_ADMIN_USER_SESSION_ATTRIBUTE_NAME);
        
        logger_.debug(sessionLoginUser.getUserId());
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }
}
