/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.dxo;

import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgenda;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussion;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasic;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingToTask;

/**
 * meeting dxo
 * 
 */
public interface MeetingDxo {

    /**
     * convert meeting basic list to meeting basic dto list.
     * 
     * @param basicList
     * @return
     */
    MeetingBasicDto[] convertMeetingBasicDtoList(MeetingBasic[] basicList);

    /**
     * convert meeting basic list with task to meeting basic dto.
     * 
     * @param basicList
     * @return
     */
    MeetingBasicDto convertMeetingBasicDtoWithTask(MeetingBasic[] basicList);

    
    /**
     * convert meeting basic dto to entity.
     * @param dto
     * @return
     */
    MeetingBasic convertMeetingBasic(MeetingBasicDto dto);
    
    /**
     * convert meeting basic to meeting basic dto.
     * 
     * @param basic
     * @return
     */
    MeetingBasicDto convertMeetingBasicDto(MeetingBasic basic);

    /**
     * convert meeting agenda list to meeting agenda dto list.
     * 
     * @param agendaList
     * @return
     */
    MeetingAgendaDto[] convertMeetingAgendaDtoList(MeetingAgenda[] agendas);

    /**
     * convert entities to dtos.
     * 
     * @param agendaList
     * @return
     */
    MeetingAgendaDto[] convertAgendaDtos(MeetingAgenda[] agendas);

    /**
     * convert meeting agenda to dto.
     * @param dto
     * @return
     */
    MeetingAgenda convertMeetingAgenda(MeetingAgendaDto dto);
    
    /**
     * convert meeting agenda to meeting agenda dto.
     * 
     * @param agenda
     * @return
     */
    MeetingAgendaDto convertMeetingAgendaDto(MeetingAgenda agenda);

    /**
     * convert meeting agenda discussion to meeting agenda discussion dto.
     * 
     * @param discussion
     * @return
     */
    MeetingAgendaDiscussionDto convertMeetingAgendaDiscussionDto(MeetingAgendaDiscussion discussion);

    /**
     * convert meeting agenda discussion dto to calendar meeting agenda
     * discussion .
     * 
     * @param discussionDto
     * @return
     */
    MeetingAgendaDiscussion convertMeetingAgendaDiscussion(MeetingAgendaDiscussionDto discussionDto);

    /**
     * convert to meeting to task.
     * 
     * @param task
     * @return
     */
    MeetingToTask convertMeetingToTask(MeetingToTaskDto toTaskDto);

    /**
     * convert meeting to task to meeting to task dto.
     * 
     * @param toTask
     * @return
     */
    MeetingToTaskDto convertMeetingToTaskDto(MeetingToTask toTask);

    /**
     * convert meeting to task array to meeting to task dto array.
     * 
     * @param toTaskList
     * @return
     */
    MeetingToTaskDto[] convertMeetingToTaskDtoList(MeetingToTask[] toTaskList);
}
