/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.dxo.impl;

import java.lang.reflect.InvocationTargetException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.beanutils.BeanUtils;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingRelatedTaskDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;
import org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgenda;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussion;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasic;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingToTask;

/**
 * meeting dxo impl.
 * 
 */
public class MeetingDxoImpl implements MeetingDxo {

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.dxo.LiveMeetingDxo#convertIntoMeetingBasicDtoList(
     *      org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasic[])
     */
    public MeetingBasicDto[] convertMeetingBasicDtoList(MeetingBasic[] basicList) {
        MeetingBasicDto[] basicDtoList = new MeetingBasicDto[basicList.length];

        for (int i = 0; i < basicList.length; i++) {
            basicDtoList[i] = this.convertMeetingBasicDto(basicList[i]);
        }
        return basicDtoList;
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo#convertMeetingBasic(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto)
     */
    public MeetingBasic convertMeetingBasic(MeetingBasicDto dto) {
        if (dto == null) return null;
        try {
            MeetingBasic entity = new MeetingBasic();
            BeanUtils.copyProperties(entity, dto);
            return entity;
            
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }
    
    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo#convertIntoMeetingBasicDto(
     *      org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasic)
     */
    public MeetingBasicDto convertMeetingBasicDto(MeetingBasic basic) {
        SimpleDateFormat sdfParse = new SimpleDateFormat("yyyyMMdd");
        SimpleDateFormat sdfFormat = new SimpleDateFormat("yyyy-MM-dd");
        DateFormat dfMedium = new SimpleDateFormat("yyyy-MM-dd HH:mm");

        try {
            MeetingBasicDto dto = new MeetingBasicDto();
            BeanUtils.copyProperties(dto, basic);
            if (dto.getBody() != null) {
                dto.setBodyConvertedTextile(dto.getBody());
            }
            if (dto.getStartDate() != null) {
                dto.setStartDate(sdfFormat.format(sdfParse.parse(dto
                        .getStartDate())));
            }
            if (dto.getRegistered() != null) {
                dto.setRegisteredFormated(dfMedium.format(new SimpleDateFormat(
                        "yyyyMMddHHmmss").parse(basic.getRegistered())));
            }
            return dto;

        } catch (ParseException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo#convertIntoMeetingBasicDtoWithTask(
     *      org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasic[])
     */
    public MeetingBasicDto convertMeetingBasicDtoWithTask(
            MeetingBasic[] basicList) {
        long basicId = -1;
        MeetingBasicDto basicDto = null;
        ArrayList<MeetingBasicDto> arrayListBasic = new ArrayList<MeetingBasicDto>();
        ArrayList<MeetingRelatedTaskDto> arrayListLink = new ArrayList<MeetingRelatedTaskDto>();

        SimpleDateFormat sdfParse = new SimpleDateFormat("yyyyMMddHHmmss");
        SimpleDateFormat sdfFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");

        try {
            for (int i = 0; i < basicList.length; i++) {
                MeetingBasic basic = basicList[i];

                if (basicId != basic.getId()) {
                    if (basicId != -1) {
                        MeetingRelatedTaskDto[] linkDtoList = new MeetingRelatedTaskDto[arrayListLink
                                .size()];
                        arrayListLink.toArray(linkDtoList);
                        arrayListLink.clear();
                        arrayListLink.trimToSize();

                        basicDto.setMeetingLinks(linkDtoList);
                        arrayListBasic.add(basicDto);
                    }
                    basicDto = convertMeetingBasicDto(basic);
                    basicDto.setRegistered(sdfFormat.format(sdfParse
                            .parse(basicDto.getRegistered())));
                    basicDto.setModified(sdfFormat.format(sdfParse
                            .parse(basicDto.getModified())));
                    basicId = basic.getId();
                }
                if (basic.getTaskId() < 1)
                    continue;

                MeetingRelatedTaskDto relatedTask = new MeetingRelatedTaskDto();
                relatedTask.setId(basic.getTaskId());
                relatedTask.setTitle(basic.getTaskTitle());
                relatedTask.setActivityId(basic.getActivityId());
                relatedTask.setActivityName(basic.getActivityName());
                relatedTask.setActivityColor(basic.getColor());
                relatedTask.setStatusId(basic.getTaskStatusId());
                relatedTask.setClassid(basic.getTaskClassId());
                relatedTask.setPersonId(basic.getTaskPersonId());
                relatedTask.setPersonFirstName(basic.getTaskPersonFirstName());
                relatedTask.setPersonLastName(basic.getTaskPersonLastName());

                arrayListLink.add(relatedTask);
            }

            if (basicList != null && basicList.length > 0) {
                MeetingRelatedTaskDto[] linkDtoList = new MeetingRelatedTaskDto[arrayListLink
                        .size()];
                arrayListLink.toArray(linkDtoList);
                arrayListLink.clear();
                arrayListLink.trimToSize();

                basicDto.setMeetingLinks(linkDtoList);
                arrayListBasic.add(basicDto);
            }

            MeetingBasicDto[] basicDtoList = new MeetingBasicDto[arrayListBasic
                    .size()];
            arrayListBasic.toArray(basicDtoList);
            arrayListBasic.clear();
            arrayListBasic.trimToSize();

            return (basicDtoList == null || basicDtoList.length == 0 ? null
                    : basicDtoList[0]);

        } catch (ParseException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo#convertAgendaDtos(
     *      org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgenda[])
     */
    public MeetingAgendaDto[] convertAgendaDtos(MeetingAgenda[] agendas) {
        if (agendas == null || agendas.length == 0)
            return null;

        List<MeetingAgendaDto> list = new ArrayList<MeetingAgendaDto>();
        try {
            for (MeetingAgenda a : agendas) {
                MeetingAgendaDto dto = new MeetingAgendaDto();

                BeanUtils.copyProperties(dto, a);
                list.add(dto);
            }
            return list.toArray(new MeetingAgendaDto[list.size()]);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo#convertIntoMeetingAgendaDtoList(
     *      org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgenda[])
     */
    public MeetingAgendaDto[] convertMeetingAgendaDtoList(
            MeetingAgenda[] agendas) {
        long agendaId = -1;
        MeetingAgendaDto agendaDto = null;
        MeetingAgendaDiscussionDto discussionDto = null;
        MeetingAgendaDiscussion discussion = null;

        ArrayList<MeetingAgendaDto> arrayListAgenda = new ArrayList<MeetingAgendaDto>();
        ArrayList<MeetingAgendaDiscussionDto> arrayListDiscussion = new ArrayList<MeetingAgendaDiscussionDto>();
        for (int i = 0; i < agendas.length; i++) {
            MeetingAgenda agenda = agendas[i];
            if (agendaId != agenda.getId()) {
                if (agendaId != -1) {
                    MeetingAgendaDiscussionDto[] discussionDtoList = new MeetingAgendaDiscussionDto[arrayListDiscussion
                            .size()];
                    arrayListDiscussion.toArray(discussionDtoList);
                    arrayListDiscussion.clear();
                    arrayListDiscussion.trimToSize();

                    agendaDto.setDiscussionList(discussionDtoList);
                    arrayListAgenda.add(agendaDto);
                }
                agendaDto = this.convertMeetingAgendaDto(agenda);
                agendaId = agenda.getId();
            }

            // there is no discussion(task and comment) in agenda
            if (agenda.getDiscussionId() < 1)
                continue;
            if (String.valueOf(MeetingAgendaDiscussion.TYPE_TODO).equals(
                    agenda.getType())
                    && agenda.getTaskTitle() == null) {
                continue;
            }
            discussion = new MeetingAgendaDiscussion();
            discussion.setId(agenda.getDiscussionId());
            discussion.setAgendaId(agenda.getId());
            discussion.setType(agenda.getType());
            discussion.setPostedUserId(agenda.getDiscussionPostedUserId());
            discussion.setFirstName(agenda.getDiscussionFirstName());
            discussion.setLastName(agenda.getDiscussionLastName());
            discussion.setRegistered(agenda.getDiscussionRegistered());
            discussion.setModified(agenda.getDiscussionModified());
            discussion.setModifiedUserId(agenda.getDiscussionModifiedUserId());

            discussionDto = this.convertMeetingAgendaDiscussionDto(discussion);
            this.assignToDisscussionDtoByDiscussionsType(agenda, discussionDto,
                    agendaDto);

            arrayListDiscussion.add(discussionDto);
        }
        if (agendas != null && agendas.length > 0) {
            MeetingAgendaDiscussionDto[] discussionDtoList = new MeetingAgendaDiscussionDto[arrayListDiscussion
                    .size()];
            arrayListDiscussion.toArray(discussionDtoList);
            arrayListDiscussion.clear();
            arrayListDiscussion.trimToSize();

            agendaDto.setDiscussionList(discussionDtoList);
            arrayListAgenda.add(agendaDto);
        }
        MeetingAgendaDto[] agendaDtoList = new MeetingAgendaDto[arrayListAgenda
                .size()];
        arrayListAgenda.toArray(agendaDtoList);
        arrayListAgenda.clear();
        arrayListAgenda.trimToSize();
        return agendaDtoList;
    }

    /**
     * assign to disscussionDto by discussion type.
     * 
     * @param agenda
     * @param discussionDto
     * @param agendaDto
     * @return
     */
    private void assignToDisscussionDtoByDiscussionsType(MeetingAgenda agenda,
            MeetingAgendaDiscussionDto discussionDto, MeetingAgendaDto agendaDto) {

        int discussionType = Integer.parseInt(discussionDto.getType());
        switch (discussionType) {
            case MeetingAgendaDiscussion.TYPE_TODO:
                discussionDto.setTaskId(agenda.getTaskId());
                discussionDto.setTaskTitle(agenda.getTaskTitle());
                discussionDto.setTaskDescription(agenda.getTaskDescription());
                discussionDto.setTaskDescriptionTextile(agenda.getTaskDescription());
                discussionDto.setTaskClassificationId(agenda.getTaskClassificationId());
                discussionDto.setTaskClassificationValue(agenda.getTaskClassificationValue());
                discussionDto.setTaskStatusId(agenda.getTaskStatusId());
                discussionDto.setTaskPersonId(agenda.getTaskPersonId());
                discussionDto.setTaskPersonFirstName(agenda.getTaskPersonFirstName());
                discussionDto.setTaskPersonLastName(agenda.getTaskPersonLastName());
                SimpleDateFormat sdfParse = new SimpleDateFormat("yyyyMMdd");
                SimpleDateFormat sdfFormat = new SimpleDateFormat("yyyy-MM-dd");
                
                if (agenda.getTaskDeadLine() != null 
                    && !"99999999".equals(agenda.getTaskDeadLine())) {
                    try {
                        discussionDto.setTaskDeadLine(
                            sdfFormat.format(sdfParse.parse(agenda.getTaskDeadLine())));
                    } catch (ParseException e) {
                        discussionDto.setTaskDeadLine("");
                    }
                }
                agendaDto.setTaskRows(agendaDto.getTaskRows() + 1);
                break;

            case MeetingAgendaDiscussion.TYPE_COMMENT:
            case MeetingAgendaDiscussion.TYPE_CONCLUSION:
                discussionDto.setBody(agenda.getDiscussionBody());

                if (agenda.getDiscussionBody() != null) {
                    discussionDto.setBodyConvertedTextile(agenda
                            .getDiscussionBody());
                }
                agendaDto.setCommentRows(agendaDto.getCommentRows() + 1);
                break;
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo#convertMeetingAgenda(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto)
     */
    public MeetingAgenda convertMeetingAgenda(MeetingAgendaDto dto) {
        if (dto == null) return null;
        try {
            MeetingAgenda agenda = new MeetingAgenda();
            BeanUtils.copyProperties(agenda, dto);
            
            return agenda;    
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }
    
    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo#convertIntoMeetingAgendaDto(
     *      org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgenda)
     */
    public MeetingAgendaDto convertMeetingAgendaDto(MeetingAgenda agenda) {
        if (agenda == null) return null;
        
        try {
            MeetingAgendaDto dto = new MeetingAgendaDto();
            BeanUtils.copyProperties(dto, agenda);
            dto.setPid(agenda.getId());

            if (dto.getBody() != null) {
                dto.setBodyConvertedTextile(dto.getBody());
            }
            SimpleDateFormat sdfParse = new SimpleDateFormat("yyyyMMddHHmmss");
            SimpleDateFormat sdfFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");

            String modified = "";
            if (dto.getModified() != null && !"".equals(dto.getModified())) {
                modified = sdfFormat.format(sdfParse.parse(dto.getModified()));
            }
            dto.setModified(modified);

            String registered = "";
            if (dto.getRegistered() != null && !"".equals(dto.getRegistered())) {
                registered = sdfFormat.format(sdfParse.parse(dto.getRegistered()));
            }
            dto.setRegistered(registered);
            return dto;

        } catch (ParseException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo#convertIntoMeetingAgendaDiscussion(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto)
     */
    public MeetingAgendaDiscussion convertMeetingAgendaDiscussion(
            MeetingAgendaDiscussionDto agendaCommentDto) {

        MeetingAgendaDiscussion topicComment = new MeetingAgendaDiscussion();
        try {
            BeanUtils.copyProperties(topicComment, agendaCommentDto);
            return topicComment;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo#convertIntoMeetingAgendaDiscussionDto(
     *      org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussion)
     */
    public MeetingAgendaDiscussionDto convertMeetingAgendaDiscussionDto(
            MeetingAgendaDiscussion discussion) {

        MeetingAgendaDiscussionDto dto = new MeetingAgendaDiscussionDto();
        try {
            BeanUtils.copyProperties(dto, discussion);
            dto.setPid(discussion.getId());

            if (dto.getBody() != null) {
                dto.setBodyConvertedTextile(dto.getBody());
            }
            SimpleDateFormat sdfParse = new SimpleDateFormat("yyyyMMddHHmmss");
            SimpleDateFormat sdfFormat =
                new SimpleDateFormat("yyyy-MM-dd HH:mm");

            String modified = "";
            if (dto.getModified() != null && !"".equals(dto.getModified())) {
                modified = sdfFormat.format(sdfParse.parse(dto.getModified()));
            }
            dto.setModified(modified);

            if (discussion.getTaskDeadLine() != null
                    && !"99999999".equals(discussion.getTaskDeadLine())) {
                try {
                    dto.setTaskDeadLine(sdfFormat.format(sdfParse
                            .parse(discussion.getTaskDeadLine())));
                } catch (ParseException e) {
                }
            }
            return dto;
        } catch (ParseException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo#convertIntoMeetingToTask(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto)
     */
    public MeetingToTask convertMeetingToTask(MeetingToTaskDto toTaskDto) {
        if (toTaskDto == null)
            return null;
        try {
            MeetingToTask liveMeetingToTask = new MeetingToTask();
            BeanUtils.copyProperties(liveMeetingToTask, toTaskDto);
            return liveMeetingToTask;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo#convertIntoMeetingToTaskDto(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTask)
     */
    public MeetingToTaskDto convertMeetingToTaskDto(MeetingToTask toTask) {
        if (toTask == null)
            return null;
        try {
            MeetingToTaskDto meetingToTaskDto = new MeetingToTaskDto();
            BeanUtils.copyProperties(meetingToTaskDto, toTask);

            SimpleDateFormat sdfParse = new SimpleDateFormat("yyyyMMdd");
            SimpleDateFormat sdfFormat = new SimpleDateFormat("yyyy-MM-dd");

            String deadline = "";
            if (meetingToTaskDto.getDeadline() != null
                    && !"99999999"
                            .equals(meetingToTaskDto.getDeadline().trim())) {
                deadline = sdfFormat.format(sdfParse.parse(meetingToTaskDto
                        .getDeadline()));
            }
            meetingToTaskDto.setDeadline(deadline);

            String registered = "";
            sdfParse = new SimpleDateFormat("yyyyMMddHHmmss");
            sdfFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");

            if (meetingToTaskDto.getRegistered() != null
                    && !"".equals(meetingToTaskDto.getRegistered())) {
                registered = sdfFormat.format(sdfParse.parse(meetingToTaskDto
                        .getRegistered()));
            }
            meetingToTaskDto.setRegistered(registered);
            return meetingToTaskDto;
        } catch (ParseException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo#convertIntoMeetingToTaskDto(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTask[])
     */
    public MeetingToTaskDto[] convertMeetingToTaskDtoList(
            MeetingToTask[] toTaskList) {
        if (toTaskList == null)
            return new MeetingToTaskDto[0];

        List<MeetingToTaskDto> meetingToTaskList = new ArrayList<MeetingToTaskDto>();
        for (MeetingToTask t : toTaskList) {
            MeetingToTaskDto dto = this.convertMeetingToTaskDto(t);
            meetingToTaskList.add(dto);
        }
        return meetingToTaskList.toArray(new MeetingToTaskDto[meetingToTaskList
                .size()]);
    }
}
