/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.helper.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingRelatedTaskDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussion;
import org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * meeting helper impl.
 * 
 * @author yosukehara
 * 
 */
public class MeetingHelperImpl implements MeetingHelper {

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper#createActivitiesJsonForTask(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto[])
     */
    public String createActivitiesJsonForTask(ActivityDto[] activities) {
        if (activities == null || activities.length == 0)
            return null;

        List<Map<?, ?>> listJson = new ArrayList<Map<?, ?>>();
        for (ActivityDto c : activities) {
            Map<String, String> map = new HashMap<String, String>();

            map.put("id", String.valueOf(c.getId()));
            map.put("value", c.getName());
            map.put("color", c.getColorValue());
            listJson.add(map);
        }
        JSONArray jsonArray = JSONArray.fromObject(listJson.toArray(new Map[listJson.size()]));
        return jsonArray.toString();
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper#createMeetingList(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto[], int,
     *      int, int)
     */
    public String createMeetingList(MeetingBasicDto[] liveMeetingBasicDtoList, int totalRecords,
            int currentPageNumber, int recordPerPage) {

        List<Map<?, ?>> listJson = new ArrayList<Map<?, ?>>();
        try {
            for (MeetingBasicDto basicDto : liveMeetingBasicDtoList) {
                Map<String, String> mapBasic = new HashMap<String, String>();
                mapBasic.put("id", String.valueOf(basicDto.getId()));
                mapBasic.put("title", basicDto.getTitle());
                mapBasic.put("activityId", String.valueOf(basicDto.getActivityId()));
                mapBasic.put("activityName", basicDto.getActivityName());
                mapBasic.put("activityColor", basicDto.getColor());
                mapBasic.put("postedByLastName", basicDto.getLastName());
                mapBasic.put("postedByFirstName", basicDto.getFirstName());
                mapBasic.put("postedAt",
                    basicDto.getRegistered().substring(0, 4)
                    + "-"
                    + basicDto.getRegistered().substring(4, 6)
                    + "-"
                    + basicDto.getRegistered().substring(6, 8));
                mapBasic.put("statusId", String.valueOf(basicDto.getStatusId()));
                mapBasic.put("startDate", basicDto.getStartDate());
                listJson.add(mapBasic);
            }
            JSONArray jsonArray = JSONArray.fromObject(listJson.toArray(new Map[listJson.size()]));

            // set page info.
            HashMap<String, Object> page = new HashMap<String, Object>();
            page.put("results", new Integer(totalRecords));
            Integer totalPages =
                (totalRecords == 0 
                    ? new Integer(1)
                    : new Integer(
                        totalRecords / recordPerPage + (totalRecords % recordPerPage == 0 ? 0 : 1)));
            page.put("totalPages", totalPages);
            page.put("currentPageNumber",
                totalRecords == 0 
                    ? new Integer(1)
                    : new Integer(currentPageNumber));

            JSONObject json = JSONObject.fromObject(page);
            return "({meetings:" + jsonArray.toString() + ", page:" + json.toString() + "})";

        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper#createMeetingBallonJson(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto)
     */
    public String createMeetingBallonJson(MeetingBasicDto basicDto) {
        if (basicDto == null)
            return "";
        Map<String, Object> mapBasic = new HashMap<String, Object>();
        mapBasic.put("id", String.valueOf(basicDto.getId()));
        mapBasic.put("title", basicDto.getTitle());
        mapBasic.put("activityId", String.valueOf(basicDto.getActivityId()));
        mapBasic.put("activityColor", basicDto.getColor());
        mapBasic.put("activityName", basicDto.getActivityName());
        mapBasic.put("postedBy", basicDto.getLastName() + "," + basicDto.getFirstName());
        mapBasic.put("postedAt", basicDto.getModified());
        mapBasic.put("statusId", String.valueOf(basicDto.getStatusId()));
        mapBasic.put("startDate", String.valueOf(basicDto.getStartDate()));
        mapBasic.put("description", basicDto.getBodyConvertedTextile());

        List<Map<?, ?>> listJson2 = new ArrayList<Map<?, ?>>();
        MeetingRelatedTaskDto[] links = basicDto.getMeetingLinks();
        for (MeetingRelatedTaskDto link : links) {
            Map<String, String> relatedTask = new HashMap<String, String>();
            relatedTask.put("id", String.valueOf(link.getId()));
            relatedTask.put("class", "T");
            relatedTask.put("type", "2");
            relatedTask.put("title", link.getTitle());
            relatedTask.put("taskStatusId", String.valueOf(link.getStatusId()));
            relatedTask.put("taskClassId", String.valueOf(link.getClassId()));
            relatedTask.put("taskActivityId", String.valueOf(link.getActivityId()));
            relatedTask.put("taskActivityColor", link.getActivityColor());
            relatedTask.put("taskPersonId", String.valueOf(link.getPersonId()));
            relatedTask.put("taskPersonFirstName", link.getPersonFirstName());
            relatedTask.put("taskPersonLastName", link.getPersonLastName());

            listJson2.add(relatedTask);
        }
        mapBasic.put("links", listJson2);
        return JSONObject.fromObject(mapBasic).toString();
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper#createAgendasJson(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto,
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto[],
     *      org.seasar.tugboat.gtd.contents.user.dto.UserDto, boolean)
     */
    public String createAgendasJson(MeetingBasicDto meetingBasicDto,
            MeetingAgendaDto[] liveMeetingAgendaDto, UserDto user, boolean hasDiscussion) {

        List<Map<?, ?>> listJson = new ArrayList<Map<?, ?>>();
        if (liveMeetingAgendaDto != null && liveMeetingAgendaDto.length > 0) {
            for (MeetingAgendaDto agenda : liveMeetingAgendaDto) {
                HashMap<String, Object> mapAgenda = new HashMap<String, Object>();
                mapAgenda.put("id", agenda.getId());
                mapAgenda.put("pid", agenda.getId());
                mapAgenda.put("priority", agenda.getPriority());
                mapAgenda.put("status", agenda.getStatus());
                mapAgenda.put("postedUserId", agenda.getPostedUserId());
                mapAgenda.put("firstName", agenda.getFirstName());
                mapAgenda.put("lastName", agenda.getLastName());
                mapAgenda.put("body", agenda.getBody());
                mapAgenda.put("registered", agenda.getRegistered());
                mapAgenda.put("modified", agenda.getModified());
                mapAgenda.put("commentCount", agenda.getCommentRows());
                mapAgenda.put("taskCount", agenda.getTaskRows());

                if (hasDiscussion) {
                    List<Map<?, ?>> listJson2 = new ArrayList<Map<?, ?>>();
                    MeetingAgendaDiscussionDto[] discussionsDtos = agenda.getDiscussionList();

                    for (MeetingAgendaDiscussionDto discussionsDto : discussionsDtos) {
                        HashMap<String, Object> mapDiscussions = new HashMap<String, Object>();
                        mapDiscussions.put("id", discussionsDto.getId());
                        mapDiscussions.put("pid", discussionsDto.getId());
                        mapDiscussions.put("type", discussionsDto.getType());
                        mapDiscussions.put("postedUserId", discussionsDto.getPostedUserId());
                        mapDiscussions.put("firstName", discussionsDto.getFirstName());
                        mapDiscussions.put("lastName", discussionsDto.getLastName());
                        mapDiscussions.put("modified", discussionsDto.getModified());

                        int discussionType = Integer.parseInt(discussionsDto.getType());

                        switch (discussionType) {
                            case MeetingAgendaDiscussion.TYPE_COMMENT:
                            case MeetingAgendaDiscussion.TYPE_CONCLUSION:
                                mapDiscussions.put("body", discussionsDto
                                        .getBodyConvertedTextile());
                                mapDiscussions.put("bodyTextile", discussionsDto.getBody());
                                break;
                            case MeetingAgendaDiscussion.TYPE_TODO:
                                mapDiscussions.put("taskId", discussionsDto.getTaskId());
                                mapDiscussions.put("taskDeadLine", discussionsDto.getTaskDeadLine());
                                mapDiscussions.put("taskPersonId", discussionsDto.getTaskPersonId());
                                mapDiscussions.put("taskPersonFirstName", discussionsDto.getTaskPersonFirstName());
                                mapDiscussions.put("taskPersonLastName", discussionsDto.getTaskPersonLastName());
                                mapDiscussions.put("taskClassificationId", discussionsDto.getTaskClassificationId());
                                mapDiscussions.put("taskClassificationValue", discussionsDto.getTaskClassificationValue());
                                mapDiscussions.put("taskTitle", discussionsDto.getTaskTitle());
                                mapDiscussions.put("taskDescription", discussionsDto.getTaskDescription());
                                mapDiscussions.put("taskDescriptionTextile", discussionsDto.getTaskDescriptionTextile());
                                mapDiscussions.put("taskStatusId", discussionsDto.getTaskStatusId());
                                break;
                        }
                        listJson2.add(mapDiscussions);
                    }
                    mapAgenda.put("discussionList", listJson2);
                }
                listJson.add(mapAgenda);
            }
        }
        HashMap<String, Object> mapLoginUser = new HashMap<String, Object>();
        mapLoginUser.put("id", String.valueOf(user.getUserId()));
        mapLoginUser.put("firstName", user.getFirstName());
        mapLoginUser.put("lastName", user.getLastName());

        HashMap<String, Object> mapOnLoadData = new HashMap<String, Object>();
        if (meetingBasicDto != null) {
            mapOnLoadData.put("meetingBasicJson", this.createBasicMap(meetingBasicDto));
        }
        mapOnLoadData.put("loginUserJson", mapLoginUser);
        mapOnLoadData.put("sectionAgendaListJson", listJson.toArray(new Map[listJson.size()]));

        JSONObject jsonArray = JSONObject.fromObject(mapOnLoadData);
        return jsonArray.toString();
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper#createBasicJson(
     *      org.seasar.tugboat.gtd.contents.meeting.MeetingBasicDto )
     */
    public String createBasicJson(MeetingBasicDto basicDto) {
        JSONObject jsonObject = JSONObject.fromObject(this.createBasicMap(basicDto));
        return jsonObject.toString();
    }

    /**
     * create basic json.
     * 
     * @param basicDto
     * @return
     */
    private Map<?, ?> createBasicMap(MeetingBasicDto basicDto) {
        if (basicDto == null)
            return null;
        HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("id", String.valueOf(basicDto.getId()));
        map.put("activityId", String.valueOf(basicDto.getActivityId()));
        map.put("activityName", basicDto.getActivityName());
        map.put("activityColor", basicDto.getColor());
        map.put("statusId", String.valueOf(basicDto.getStatusId()));
        map.put("title", basicDto.getTitle());
        map.put("description", basicDto.getBodyConvertedTextile());
        map.put("descriptionTextile", basicDto.getBody());
        
        map.put("activeAgendas", basicDto.getActiveAgendas());
        map.put("doneAgendas", basicDto.getDoneAgendas());
        
        map.put("startDate", basicDto.getStartDate());
        map.put("registered", basicDto.getRegisteredFormated());
        map.put("firstName", basicDto.getFirstName());
        map.put("lastName", basicDto.getLastName());
        return map;
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper#createAgendaJson(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto )
     */
    public String createAgendaJson(MeetingAgendaDto agendaDto) {
        HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("id", agendaDto.getId());
        map.put("pid", agendaDto.getPid());
        map.put("priority", agendaDto.getPriority());
        map.put("status", agendaDto.getStatus());
        map.put("postedUserId", agendaDto.getPostedUserId());
        map.put("firstName", agendaDto.getFirstName());
        map.put("lastName", agendaDto.getLastName());
        map.put("body", agendaDto.getBodyConvertedTextile());
        map.put("bodyTextile", agendaDto.getBody());
        map.put("modified", agendaDto.getModified());
        map.put("commentCount", new Integer(0));
        map.put("taskCount", new Integer(0));
        return JSONObject.fromObject(map).toString();
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper#createAgendaDiscussionJson(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto)
     */
    public String createAgendaDiscussionJsonWithMeetingBasic(
            MeetingBasicDto basicDto,
            MeetingAgendaDiscussionDto agendaCommentDto, TaskDto taskDto,
            TaskRelationDto relationsDto) {

        HashMap<String, Object> map = new HashMap<String, Object>();
        map.put("id", agendaCommentDto.getId());
        map.put("pid", agendaCommentDto.getPid());
        map.put("type", agendaCommentDto.getType());
        map.put("postedUserId", agendaCommentDto.getPostedUserId());
        map.put("firstName", agendaCommentDto.getFirstName());
        map.put("lastName", agendaCommentDto.getLastName());
        map.put("modified", agendaCommentDto.getModified());

        switch (Integer.parseInt(agendaCommentDto.getType())) {
            case 1:// comment
            case 2:// conclusion
                map.put("body", agendaCommentDto.getBodyConvertedTextile());
                map.put("bodyTextile", agendaCommentDto.getBody());
                break;

            case 5:// todo(task)
                map.put("taskDeadLine", agendaCommentDto.getTaskDeadLine());
                map.put("taskPersonId", agendaCommentDto.getTaskPersonId());
                map.put("taskPersonFirstName", agendaCommentDto.getTaskPersonFirstName());
                map.put("taskPersonLastName", agendaCommentDto.getTaskPersonLastName());
                map.put("taskClassificationId", agendaCommentDto.getTaskClassificationId());
                map.put("taskClassificationValue", agendaCommentDto
                        .getTaskClassificationValue());
                map.put("taskTitle", taskDto.getTitle());
                map.put("taskDescription", taskDto.getDescription());
                map.put("taskDescriptionTextile", taskDto.getDescriptionTextile());
                map.put("taskStatusId", taskDto.getStatusId());
                break;
        }
        HashMap<String, Object> mapResponse = new HashMap<String, Object>();
        mapResponse.put("meetingBasicJson", this.createBasicMap(basicDto));
        mapResponse.put("meetingDiscussionJson", map);
        return JSONObject.fromObject(mapResponse).toString();
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper#createSidebarMembers(
     *      org.seasar.tugboat.gtd.container.dto.ActivityMembersDto[] )
     */
    public String createSidebarMembers(ActivityMemberDto[] activityMembersDto) {
        List<Map<?, ?>> listJson = new ArrayList<Map<?, ?>>();
        for (ActivityMemberDto c : activityMembersDto) {
            Map<String, String> map = new HashMap<String, String>();
            map.put("id", String.valueOf(c.getUserId()));
            map.put("lastName", c.getLastName());
            map.put("firstName", c.getFirstName());
            map.put("photo", c.getPhotoImgUrl());

            listJson.add(map);
        }
        JSONArray jsonArray = JSONArray.fromObject(listJson.toArray(new Map[listJson.size()]));
        return jsonArray.toString();
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper#createSidebarTasks(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto[], int,
     *      int ,int )
     */
    public String createSidebarTasks(MeetingToTaskDto[] liveMeetingToTaskDtoList, int totalRecords,
            int currentPageNumber, int recordPerPage) {

        List<Map<?, ?>> listJson = new ArrayList<Map<?, ?>>();
        for (MeetingToTaskDto dto : liveMeetingToTaskDtoList) {
            Map<String, String> mapBasic = new HashMap<String, String>();
            mapBasic.put("statusId", String.valueOf(dto.getStatusId()));
            mapBasic.put("classificationId", String.valueOf(dto.getClassificationId()));
            mapBasic.put("taskId", String.valueOf(dto.getTaskId()));
            mapBasic.put("title", dto.getTitle());
            mapBasic.put("deadLine", dto.getDeadline());
            mapBasic.put("firstName", dto.getFirstName());
            mapBasic.put("lastName", dto.getLastName());
            mapBasic.put("createdAt", dto.getRegistered());
            listJson.add(mapBasic);
        }
        JSONArray jsonArray =
            JSONArray.fromObject(listJson.toArray(new Map[listJson.size()]));

        HashMap<String, Object> page = new HashMap<String, Object>();
        page.put("results", new Integer(totalRecords));
        Integer totalPages =
            (totalRecords == 0 
                ? new Integer(1)
                : new Integer(
                    totalRecords / recordPerPage + (totalRecords % recordPerPage == 0 ? 0 : 1)));
        page.put("totalPages", totalPages);
        page.put("currentPageNumber",
            totalRecords == 0
            ? new Integer(1)
            : new Integer(currentPageNumber));

        JSONObject json = JSONObject.fromObject(page);
        String result = "({tasks:" + jsonArray.toString() + ", page:" + json.toString() + "})";
        return result;
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper#createBrowserRegisterJson(
     *      org.seasar.tugboat.gtd.container.dto.ActivityMembersDto[])
     */
    public String createBrowserRegisterJson(ActivityMemberDto[] activityMembersDtoList) {
        List<Map<?, ?>> listJson = new ArrayList<Map<?, ?>>();
        for (ActivityMemberDto memberDto : activityMembersDtoList) {
            Map<String, Object> mapMember = new HashMap<String, Object>();
            mapMember.put("id", memberDto.getUserId());
            mapMember.put("value", memberDto.getLastName() + "," + memberDto.getFirstName());
            listJson.add(mapMember);
        }
        JSONArray jsonArray = JSONArray.fromObject(listJson.toArray(new Map[listJson.size()]));
        return jsonArray.toString();
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper#createFolderMeetingCountJson(
     *      int)
     */
    public String createFolderMeetingCountJson(MeetingBasicDto basicDto) {
        HashMap<String, String> map = new HashMap<String, String>();
        map.put("libraryCount", String.valueOf(basicDto.getLibraryCount()));
        map.put("archivesCount", String.valueOf(basicDto.getArchivesCount()));

        JSONObject jsonObject = JSONObject.fromObject(map);
        return jsonObject.toString();
    }
}
