/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.page.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.commons.beanutils.BeanUtils;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityService;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto;
import org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper;
import org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax;
import org.seasar.tugboat.gtd.contents.meeting.service.MeetingService;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * meeting ajax impl.
 * 
 */
@FormAnnotation(formId = "meetingAjax", isAuthorizationCheck = true, isDirectAccessCheck = true)
public class MeetingAjaxImpl implements MeetingAjax {

    /* query param status id */
    private String queryStatusId_;

    public String getQueryStatusId() {
        return queryStatusId_;
    }

    public void setQueryStatusId(String queryStatusId) {
        this.queryStatusId_ = queryStatusId;
    }

    /* query param activity id */
    private String queryActivityId_;

    public String getQueryActivityId() {
        return queryActivityId_;
    }

    public void setQueryActivityId(String queryActivityId) {
        this.queryActivityId_ = queryActivityId;
    }

    /* query param register id */
    private String queryRegisterId_;

    public String getQueryRegisterId() {
        return queryRegisterId_;
    }

    public void setQueryRegisterId(String queryRegisterId) {
        this.queryRegisterId_ = queryRegisterId;
    }

    /* current page number */
    private int currentPageNumber_;

    public int getCurrentPageNumber() {
        return currentPageNumber_;
    }

    public void setCurrentPageNumber(int currentPageNumber) {
        this.currentPageNumber_ = currentPageNumber;
    }

    /* offset */
    private int offset_;

    public int getOffset() {
        return offset_;
    }

    public void setOffset(int offset) {
        this.offset_ = offset;
    }

    /* limit */
    private int limit_;

    public int getLimit() {
        return limit_;
    }

    public void setLimit(int limit) {
        this.limit_ = limit;
    }

    /* activity id */
    private long activityId_;

    public long getActivityId() {
        return activityId_;
    }

    public void setActivityId(long activityId) {
        this.activityId_ = activityId;
    }

    /* title */
    private String title_;

    public String getTitle() {
        return title_;
    }

    public void setTitle(String title) {
        this.title_ = title;
    }

    /* descriptionTextile */
    private String descriptionTextile_;

    public String getDescriptionTextile() {
        return descriptionTextile_;
    }

    public void setDescriptionTextile(String descriptionTextile) {
        this.descriptionTextile_ = descriptionTextile;
    }

    /* basic id */
    private long id_;

    public long getId() {
        return id_;
    }

    public void setId(long id) {
        this.id_ = id;
    }

    /* meeting Helper */
    private MeetingHelper meetingHelper_;

    public void setMeetingHelper(MeetingHelper meetingHelper) {
        this.meetingHelper_ = meetingHelper;
    }

    /* meeting service */
    private MeetingService meetingService_;

    public void setMeetingService(MeetingService meetingService) {
        this.meetingService_ = meetingService;
    }

    /* activity members service */
    private ActivityMembersService activityMemberService_;

    public void setActivityMemberService(ActivityMembersService activityMemberService) {
        this.activityMemberService_ = activityMemberService;
    }

    /* activity service */
    private ActivityService activitySerice_;

    public void setActivityService(ActivityService activityService) {
        this.activitySerice_ = activityService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#getFolderMeetingCountJson()
     */
    public String getFolderMeetingCountJson() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        MeetingBasicDto conditionDto = new MeetingBasicDto();
        conditionDto.setUserId(sessionLoginUser.getUserId());
        conditionDto.setQueryStatusIds(splitTokenToInteger(this.queryStatusId_, ","));
        conditionDto.setQueryActivityIds(splitTokenToLong(this.queryActivityId_, ","));
        conditionDto.setQueryRegisterIds(splitTokenToInteger(this.queryRegisterId_, ","));

        MeetingBasicDto basicDto = this.meetingService_.getDefaultCountByFolder(conditionDto);

        return this.meetingHelper_.createFolderMeetingCountJson(basicDto);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#getMeetingListByQueryParams()
     */
    public String getMeetingListByQueryParams() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        MeetingBasicDto conditionDto = new MeetingBasicDto();
        conditionDto.setUserId(sessionLoginUser.getUserId());
        conditionDto.setQueryStatusIds(splitTokenToInteger(this.queryStatusId_, ","));
        conditionDto.setQueryActivityIds(splitTokenToLong(this.queryActivityId_, ","));
        conditionDto.setQueryRegisterIds(splitTokenToInteger(this.queryRegisterId_, ","));
        conditionDto.setOffset(this.offset_);
        conditionDto.setLimit(this.limit_);

        MeetingBasicDto[] basicDtoList = this.meetingService_.getMeetingBasicList(conditionDto);
        MeetingBasicDto basicDto = this.meetingService_.getMeetingCountByQueryParams(conditionDto);

        String result = meetingHelper_.createMeetingList(basicDtoList, basicDto.getRowCount(),
                this.currentPageNumber_, this.limit_);
        return result;
    }

    /**
     * split token
     * 
     * @param tokens
     * @param delimiter
     * 
     * @return
     */
    private Integer[] splitTokenToInteger(String tokens, String delimiter) {
        if (tokens == null)
            return null;

        StringTokenizer tokenClass = new StringTokenizer(tokens, delimiter);
        List<Integer> paramList = new ArrayList<Integer>();

        String classId;
        while (tokenClass.hasMoreTokens()) {
            classId = tokenClass.nextToken();
            if ("0".equals(classId))
                continue;
            try {
                paramList.add(new Integer(classId));
            } catch (NumberFormatException e) {
                throw new SystemUnexpectedRuntimeException(e);
            }
        }
        return paramList.size() == 0 ? null : paramList.toArray(new Integer[paramList.size()]);
    }

    /**
     * split token to long.
     * 
     * @param tokens
     * @param delimiter
     * @return
     */
    private Long[] splitTokenToLong(String tokens, String delimiter) {
        if (tokens == null)
            return null;

        StringTokenizer tokenClass = new StringTokenizer(tokens, delimiter);
        List<Long> paramList = new ArrayList<Long>();

        String classId;
        while (tokenClass.hasMoreTokens()) {
            classId = tokenClass.nextToken();
            if ("0".equals(classId))
                continue;
            try {
                paramList.add(new Long(classId));
            } catch (NumberFormatException e) {
                throw new SystemUnexpectedRuntimeException(e);
            }
        }
        return paramList.size() == 0 ? null : paramList.toArray(new Long[paramList.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#getActivitiesJson()
     */
    public String getActivitiesJson() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        ActivityDto[] activities = this.activitySerice_.getOwnActivities(sessionLoginUser
                .getUserId());

        return this.meetingHelper_.createActivitiesJsonForTask(activities);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#getRegisterJson()
     */
    public String getRegisterJson() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        ActivityMemberDto[] activityMembersDto = this.activityMemberService_
                .getMembersListRelatedToUser(sessionLoginUser.getUserId());

        return meetingHelper_.createBrowserRegisterJson(activityMembersDto);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#getMeetingBalloonJson()
     */
    public String getMeetingBalloonJson() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        MeetingBasicDto basicDto =
            this.meetingService_.getMeetingBasicWithTask(
                this.id_, sessionLoginUser.getUserId());

        return meetingHelper_.createMeetingBallonJson(basicDto);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#add()
     */
    public long add() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        try {
            MeetingBasicDto basicDto = new MeetingBasicDto();
            BeanUtils.copyProperties(basicDto, this);
            basicDto.setBody(this.getDescriptionTextile());

            basicDto.setPostedUserId(sessionLoginUser.getUserId());
            basicDto.setUserId(sessionLoginUser.getUserId());

            MeetingBasicDto resultBasicDto = this.meetingService_.addMeeting(basicDto);
            return resultBasicDto.getId();
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }
}
