/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.page.impl;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.HashMap;

import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.commons.util.SystemConfiguration;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;
import org.seasar.tugboat.gtd.contents.meeting.page.MeetingDetailAction;
import org.seasar.tugboat.gtd.contents.meeting.service.MeetingService;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * meeting detail action impl.
 * 
 * @author naoya.saito
 * 
 */
@FormAnnotation(formId = "meetingDetailAction", isAuthorizationCheck = true, isDirectAccessCheck = true, errorPage = "error")
public class MeetingDetailActionImpl implements MeetingDetailAction {

    /* meeting basic id */
    private String id_;

    public void setId(String id) {
        this.id_ = id;
    }

    /* action */
    private String action_;

    public void setAction(String action) {
        this.action_ = action;
    }

    /* meeting basic dto */
    private MeetingBasicDto meetingBasicDto_;

    public MeetingBasicDto getMeetingBasicDto() {
        return meetingBasicDto_;
    }

    public void setMeetingBasicDto(MeetingBasicDto meetingBasicDto) {
        this.meetingBasicDto_ = meetingBasicDto;
    }

    /* http response */
    private HttpServletResponse response_;

    public void setHttpServletResponse(HttpServletResponse response) {
        this.response_ = response;
    }

    /* meeting service */
    private MeetingService meetingService_;

    public void setMeetingService(MeetingService meetingService) {
        this.meetingService_ = meetingService;
    }

    /* activity member service */
    private ActivityMembersService activityMemberService_;

    public void setActivityMemberService(ActivityMembersService activityMemberService) {
        this.activityMemberService_ = activityMemberService;
    }

    /* user name */
    private String userName_;

    public String getUserName() {
        return userName_;
    }

    public void setUserName(String userName) {
        this.userName_ = userName;
    }

    /* version */
    private String version_;

    public String getVersion() {
        return version_;
    }

    public void setVersion(String version) {
        this.version_ = version;
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAction#initialize()
     */
    public String initialize() {
        if (this.action_ != null && !"".equals(this.action_)) {
            return this.downloadAgenda();
        }
        MeetingBasicDto dto = new MeetingBasicDto();
        try {
            dto.setId(Integer.parseInt(this.id_));
            this.setMeetingBasicDto(dto);

            UserDto sessionLoginUser = SessionUtil.getLoginUser();
            this.userName_ = sessionLoginUser.getLastName() + "," + sessionLoginUser.getFirstName();

            return null;

        } catch (NumberFormatException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingDetailAction#agendaDownload()
     */
    public String downloadAgenda() {
        long userId = SessionUtil.getLoginUser().getUserId();
        int meetingId = new Integer(this.id_);

        MeetingBasicDto dto = new MeetingBasicDto();
        dto.setId(meetingId);
        dto.setUserId(userId);

        MeetingBasicDto meetingDto = this.meetingService_.getMeetingBasic(dto);
        meetingDto.setBody(StringUtils.replace(meetingDto.getBody(), "\n", "<br/>"));

        MeetingAgendaDto[] agendas = this.meetingService_.getAgendasWithDiscuss(meetingId);
        for (MeetingAgendaDto m : agendas) {
            m.setBody(StringUtils.replace(m.getBody(), "\n", "<br/>"));

            for (MeetingAgendaDiscussionDto d : m.getDiscussionList()) {
                d.setBody(StringUtils.replace(d.getBody(), "\n", "<br/>"));
            }
        }

        MeetingToTaskDto meetingToTaskDto = new MeetingToTaskDto();
        meetingToTaskDto.setBasicId(meetingId);
        meetingToTaskDto = this.meetingService_.getTaskCountByBasicId(meetingToTaskDto);
        meetingToTaskDto.setBasicId(meetingId);
        meetingToTaskDto.setOffset(0);
        meetingToTaskDto.setLimit(meetingToTaskDto.getRowCount());
        MeetingToTaskDto[] meetingToTaskDtos = this.meetingService_
                .getTasksByBasicId(meetingToTaskDto);

        HashMap<Long, String> discussionIdToTaskURL = new HashMap<Long, String>();
        for (MeetingToTaskDto m : meetingToTaskDtos) {
            discussionIdToTaskURL.put(m.getDiscussionId(), SystemConfiguration
                    .getValue("tugboat.gtd.context_path")
                    + "/browse/tasks/" + m.getTaskId());
        }

        ActivityMemberDto[] members = activityMemberService_.getActivityMembersList(meetingDto
                .getActivityId());

        // invoke velocity template engine.
        VelocityContext vc = new VelocityContext();
        vc.put("url", SystemConfiguration.getValue("tugboat.gtd.context_path")
                + "/browse/meetings/" + meetingDto.getId());

        vc.put("header", meetingDto);
        vc.put("agendas", agendas);
        vc.put("members", members);
        vc.put("mapTaskURL", discussionIdToTaskURL);

        StringWriter stringWriter = new StringWriter();
        String body = "";
        try {
            InputStream is = this.getClass().getResourceAsStream("meeting_agenda.vm");

            Velocity.evaluate(vc, stringWriter, "BODY", new InputStreamReader(is, "UTF-8"));

            body = stringWriter.toString();
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }

        this.response_.setHeader("Cache-Control", "no-cache");
        this.response_.setHeader("Pragma", "no-cache");
        this.response_.setContentType("text/html; charset=utf-8");
        this.response_.setCharacterEncoding("UTF-8");

        try {
            PrintWriter pw = this.response_.getWriter();
            pw.write(body);
            pw.close();
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
        FacesContext context = FacesContext.getCurrentInstance();
        context.responseComplete();
        return null;
    }
}
