/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.page.impl;

import java.lang.reflect.InvocationTargetException;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityService;
import org.seasar.tugboat.gtd.contents.common.entity.Classification;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;
import org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper;
import org.seasar.tugboat.gtd.contents.meeting.page.MeetingDetailAjax;
import org.seasar.tugboat.gtd.contents.meeting.service.MeetingService;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.mailservices.service.MailService;

/**
 * meeting detail ajax impl.
 * 
 */
@FormAnnotation(formId = "meetingDetailAjax", isAuthorizationCheck = true, isDirectAccessCheck = true)
public class MeetingDetailAjaxImpl implements MeetingDetailAjax {

    /* basic id */
    private int basicId_;

    public int getBasicId() {
        return basicId_;
    }

    public void setBasicId(int eventId) {
        this.basicId_ = eventId;
    }

    /* status id */
    private int statusId_;

    public int getStatusId() {
        return statusId_;
    }

    public void setStatusId(int statusId) {
        this.statusId_ = statusId;
    }

    /* activity id */
    private long activityId_;

    public long getActivityId() {
        return activityId_;
    }

    public void setActivityId(long activityId) {
        this.activityId_ = activityId;
    }

    /* agenda id */
    private int agendaId_;

    public void setAgendaId(int topicId) {
        this.agendaId_ = topicId;
    }

    public int getAgendaId() {
        return this.agendaId_;
    }

    /* agenda priority */
    private String priority_ = "0";

    public void setPriority(String priority) {
        this.priority_ = priority;
    }

    public String getPriority() {
        return this.priority_;
    }

    /* agenda or discussion posted user id */
    private int postedUserId_ = -1;

    public void setPostedUserId(int postedUserId) {
        this.postedUserId_ = postedUserId;
    }

    public int getPostedUserId() {
        return this.postedUserId_;
    }

    /* discussion id */
    private int discussionId_;

    public void setDiscussionId(int discussionId) {
        this.discussionId_ = discussionId;
    }

    public int getDiscussionId() {
        return this.discussionId_;
    }

    /* discussion posted type */
    private String postedType_ = null;

    public void setPostedType(String postedType) {
        this.postedType_ = postedType;
    }

    public String getPostedType() {
        return this.postedType_;
    }

    /* bodyTextile */
    private String body_;

    public String getBody() {
        return body_;
    }

    public void setBody(String body) {
        this.body_ = body;
    }

    /* title */
    private String title_;

    public String getTitle() {
        return title_;
    }

    public void setTitle(String title) {
        this.title_ = title;
    }

    /* descriptionTextile */
    private String descriptionTextile_;

    public String getDescriptionTextile() {
        return descriptionTextile_;
    }

    public void setDescriptionTextile(String descriptionTextile) {
        this.descriptionTextile_ = descriptionTextile;
    }

    /* personInChargeId */
    private String personInChargeId_;

    public String getPersonInChargeId() {
        return personInChargeId_;
    }

    public void setPersonInChargeId(String personInChargeId) {
        this.personInChargeId_ = personInChargeId;
    }

    /* current page number */
    private int currentPageNumber_;

    public int getCurrentPageNumber() {
        return currentPageNumber_;
    }

    public void setCurrentPageNumber(int currentPageNumber) {
        this.currentPageNumber_ = currentPageNumber;
    }

    /* offset */
    private int offset_;

    public int getOffset() {
        return offset_;
    }

    public void setOffset(int offset) {
        this.offset_ = offset;
    }

    /* limit */
    private int limit_;

    public int getLimit() {
        return limit_;
    }

    public void setLimit(int limit) {
        this.limit_ = limit;
    }

    /* task title */
    private String taskTitle_;

    public String getTaskTitle() {
        return taskTitle_;
    }

    public void setTaskTitle(String taskTitle) {
        this.taskTitle_ = taskTitle;
    }

    /* task description */
    private String taskDescription_;

    public String getTaskDescription() {
        return taskDescription_;
    }

    public void setTaskDescription(String taskDescription) {
        this.taskDescription_ = taskDescription;
    }

    /* task dead line */
    private String taskDeadLine_;

    public String getTaskDeadLine() {
        return taskDeadLine_;
    }

    public void setTaskDeadLine(String taskDeadLine) {
        this.taskDeadLine_ = taskDeadLine;
    }

    /* task person id */
    private int taskPersonId_;

    public int getTaskPersonId() {
        return taskPersonId_;
    }

    public void setTaskPersonId(int taskPersonId) {
        this.taskPersonId_ = taskPersonId;
    }

    /* task classification id */
    private int taskClassificationId_;

    public int getTaskClassificationId() {
        return taskClassificationId_;
    }

    public void setTaskClassificationId(int taskClassificationId) {
        this.taskClassificationId_ = taskClassificationId;
    }

    /* task status id */
    private int taskStatusId_;

    public int getTaskStatusId() {
        return taskStatusId_;
    }

    public void setTaskStatusId(int taskStatusId) {
        this.taskStatusId_ = taskStatusId;
    }

    /* meeting Helper */
    private MeetingHelper meetingHelper_;

    public void setMeetingHelper(MeetingHelper meetingHelper) {
        this.meetingHelper_ = meetingHelper;
    }

    /* meeting service */
    private MeetingService meetingService_;

    public void setMeetingService(MeetingService meetingService) {
        this.meetingService_ = meetingService;
    }

    /* activity members service */
    private ActivityMembersService activityMemberService_;

    public void setActivityMemberService(ActivityMembersService activityMemberService) {
        this.activityMemberService_ = activityMemberService;
    }

    /* mail service */
    private MailService mailService_;

    public void setMailService(MailService mailService) {
        this.mailService_ = mailService;
    }

    /* activity service */
    private ActivityService activityService_;

    public void setActivityService(ActivityService activityService) {
        this.activityService_ = activityService;
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#getAgendaListOnLoad()
     */
    public String getAgendaListOnLoad() {
        UserDto userInfo = SessionUtil.getLoginUser();

        MeetingBasicDto dto = new MeetingBasicDto();
        dto.setId(this.basicId_);
        dto.setUserId(userInfo.getUserId());

        MeetingBasicDto meetingInfo = this.meetingService_.getMeetingBasic(dto);
        MeetingAgendaDto[] agendas = this.meetingService_.getAgendaList(this.basicId_);

        return this.meetingHelper_.createAgendasJson(meetingInfo, agendas, userInfo, false);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#updateMeeting()
     */
    public String updateMeeting() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        MeetingBasicDto basicDto = new MeetingBasicDto();
        basicDto.setId(this.basicId_);
        basicDto.setStatusId(this.statusId_);
        basicDto.setActivityId(this.activityId_);
        basicDto.setTitle(this.title_);
        basicDto.setBody(this.descriptionTextile_);
        basicDto.setPostedUserId(sessionLoginUser.getUserId());
        basicDto.setUserId(sessionLoginUser.getUserId());
        basicDto = this.meetingService_.updateMeeting(basicDto);

        return this.meetingHelper_.createBasicJson(basicDto);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingDetailAjax#updateStatus()
     */
    public String updateStatus() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        MeetingBasicDto basicDto =
            this.meetingService_.updateStatus(
                this.basicId_, this.statusId_, sessionLoginUser.getUserId());
        return this.meetingHelper_.createBasicJson(basicDto);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#addAgenda()
     */
    public String addAgenda() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        MeetingAgendaDto agendaDto = new MeetingAgendaDto();
        agendaDto.setPriority(this.priority_);
        agendaDto.setBody(this.body_);
        agendaDto.setPostedUserId(this.postedUserId_);
        agendaDto.setModifiedUserId(sessionLoginUser.getUserId());
        agendaDto = this.meetingService_.addAgenda(agendaDto, this.basicId_);

        return this.meetingHelper_.createAgendaJson(agendaDto);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingDetailAjax#getAgendaDiscussion()
     */
    public String getAgendaDiscussion() {
        UserDto userInfo = SessionUtil.getLoginUser();
        MeetingAgendaDto[] agendas = this.meetingService_.getAgendasWithDiscussionByIdAndAgendaId(
                this.basicId_, this.agendaId_);

        return this.meetingHelper_.createAgendasJson(null, agendas, userInfo, true);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#addAgendaDiscussion()
     */
    public String addAgendaDiscussion() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        
        MeetingBasicDto basicDto = new MeetingBasicDto();
        basicDto.setId(this.basicId_);
        basicDto.setStatusId(MeetingBasicDto.STATUS_OPEN);
        basicDto.setPostedUserId(sessionLoginUser.getUserId());
        basicDto.setUserId(sessionLoginUser.getUserId());

        MeetingAgendaDiscussionDto discussionDto = new MeetingAgendaDiscussionDto();
        discussionDto.setAgendaId(this.agendaId_);
        discussionDto.setType(this.postedType_);
        discussionDto.setBody(this.body_);
        discussionDto.setPostedUserId(this.postedUserId_);
        discussionDto.setModifiedUserId(sessionLoginUser.getUserId());

        MeetingToTaskDto toTaskDto = new MeetingToTaskDto();
        
        try {
            BeanUtils.copyProperties(toTaskDto, this);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
        TaskDto taskDto = new TaskDto();
        taskDto.setTitle(this.taskTitle_);
        taskDto.setDescription(this.taskDescription_);
        taskDto.setDescriptionTextile(this.taskDescription_);
        taskDto.setDeadline(this.taskDeadLine_);
        taskDto.setClassificationId(this.taskClassificationId_);
        taskDto.setStatusId(this.taskStatusId_);
        taskDto.setUserId(sessionLoginUser.getUserId());
        taskDto.setActivityId(this.activityId_);

        TaskRelationDto relationsDto = new TaskRelationDto();
        relationsDto.setActivityId(this.activityId_);
        relationsDto.setStatusId(TaskRelationDto.STATUS_TODO);
        relationsDto.setClassificationId(Classification.INBOX);

        relationsDto.setPersonInChargeId(taskPersonId_);
        relationsDto.setUserId(sessionLoginUser.getUserId());

        discussionDto =
            this.meetingService_.addAgendaDiscussion(discussionDto, taskDto, relationsDto, toTaskDto);

        MeetingBasicDto meetingBasicDto = this.meetingService_.getMeetingBasic(basicDto);
        if (meetingBasicDto.getActiveAgendas() == 0) {
            this.meetingService_.updateStatus(
                meetingBasicDto.getId(), MeetingBasicDto.STATIS_CLOSE, sessionLoginUser.getUserId());
        }
        return this.meetingHelper_.createAgendaDiscussionJsonWithMeetingBasic(
                meetingBasicDto, discussionDto, taskDto, relationsDto);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#updateAgenda()
     */
    public String updateAgenda() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        MeetingAgendaDto agendaDto = new MeetingAgendaDto();
        agendaDto.setId(this.agendaId_);
        agendaDto.setPostedUserId(this.postedUserId_);
        agendaDto.setBody(this.body_);
        agendaDto.setModifiedUserId(sessionLoginUser.getUserId());
        agendaDto = this.meetingService_.updateAgenda(agendaDto);

        return this.meetingHelper_.createAgendaJson(agendaDto);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#deleteAgenda()
     */
    public void deleteAgenda() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        MeetingAgendaDto agendaDto = new MeetingAgendaDto();
        agendaDto.setId(this.agendaId_);
        agendaDto.setModifiedUserId(sessionLoginUser.getUserId());

        this.meetingService_.deleteAgenda(agendaDto);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#updateAgendaDiscussion()
     */
    public String updateAgendaDiscussion() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        MeetingAgendaDiscussionDto discussionDto = new MeetingAgendaDiscussionDto();
        discussionDto.setId(this.discussionId_);
        discussionDto.setAgendaId(this.agendaId_);
        discussionDto.setType(this.postedType_);
        discussionDto.setPostedUserId(this.postedUserId_);
        discussionDto.setBody(this.body_);
        discussionDto.setModifiedUserId(sessionLoginUser.getUserId());

        MeetingToTaskDto toTaskDto = new MeetingToTaskDto();
        
        try {
            BeanUtils.copyProperties(toTaskDto, this);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
        
        TaskDto taskDto = new TaskDto();
        taskDto.setTitle(this.taskTitle_);
        taskDto.setDescription(this.taskDescription_);
        taskDto.setDescriptionTextile(this.taskDescription_);
        taskDto.setDeadline(this.taskDeadLine_);
        taskDto.setUserId(sessionLoginUser.getUserId());

        TaskRelationDto relationsDto = new TaskRelationDto();
        relationsDto.setActivityId(this.activityId_);
        relationsDto.setStatusId(TaskRelationDto.STATUS_TODO);
        relationsDto.setClassificationId(Classification.INBOX);

        relationsDto.setPersonInChargeId(taskPersonId_);
        relationsDto.setUserId(sessionLoginUser.getUserId());

        discussionDto =
            this.meetingService_.updateAgendaDiscussion(discussionDto, toTaskDto, taskDto, relationsDto);
        return this.meetingHelper_.createAgendaDiscussionJsonWithMeetingBasic(
            null, discussionDto, taskDto, null);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#deleteAgendaDiscussion()
     */
    public void deleteAgendaDiscussion() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        MeetingAgendaDiscussionDto discussionDto = new MeetingAgendaDiscussionDto();
        discussionDto.setId(this.discussionId_);
        discussionDto.setAgendaId(this.agendaId_);
        discussionDto.setType(this.postedType_);
        discussionDto.setPostedUserId(this.postedUserId_);
        discussionDto.setActivityId(this.activityId_);
        discussionDto.setMeetingBasicId(this.basicId_);
        discussionDto.setModifiedUserId(sessionLoginUser.getUserId());

        this.meetingService_.deleteAgendaDiscussion(discussionDto);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#getSidebarMembersContents()
     */
    public String getSidebarMembersContents() {
        ActivityMemberDto[] activityMembersDto = this.activityMemberService_
                .getActivityMembersList(this.activityId_);

        return this.meetingHelper_.createSidebarMembers(activityMembersDto);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#getSidebarTasksContents()
     */
    public String getSidebarTasksContents() {
        MeetingToTaskDto conditionDto = new MeetingToTaskDto();
        conditionDto.setBasicId(this.basicId_);
        conditionDto.setOffset(this.offset_);
        conditionDto.setLimit(this.limit_);

        MeetingToTaskDto[] liveMeetingToTaskDto =
            this.meetingService_.getTasksByBasicId(conditionDto);
        MeetingToTaskDto toTaskDto = this.meetingService_.getTaskCountByBasicId(conditionDto);
        String result =
            this.meetingHelper_.createSidebarTasks(
                liveMeetingToTaskDto, toTaskDto.getRowCount(),
                this.currentPageNumber_, this.limit_);
        return result;
    }

    /*
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return ReflectionToStringBuilder.toString(this, ToStringStyle.MULTI_LINE_STYLE);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingDetailAjax#sendMail()
     */
    public String sendMail() {
        long userId = SessionUtil.getLoginUser().getUserId();
        MeetingBasicDto dto = new MeetingBasicDto();
        dto.setId(basicId_);
        dto.setUserId(userId);
        MeetingBasicDto eventHeaderDto = this.meetingService_.getMeetingBasic(dto);
        MeetingAgendaDto[] topicDtoList = this.meetingService_.getAgendasWithDiscuss(basicId_);

        ActivityDto activityDto =
            this.activityService_.getActivityById(eventHeaderDto.getActivityId());

        MeetingToTaskDto meetingToTaskDto = new MeetingToTaskDto();
        meetingToTaskDto.setBasicId(basicId_);
        meetingToTaskDto = meetingService_.getTaskCountByBasicId(meetingToTaskDto);
        meetingToTaskDto.setBasicId(basicId_);
        meetingToTaskDto.setOffset(0);
        meetingToTaskDto.setLimit(meetingToTaskDto.getRowCount());
        MeetingToTaskDto[] meetingToTaskDtos = meetingService_.getTasksByBasicId(meetingToTaskDto);

        this.mailService_.send(
            eventHeaderDto, topicDtoList, activityDto, meetingToTaskDtos, userId);
        return "";
    }
    
    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingDetailAjax#changeType()
     */
    public String changeType() {
        long userId = SessionUtil.getLoginUser().getUserId();
        int agendaStatus = 
            this.meetingService_.updateAgendaDiscussionType(
                this.agendaId_, this.discussionId_, this.postedType_, userId);
        
        MeetingBasicDto meeting = new MeetingBasicDto();
        meeting.setId(this.basicId_);
        meeting.setUserId(userId);
        
        MeetingBasicDto meetingBasicDto = this.meetingService_.getMeetingBasic(meeting);
        int meetingStatus = 
            meetingBasicDto.getActiveAgendas() == 0 
            ? MeetingBasicDto.STATIS_CLOSE
            : MeetingBasicDto.STATUS_OPEN;
        
        this.meetingService_.updateStatus(
            meetingBasicDto.getId(), meetingStatus, userId);
        
        StringBuilder json = new StringBuilder();
        json.append("({")
            .append("'meetingStatus':")
            .append(meetingStatus)
            .append(",")
            .append("'agendaStatus':")
            .append(agendaStatus)
            .append("})");
        return json.toString();
    }

}
