/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.service;

import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;

/**
 * meeting service
 * 
 */
public interface MeetingService {

    /**
     * create meeting basic List.
     * 
     * @param basicDto
     */
    MeetingBasicDto[] getMeetingBasicList(MeetingBasicDto basicDto);

    /**
     * get meeting count by folder.
     * 
     * @param basicDto
     */
    MeetingBasicDto getDefaultCountByFolder(MeetingBasicDto basicDto);

    /**
     * get meeting count by query params.
     * 
     * @param basicDto
     */
    MeetingBasicDto getMeetingCountByQueryParams(MeetingBasicDto basicDto);

    /**
     * create meeting basic with task.
     * 
     * @param basicId
     * @param userId
     */
    MeetingBasicDto getMeetingBasicWithTask(long id, long userId);

    /**
     * add meeting.
     * 
     * @param dto
     */
    MeetingBasicDto addMeeting(MeetingBasicDto dto);

    /**
     * create meeting basic.
     * 
     * @param dto
     */
    MeetingBasicDto getMeetingBasic(MeetingBasicDto dto);

    /**
     * create agenda list.
     * 
     * @param id
     */
    MeetingAgendaDto[] getAgendaList(long id);

    /**
     * get agenda with discussion by id and agenda id.
     * 
     * @param id
     * @param agendaId
     * @return
     */
    MeetingAgendaDto[] getAgendasWithDiscussionByIdAndAgendaId(long id, long agendaId);

    /**
     * create agenda list with discuss.
     * 
     * @param id
     * @return
     */
    MeetingAgendaDto[] getAgendasWithDiscuss(long id);

    /**
     * update meeting.
     * 
     * @param dto
     */
    MeetingBasicDto updateMeeting(MeetingBasicDto dto);

    /**
     * update meeting status.
     * 
     * @param id
     * @param statusId
     * @param userId
     * @return
     */
    MeetingBasicDto updateStatus(long id, int statusId, long userId);

    /**
     * add meeting agenda.
     * 
     * @param dto
     * @param id
     */
    MeetingAgendaDto addAgenda(MeetingAgendaDto dto, long id);

    /**
     * update meeting agenda.
     * 
     * @param dto
     */
    MeetingAgendaDto updateAgenda(MeetingAgendaDto dto);

    /**
     * delete meeting agenda.
     * 
     * @param dto
     */
    void deleteAgenda(MeetingAgendaDto dto);

    /**
     * add meeting agenda discussion.
     * 
     * @param dto
     * @param taskDto
     * @param taskRelationsDto
     * @param toTaskDto
     */
    MeetingAgendaDiscussionDto addAgendaDiscussion(MeetingAgendaDiscussionDto dto, TaskDto taskDto,
            TaskRelationDto taskRelationsDto, MeetingToTaskDto toTaskDto);

    /**
     * update meeting agenda discussion.
     * 
     * @param discussionDto
     * @param toTaskDto
     * @param taskDto
     */
    MeetingAgendaDiscussionDto updateAgendaDiscussion(MeetingAgendaDiscussionDto discussionDto,
            MeetingToTaskDto toTaskDto, TaskDto taskDto, TaskRelationDto relationDto);
    
    /**
     * update agenda discussion type.
     * 
     * @param agendaId
     * @param discussId
     * @param discussionType
     * @param userId
     * @return
     */
    int updateAgendaDiscussionType(
        long agendaId, long discussId, String discussionType, long userId);
    

    /**
     * delete meeting agenda discussion.
     * 
     * @param discussionDto
     */
    void deleteAgendaDiscussion(MeetingAgendaDiscussionDto discussionDto);

    /**
     * get tasks by basic id.
     * 
     * @param toTaskDto
     * @return
     */
    MeetingToTaskDto[] getTasksByBasicId(MeetingToTaskDto toTaskDto);

    /**
     * get tasks count by basic id.
     * 
     * @param toTaskDto
     * @return
     */
    MeetingToTaskDto getTaskCountByBasicId(MeetingToTaskDto toTaskDto);

    /**
     * get meeting to tasks by task id.
     * 
     * @param taskId
     * @return
     */
    MeetingToTaskDto[] getMeetingToTasksByTaskId(long taskId);

    /**
     * check belong activity meeting.
     * 
     * @param id
     * @param userId
     * @return
     */
    boolean checkBelongActivityMeeting(long id, long userId);
}
