/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.beanutils.BeanUtils;
import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.container.service.SearchService;
import org.seasar.tugboat.gtd.contents.common.entity.Classification;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingAgendaBodiesDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingAgendaDiscussionBodiesDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingAgendaDiscussionsDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingAgendasDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingBasicBodiesDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingBasicsDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingRelationsDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingToTasksDao;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;
import org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgenda;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaBody;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussion;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussionBody;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasic;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasicBody;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBrowsePageCondition;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingRelation;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingToTask;
import org.seasar.tugboat.gtd.contents.meeting.service.MeetingService;
import org.seasar.tugboat.gtd.contents.task.dao.TaskRelationsDao;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBodyDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo;
import org.seasar.tugboat.gtd.contents.task.entity.TaskRelation;
import org.seasar.tugboat.gtd.contents.task.service.TaskService;

/**
 * meeting service impl.
 * 
 */
public class MeetingServiceImpl implements MeetingService {

    /* meeting dxo */
    private MeetingDxo meetingDxo_;

    public void setMeetingDxo(MeetingDxo meetingDxo) {
        this.meetingDxo_ = meetingDxo;
    }

    /* meeting basic dao */
    private MeetingBasicsDao meetingBasicsDao_;

    public void setMeetingBasicsDao(MeetingBasicsDao meetingBasicsDao) {
        this.meetingBasicsDao_ = meetingBasicsDao;
    }

    /* meeting agenda Dao */
    private MeetingAgendasDao meetingAgendasDao_;

    public void setMeetingAgendasDao(MeetingAgendasDao meetingAgendasDao) {
        this.meetingAgendasDao_ = meetingAgendasDao;
    }

    /* meeting agenda comment Dao */
    private MeetingAgendaDiscussionsDao meetingAgendaDiscussionsDao_;

    public void setMeetingAgendaDiscussionsDao(
            MeetingAgendaDiscussionsDao meetingAgendaDiscussionsDao) {
        this.meetingAgendaDiscussionsDao_ = meetingAgendaDiscussionsDao;
    }

    /* meeting relations Dao */
    private MeetingRelationsDao meetingRelationsDao_;

    public void setMeetingRelationsDao(MeetingRelationsDao meetingRelationsDao) {
        this.meetingRelationsDao_ = meetingRelationsDao;
    }

    /* meeting to task Dao */
    private MeetingToTasksDao meetingToTasksDao_;

    public void setMeetingToTasksDao(MeetingToTasksDao meetingToTasksDao) {
        this.meetingToTasksDao_ = meetingToTasksDao;
    }

    /* meeting basic bodies Dao */
    private MeetingBasicBodiesDao meetingBasicBodiesDao_;

    public void setMeetingBasicBodiesDao(MeetingBasicBodiesDao meetingBasicBodiesDao) {
        this.meetingBasicBodiesDao_ = meetingBasicBodiesDao;
    }

    /* meeting agenda bodies Dao */
    private MeetingAgendaBodiesDao meetingAgendaBodiesDao_;

    public void setMeetingAgendaBodiesDao(MeetingAgendaBodiesDao meetingAgendaBodiesDao) {
        this.meetingAgendaBodiesDao_ = meetingAgendaBodiesDao;
    }

    /* meeting agenda discussion bodies Dao */
    private MeetingAgendaDiscussionBodiesDao meetingAgendaDiscussionBodiesDao_;

    public void setMeetingAgendaDiscussionBodiesDao(
            MeetingAgendaDiscussionBodiesDao meetingAgendaDiscussionBodiesDao) {
        this.meetingAgendaDiscussionBodiesDao_ = meetingAgendaDiscussionBodiesDao;
    }

    /* task service */
    private TaskService taskService_;

    public void setTaskService(TaskService taskService) {
        this.taskService_ = taskService;
    }

    /* task dxo */
    private TaskDxo taskDxo_;

    public void setTaskDxo(TaskDxo taskDxo) {
        this.taskDxo_ = taskDxo;
    }

    /* taskRelationsDao Dao */
    private TaskRelationsDao taskRelationsDao_;

    public void setTaskRelationsDao(TaskRelationsDao taskRelationsDao) {
        this.taskRelationsDao_ = taskRelationsDao;
    }

    /* search service */
    private SearchService searchService_;

    public void setSearchService(SearchService searchService) {
        this.searchService_ = searchService;
    }

    /* mail service 
    private MailService mailService_;

    public void setMailService(MailService mailService) {
        this.mailService_ = mailService;
    }
    */

    /* activity members dao 
    private ActivityMembersDao activityMembersDao_;

    public void setActivityMembersDao(ActivityMembersDao activityMembersDao) {
        this.activityMembersDao_ = activityMembersDao;
    }
    */

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#createMeetingBasicList(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto)
     */
    public MeetingBasicDto[] getMeetingBasicList(MeetingBasicDto meetingBasicDto) {
        MeetingBrowsePageCondition condition = new MeetingBrowsePageCondition();
        condition.setUserId(meetingBasicDto.getUserId());
        condition.setQueryStatusIds(meetingBasicDto.getQueryStatusIds());
        condition.setQueryActivityIdsIds(meetingBasicDto.getQueryActivityIds());
        condition.setQueryRegistrationIds(meetingBasicDto.getQueryRegisterIds());
        condition.setOffset(meetingBasicDto.getOffset());
        condition.setLimit(meetingBasicDto.getLimit());

        MeetingBasic[] basicList = this.meetingBasicsDao_.findByUserId(condition);

        return this.meetingDxo_.convertMeetingBasicDtoList(basicList);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#getDefaultCountByFolder(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto)
     */
    public MeetingBasicDto getDefaultCountByFolder(MeetingBasicDto meetingBasicDto) {
        MeetingBrowsePageCondition condition = new MeetingBrowsePageCondition();
        condition.setUserId(meetingBasicDto.getUserId());

        MeetingBasic basic = this.meetingBasicsDao_.findDefaultCountByFolder(condition);

        return this.meetingDxo_.convertMeetingBasicDto(basic);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#getMeetingCountByQueryParams(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto)
     */
    public MeetingBasicDto getMeetingCountByQueryParams(MeetingBasicDto meetingBasicDto) {
        MeetingBrowsePageCondition condition = new MeetingBrowsePageCondition();
        condition.setUserId(meetingBasicDto.getUserId());
        condition.setQueryStatusIds(meetingBasicDto.getQueryStatusIds());
        condition.setQueryActivityIdsIds(meetingBasicDto.getQueryActivityIds());
        condition.setQueryRegistrationIds(meetingBasicDto.getQueryRegisterIds());

        MeetingBasic basic = this.meetingBasicsDao_.findCountByQueryParams(condition);

        return this.meetingDxo_.convertMeetingBasicDto(basic);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#createMeetingBasic(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto)
     */
    public MeetingBasicDto getMeetingBasic(MeetingBasicDto dto) {
        MeetingBasic basic = this.meetingBasicsDao_.findById(dto.getUserId(), dto.getId());

        return this.meetingDxo_.convertMeetingBasicDto(basic);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#getMeetingBasicWithTask(long,
     *      long)
     */
    public MeetingBasicDto getMeetingBasicWithTask(long id, long userId) {
        MeetingBasic[] basicList = this.meetingBasicsDao_.findWithTaskById(id, userId);

        return this.meetingDxo_.convertMeetingBasicDtoWithTask(basicList);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#createAgendaList(long)
     */
    public MeetingAgendaDto[] getAgendaList(long id) {
        MeetingAgenda[] agendas = this.meetingAgendasDao_.findByMeetingId(id);

        List<Long> list = new ArrayList<Long>();
        for (MeetingAgenda a : agendas) {
            list.add(a.getId());
        }
        Long[] ids = list.toArray(new Long[list.size()]);

        // get discussions rows.
        if (ids != null && ids.length > 0) {
            MeetingAgenda[] commentRows = this.meetingAgendasDao_.getCommentRows(ids);
            MeetingAgenda[] taskRows = this.meetingAgendasDao_.getTaskRows(ids);

            for (MeetingAgenda a : agendas) {
                for (MeetingAgenda c : commentRows) {
                    if (a.getId() == c.getId()) {
                        a.setCommentRows(c.getCommentRows());
                        break;
                    }
                }
                for (MeetingAgenda t : taskRows) {
                    if (a.getId() == t.getId()) {
                        a.setTaskRows(t.getTaskRows());
                        break;
                    }
                }
            }
        }
        return this.meetingDxo_.convertAgendaDtos(agendas);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#getAgendasWithDiscussionByIdAndAgendaId(long,
     *      long)
     */
    public MeetingAgendaDto[] getAgendasWithDiscussionByIdAndAgendaId(long id, long agendaId) {
        MeetingAgenda[] agendas = this.meetingAgendasDao_.findWithDiscussionByMeetingIdAndAgendaId(
                id, agendaId);

        return this.meetingDxo_.convertMeetingAgendaDtoList(agendas);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#getAgendasWithDiscuss(long)
     */
    public MeetingAgendaDto[] getAgendasWithDiscuss(long id) {
        MeetingAgenda[] agendas = this.meetingAgendasDao_.findWithDiscussionByMeetingId(id);

        return this.meetingDxo_.convertMeetingAgendaDtoList(agendas);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#addMeeting(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto)
     */
    public MeetingBasicDto addMeeting(MeetingBasicDto dto) {
        MeetingBasic nextId = this.meetingBasicsDao_.findNewId();
        dto.setId(nextId.getId());

        MeetingBasic basic = this.meetingDxo_.convertMeetingBasic(dto);
        basic.setStartDate(CalendarUtil.now("yyyyMMdd"));
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        basic.setRegistered(nowString);
        basic.setModified(nowString);
        
        this.meetingBasicsDao_.insert(basic);
        MeetingBasicBody basicBody = new MeetingBasicBody();
        basicBody.setBody(basic.getBody());
        basicBody.setBasicId(nextId.getId());

        this.meetingBasicBodiesDao_.insert(basicBody);
        basic = this.meetingBasicsDao_.findById(dto.getUserId(), basic.getId());

        this.searchService_.generateMeetingIndex(basic);

        return this.meetingDxo_.convertMeetingBasicDto(basic);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#updateMeeting(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto)
     */
    public MeetingBasicDto updateMeeting(MeetingBasicDto dto) {
        MeetingBasic basic = this.meetingDxo_.convertMeetingBasic(dto);
        basic.setModified(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
        this.meetingBasicsDao_.update(basic);

        MeetingBasicBody basicBody = new MeetingBasicBody();
        basicBody.setBody(basic.getBody());
        basicBody.setBasicId(basic.getId());
        this.meetingBasicBodiesDao_.update(basicBody);

        basic = this.meetingBasicsDao_.findById(dto.getUserId(), basic.getId());
        searchService_.generateMeetingIndex(basic);

        return this.meetingDxo_.convertMeetingBasicDto(basic);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#updateStatus(long,
     *      int, long)
     */
    public MeetingBasicDto updateStatus(long id, int statusId, long userId) {
        MeetingBasic entity = new MeetingBasic();
        entity.setId(id);
        entity.setStatusId(statusId);
        entity.setPostedUserId(userId);
        entity.setModified(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
        this.meetingBasicsDao_.updateStatus(entity);

        MeetingBasic basic = this.meetingBasicsDao_.findById(userId, id);
        return this.meetingDxo_.convertMeetingBasicDto(basic);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#addAgenda(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto, long)
     */
    public MeetingAgendaDto addAgenda(MeetingAgendaDto dto, long id) {
        MeetingAgenda nextId = this.meetingAgendasDao_.findNewId();
        dto.setId(nextId.getId());

        this.meetingDxo_.convertMeetingAgenda(dto);
        
        MeetingAgenda agenda = (MeetingAgenda)dto;

        MeetingRelation relations = new MeetingRelation();
        relations.setBasicId(id);
        relations.setAgendaId(nextId.getId());

        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        agenda.setRegistered(nowString);
        agenda.setModified(nowString);
        
        this.meetingAgendasDao_.insert(agenda);
        this.meetingRelationsDao_.insert(relations);

        MeetingAgendaBody agendaBody = new MeetingAgendaBody();
        agendaBody.setAgendaId(nextId.getId());
        agendaBody.setBody(agenda.getBody());
        this.meetingAgendaBodiesDao_.insert(agendaBody);

        agenda = this.meetingAgendasDao_.findAgendaRelationById(agenda.getId());
        searchService_.generateMeetingAgendaIndex(agenda);

        return this.meetingDxo_.convertMeetingAgendaDto(agenda);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#updateAgenda(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto)
     */
    public MeetingAgendaDto updateAgenda(MeetingAgendaDto dto) {
        MeetingAgenda agenda = this.meetingDxo_.convertMeetingAgenda(dto);
        agenda.setModified(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
        this.meetingAgendasDao_.update(agenda);

        MeetingAgendaBody agendaBody = new MeetingAgendaBody();
        agendaBody.setAgendaId(agenda.getId());
        agendaBody.setBody(agenda.getBody());
        this.meetingAgendaBodiesDao_.update(agendaBody);

        agenda = this.meetingAgendasDao_.findAgendaRelationById(agenda.getId());
        searchService_.generateMeetingAgendaIndex(agenda);

        return this.meetingDxo_.convertMeetingAgendaDto(agenda);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#deleteAgenda(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto)
     */
    public void deleteAgenda(MeetingAgendaDto dto) {
        MeetingAgenda agenda = this.meetingDxo_.convertMeetingAgenda(dto);
        agenda.setModified(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
        this.meetingAgendasDao_.updateDelFlag(agenda);

        MeetingToTask[] toTasks = this.meetingToTasksDao_.findTasksByAgendaId(agenda.getId());
        long[] taskIds = new long[toTasks.length];
        for (int i = 0; i < toTasks.length; i++) {
            taskIds[i] = toTasks[i].getTaskId();
        }
        // When task is trashed, status is todo and Classification is TRASH.
        this.taskService_.updateClassificationId(Classification.TRASH, TaskRelation.STATUS_TODO,
                taskIds, agenda.getModifiedUserId());

        agenda = this.meetingAgendasDao_.findAgendaRelationById(agenda.getId());
        agenda.setBody("");
        this.searchService_.generateMeetingAgendaIndex(agenda);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#addAgendaDiscussion(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto,
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskDto,
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto)
     */
    public MeetingAgendaDiscussionDto addAgendaDiscussion(
            MeetingAgendaDiscussionDto dto, TaskDto taskDto,
            TaskRelationDto taskRelationsDto, MeetingToTaskDto toTaskDto) {

        MeetingAgendaDiscussion agendaId = this.meetingAgendaDiscussionsDao_.findNewId();
        dto.setId(agendaId.getId());

        MeetingAgendaDiscussion discussion = this.meetingDxo_.convertMeetingAgendaDiscussion(dto);
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        discussion.setRegistered(nowString);
        discussion.setModified(nowString);
        this.meetingAgendaDiscussionsDao_.insert(discussion);

        if (String.valueOf(MeetingAgendaDiscussion.TYPE_TODO).equals(discussion.getType())) {
            toTaskDto.setDiscussionId(discussion.getId());
            this.addTask(taskDto, taskRelationsDto, toTaskDto);

        } else {
            MeetingAgendaDiscussionBody discussionBody = new MeetingAgendaDiscussionBody();
            discussionBody.setDiscussionId(discussion.getId());
            discussionBody.setAgendaId(discussion.getAgendaId());
            discussionBody.setBody(discussion.getBody());
            this.meetingAgendaDiscussionBodiesDao_.insert(discussionBody);
            
            if (String.valueOf(MeetingAgendaDiscussion.TYPE_CONCLUSION).equals(discussion.getType())) {
                MeetingAgenda agenda = new MeetingAgenda();
                agenda.setId(discussion.getAgendaId());
                agenda.setStatus(MeetingAgenda.STATUS_DONE);
                agenda.setModified(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
                agenda.setModifiedUserId(dto.getModifiedUserId());
                this.meetingAgendasDao_.updateStatus(agenda);
            }
        }
        discussion =
            this.meetingAgendaDiscussionsDao_.findDiscussionRelationById(discussion.getId());
        MeetingAgendaDiscussionDto discussionDto =
            this.meetingDxo_.convertMeetingAgendaDiscussionDto(discussion);

        if (String.valueOf(MeetingAgendaDiscussion.TYPE_TODO).equals(discussion.getType())) {
            discussionDto.setBody(taskDto.getTitle());
            discussionDto.setBodyConvertedTextile(taskDto.getTitle());
        } else {
            searchService_.generateMeetingAgendaDiscussionIndex(discussion);
        }
        return discussionDto;
    }

    /**
     * add task.
     * 
     * @param taskDto
     * @param taskRelationsDto
     * @param toTaskDto
     */
    private void addTask(TaskDto taskDto, TaskRelationDto taskRelationsDto,
            MeetingToTaskDto toTaskDto) {
        TaskBodyDto bodyDto = new TaskBodyDto();
        bodyDto.setBody(taskDto.getDescriptionTextile());

        long taskId = this.taskService_.addTask(taskDto, bodyDto, taskRelationsDto);

        toTaskDto.setTaskId(taskId);
        MeetingToTask toTask = this.meetingDxo_.convertMeetingToTask(toTaskDto);
        this.meetingToTasksDao_.insert(toTask);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#updateAgendaDiscussion(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto,
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto,
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskDto )
     */
    public MeetingAgendaDiscussionDto updateAgendaDiscussion(
            MeetingAgendaDiscussionDto discussionDto, MeetingToTaskDto toTaskDto, TaskDto taskDto,
            TaskRelationDto taskRelationDto) {

        MeetingAgendaDiscussion discussion =
            this.meetingDxo_.convertMeetingAgendaDiscussion(discussionDto);
        discussion.setModified(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
        this.meetingAgendaDiscussionsDao_.update(discussion);

        if (String.valueOf(MeetingAgendaDiscussion.TYPE_TODO).equals(discussion.getType())) {
            MeetingToTask toTask = this.meetingDxo_.convertMeetingToTask(toTaskDto);
            TaskRelation taskRelations =
                this.taskRelationsDao_.findTaskByDiscussionId(toTask.getDiscussionId());
            TaskRelationDto taskRelationsDto =
                this.taskDxo_.convertTaskDtoWithoutDescription(taskRelations);
            taskRelationsDto.setTitle(taskDto.getTitle());
            taskRelationsDto.setDescription(taskDto.getDescription());
            taskRelationsDto.setDescriptionTextile(taskDto.getDescriptionTextile());
            taskRelationsDto.setDeadline(taskDto.getDeadline());
            taskRelationsDto.setUserId(taskDto.getUserId());
            taskRelationsDto.setPersonInChargeId(taskRelationDto.getPersonInChargeId());

            try {
                BeanUtils.copyProperties(taskDto, taskRelationsDto);
                taskDto.setId(taskRelationsDto.getTaskId());

                TaskBodyDto taskBodyDto = new TaskBodyDto();
                BeanUtils.copyProperties(taskBodyDto, taskRelationsDto);
                taskBodyDto.setBody(taskRelationsDto.getDescription());

                taskRelations =
                    this.taskService_.updateTask(taskDto, taskBodyDto, taskRelationsDto, null);

            } catch (IllegalAccessException e) {
                throw new SystemUnexpectedRuntimeException(e);
            } catch (InvocationTargetException e) {
                throw new SystemUnexpectedRuntimeException(e);
            }

        } else {
            MeetingAgendaDiscussionBody discussionBody = new MeetingAgendaDiscussionBody();
            discussionBody.setDiscussionId(discussion.getId());
            discussionBody.setAgendaId(discussion.getAgendaId());
            discussionBody.setBody(discussion.getBody());
            this.meetingAgendaDiscussionBodiesDao_.update(discussionBody);
            
            if (String.valueOf(MeetingAgendaDiscussion.TYPE_CONCLUSION).equals(discussion.getType())) {
                MeetingAgenda agenda = new MeetingAgenda();
                agenda.setId(discussion.getAgendaId());
                agenda.setStatus(MeetingAgenda.STATUS_DONE);
                agenda.setModified(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
                agenda.setModifiedUserId(discussionDto.getModifiedUserId());
                this.meetingAgendasDao_.updateStatus(agenda);
                
                
            }
        }
        discussion =
            this.meetingAgendaDiscussionsDao_.findDiscussionRelationById(discussion.getId());
        discussionDto = this.meetingDxo_.convertMeetingAgendaDiscussionDto(discussion);

        if (String.valueOf(MeetingAgendaDiscussion.TYPE_TODO).equals(discussion.getType())) {
            discussionDto.setBody(taskDto.getTitle());
            discussionDto.setBodyConvertedTextile(taskDto.getTitle());
        } else {
            searchService_.generateMeetingAgendaDiscussionIndex(discussion);
        }
        return discussionDto;
    }
    
    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#updateAgendaDiscussionType(
     *      long, long, int)
     */
    public int updateAgendaDiscussionType(
        long agendaId, long discussId, String discussionType, long userId) {
        
        MeetingAgendaDiscussion discussion = new MeetingAgendaDiscussion();
        discussion.setId(discussId);
        discussion.setType(discussionType);
        discussion.setPostedUserId(userId);
        discussion.setModifiedUserId(userId);
        discussion.setModified(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
        
        this.meetingAgendaDiscussionsDao_.update(discussion);
        
        MeetingAgendaDiscussion entity = 
            this.meetingAgendaDiscussionsDao_.findExistsDiscussionByAgendaId(agendaId);

        MeetingAgenda agenda = new MeetingAgenda();
        agenda.setId(agendaId);
        agenda.setModifiedUserId(userId);
        agenda.setModified(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
        agenda.setStatus( entity == null ? MeetingAgenda.STATUS_ACTIVE : MeetingAgenda.STATUS_DONE);
        this.meetingAgendasDao_.updateStatus(agenda);
        
        return ( entity == null ? MeetingAgenda.STATUS_ACTIVE : MeetingAgenda.STATUS_DONE);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#deleteAgendaDiscussion(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto)
     */
    public void deleteAgendaDiscussion(MeetingAgendaDiscussionDto discussionDto) {
        MeetingAgendaDiscussion discussion =
            this.meetingDxo_.convertMeetingAgendaDiscussion(discussionDto);
        discussion.setModified(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
        this.meetingAgendaDiscussionsDao_.updateDelFlag(discussion);

        if (String.valueOf(MeetingAgendaDiscussion.TYPE_TODO).equals(discussion.getType())) {

            TaskRelation taskRelation = this.taskRelationsDao_.findTaskByDiscussionId(discussionDto
                    .getId());

            // When task is trashed, status is todo and Classification is TRASH.
            this.taskService_.updateClassificationId(Classification.TRASH,
                    TaskRelation.STATUS_TODO, new long[] { taskRelation.getTaskId() },
                    discussionDto.getModifiedUserId());
        }
        discussion = this.meetingAgendaDiscussionsDao_.findDiscussionRelationById(discussion
                .getId());

        if (!String.valueOf(MeetingAgendaDiscussion.TYPE_TODO).equals(discussion.getType())) {
            discussion.setBody("");
            this.searchService_.generateMeetingAgendaDiscussionIndex(discussion);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#getTasksByBasicId(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto)
     */
    public MeetingToTaskDto[] getTasksByBasicId(MeetingToTaskDto meetingToTaskDto) {
        MeetingToTask[] toTasks =
            this.meetingToTasksDao_.findTasksByBasicId(meetingToTaskDto.getBasicId());

        return this.meetingDxo_.convertMeetingToTaskDtoList(toTasks);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#getTaskCountByBasicId(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto.MeetingToTaskDto)
     */
    public MeetingToTaskDto getTaskCountByBasicId(MeetingToTaskDto liveMeetingToTaskDto) {
        MeetingToTask toTask = this.meetingToTasksDao_.findTaskCountByBasicId(liveMeetingToTaskDto
                .getBasicId());

        return this.meetingDxo_.convertMeetingToTaskDto(toTask);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#getMeetingToTasksByTaskId(long)
     */
    public MeetingToTaskDto[] getMeetingToTasksByTaskId(long taskId) {
        MeetingToTask[] meetingToTasks = this.meetingToTasksDao_.findByTaskId(taskId);

        return this.meetingDxo_.convertMeetingToTaskDtoList(meetingToTasks);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#checkBelongActivityMeeting(long,
     *      long)
     */
    public boolean checkBelongActivityMeeting(long id, long userId) {
        MeetingBasic basic = meetingBasicsDao_.findById(userId, id);
        return basic != null;
    }
}
