/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dao;

import org.seasar.dao.annotation.tiger.Arguments;
import org.seasar.dao.annotation.tiger.S2Dao;
import org.seasar.tugboat.gtd.contents.task.entity.TaskGroupRelation;

/**
 * task group relations.
 * 
 * @author yousuke
 *
 */
@S2Dao(bean=TaskGroupRelation.class)
public interface TaskGroupRelationsDao {
    
    /**
     * insert.
     * 
     * @param taskGroupRelation
     * @return
     */
    @Arguments("taskGroupRelation")
    int insert(TaskGroupRelation taskGroupRelation);
    
    /**
     * delete.
     * 
     * @param taskGroupRelation
     * @return
     */
    @Arguments("taskGroupId")
    int delete(long taskGroupId);
    
    /**
     * delete by task group id and task id.
     * 
     * @param taskGroupId
     * @param taskId
     * @return
     */
    @Arguments({"taskIds"})
    int deleteByTaskIds(long[] taskIds);
    
    /**
     * find by task group id.
     * 
     * @param taskGroupId
     * @return
     */
    @Arguments({"taskGroupId", "userId"})
    TaskGroupRelation findByTaskGroupId(long taskGroupId, long userId);
    
    /**
     * find active groups by user id.
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    TaskGroupRelation[] findActiveGroupsByUserId(long userId);
    
    /**
     * find archive groups by user id.
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    TaskGroupRelation[] findArchiveGroupsByUserId(long userId);
    
    /**
     * find by activity id.
     * 
     * @param activityId
     * @param userId
     * @return
     */
    @Arguments({"activityId", "userId"})
    TaskGroupRelation[] findByActivityId(long activityId, long userId);
    
    /**
     * get tasks by task group id.
     * 
     * @param taskGroupId
     * @return
     */
    @Arguments({"taskGroupId", "userId"})
    TaskGroupRelation[] getTasksByTaskGroupId(long taskGroupId, long userId);
    
    /**
     * find by task id.
     * 
     * @param taskId
     * @return
     */
    TaskGroupRelation findByTaskId(long taskId);
}
