/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dao;

import org.seasar.dao.annotation.tiger.Arguments;
import org.seasar.dao.annotation.tiger.S2Dao;
import org.seasar.tugboat.gtd.contents.task.entity.TaskTag;

/**
 * task tags dao.
 * 
 * @author yosuke
 * 
 */
@S2Dao(bean = TaskTag.class)
public interface TaskTagsDao {

    /**
     * insert.
     * 
     * @param entity
     * @return
     */
    @Arguments("entity")
    int insert(TaskTag entity);

    /**
     * delete.
     * 
     * @param entity
     * @return
     */
    @Arguments("taskId")
    int delete(long taskId);

    /**
     * find by user id.
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    TaskTag[] findByUserId(long userId);

    /**
     * get task ids by user id.
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    TaskTag[] getTaskIdsByUserId(long userId);

    /**
     * get task ids by tag ids and user id.
     * 
     * @param tagIds
     * @param userId
     * @return
     */
    @Arguments( { "tagIds", "userId" })
    TaskTag[] getTaskIdsByTagIdsAndUserId(Long[] tagIds, long userId);

    /**
     * find by task id and user id.
     * 
     * @param userId
     * @param taskId
     * @return
     */
    @Arguments( { "taskId", "userId" })
    TaskTag[] findByTaskIdAndUserId(long taskId, long userId);

    /**
     * find by task ids and user id.
     * 
     * @param taskIds
     * @param userId
     * @return
     */
    @Arguments( { "taskIds", "userId" })
    TaskTag[] findByTaskIdsAndUserId(long[] taskIds, long userId);

}
