/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dao;

import org.seasar.dao.annotation.tiger.Arguments;
import org.seasar.dao.annotation.tiger.S2Dao;
import org.seasar.tugboat.gtd.contents.task.entity.Task;

/**
 * Tasks DAO.
 * 
 * @author yousuke
 */
@S2Dao(bean = Task.class)
public interface TasksDao {
    /**
     * get new task id.
     * 
     * @return
     */
    long getId();

    /**
     * insert task.
     * 
     * @param task
     * @return
     */
    @Arguments("task")
    int insert(Task task);

    /**
     * update task.
     * 
     * @param task
     * @return
     */
    @Arguments("task")
    int update(Task task);

    /**
     * update delete flag.
     * 
     * @param task
     * @return
     */
    @Arguments("task")
    int updateDeleteFlag(Task task);

    /**
     * update delete flag by user id.
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    int updateDeleteFlagByUserId(long userId);

    /**
     * get update target by date.
     * 
     * @param date
     * @return
     */
    @Arguments("date")
    Task[] getUpdateTargetByDate(String date);
}
