/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dto;

import java.io.Serializable;
import java.util.Arrays;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.tugboat.gtd.contents.task.entity.Task;

/**
 * Task dto.
 * 
 * @author yousuke.hara
 * 
 */
public class TaskDto extends Task implements Serializable {
    private static final long serialVersionUID = 2024545417812076850L;

    /**
     * get summary
     * 
     * @return
     */
    public String getSummary() {
        return StringUtils.abbreviate(this.getDescription(), 100);
    }

    /* query status ids. */
    private Integer[] queryStatusIds_;

    public Integer[] getQueryStatusIds() {
        return queryStatusIds_;
    }

    public void setQueryStatusIds(Integer[] queryStatusIds) {
        this.queryStatusIds_ = queryStatusIds;
    }

    /* query classification ids. */
    private Integer[] queryClassificationIds_;

    public Integer[] getQueryClassificationIds() {
        return queryClassificationIds_;
    }

    public void setQueryClassificationIds(Integer[] queryClassificationIds) {
        this.queryClassificationIds_ = queryClassificationIds;
    }

    /* query activity ids. */
    private Long[] queryActivityIds_;

    public Long[] getQueryActivityIds() {
        return queryActivityIds_;
    }

    public void setQueryActivityIds(Long[] queryActivityIds) {
        this.queryActivityIds_ = queryActivityIds;
    }

    /* query person ids */
    private Long[] queryPersonIds_;

    public Long[] getQueryPersonIds() {
        return queryPersonIds_;
    }

    public void setQueryPersonIds(Long[] queryPersonIds) {
        this.queryPersonIds_ = queryPersonIds;
    }

    /* query person Is Null */
    private boolean queryPersonIsNull_;

    public boolean getQueryPersonIsNull() {
        return queryPersonIsNull_;
    }

    public void setQueryPersonIsNull(boolean queryPersonIsNull) {
        this.queryPersonIsNull_ = queryPersonIsNull;
    }

    /* offset */
    private int offset_;

    public int getOffset() {
        return offset_;
    }

    public void setOffset(int offset) {
        this.offset_ = offset;
    }

    /* limit */
    private int limit_;

    public int getLimit() {
        return limit_;
    }

    public void setLimit(int limit) {
        this.limit_ = limit;
    }

    /* start dateformatted string */
    private String startDateformattedString_;

    public String getStartDateformattedString() {
        return startDateformattedString_;
    }

    /* bufferline dateformatted string */
    private String bufferlineDateformattedString_;

    public String getBufferlineDateformattedString() {
        return bufferlineDateformattedString_;
    }

    /* deadline dateformatted string */
    private String deadlineDateformattedString_;

    public String getDeadlineDateformattedString() {
        return deadlineDateformattedString_;
    }

    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this, ToStringStyle.MULTI_LINE_STYLE);
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = super.hashCode();
        result = prime
                * result
                + ((bufferlineDateformattedString_ == null) ? 0
                        : bufferlineDateformattedString_.hashCode());
        result = prime
                * result
                + ((deadlineDateformattedString_ == null) ? 0
                        : deadlineDateformattedString_.hashCode());
        result = prime * result + limit_;
        result = prime * result + offset_;
        result = prime * result + Arrays.hashCode(queryActivityIds_);
        result = prime * result + Arrays.hashCode(queryClassificationIds_);
        result = prime * result + Arrays.hashCode(queryPersonIds_);
        result = prime * result + (queryPersonIsNull_ ? 1231 : 1237);
        result = prime * result + Arrays.hashCode(queryStatusIds_);
        result = prime
                * result
                + ((startDateformattedString_ == null) ? 0
                        : startDateformattedString_.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (!super.equals(obj))
            return false;
        if (getClass() != obj.getClass())
            return false;
        final TaskDto other = (TaskDto) obj;
        if (bufferlineDateformattedString_ == null) {
            if (other.bufferlineDateformattedString_ != null)
                return false;
        } else if (!bufferlineDateformattedString_
                .equals(other.bufferlineDateformattedString_))
            return false;
        if (deadlineDateformattedString_ == null) {
            if (other.deadlineDateformattedString_ != null)
                return false;
        } else if (!deadlineDateformattedString_
                .equals(other.deadlineDateformattedString_))
            return false;
        if (limit_ != other.limit_)
            return false;
        if (offset_ != other.offset_)
            return false;
        if (!Arrays.equals(queryActivityIds_, other.queryActivityIds_))
            return false;
        if (!Arrays.equals(queryClassificationIds_,
                other.queryClassificationIds_))
            return false;
        if (!Arrays.equals(queryPersonIds_, other.queryPersonIds_))
            return false;
        if (queryPersonIsNull_ != other.queryPersonIsNull_)
            return false;
        if (!Arrays.equals(queryStatusIds_, other.queryStatusIds_))
            return false;
        if (startDateformattedString_ == null) {
            if (other.startDateformattedString_ != null)
                return false;
        } else if (!startDateformattedString_
                .equals(other.startDateformattedString_))
            return false;
        return true;
    }
    
}
