/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dto;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

/**
 * task view state relation dto.
 * 
 * @author yosuke
 * 
 */
public class TaskViewStateRelationDto implements Serializable {
    private static final long serialVersionUID = 7018237177403753014L;

    public TaskViewStateRelationDto() {
    }

    /* task browser activity dto */
    private TaskBrowseActivityDto[] taskBrowseActivityDtos_;

    public TaskBrowseActivityDto[] getTaskBrowseActivityDtos() {
        return taskBrowseActivityDtos_;
    }

    public void setTaskBrowseActivityDtos(TaskBrowseActivityDto[] taskBrowseActivityDtos) {
        this.taskBrowseActivityDtos_ = taskBrowseActivityDtos;
    }

    public List<Long> getTaskBrowseActivities() {
        if (this.taskBrowseActivityDtos_ == null || this.taskBrowseActivityDtos_.length == 0)
            return null;

        List<Long> result = new ArrayList<Long>();
        for (int i = 0; i < this.taskBrowseActivityDtos_.length; i++) {
            result.add(this.taskBrowseActivityDtos_[i].getActivityId());
        }
        return result;
    }

    /* task browse classification dto */
    private TaskBrowseClassificationDto[] taskBrowseClassificationDtos_;

    public TaskBrowseClassificationDto[] getTaskBrowseClassificationDtos() {
        return taskBrowseClassificationDtos_;
    }

    public void setTaskBrowseClassificationDtos(
            TaskBrowseClassificationDto[] taskBrowseClassificationDtos) {
        this.taskBrowseClassificationDtos_ = taskBrowseClassificationDtos;
    }

    public List<Integer> getTaskBrowseClassifications() {
        if (this.taskBrowseClassificationDtos_ == null
                || this.taskBrowseClassificationDtos_.length == 0)
            return null;

        List<Integer> result = new ArrayList<Integer>();
        for (int i = 0; i < this.taskBrowseClassificationDtos_.length; i++) {
            result.add(this.taskBrowseClassificationDtos_[i].getClassId());
        }
        return result;
    }

    /* task browse person dtos */
    private TaskBrowsePersonDto[] taskBrowsePersonDtos_;

    public TaskBrowsePersonDto[] getTaskBrowsePersonDtos() {
        return taskBrowsePersonDtos_;
    }

    public void setTaskBrowsePersonDto(TaskBrowsePersonDto[] taskBrowsePersonDtos) {
        this.taskBrowsePersonDtos_ = taskBrowsePersonDtos;
    }

    public List<Integer> getTaskBrowsePersons() {
        if (this.taskBrowsePersonDtos_ == null || this.taskBrowsePersonDtos_.length == 0)
            return null;

        List<Integer> result = new ArrayList<Integer>();
        for (int i = 0; i < this.taskBrowsePersonDtos_.length; i++) {
            result.add(this.taskBrowsePersonDtos_[i].getPersonId());
        }
        return result;
    }

    /* task browse status dtos */
    private TaskBrowseStatusDto[] taskBrowseStatusDtos_;

    public TaskBrowseStatusDto[] getTaskBrowseStatusDtos() {
        return taskBrowseStatusDtos_;
    }

    public void setTaskBrowseStatusDtos(TaskBrowseStatusDto[] taskBrowseStatusDtos) {
        this.taskBrowseStatusDtos_ = taskBrowseStatusDtos;
    }

    public List<Integer> getTaskBrowseStatuses() {
        if (this.taskBrowseStatusDtos_ == null || this.taskBrowseStatusDtos_.length == 0)
            return null;

        List<Integer> result = new ArrayList<Integer>();
        for (int i = 0; i < this.taskBrowseStatusDtos_.length; i++) {
            result.add(this.taskBrowseStatusDtos_[i].getStatusId());
        }
        return result;
    }

    /* task select view dto */
    private TaskSelectViewDto taskSelectViewDto_;

    public TaskSelectViewDto getTaskSelectViewDto() {
        return taskSelectViewDto_;
    }

    public void setTaskSelectViewDto(TaskSelectViewDto taskSelectViewDto) {
        this.taskSelectViewDto_ = taskSelectViewDto;
    }

    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this, ToStringStyle.MULTI_LINE_STYLE);
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + Arrays.hashCode(taskBrowseActivityDtos_);
        result = prime * result
                + Arrays.hashCode(taskBrowseClassificationDtos_);
        result = prime * result + Arrays.hashCode(taskBrowsePersonDtos_);
        result = prime * result + Arrays.hashCode(taskBrowseStatusDtos_);
        result = prime
                * result
                + ((taskSelectViewDto_ == null) ? 0 : taskSelectViewDto_
                        .hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final TaskViewStateRelationDto other = (TaskViewStateRelationDto) obj;
        if (!Arrays.equals(taskBrowseActivityDtos_,
                other.taskBrowseActivityDtos_))
            return false;
        if (!Arrays.equals(taskBrowseClassificationDtos_,
                other.taskBrowseClassificationDtos_))
            return false;
        if (!Arrays.equals(taskBrowsePersonDtos_, other.taskBrowsePersonDtos_))
            return false;
        if (!Arrays.equals(taskBrowseStatusDtos_, other.taskBrowseStatusDtos_))
            return false;
        if (taskSelectViewDto_ == null) {
            if (other.taskSelectViewDto_ != null)
                return false;
        } else if (!taskSelectViewDto_.equals(other.taskSelectViewDto_))
            return false;
        return true;
    }
    
}
