/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dxo.impl;

import java.lang.reflect.InvocationTargetException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.StringUtils;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.common.entity.Classification;
import org.seasar.tugboat.gtd.contents.task.dto.TagDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBodyDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBrowseActivityDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBrowseClassificationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBrowsePersonDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBrowseStatusDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskCommentBodyDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskCommentDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskGroupActivityDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskGroupDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskGroupRelationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskSelectViewDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskToTaskDto;
import org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo;
import org.seasar.tugboat.gtd.contents.task.entity.Tag;
import org.seasar.tugboat.gtd.contents.task.entity.Task;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBody;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBrowseActivity;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBrowseClassification;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBrowsePerson;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBrowseStatus;
import org.seasar.tugboat.gtd.contents.task.entity.TaskComment;
import org.seasar.tugboat.gtd.contents.task.entity.TaskCommentBody;
import org.seasar.tugboat.gtd.contents.task.entity.TaskGroup;
import org.seasar.tugboat.gtd.contents.task.entity.TaskGroupActivity;
import org.seasar.tugboat.gtd.contents.task.entity.TaskGroupRelation;
import org.seasar.tugboat.gtd.contents.task.entity.TaskHistory;
import org.seasar.tugboat.gtd.contents.task.entity.TaskRelation;
import org.seasar.tugboat.gtd.contents.task.entity.TaskSelectView;
import org.seasar.tugboat.gtd.contents.task.entity.TaskTag;
import org.seasar.tugboat.gtd.contents.task.entity.TaskToTask;

/**
 * task dxo impl.
 * 
 * @author yosuke
 * 
 */
public class TaskDxoImpl implements TaskDxo {

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertCalendarTaskDto(
     *      org.seasar.tugboat.gtd.contents.task.entity.Task)
     */
    public TaskDto convertTaskDto(Task task) {
        if (task == null)
            return null;
        try {
            TaskDto dto = new TaskDto();
            BeanUtils.copyProperties(dto, task);
            return dto;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertCalendarTaskDtos(
     *      org.seasar.tugboat.gtd.contents.task.entity.Task[])
     */
    public TaskDto[] convertTaskDtos(Task[] tasks) {
        if (tasks == null || tasks.length == 0)
            return null;
        try {
            List<TaskDto> list = new ArrayList<TaskDto>();
            for (Task t : tasks) {
                TaskDto dto = new TaskDto();
                BeanUtils.copyProperties(dto, t);
                list.add(dto);
            }
            return list.toArray(new TaskDto[list.size()]);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskRelations(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskRelationsDto)
     */
    public TaskRelation convertTaskRelations(TaskRelationDto taskRelationsDto) {
        if (taskRelationsDto == null)
            return null;
        try {
            TaskRelation relations = new TaskRelation();
            BeanUtils.copyProperties(relations, taskRelationsDto);
            return relations;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskDtoWithoutDescription(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskRelations)
     */
    public TaskRelationDto convertTaskDtoWithoutDescription(
            TaskRelation taskRelations) {
        if (taskRelations == null)
            return null;
        try {
            TaskRelationDto dto = new TaskRelationDto();
            BeanUtils.copyProperties(dto, taskRelations);

            DateFormat dfShort = new SimpleDateFormat("yyyy-MM-dd");
            DateFormat dfMedium = new SimpleDateFormat("yyyy-MM-dd HH:mm");

            if (dto.getRegistered() == null
            || "".equals(dto.getRegistered().trim())
            || dto.getRegistered().length() < 14) {
                dto.setRegisteredFormated("");
            } else {
                dto.setRegisteredFormated(dfMedium.format(new SimpleDateFormat(
                        "yyyyMMddHHmmss").parse(dto.getRegistered())));
            }
            if (dto.getStartDate() == null
            || "".equals(dto.getStartDate().trim())
            || dto.getStartDate().trim().length() < 8) {
                dto.setStartDateFormated("");
            } else {
                dto.setStartDateFormated(dfShort.format(new SimpleDateFormat(
                        "yyyyMMdd").parse(dto.getStartDate())));
            }
            if (dto.getDeadline() == null
            || "".equals(dto.getDeadline().trim())
            || "99999999".equals(dto.getDeadline())) {
                dto.setDeadlineFormated("");
            } else {
                dto.setDeadlineFormated(
                    dfShort.format(
                        new SimpleDateFormat(
                            "yyyyMMdd").parse(dto.getDeadline())));
            }
            return dto;
        } catch (ParseException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskDtosWithoutDescription(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskRelations[])
     */
    public TaskRelationDto[] convertTaskDtosWithoutDescription(
            TaskRelation[] taskRelations) {
        if (taskRelations == null || taskRelations.length == 0)
            return null;

        List<TaskRelationDto> dtos = new ArrayList<TaskRelationDto>();
        for (TaskRelation task : taskRelations) {
            dtos.add(this.convertTaskDtoWithoutDescription(task));
        }
        return dtos.toArray(new TaskRelationDto[dtos.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskRelationsDtos(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskRelations[])
     */
    public TaskRelationDto[] convertTaskRelationsDtos(
            TaskRelation[] taskRelations) {
        if (taskRelations == null || taskRelations.length == 0)
            return null;

        List<TaskRelationDto> tempTaskList = new ArrayList<TaskRelationDto>();

        for (TaskRelation r : taskRelations) {
            TaskRelationDto dto = this.convertTaskDtoWithoutDescription(r);
            tempTaskList.add(dto);
        }
        return tempTaskList.toArray(new TaskRelationDto[tempTaskList.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskRelationsDto(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskRelation,
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskTag[])
     */
    public TaskRelationDto convertTaskRelationsDto(TaskRelation taskRelations,
            TaskTag[] taskTags) {
        if (taskRelations == null)
            return null;

        try {
            TaskRelationDto dto = this
                    .convertTaskDtoWithoutDescription(taskRelations);

            List<TaskTagDto> list = new ArrayList<TaskTagDto>();
            for (TaskTag t : taskTags) {
                TaskTagDto taskTagDto = new TaskTagDto();
                BeanUtils.copyProperties(taskTagDto, t);

                list.add(taskTagDto);
            }
            dto.setTaskTagDtos(list.toArray(new TaskTagDto[list.size()]));
            return dto;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskDtosByCalendarId(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskRelations[])
     */
    public TaskRelationDto[] convertTaskDtosIncludeTags(TaskRelation[] relations) {
        if (relations == null || relations.length == 0)
            return null;

        int counter = 0;
        long beforeTaskId = 0;
        TaskRelation beforeTaskRelation = null;
        TaskRelationDto taskRelation = null;

        List<TaskTagDto> tempTaskTagList = new ArrayList<TaskTagDto>();
        List<TaskRelationDto> tempTaskRelationList = new ArrayList<TaskRelationDto>();
        Iterator<TaskRelation> itTask = Arrays.asList(relations).iterator();
        DateFormat dfShort = new SimpleDateFormat("yyyy-MM-dd");

        do {
            TaskRelation r = itTask.hasNext() ? itTask.next() : null;
            if (beforeTaskId == 0) {
                taskRelation = new TaskRelationDto();

            } else {
                if (r != null && beforeTaskId == r.getTaskId()) {
                } else {
                    try {
                        BeanUtils.copyProperties(taskRelation,
                                beforeTaskRelation);
                        taskRelation
                                .setDeadlineFormated(taskRelation.getDeadline() == null
                                        || "".equals(taskRelation.getDeadline()
                                                .trim())
                                        || "99999999".equals(taskRelation
                                                .getDeadline()) ? ""
                                        : dfShort.format(new SimpleDateFormat(
                                                "yyyyMMdd").parse(taskRelation
                                                .getDeadline())));
                    } catch (ParseException e) {
                        throw new SystemUnexpectedRuntimeException(e);
                    } catch (IllegalAccessException e) {
                        throw new SystemUnexpectedRuntimeException(e);
                    } catch (InvocationTargetException e) {
                        throw new SystemUnexpectedRuntimeException(e);
                    }
                    taskRelation.setTaskTagDtos(tempTaskTagList
                            .toArray(new TaskTagDto[tempTaskTagList.size()]));

                    tempTaskRelationList.add(taskRelation);
                    taskRelation = new TaskRelationDto();
                    tempTaskTagList = new ArrayList<TaskTagDto>();
                }
            }
            if (r == null) {
            } else {
                try {
                    if (r.getTagId() != null) {
                        tempTaskTagList.add(new TaskTagDto(r.getTagId(), r
                                .getTagValue()));
                    }
                    beforeTaskRelation = new TaskRelation();
                    BeanUtils.copyProperties(beforeTaskRelation, r);
                } catch (IllegalAccessException e) {
                    throw new SystemUnexpectedRuntimeException(e);
                } catch (InvocationTargetException e) {
                    throw new SystemUnexpectedRuntimeException(e);
                }
                beforeTaskId = r.getTaskId();
            }
            counter++;
        } while (counter <= relations.length);

        return tempTaskRelationList
                .toArray(new TaskRelationDto[tempTaskRelationList.size()]);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTask(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskDto)
     */
    public Task convertTask(TaskDto taskDto) {
        if (taskDto == null)
            return null;
        try {
            Task task = new Task();
            BeanUtils.copyProperties(task, taskDto);

            if (task.getDeadline() != null
                    && !"".equals(task.getDeadline().trim())
                    && task.getDeadline().trim().length() == 8) {
                task.setDeadlineYear(Integer.parseInt(task.getDeadline()
                        .substring(0, 4)));
                task.setDeadlineMonth(Integer.parseInt(task.getDeadline()
                        .substring(4, 6)));
                task.setDeadlineDayOfMonth(Integer.parseInt(task.getDeadline()
                        .substring(6, 8)));
            } else {
                task.setDeadline("99999999");
            }
            return task;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskBody(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskBodyDto)
     */
    public TaskBody convertTaskBody(TaskBodyDto taskBodyDto) {
        if (taskBodyDto == null)
            return null;
        try {
            TaskBody taskBody = new TaskBody();
            BeanUtils.copyProperties(taskBody, taskBodyDto);

            return taskBody;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskHistory(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskRelationsDto)
     */
    public TaskHistory convertTaskHistory(TaskRelationDto taskRelationsDto) {
        if (taskRelationsDto == null)
            return null;
        TaskHistory history = new TaskHistory();
        history.setTaskId(taskRelationsDto.getTaskId());
        history.setStatusId(taskRelationsDto.getStatusId());
        history.setUserId(taskRelationsDto.getUserId());
        return history;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertCalendarTaskCommentDto(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskComment)
     */
    public TaskCommentDto convertTaskCommentDto(TaskComment entity) {
        if (entity == null)
            return null;
        try {
            TaskCommentDto dto = new TaskCommentDto();
            BeanUtils.copyProperties(dto, entity);
            return dto;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertCalendarTaskCommentDtos(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskComment[])
     */
    public TaskCommentDto[] convertTaskCommentDtos(TaskComment[] entities) {
        if (entities == null || entities.length == 0)
            return null;
        try {
            List<TaskCommentDto> list = new ArrayList<TaskCommentDto>();

            for (TaskComment comment : entities) {
                TaskCommentDto dto = new TaskCommentDto();
                BeanUtils.copyProperties(dto, comment);

                DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");
                if (dto.getRegistered() == null
                || "".equals(dto.getRegistered().trim())
                || dto.getRegistered().trim().length() < 14) {
                    dto.setRegisteredFormated("");
                } else {
                    dto.setRegisteredFormated(
                        df.format(
                            new SimpleDateFormat(
                                "yyyyMMddHHmmss").parse(dto.getRegistered())));
                }
                list.add(dto);
            }
            return list.toArray(new TaskCommentDto[list.size()]);
        } catch (ParseException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertCalendarTaskComment(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskCommentDto)
     */
    public TaskComment convertTaskComment(TaskCommentDto dto) {
        if (dto == null)
            return null;
        try {
            TaskComment entity = new TaskComment();
            BeanUtils.copyProperties(entity, dto);
            return entity;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskCommentBody(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskCommentBodyDto)
     */
    public TaskCommentBody convertTaskCommentBody(TaskCommentBodyDto dto) {
        if (dto == null)
            return null;
        try {
            TaskCommentBody taskCommentBody = new TaskCommentBody();
            BeanUtils.copyProperties(taskCommentBody, dto);
            return taskCommentBody;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskToTasks(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskToTaskDto)
     */
    public TaskToTask convertTaskToTasks(TaskToTaskDto taskToTaskDto) {
        if (taskToTaskDto == null)
            return null;
        try {
            TaskToTask taskToTask = new TaskToTask();
            BeanUtils.copyProperties(taskToTask, taskToTaskDto);
            return taskToTask;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTasktoTaskDtos(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskToTask[])
     */
    public TaskToTaskDto[] convertTasktoTaskDtos(TaskToTask[] taskToTasks) {
        if (taskToTasks == null || taskToTasks.length == 0)
            return null;
        try {
            List<TaskToTaskDto> list = new ArrayList<TaskToTaskDto>();
            for (TaskToTask t : taskToTasks) {
                TaskToTaskDto dto = new TaskToTaskDto();
                BeanUtils.copyProperties(dto, t);

                DateFormat dfMedium = new SimpleDateFormat("yyyy-MM-dd");

                if (dto.getDestinationDue() == null
                        || "".equals(dto.getDestinationDue())
                        || "99999999".equals(dto.getDestinationDue())) {

                    dto.setDestinationDue("");
                } else {
                    dto.setDestinationDue(dfMedium.format(new SimpleDateFormat(
                            "yyyyMMdd").parse(dto.getDestinationDue())));
                }
                if (dto.getSourceDue() == null || "".equals(dto.getSourceDue())
                        || "99999999".equals(dto.getSourceDue())) {

                    dto.setSourceDue("");
                } else {
                    dto.setSourceDue(dfMedium.format(new SimpleDateFormat(
                            "yyyyMMdd").parse(dto.getSourceDue())));
                }
                list.add(dto);
            }
            return list.toArray(new TaskToTaskDto[list.size()]);
        } catch (ParseException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertFromTaskToTaskToTask(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskToTask)
     */
    public Task convertFromTaskToTaskToTask(TaskToTask taskToTask) {
        Task task = new Task();
        task.setId(taskToTask.getDestinationTaskId());
        task.setTitle(taskToTask.getDestinationTitle());
        task.setUserId(taskToTask.getDestinationPostedUserId());
        task.setDeadline(taskToTask.getDestinationDue());

        if (taskToTask.getDestinationDue() != null
                && !"".equals(taskToTask.getDestinationDue())
                && taskToTask.getDestinationDue().length() == 8
                && StringUtils.isNumeric(taskToTask.getDestinationDue())) {

            task.setDeadlineYear(Integer.parseInt(taskToTask
                    .getDestinationDue().substring(0, 4)));
            task.setDeadlineMonth(Integer.parseInt(taskToTask
                    .getDestinationDue().substring(4, 6)));
            task.setDeadlineDayOfMonth(Integer.parseInt(taskToTask
                    .getDestinationDue().substring(6, 8)));
        }
        return task;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertFromTaskToTaskToTaskBody(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskToTask)
     */
    public TaskBody convertFromTaskToTaskToTaskBody(TaskToTask taskToTask) {
        TaskBody body = new TaskBody();
        body.setTaskId(taskToTask.getDestinationTaskId());
        body.setBody(taskToTask.getDestinationDescription());
        return body;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertFromTaskToTaskToTaskRelations(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskToTask)
     */
    public TaskRelation convertFromTaskToTaskToTaskRelations(
            TaskToTask taskToTask) {
        TaskRelation relations = new TaskRelation();
        relations.setTaskId(taskToTask.getDestinationTaskId());
        relations.setActivityId(taskToTask.getDestinationActivityId());
        relations.setClassificationId(Classification.INBOX);
        relations.setStatusId(Task.STATUS_TODO);
        relations.setPersonInChargeId(taskToTask
                .getDestinationPersonInChargeId());
        relations.setUserId(taskToTask.getDestinationPostedUserId());
        return relations;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertFromTaskToTaskToTaskHistory(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskToTask)
     */
    public TaskHistory convertFromTaskToTaskToTaskHistory(TaskToTask taskToTask) {
        TaskHistory history = new TaskHistory();
        history.setTaskId(taskToTask.getDestinationTaskId());
        history.setStatusId(Task.STATUS_TODO);
        history.setUserId(taskToTask.getDestinationPostedUserId());
        return history;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskBrowseStatusDtos(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskBrowseStatus[])
     */
    public TaskBrowseStatusDto[] convertTaskBrowseStatusDtos(
            TaskBrowseStatus[] browseStatuses) {

        if (browseStatuses == null || browseStatuses.length == 0)
            return null;
        try {
            List<TaskBrowseStatusDto> list = new ArrayList<TaskBrowseStatusDto>();
            for (TaskBrowseStatus b : browseStatuses) {
                TaskBrowseStatusDto dto = new TaskBrowseStatusDto();
                BeanUtils.copyProperties(dto, b);

                list.add(dto);
            }
            return list.toArray(new TaskBrowseStatusDto[list.size()]);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskBrowseClassficationDtos(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskBrowseClassification[])
     */
    public TaskBrowseClassificationDto[] convertTaskBrowseClassficationDtos(
            TaskBrowseClassification[] browseClassifications) {

        if (browseClassifications == null || browseClassifications.length == 0)
            return null;
        try {
            List<TaskBrowseClassificationDto> list = new ArrayList<TaskBrowseClassificationDto>();
            for (TaskBrowseClassification b : browseClassifications) {
                TaskBrowseClassificationDto dto = new TaskBrowseClassificationDto();
                BeanUtils.copyProperties(dto, b);

                list.add(dto);
            }
            return list.toArray(new TaskBrowseClassificationDto[list.size()]);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskBrowseActivityDtos(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskBrowseActivity[])
     */
    public TaskBrowseActivityDto[] convertTaskBrowseActivityDtos(
            TaskBrowseActivity[] browseActivities) {

        if (browseActivities == null || browseActivities.length == 0)
            return null;
        try {
            List<TaskBrowseActivityDto> list = new ArrayList<TaskBrowseActivityDto>();
            for (TaskBrowseActivity b : browseActivities) {
                TaskBrowseActivityDto dto = new TaskBrowseActivityDto();
                BeanUtils.copyProperties(dto, b);

                list.add(dto);
            }
            return list.toArray(new TaskBrowseActivityDto[list.size()]);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskBrowsePersonDtos(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskBrowsePerson[])
     */
    public TaskBrowsePersonDto[] convertTaskBrowsePersonDtos(
            TaskBrowsePerson[] browsePersons) {

        if (browsePersons == null || browsePersons.length == 0)
            return null;
        try {
            List<TaskBrowsePersonDto> list = new ArrayList<TaskBrowsePersonDto>();
            for (TaskBrowsePerson b : browsePersons) {
                TaskBrowsePersonDto dto = new TaskBrowsePersonDto();
                BeanUtils.copyProperties(dto, b);

                list.add(dto);
            }
            return list.toArray(new TaskBrowsePersonDto[list.size()]);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskSelectViewDto(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskSelectView)
     */
    public TaskSelectViewDto convertTaskSelectViewDto(
            TaskSelectView taskSelectView) {
        if (taskSelectView == null)
            return new TaskSelectViewDto();
        try {
            TaskSelectViewDto dto = new TaskSelectViewDto();
            BeanUtils.copyProperties(dto, taskSelectView);

            return dto;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTagDtos(
     *      org.seasar.tugboat.gtd.contents.task.entity.Tag[])
     */
    public TagDto[] convertTagDtos(Tag[] tags) {
        if (tags == null)
            return null;
        try {
            List<TagDto> list = new ArrayList<TagDto>();
            for (Tag t : tags) {
                TagDto dto = new TagDto();
                BeanUtils.copyProperties(dto, t);

                list.add(dto);
            }
            return list.toArray(new TagDto[list.size()]);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskTagDtos(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskTag[])
     */
    public TaskTagDto[] convertTaskTagDtos(TaskTag[] taskTags) {
        if (taskTags == null || taskTags.length == 0)
            return null;
        try {
            List<TaskTagDto> list = new ArrayList<TaskTagDto>();
            for (TaskTag t : taskTags) {
                TaskTagDto dto = new TaskTagDto();
                BeanUtils.copyProperties(dto, t);

                list.add(dto);
            }
            return list.toArray(new TaskTagDto[list.size()]);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskGroup(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskGroupDto)
     */
    public TaskGroup convertTaskGroup(TaskGroupDto taskGroupDto) {
        if (taskGroupDto == null)
            return null;
        try {
            TaskGroup taskGroup = new TaskGroup();
            BeanUtils.copyProperties(taskGroup, taskGroupDto);
            return taskGroup;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskGroupDto(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskGroup)
     */
    public TaskGroupDto convertTaskGroupDto(TaskGroup taskGroup) {
        if (taskGroup == null)
            return null;
        try {
            TaskGroupDto dto = new TaskGroupDto();
            BeanUtils.copyProperties(dto, taskGroup);
            return dto;

        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskGroupDtos(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskGroup[])
     */
    public TaskGroupDto[] convertTaskGroupDtos(TaskGroup[] taskGroups) {
        if (taskGroups == null || taskGroups.length == 0)
            return null;
        try {
            List<TaskGroupDto> list = new ArrayList<TaskGroupDto>();
            for (TaskGroup t : taskGroups) {
                TaskGroupDto dto = new TaskGroupDto();
                BeanUtils.copyProperties(dto, t);
                list.add(dto);
            }
            return list.toArray(new TaskGroupDto[list.size()]);
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskGroupRelation(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskGroupRelationDto)
     */
    public TaskGroupRelation convertTaskGroupRelation(
            TaskGroupRelationDto taskGroupRelationDto) {
        if (taskGroupRelationDto == null)
            return null;
        try {
            TaskGroupRelation taskGroupRelation = new TaskGroupRelation();
            BeanUtils.copyProperties(taskGroupRelation, taskGroupRelationDto);
            return taskGroupRelation;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskGroupRelationDto(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskGroupRelation)
     */
    public TaskGroupRelationDto convertTaskGroupRelationDto(
            TaskGroupRelation taskGroupRelation) {
        if (taskGroupRelation == null)
            return null;
        try {
            TaskGroupRelationDto dto = new TaskGroupRelationDto();
            BeanUtils.copyProperties(dto, taskGroupRelation);
            return dto;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskGroupRelationDtos(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskGroupRelation[])
     */
    public TaskGroupRelationDto[] convertTaskGroupRelationDtos(
            TaskGroupRelation[] taskGroupRelations) {
        
        if (taskGroupRelations == null || taskGroupRelations.length == 0) return null;
        try {
            List<TaskGroupRelationDto> list = new ArrayList<TaskGroupRelationDto>();
            for (TaskGroupRelation t : taskGroupRelations) {
                TaskGroupRelationDto dto = new TaskGroupRelationDto();
                BeanUtils.copyProperties(dto, t);
                list.add(dto);
            }
            return list.toArray(new TaskGroupRelationDto[list.size()]);    
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskGroupActivity(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskGroupActivityDto)
     */
    public TaskGroupActivity convertTaskGroupActivity(
            TaskGroupActivityDto taskGroupActivityDto) {
        if (taskGroupActivityDto == null)
            return null;
        try {
            TaskGroupActivity taskGroupActivity = new TaskGroupActivity();
            BeanUtils.copyProperties(taskGroupActivity, taskGroupActivityDto);
            return taskGroupActivity;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo#convertTaskGroupActivityDto(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskGroupActivity)
     */
    public TaskGroupActivityDto convertTaskGroupActivityDto(
            TaskGroupActivity taskGroupActivity) {
        if (taskGroupActivity == null)
            return null;
        try {
            TaskGroupActivityDto dto = new TaskGroupActivityDto();
            BeanUtils.copyProperties(dto, taskGroupActivity);
            return dto;
        } catch (IllegalAccessException e) {
            throw new SystemUnexpectedRuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

}
