/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.entity;

import java.io.Serializable;
import java.util.Arrays;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.dao.pager.DefaultPagerCondition;

/**
 * Task browse page condition.
 * 
 * @author yousuke.hara
 * 
 */
public class TaskBrowsePageCondition extends DefaultPagerCondition implements Serializable {
    private static final long serialVersionUID = 8325296927953229058L;
    
    /* user id. */
    private long userId_;

    public long getUserId() {
        return userId_;
    }

    public void setUserId(long userId) {
        this.userId_ = userId;
    }

    /* keyword. */
    private String keyword_;

    public String getKeyword() {
        return keyword_;
    }

    public void setKeyword(String keyword) {
        this.keyword_ = keyword;
    }

    /* query status ids. */
    private Integer[] queryStatusIds_;

    public Integer[] getQueryStatusIds() {
        return queryStatusIds_;
    }

    public void setQueryStatusIds(Integer[] queryStatusIds) {
        this.queryStatusIds_ = queryStatusIds;
    }

    /* query classification ids. */
    private Integer[] queryClassificationIds_;

    public Integer[] getQueryClassificationIds() {
        return queryClassificationIds_;
    }

    public void setQueryClassificationIds(Integer[] queryClassificationIds) {
        this.queryClassificationIds_ = queryClassificationIds;
    }

    /* query activity ids. */
    private Long[] queryActivityIds_;

    public Long[] getQueryActivityIds() {
        return queryActivityIds_;
    }

    public void setQueryActivityIds(Long[] queryActivityIds) {
        this.queryActivityIds_ = queryActivityIds;
    }

    /* query person ids. */
    private Long[] queryPersonIds_;

    public Long[] getQueryPersonIds() {
        return queryPersonIds_;
    }

    public void setQueryPersonIds(Long[] queryPersonIds) {
        this.queryPersonIds_ = queryPersonIds;
    }

    /* query person Is Null */
    private boolean queryPersonIsNull_;

    public boolean getQueryPersonIsNull() {
        return queryPersonIsNull_;
    }

    public void setQueryPersonIsNull(boolean queryPersonIsNull) {
        this.queryPersonIsNull_ = queryPersonIsNull;
    }

    /* query tag which has task ids. */
    private Long[] taskIds_;

    public Long[] getTaskIds() {
        return taskIds_;
    }

    public void setTaskIds(Long[] taskIds) {
        this.taskIds_ = taskIds;
    }

    /* query tag ids */
    private Long[] tagIds_;

    public Long[] getTagIds() {
        return tagIds_;
    }

    public void setTagIds(Long[] tagIds) {
        this.tagIds_ = tagIds;
    }

    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this, ToStringStyle.MULTI_LINE_STYLE);
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result
                + ((keyword_ == null) ? 0 : keyword_.hashCode());
        result = prime * result + Arrays.hashCode(queryActivityIds_);
        result = prime * result + Arrays.hashCode(queryClassificationIds_);
        result = prime * result + Arrays.hashCode(queryPersonIds_);
        result = prime * result + (queryPersonIsNull_ ? 1231 : 1237);
        result = prime * result + Arrays.hashCode(queryStatusIds_);
        result = prime * result + Arrays.hashCode(tagIds_);
        result = prime * result + Arrays.hashCode(taskIds_);
        result = prime * result + (int) (userId_ ^ (userId_ >>> 32));
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final TaskBrowsePageCondition other = (TaskBrowsePageCondition) obj;
        if (keyword_ == null) {
            if (other.keyword_ != null)
                return false;
        } else if (!keyword_.equals(other.keyword_))
            return false;
        if (!Arrays.equals(queryActivityIds_, other.queryActivityIds_))
            return false;
        if (!Arrays.equals(queryClassificationIds_,
                other.queryClassificationIds_))
            return false;
        if (!Arrays.equals(queryPersonIds_, other.queryPersonIds_))
            return false;
        if (queryPersonIsNull_ != other.queryPersonIsNull_)
            return false;
        if (!Arrays.equals(queryStatusIds_, other.queryStatusIds_))
            return false;
        if (!Arrays.equals(tagIds_, other.tagIds_))
            return false;
        if (!Arrays.equals(taskIds_, other.taskIds_))
            return false;
        if (userId_ != other.userId_)
            return false;
        return true;
    }
    
}
