/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.page.impl;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.container.dto.BrowseInfoDto;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.meeting.service.MeetingService;
import org.seasar.tugboat.gtd.contents.task.page.TaskAction;
import org.seasar.tugboat.gtd.contents.task.service.TaskService;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.webservices.service.FeedTokenService;

/**
 * task action impl.
 * 
 * @author yousuke.hara
 * 
 */
@FormAnnotation(
  formId = "taskAction", isAuthorizationCheck = true, isDirectAccessCheck = true, errorPage = "error")
public class TaskActionImpl implements TaskAction {

    /* LOGGER */
    Logger logger_ = Logger.getLogger(TaskActionImpl.class);

    /* calendar common service */
    TaskService taskService_;

    public void setTaskService(TaskService taskService) {
        this.taskService_ = taskService;
    }
    
    /* meeting service */
    MeetingService meetingService_;
    
    public void setMeetingService(MeetingService meetingService) {
        this.meetingService_ = meetingService;
    }
    
    /* feed token service */
    FeedTokenService feedTokenService_;
    
    public void setFeedTokenService(FeedTokenService feedTokenService) {
        this.feedTokenService_ = feedTokenService;
    }

    /* week start date */
    String weekStartDate_;

    public String getWeekStartDate() {
        return weekStartDate_;
    }
    public void setWeekStartDate(String weekStartDate) {
        this.weekStartDate_ = weekStartDate;
    }

    /* week end date */
    String weekEndDate_;

    public String getWeekEndDate() {
        return weekEndDate_;
    }
    public void setWeekEndDate(String weekEndDate) {
        this.weekEndDate_ = weekEndDate;
    }

    /* forward html */
    String forwardHtml_;

    public String getForwardHtml() {
        return forwardHtml_;
    }
    public void setForwardHtml(String forwardHtml) {
        this.forwardHtml_ = forwardHtml;
    }
    
    /* http servlet response */
    HttpServletResponse response_;

    public void setHttpServletResponse(HttpServletResponse response) {
        this.response_ = response;
    }
    
    /* http servlet request*/
    HttpServletRequest request_;

    public void setHttpServletRequest(HttpServletRequest request) {
        this.request_= request;
    }
    
    /* user name */
    String userName_;
    
    public String getUserName() {
    	return userName_;
    }
    public void setUserName(String userName) {
    	this.userName_ = userName;
    }
    
    /* version */
    String version_;
    
    public String getVersion() {
    	return version_;
    }
    public void setVersion(String version) {
    	this.version_ = version;
    }
    
    /* RSS2.0 publish-URL */
    String rss20URL_;
    
    public String getRss20URL() {
        return rss20URL_;
    }
    public void setRss20URL(String rss20URL) {
        this.rss20URL_ = rss20URL;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.calendar.action.CalendarTaskAction#initialize()
     */
    public String initialize() {
        final String PATH_TASKS         = "tasks";
        final String URL_ERROR          = "/view/invalidError.html";
        BrowseInfoDto browseInfo = SessionUtil.getBrowseInfo();
        String url = browseInfo.getUrl();
        String type = browseInfo.getType();
        String pid = browseInfo.getPid();
        UserDto user = SessionUtil.getLoginUser();
        
        if (browseInfo != null && url != null) {
            try {
                boolean isBelongActivityInfo = false;
                if (PATH_TASKS.equals(type)) {
                    
                    isBelongActivityInfo = 
                        taskService_.checkBelongActivityTask(
                                Integer.parseInt(pid),user.getUserId());

                } else {
                    isBelongActivityInfo = 
                        meetingService_.checkBelongActivityMeeting(
                                Integer.parseInt(pid), user.getUserId());
                }
                if (! isBelongActivityInfo) {
                    this.response_.sendRedirect(request_.getContextPath() + URL_ERROR);
                } else {
                	this.response_.setCharacterEncoding("UTF-8");
                    this.response_.sendRedirect(url);
                }
                return null;
            } catch (Exception e) {
                e.printStackTrace();
                throw new SystemUnexpectedRuntimeException(e);
            }
        }
        SessionUtil.clearBrowseInfo();
        
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        this.userName_ = sessionLoginUser.getLastName() + "," + sessionLoginUser.getFirstName();
        
        String[] weekStartDateAndEndDate = 
            this.taskService_.getWeekStartDateAndWeekEndDate(sessionLoginUser.getUserId());

        if (weekStartDateAndEndDate != null
         && weekStartDateAndEndDate.length == 2) {
            this.weekStartDate_ = weekStartDateAndEndDate[0];
            this.weekEndDate_ = weekStartDateAndEndDate[1];
        }
        
        // get rss2.0 url.
        this.rss20URL_ =
            "../rss/tasks/" + this.feedTokenService_.create(user.getUserId());
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskAction#forward()
     */
    public String forwardTasksView() {
        return "gtd_tasks";
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }
}
