/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.page.impl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityService;
import org.seasar.tugboat.gtd.contents.common.entity.Classification;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBodyDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskViewStateRelationDto;
import org.seasar.tugboat.gtd.contents.task.helper.TaskHelper;
import org.seasar.tugboat.gtd.contents.task.page.TaskAjax;
import org.seasar.tugboat.gtd.contents.task.service.TaskService;
import org.seasar.tugboat.gtd.contents.task.service.TaskTagService;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * gtd task Ajax implement.
 * 
 * @author yousuke.hara
 * 
 */
@FormAnnotation(formId = "taskAjax", isAuthorizationCheck = true, isDirectAccessCheck = true)
public class TaskAjaxImpl implements TaskAjax {
	Logger logger_ = Logger.getLogger(TaskAjaxImpl.class);

	/* task.id */
	private int id_;

	public int getId() {
		return id_;
	}

	public void setId(int id) {
		this.id_ = id;
	}

	/* activity id */
	private long activityId_;

	public long getActivityId() {
		return activityId_;
	}

	public void setActivityId(long activityId) {
		this.activityId_ = activityId;
	}

	/* staus id */
	private int statusId_;

	public int getStatusId() {
		return statusId_;
	}

	public void setStatusId(int statusId) {
		this.statusId_ = statusId;
	}

	/* classification id */
	private int classificationId_;

	public int getClassificationId() {
		return classificationId_;
	}

	public void setClassificationId(int classificationId) {
		this.classificationId_ = classificationId;
	}

	/* title */
	private String title_;

	public String getTitle() {
		return title_;
	}

	public void setTitle(String title) {
		this.title_ = title;
	}

	/* descriptionTextile */
	private String description_;

	public String getDescription() {
		return description_;
	}

	public void setDescription(String description) {
		this.description_ = description;
	}

	/* person id */
	private int personId_;

	public int getPersonId() {
		return personId_;
	}

	public void setPersonId(int personId) {
		this.personId_ = personId;
	}

	/* deadline */
	private String deadline_ = "";

	public String getDeadline() {
		return deadline_;
	}

	public void setDeadline(String deadline) {
		this.deadline_ = deadline;
	}

	/* search keyword */
	private String keyword_;

	public String getKeyword() {
		return keyword_;
	}

	public void setKeyword(String keyword) {
		this.keyword_ = keyword;
	}

	/* query param status id */
	private String queryStatusId_;

	public String getQueryStatusId() {
		return queryStatusId_;
	}

	public void setQueryStatusId(String queryStatusId) {
		this.queryStatusId_ = queryStatusId;
	}

	/* query param classification id */
	private String queryClassificationId_;

	public String getQueryClassificationId() {
		return queryClassificationId_;
	}

	public void setQueryClassificationId(String queryClassificationId) {
		this.queryClassificationId_ = queryClassificationId;
	}

	/* query param activity id */
	private String queryActivityId_;

	public String getQueryActivityId() {
		return queryActivityId_;
	}

	public void setQueryActivityId(String queryActivityId) {
		this.queryActivityId_ = queryActivityId;
	}

	/* query param person id */
	private String queryPersonId_;

	public String getQueryPersonId() {
		return queryPersonId_;
	}

	public void setQueryPersonId(String queryPersonId) {
		this.queryPersonId_ = queryPersonId;
	}

	/* offset */
	private int offset_;

	public int getOffset() {
		return offset_;
	}

	public void setOffset(int offset) {
		this.offset_ = offset;
	}

	/* limit */
	private int limit_;

	public int getLimit() {
		return limit_;
	}

	public void setLimit(int limit) {
		this.limit_ = limit;
	}

	/* current page number */
	private int currentPageNumber_;

	public int getCurrentPageNumber() {
		return currentPageNumber_;
	}

	public void setCurrentPageNumber(int currentPageNumber) {
		this.currentPageNumber_ = currentPageNumber;
	}

	/* params */
	private String params_;

	public void setParams(String params) {
		this.params_ = params;
	}

	/* select mode */
	private int selectMode_;

	public void setSelectMode(int selectMode) {
		this.selectMode_ = selectMode;
	}

	/* select folder */
	private int selectFolderId_;

	public void setSelectFolderId(int selectFolderId) {
		this.selectFolderId_ = selectFolderId;
	}

	/* is visible browser */
	private int isVisibleBrowser_;

	public void setIsVisibleBrowser(int isVisibleBrowser) {
		this.isVisibleBrowser_ = isVisibleBrowser;
	}

	/* selected tab id */
	private int selectedTabId_;

	public void setSelectedTabId(int selectedTabId) {
		this.selectedTabId_ = selectedTabId;
	}

	/* tags */
	private String tags_;

	public String getTags() {
		return tags_;
	}

	public void setTags(String tags) {
		this.tags_ = tags;
	}

	/* is update view status */
	private boolean isViewStatusUpdate_ = true;

	public boolean getIsViewStatusUpdate() {
		return isViewStatusUpdate_;
	}

	public void setIsViewStatusUpdate(boolean isViewStatusUpdate) {
		this.isViewStatusUpdate_ = isViewStatusUpdate;
	}

	/* tag ids string */
	private String tagIdsString_ = "";

	public void setTagIdsString(String tagIdsString) {
		this.tagIdsString_ = tagIdsString;
	}
	
	/* task group id */
	private long taskGroupId_;
	
	public long getTaskGroupId() {
	    return taskGroupId_;
	}
	
	public void setTaskGroupId(long taskGroupId) {
	    this.taskGroupId_ = taskGroupId;
	}

	/* calendar task service. */
	private TaskService taskService_;

	public void setTaskService(TaskService taskService) {
		this.taskService_ = taskService;
	}

	/* task tag service */
	private TaskTagService taskTagService_;

	public void setTaskTagService(TaskTagService taskTagService) {
		this.taskTagService_ = taskTagService;
	}

	/* activity service */
	private ActivityService activityService_;

	public void setActivityService(ActivityService activityService) {
		this.activityService_ = activityService;
	}
	
	/* task helper */
	private TaskHelper taskHelper_;

	public void setTaskHelper(TaskHelper taskHelper) {
		this.taskHelper_ = taskHelper;
	}

	/*
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#add()
	 */
	public String add() {
		UserDto sessionLoginUser = SessionUtil.getLoginUser();

		String[] tags = null;
		if (this.tags_ != null && !"".equals(this.tags_.trim())) {
			StringTokenizer token = new StringTokenizer(this.tags_, ",");
			Set<String> tagList = new HashSet<String>();
			while (token.hasMoreTokens()) {
				String tag = token.nextToken();
				if (tag == null || "".equals(tag.trim()))
					continue;

				tagList.add(tag.trim());
			}

			tags = new String[tagList.size()];
			if (!tagList.isEmpty()) {
				Iterator<String> it = tagList.iterator();
				int i = 0;
				while (it.hasNext()) {
					tags[i] = it.next();
					i++;
				}
			}
		}

		try {
			TaskDto taskDto = new TaskDto();
			BeanUtils.copyProperties(taskDto, this);
			taskDto.setUserId(sessionLoginUser.getUserId());

			TaskBodyDto bodyDto = new TaskBodyDto();
			bodyDto.setBody(this.description_);

			TaskRelationDto relationsDto = new TaskRelationDto();
			relationsDto.setActivityId(this.activityId_);
			relationsDto.setStatusId(TaskRelationDto.STATUS_TODO);
			relationsDto.setClassificationId(Classification.INBOX);
			relationsDto.setPersonInChargeId(sessionLoginUser.getUserId());
			relationsDto.setUserId(sessionLoginUser.getUserId());
			relationsDto.setTaskGroupId(this.taskGroupId_);

			this.taskService_.addTask(taskDto, bodyDto, relationsDto, tags);

			return "";
		} catch (Exception e) {
			throw new SystemUnexpectedRuntimeException(e);
		}
	}

	/*
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#getActivitiesJSON()
	 */
	public String getActivitiesJSON() {
		UserDto sessionLoginUser = SessionUtil.getLoginUser();
		;
		ActivityDto[] activities = this.activityService_
				.getOwnActivities(sessionLoginUser.getUserId());

		return this.taskHelper_.createActivitiesJsonForTask(activities);
	}

	/*
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#getTaskListByQueryParams()
	 */
	public String getTaskListByQueryParams() {
		List<Long> personParams = new ArrayList<Long>();
		UserDto sessionLoginUser = SessionUtil.getLoginUser();
		boolean queryPersonIsNull = false;

		if (this.queryPersonId_ != null) {
			StringTokenizer tokenPerson = new StringTokenizer(this.queryPersonId_, ",");
			String personId;

			while (tokenPerson.hasMoreTokens()) {
				personId = tokenPerson.nextToken();

				if ("-1".equals(personId)) {
					queryPersonIsNull = true;
				} else if (!"0".equals(personId)) {
					personParams.add(new Long(personId));
				}
			}
		}
		TaskDto taskDto = new TaskDto();
		taskDto.setUserId(sessionLoginUser.getUserId());
		taskDto.setQueryStatusIds(splitTokenToInteger(this.queryStatusId_, ","));
		taskDto.setQueryClassificationIds(splitTokenToInteger(this.queryClassificationId_, ","));
		taskDto.setQueryActivityIds(splitTokenToLong(this.queryActivityId_, ","));
		taskDto.setQueryPersonIds(personParams.size() == 0 
				? null
				: personParams.toArray(new Long[personParams.size()]));

		taskDto.setQueryPersonIsNull(queryPersonIsNull);
		taskDto.setOffset(this.offset_);
		taskDto.setLimit(this.limit_);

		int rows = this.taskService_.getTaskCountByQueryParams(taskDto);

		TaskRelationDto[] tasks = this.taskService_.getTasksByQueryParams(
				taskDto, this.selectFolderId_, this.isViewStatusUpdate_);

		return this.taskHelper_.createTasksJSON(
	        tasks, rows, this.currentPageNumber_, this.limit_, sessionLoginUser.getUserId());
	}

	/**
	 * split token to integer.
	 * 
	 * @param tokens
	 * @param delimiter
	 * 
	 * @return
	 */
	private Integer[] splitTokenToInteger(String tokens, String delimiter) {
		if (tokens == null)
			return null;
		StringTokenizer tokenClass = new StringTokenizer(tokens, delimiter);
		List<Integer> paramList = new ArrayList<Integer>();

		String classId;
		while (tokenClass.hasMoreTokens()) {
			classId = tokenClass.nextToken();
			if ("0".equals(classId))
				continue;
			try {
				paramList.add(new Integer(classId));
			} catch (NumberFormatException e) {
			}
		}
		return paramList.size() == 0 ? null : paramList
				.toArray(new Integer[paramList.size()]);
	}
	
	/**
	 * split token to long.
	 * 
	 * @param tokens
	 * @param delimiter
	 * @return
	 */
	private Long[] splitTokenToLong(String tokens, String delimiter) {
		if (tokens == null)
			return null;
		StringTokenizer tokenClass = new StringTokenizer(tokens, delimiter);
		List<Long> paramList = new ArrayList<Long>();

		String classId;
		while (tokenClass.hasMoreTokens()) {
			classId = tokenClass.nextToken();
			if ("0".equals(classId))
				continue;
			try {
				paramList.add(new Long(classId));
			} catch (NumberFormatException e) {
			}
		}
		return paramList.size() == 0 
		    ? null 
		    : paramList.toArray(new Long[paramList.size()]);
	}

	/*
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#getDefaultFolderTasks()
	 */
	public String getDefaultFolderTasks() {
		UserDto sessionLoginUser = SessionUtil.getLoginUser();
		TaskRelationDto folderTasks = this.taskService_
				.getDefaultFoloderTasks(sessionLoginUser.getUserId());

		return this.taskHelper_.createDefaultFolderTasksJSON(folderTasks);
	}

	/*
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#getTaskForBalloonJSON()
	 */
	public String getTaskForBalloonJSON() {
		UserDto sessionLoginUser = SessionUtil.getLoginUser();
		TaskRelationDto taskDto = 
		    this.taskService_.getTaskById(
	            this.id_, sessionLoginUser.getUserId());

		return this.taskHelper_.createBalloonDetailJSON(taskDto);
	}

	/*
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#updateClassification()
	 */
	public String updateClassification() {
		if (this.params_ == null || "".equals(this.params_.trim()))
			return null;

		JSONObject jsonObject = JSONObject.fromObject(this.params_);
		int classId = new Integer((String) jsonObject.get("folderId"));
		int statusId = new Integer((String) jsonObject.get("statusId"));
		JSONArray paramsJson = jsonObject.getJSONArray("params");

		long[] updateIds = new long[paramsJson.size()];
		for (int i = 0; i < paramsJson.size(); i++) {
			updateIds[i] = Integer.parseInt((String) paramsJson
					.getJSONObject(i).get("id"));
		}

		UserDto sessionLoginUser = SessionUtil.getLoginUser();
		this.taskService_.updateClassificationId(classId, statusId, updateIds,
				sessionLoginUser.getUserId());

		if (!"".equals(this.tagIdsString_)) {
		    return this.findTasksByHasTags();
		    
		} else if (!"".equals(this.deadline_)) {
		    return this.getTasksByDue();
		    
		} else if (this.taskGroupId_ > 0) {
		    return this.getTasksByTaskGroupId();
		    
		} else {
		    return this.getTaskListByQueryParams();
		}
	}

	/*
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#findTasksByHasTags()
	 */
	public String findTasksByHasTags() {
		UserDto sessionLoginUser = SessionUtil.getLoginUser();
		long userId = sessionLoginUser.getUserId();

		Long[] tagIds = null;
		if (this.tagIdsString_ != null && !"".equals(this.tagIdsString_)) {
			StringTokenizer token = new StringTokenizer(this.tagIdsString_, ",");
			List<Long> list = new ArrayList<Long>();

			while (token.hasMoreTokens()) {
				String value = (String) token.nextElement();
				list.add(new Long(value));
			}
			tagIds = list.toArray(new Long[list.size()]);
		}
		int totalRecords = this.taskTagService_.getRowsByHasTasks(tagIds,userId);

		TaskRelationDto[] tasks = 
		    this.taskTagService_.findTasksByHasTasks(
	            tagIds, userId, this.offset_, this.limit_);

		return this.taskHelper_.createTasksJSON(tasks, totalRecords,
				this.currentPageNumber_, this.limit_, userId);
	}

	/*
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#emptyTrashBox()
	 */
	public String emptyTrashBox() {
		UserDto sessionLoginUser = SessionUtil.getLoginUser();

		int result = this.taskService_.emptyTrashBox(sessionLoginUser
				.getUserId());
		return String.valueOf(result);
	}

	/*
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#selectModeBlockOrList()
	 */
	public String selectModeBlockOrList() {
		UserDto sessionLoginUser = SessionUtil.getLoginUser();

		this.taskService_.updateSelectedMode(this.selectMode_, sessionLoginUser
				.getUserId());
		return null;
	}

	/*
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#getBeforeViewStatusesJSON()
	 */
	public String getBeforeViewStatusesJSON() {
		UserDto sessionLoginUser = SessionUtil.getLoginUser();

		TaskViewStateRelationDto viewStatuses = this.taskService_
				.getViewStateRelations(sessionLoginUser.getUserId());

		return this.taskHelper_.createBeforeViewStatusesJSON(viewStatuses,
				sessionLoginUser.getUserId());
	}

	/*
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#updateIsVisibleBrowser()
	 */
	public String updateIsVisibleBrowser() {
		UserDto sessionLoginUser = SessionUtil.getLoginUser();
		this.taskService_.updateIsVisibleBrowser(this.isVisibleBrowser_,
				sessionLoginUser.getUserId());

		return null;
	}

	/*
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#updateSelectedTab()
	 */
	public String updateSelectedTab() {
		UserDto sessionLoginUser = SessionUtil.getLoginUser();
		this.taskService_.updateSelectedTab(this.selectedTabId_,
				sessionLoginUser.getUserId());
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#getCalendarList()
	 */
	public String getDueList() {
		UserDto sessionLoginUser = SessionUtil.getLoginUser();

		TaskRelationDto[] taskRelations = this.taskService_
				.getDueListByUserId(sessionLoginUser.getUserId());

		return this.taskHelper_.createCalendarDeadlinesJSON(taskRelations);
	}

	/*
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#getTasksByDue()
	 */
	public String getTasksByDue() {
		UserDto sessionLoginUser = SessionUtil.getLoginUser();
		long userId = sessionLoginUser.getUserId();

		TaskRelationDto[] tasks = 
		    this.taskService_.getTasksByDue(this.deadline_, userId);

		int totalRecords = tasks == null ? 0 : tasks.length;

		return this.taskHelper_.createTasksJSON(tasks, totalRecords,
				this.currentPageNumber_, this.limit_, userId);
	}

	/*
	 * @see org.seasar.tugboat.gtd.contents.task.page.TaskAjax#getTasksByTaskGroupId()
	 */
	public String getTasksByTaskGroupId() {
	    long userId = SessionUtil.getLoginUser().getUserId();
	    TaskRelationDto[] tasks = 
    	    this.taskService_.getTasksByTaskGroupId(
                this.taskGroupId_, userId);
	    
	    int totalRecords = tasks == null ? 0 : tasks.length;
	    
	    return this.taskHelper_.createTasksJSON(tasks, totalRecords, 0, totalRecords, userId);
	}
	
	/*
	 * @see java.lang.Object#toString()
	 */
    @Override
	public String toString() {
		return ReflectionToStringBuilder.toString(this,
				ToStringStyle.MULTI_LINE_STYLE);
	}
}
