/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.page.impl;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.contents.task.dto.TaskGroupDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskGroupRelationDto;
import org.seasar.tugboat.gtd.contents.task.helper.TaskHelper;
import org.seasar.tugboat.gtd.contents.task.page.TaskGroupAjax;
import org.seasar.tugboat.gtd.contents.task.service.TaskGroupService;

/**
 * task group ajax implementation.
 * 
 * @author yousuke
 * 
 */
@FormAnnotation(formId = "taskGroupAjax", isAuthorizationCheck = true, isDirectAccessCheck = true)
public class TaskGroupAjaxImpl implements TaskGroupAjax {
    Logger logger_ = Logger.getLogger(TaskGroupAjaxImpl.class);

    /** task group id */ 
    private long taskGroupId_;

    public void setTaskGroupId(long taskGroupId) {
        this.taskGroupId_ = taskGroupId;
    }

    /** activity id */
    private long activityId_;

    public void setActivityId(long activityId) {
        this.activityId_ = activityId;
    }

    /** titile */
    private String title_;

    public void setTitle(String title) {
        this.title_ = title;
    }

    /** description */
    private String description_;

    public void setDescription(String description) {
        this.description_ = description;
    }

    /** start date */
    private String startDate_;

    public void setStartDate(String startDate) {
        this.startDate_ = startDate;
    }

    /** end date */
    private String deadline_;

    public void setDeadline(String deadline) {
        this.deadline_ = deadline;
    }
    
    /** params */
    private String params_;
    
    public String getParams() {
        return params_;
    }
    public void setParams(String params) {
        this.params_ = params;
    }
    
    /** task id */
    private long taskId_;
    
    public long getTaskId() {
        return taskId_;
    }
    public void setTaskId(long taskId) {
        this.taskId_ = taskId;
    }
    
    /** tasks */
    private String tasks_;
    
    public String getTasks() {
        return tasks_;
    }
    public void setTasks(String tasks) {
        this.tasks_ = tasks;
    }

    /* task group */
    private TaskGroupService taskGroupService_;

    public void setTaskGroupService(TaskGroupService taskGroupService) {
        this.taskGroupService_ = taskGroupService;
    }

    /* task helper */
    private TaskHelper taskHelper_;

    public void setTaskHelper(TaskHelper taskHelper) {
        this.taskHelper_ = taskHelper;
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskGroupAjax#create()
     */
    public String add() {
        TaskGroupDto dto = new TaskGroupDto();
        dto.setName(this.title_);
        dto.setDescription(this.description_);
        dto.setStartDate(this.startDate_);
        dto.setDeadline(this.deadline_);
        dto.setUserId(SessionUtil.getLoginUser().getUserId());
        
        long id = this.taskGroupService_.add(this.activityId_, dto);
        return "({'id':" + id + "})";
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskGroupAjax#delete()
     */
    public String delete() {
        TaskGroupDto taskGroupDto = new TaskGroupDto();
        taskGroupDto.setId(this.taskGroupId_);
        this.taskGroupService_.delete(taskGroupDto);
        return "";
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskGroupAjax#select()
     */
    public String select() {
        long userId = SessionUtil.getLoginUser().getUserId();
        TaskGroupRelationDto[] groups = 
            this.taskGroupService_.findActiveGroupsByUserId(userId);

        TaskGroupRelationDto[] archives = 
            this.taskGroupService_.findArchiveGroupsByUserId(userId);
        
        return this.taskHelper_.createTaskGroupsJSON(groups, archives);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskGroupAjax#update()
     */
    public String update() {
        TaskGroupDto taskGroupDto = new TaskGroupDto();
        taskGroupDto.setId(this.taskGroupId_);
        taskGroupDto.setName(this.title_);
        taskGroupDto.setDescription(this.description_);
        taskGroupDto.setStartDate(this.startDate_);
        taskGroupDto.setDeadline(this.deadline_.trim());
        taskGroupDto.setUserId(SessionUtil.getLoginUser().getUserId());
        
        this.taskGroupService_.update(this.activityId_, taskGroupDto);
        return "";
    }
    
    /*
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskGroupAjax#updateByDragnDrop()
     */
    public String updateByDragnDrop() {
        if (this.params_ == null || "".equals(this.params_.trim())) return "";
        
        JSONObject jsonObject = JSONObject.fromObject(this.params_);
        long groupId = new Long((String)jsonObject.get("groupId"));
        
        JSONArray paramsJson = jsonObject.getJSONArray("params");
        
        long[] taskIds = new long[paramsJson.size()];
        for (int i = 0; i < paramsJson.size(); i++) {
            taskIds[i] = 
                Integer.parseInt((String) paramsJson.getJSONObject(i).get("id"));
        }
        this.taskGroupService_.updateByTaskGroupIdAndTaskId(groupId, taskIds);

        return this.select();
    }
    
    /*
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskGroupAjax#findById()
     */
    public String findById() {
        TaskGroupRelationDto taskGroupRelation = 
            this.taskGroupService_.findById(
                this.taskGroupId_, SessionUtil.getLoginUser().getUserId());
        
        return  this.taskHelper_.createTaskGroupJSON(taskGroupRelation);
    }
    
    /*
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskGroupAjax#divideTaskAndCreateTaskGroup()
     */
    public String divideTaskAndCreateTaskGroup() {
        String[] tasks = this.tasks_.split("\n");
        long[] taskIds =
            this.taskGroupService_.divideTaskAndCreateTaskGroup(
                this.activityId_, this.taskId_, this.title_, tasks, SessionUtil.getLoginUser().getUserId());

        return this.taskHelper_.createNewTaskIdsJSON(taskIds);
    }

    /*
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }

}
