/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.service;

import org.seasar.tugboat.gtd.contents.task.dto.TaskCommentBodyDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskCommentDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskToTaskDto;

/**
 * task detail service.
 * 
 * @author yosuke
 * 
 */
public interface TaskDetailService {
    /**
     * add task comment.
     * 
     * @param commentDto
     * @param bodyDto
     * @return
     */
    TaskCommentDto[] addTaskComment(TaskCommentDto commentDto, TaskCommentBodyDto bodyDto);

    /**
     * get comment by task id.
     * 
     * @param taskId
     * @return
     */
    TaskCommentDto[] getTaskCommentsByTaskId(long taskId);

    /**
     * get task comment by comment id.
     * 
     * @param commentId
     * @return
     */
    TaskCommentDto getTaskCommentByCommentId(long commentId);

    /**
     * update comment.
     * 
     * @param bodyDto
     */
    void updateTaskComment(TaskCommentBodyDto bodyDto);

    /**
     * delete comment.
     * 
     * @param bodyDto
     */
    void deleteTaskComment(TaskCommentBodyDto bodyDto);

    /**
     * add task to tasks.
     * 
     * @param taskToTasksDto
     */ 
     void addTaskToTasks(TaskToTaskDto taskToTasksDto);
     

    /**
     * get task to tasks by task id.
     * 
     * @param taskId
     * @return
     */ 
     TaskToTaskDto[] getTaskToTasksByTaskId(long taskId);
     
     /**
      * remove child task.
      * 
      * @param taskId
      * @param childTaskId
      * @param userId
      * @return
      */
     TaskToTaskDto[] removeChildTask(long taskId, long childTaskId);
}
