/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.service;

import org.seasar.tugboat.gtd.contents.task.dto.TaskGroupDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskGroupRelationDto;

/**
 * task group service.
 * 
 * @author yousuke
 * 
 */
public interface TaskGroupService {

    /**
     * add.
     * 
     * @param activityId
     * @param taskGroupDto
     */
    long add(long activityId, TaskGroupDto taskGroupDto);

    /**
     * update.
     * 
     * @param activityId
     * @param taskGroupDto
     */
    void update(long activityId, TaskGroupDto taskGroupDto);

    /**
     * delete.
     * 
     * @param taskGroupDto
     */
    void delete(TaskGroupDto taskGroupDto);

    /**
     * find by id.
     * 
     * @param taskGroupDto
     * @return
     */
    TaskGroupRelationDto findById(long taskGroupId, long userId);
    
    /**
     * find by activity id.
     * 
     * @param activityId
     * @param userId
     * @return
     */
    TaskGroupRelationDto[] findByActivityId(long activityId, long userId);
    
    /**
     * find active group by user id.
     * 
     * @param userId
     * @return
     */
    TaskGroupRelationDto[] findActiveGroupsByUserId(long userId);
    
    /**
     * find archive groups by user id.
     * @param userId
     * @return
     */
    TaskGroupRelationDto[] findArchiveGroupsByUserId(long userId);
    
    /**
     * find by task id.
     * 
     * @param taskId
     * @return
     */
    TaskGroupRelationDto findByTaskId(long taskId);
    
    /**
     * get tasks by task group id.
     * 
     * @param taskGroupId
     * @param userId
     * @return
     */
    TaskGroupRelationDto[] getTasksByTaskGroupId(long taskGroupId, long userId);
    
    /**
     * update by task group id and task id.
     * 
     * @param taskGroupId
     * @param taskIds
     * @return
     */
    void updateByTaskGroupIdAndTaskId(long taskGroupId, long[] taskIds);
    
    /**
     * divide task and create task group.
     * 
     * @param activityId
     * @param taskId
     * @param taskGroupName
     * @param tasks
     * @param userId
     * @return - taskIds
     */
    long[] divideTaskAndCreateTaskGroup(
        long activityId, long taskId, String taskGroupName, String[] tasks, long userId);
}
