/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.service.impl;

import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.contents.common.entity.Classification;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingAgendaDiscussionsDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingToTasksDao;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussion;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingToTask;
import org.seasar.tugboat.gtd.contents.task.dao.TaskBodiesDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskCommentsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskGroupActivitiesDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskGroupRelationsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskGroupsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskHistoriesDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskRelationsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskToTasksDao;
import org.seasar.tugboat.gtd.contents.task.dao.TasksDao;
import org.seasar.tugboat.gtd.contents.task.dto.TaskGroupDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskGroupRelationDto;
import org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo;
import org.seasar.tugboat.gtd.contents.task.entity.Task;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBody;
import org.seasar.tugboat.gtd.contents.task.entity.TaskComment;
import org.seasar.tugboat.gtd.contents.task.entity.TaskGroup;
import org.seasar.tugboat.gtd.contents.task.entity.TaskGroupActivity;
import org.seasar.tugboat.gtd.contents.task.entity.TaskGroupRelation;
import org.seasar.tugboat.gtd.contents.task.entity.TaskHistory;
import org.seasar.tugboat.gtd.contents.task.entity.TaskRelation;
import org.seasar.tugboat.gtd.contents.task.entity.TaskStatus;
import org.seasar.tugboat.gtd.contents.task.entity.TaskToTask;
import org.seasar.tugboat.gtd.contents.task.service.TaskGroupService;

/**
 * task group service implement.
 * 
 * @author yousuke
 * 
 */
public class TaskGroupServiceImpl implements TaskGroupService {
    //private static Logger LOG_ = Logger.getLogger(TaskGroupServiceImpl.class);

    /* task groups dao */
    private TaskGroupsDao taskGroupsDao_;

    public void setTaskGroupsDao(TaskGroupsDao taskGroupsDao) {
        this.taskGroupsDao_ = taskGroupsDao;
    }

    /* task group relations dao */
    private TaskGroupRelationsDao taskGroupRelationDao_;

    public void setTaskGroupRelationsDao(
            TaskGroupRelationsDao taskGroupRelationsDao) {
        this.taskGroupRelationDao_ = taskGroupRelationsDao;
    }

    /* task group activities dao */
    private TaskGroupActivitiesDao taskGroupActivitiesDao_;

    public void setTaskGroupActivitiesDao(
            TaskGroupActivitiesDao taskGroupActivitiesDao) {
        this.taskGroupActivitiesDao_ = taskGroupActivitiesDao;
    }
    
    /* tasks dao */
    private TasksDao tasksDao_;

    public void setTasksDao(TasksDao tasksDao) {
        this.tasksDao_ = tasksDao;
    }
    
    /* task bodies dao */
    private TaskBodiesDao taskBodiesDao_;

    public void setTaskBodiesDao(TaskBodiesDao taskBodiesDao) {
        this.taskBodiesDao_ = taskBodiesDao;
    }
    
    /* task histories dao */
    private TaskHistoriesDao taskHistoriesDao_;

    public void setTaskHistoriesDao(TaskHistoriesDao taskHistoriesDao) {
        this.taskHistoriesDao_ = taskHistoriesDao;
    }
    
    /* task relations dao */
    private TaskRelationsDao taskRelationsDao_;
    
    public void setTaskRelationsDao(TaskRelationsDao taskRelationsDao) {
        this.taskRelationsDao_ = taskRelationsDao;
    }
    
    /* task comments dao */
    private TaskCommentsDao taskCommentsDao_;
    
    public void setTaskCommentsDao(TaskCommentsDao taskCommentsDao) {
        this.taskCommentsDao_ = taskCommentsDao;
    }
    
    /* task to tasks dao */
    private TaskToTasksDao taskToTasksDao_;

    public void setTaskToTasksDao(TaskToTasksDao taskToTasksDao) {
        this.taskToTasksDao_ = taskToTasksDao;
    }

    /* meeting agenda discussions dao */
    private MeetingAgendaDiscussionsDao meetingAgendaDiscussionsDao_;
    
    public void setMeetingAgendaDiscussionsDao(MeetingAgendaDiscussionsDao meetingAgendaDiscussionsDao) {
        this.meetingAgendaDiscussionsDao_ = meetingAgendaDiscussionsDao;
    }
    
    /* meeting to tasks dao */
    private MeetingToTasksDao meetingToTasksDao_;
    
    public void setMeetingToTasksDao(MeetingToTasksDao meetingToTasksDao) {
        this.meetingToTasksDao_ = meetingToTasksDao;
    }

    /* task dxo */
    private TaskDxo taskDxo_;

    public void setTaskDxo(TaskDxo taskDxo) {
        this.taskDxo_ = taskDxo;
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskGroupService#add(
     *      long, org.seasar.tugboat.gtd.contents.task.dto.TaskGroupDto)
     */
    public long add(long activityId, TaskGroupDto taskGroupDto) {
        long id = this.taskGroupsDao_.getId();
        taskGroupDto.setId(id);

        TaskGroup taskGroup = this.taskDxo_.convertTaskGroup(taskGroupDto);
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        taskGroup.setRegistered(nowString);
        taskGroup.setModified(nowString);
        this.taskGroupsDao_.insert(taskGroup);

        TaskGroupActivity taskGroupActivity = new TaskGroupActivity();
        taskGroupActivity.setActivityId(activityId);
        taskGroupActivity.setTaskGroupId(id);
        this.taskGroupActivitiesDao_.insert(taskGroupActivity);
        return id; 
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskGroupService#delete(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskGroupDto)
     */
    public void delete(TaskGroupDto taskGroupDto) {
        TaskGroup taskGroup = this.taskDxo_.convertTaskGroup(taskGroupDto);
        this.taskGroupsDao_.updateDelFlag(taskGroup);
        this.taskGroupRelationDao_.delete(taskGroupDto.getId());
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskGroupService#findById(long,
     *      long)
     */
    public TaskGroupRelationDto findById(long taskGroupId, long userId) {
        TaskGroupRelation taskGroupRelation = 
            this.taskGroupRelationDao_.findByTaskGroupId(taskGroupId, userId);
        return this.taskDxo_.convertTaskGroupRelationDto(taskGroupRelation);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskGroupService#update(
     *      long, org.seasar.tugboat.gtd.contents.task.dto.TaskGroupDto)
     */
    public void update(long activityId, TaskGroupDto taskGroupDto) {
        long taskGroupId = taskGroupDto.getId();
        
        TaskGroupActivity taskGroupActivity = 
            this.taskGroupActivitiesDao_.findByTaskGroupId(taskGroupId);
        if (taskGroupActivity.getActivityId() != activityId) {
            this.taskGroupRelationDao_.delete(taskGroupId);
            this.taskGroupActivitiesDao_.update(new TaskGroupActivity(activityId, taskGroupId));
        }
        TaskGroup taskGroup = this.taskDxo_.convertTaskGroup(taskGroupDto);
        taskGroup.setModified(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
        this.taskGroupsDao_.update(taskGroup);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskGroupService#findByActivityId(long,
     *      long)
     */
    public TaskGroupRelationDto[] findByActivityId(long activityId, long userId) {
        TaskGroupRelation[] taskGroupRelations = this.taskGroupRelationDao_
                .findByActivityId(activityId, userId);
        return this.taskDxo_.convertTaskGroupRelationDtos(taskGroupRelations);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskGroupService#findByUserId(long)
     */
    public TaskGroupRelationDto[] findActiveGroupsByUserId(long userId) {
        TaskGroupRelation[] taskGroupRelations =
            this.taskGroupRelationDao_.findActiveGroupsByUserId(userId);
        return this.taskDxo_.convertTaskGroupRelationDtos(taskGroupRelations);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskGroupService#findArchiveGroupsByUserId(long)
     */
    public TaskGroupRelationDto[] findArchiveGroupsByUserId(long userId) {
        TaskGroupRelation[] taskGroupRelations =
            this.taskGroupRelationDao_.findArchiveGroupsByUserId(userId);
        return this.taskDxo_.convertTaskGroupRelationDtos(taskGroupRelations);
    }
    
    /*
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskGroupService#findByTaskId(long)
     */
    public TaskGroupRelationDto findByTaskId(long taskId) {
        TaskGroupRelation taskGroupRelation = this.taskGroupRelationDao_.findByTaskId(taskId);
        return this.taskDxo_.convertTaskGroupRelationDto(taskGroupRelation);
    }
    
    /*
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskGroupService#getTasksByTaskGroupId(long,
     *      long)
     */
    public TaskGroupRelationDto[] getTasksByTaskGroupId(long taskGroupId, long userId) {
        TaskGroupRelation[] taskGroupRelations = this.taskGroupRelationDao_
                .getTasksByTaskGroupId(taskGroupId, userId);
        return this.taskDxo_.convertTaskGroupRelationDtos(taskGroupRelations);
    }
    
    /*
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskGroupService#updateByTaskGroupIdAndTaskId(
     *      long, long[])
     */
    public void updateByTaskGroupIdAndTaskId(long taskGroupId, long[] taskIds) {
        this.taskGroupRelationDao_.deleteByTaskIds(taskIds);

        for (long taskId : taskIds) {
            TaskGroupRelation relation = new TaskGroupRelation(taskGroupId, taskId);
            this.taskGroupRelationDao_.insert(relation);
        }
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskGroupService#divideTaskAndCreateTaskGroup(
     *      long, long, java.lang.String, java.lang.String[], long)
     */
    public long[] divideTaskAndCreateTaskGroup(
        long activityId, long taskId, String taskGroupName, String[] tasks, long userId) {
        
        TaskComment[] comments = this.taskCommentsDao_.findByTaskId(taskId);
        boolean hasComments = comments != null && comments.length > 0;
        
        TaskToTask[] task2Tasks = 
            this.taskToTasksDao_.findTaskToTasksByTaskId(taskId);
        boolean hasTask2Tasks = task2Tasks != null && task2Tasks.length > 0;
        
        // new task group.
        long taskGroupId = this.taskGroupsDao_.getId();
        TaskRelation sourceTask = this.taskRelationsDao_.findByTaskId(taskId, userId);
        
        TaskGroupDto group = new TaskGroupDto();
        group.setId(taskGroupId);
        group.setName(taskGroupName);
        group.setDescription(sourceTask.getDescription());
        group.setStartDate(sourceTask.getStartDate());
        group.setDeadline(sourceTask.getDeadline());
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        group.setRegistered(nowString);
        group.setModified(nowString);
        group.setUserId(userId);
        this.add(activityId, group);

        // if exists comments or child tasks.
        if (hasComments || hasTask2Tasks) {
            this.taskRelationsDao_.updateClassification(
                taskId, Classification.ARCHIVES, TaskStatus.DONE);
            
            this.taskGroupRelationDao_.deleteByTaskIds(new long[]{taskId});
            this.taskGroupRelationDao_.insert(new TaskGroupRelation(taskGroupId, taskId));

        } else {
            Task task = new Task();
            task.setId(taskId);
            task.setDeleteFlag("1");
            this.tasksDao_.updateDeleteFlag(task);
            this.taskRelationsDao_.updateClassification(taskId, Classification.TRASH, TaskStatus.TODO);
        }
        
        // new tasks.
        long[] taskIds = new long[tasks.length];
        for (int i = 0; i < tasks.length; i++) {
            String title = tasks[i];
            
            Task task = new Task();
            long _taskId = this.tasksDao_.getId();
            taskIds[i] = _taskId;
            task.setId(_taskId);
            task.setTitle(title);
            
            String startDate = sourceTask.getStartDate();
            task.setStartDate(startDate);
            task.setStartYear(Integer.parseInt(startDate.substring(0, 4)));
            task.setStartMonth(Integer.parseInt(startDate.substring(4, 6)));
            task.setStartDayOfMonth(Integer.parseInt(startDate.substring(6, 8)));
            
            task.setRegistered(nowString);
            this.tasksDao_.insert(task);
            this.taskBodiesDao_.insert(new TaskBody(_taskId, ""));
            
            TaskHistory history = new TaskHistory();
            history.setTaskId(_taskId);
            history.setStatusId(TaskStatus.TODO);
            history.setRegistered(nowString);
            history.setUserId(userId);
            this.taskHistoriesDao_.insert(history);
            
            TaskRelation taskRelation = new TaskRelation();
            taskRelation.setTaskId(_taskId);
            taskRelation.setStatusId(TaskStatus.TODO);
            taskRelation.setActivityId(activityId);
            taskRelation.setClassificationId(Classification.INBOX);
            taskRelation.setPersonInChargeId(userId);
            taskRelation.setUserId(userId);
            this.taskRelationsDao_.insert(taskRelation);

            this.taskGroupRelationDao_.insert(new TaskGroupRelation(taskGroupId, _taskId));
        }
        
        // meeting relations.
        MeetingToTask[] meetingToTasks = this.meetingToTasksDao_.findByTaskId(taskId);
        for (MeetingToTask m : meetingToTasks) {
            MeetingAgendaDiscussion discussion = new MeetingAgendaDiscussion();
            discussion.setId(m.getDiscussionId());
            discussion.setModifiedUserId(userId);
            discussion.setModified(nowString);
            this.meetingAgendaDiscussionsDao_.updateDelFlag(discussion);
            
            long discussionId = this.meetingAgendaDiscussionsDao_.findNewId().getId();
            discussion = new MeetingAgendaDiscussion();
            discussion.setId(discussionId);
            discussion.setAgendaId(m.getAgendaId());
            discussion.setType(String.valueOf(MeetingAgendaDiscussion.TYPE_TODO));
            discussion.setPostedUserId(userId);
            discussion.setRegistered(nowString);
            discussion.setModified(nowString);
            discussion.setModifiedUserId(userId);
            this.meetingAgendaDiscussionsDao_.insert(discussion);
            
            for (long t : taskIds) {
                MeetingToTask meetingToTask = new MeetingToTask();
                meetingToTask.setBasicId(m.getBasicId());
                meetingToTask.setAgendaId(m.getAgendaId());
                meetingToTask.setDiscussionId(discussionId);
                meetingToTask.setTaskId(t);
                this.meetingToTasksDao_.insert(meetingToTask);
            }
        }
        return taskIds;
    }

}
