package org.seasar.tugboat.gtd.contents.task.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.contents.activity.dao.ActivityMembersDao;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityMember;
import org.seasar.tugboat.gtd.contents.task.dao.TagsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskRelationsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskTagsDao;
import org.seasar.tugboat.gtd.contents.task.dto.TagDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto;
import org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo;
import org.seasar.tugboat.gtd.contents.task.entity.Tag;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBrowsePageCondition;
import org.seasar.tugboat.gtd.contents.task.entity.TaskRelation;
import org.seasar.tugboat.gtd.contents.task.entity.TaskTag;
import org.seasar.tugboat.gtd.contents.task.service.TaskTagService;

/**
 * task tag service impl.
 * 
 * @author yosuke
 * 
 */
public class TaskTagServiceImpl implements TaskTagService {

    /* task tag dao */
    TaskTagsDao taskTagDao_;
    public void setTaskTagsDao(TaskTagsDao taskTagDao) {
        this.taskTagDao_ = taskTagDao;
    }
    
    /* tags dao */
    TagsDao tagsDao_;
    public void setTagsDao(TagsDao tagsDao) {
        this.tagsDao_ = tagsDao;
    }
    
    /* task relations dao */
    TaskRelationsDao taskRelationsDao_;
    public void setTaskRelationsDao(TaskRelationsDao taskRelationsDao) {
        this.taskRelationsDao_ = taskRelationsDao;
    }
    
    /* activity members dao */
    ActivityMembersDao activityMembersDao_;
    public void setActivityMembersDao(ActivityMembersDao activityMembersDao) {
        this.activityMembersDao_ = activityMembersDao;
    }
    
    /* task dxo */
    TaskDxo taskDxo_;
    public void setTaskDxo(TaskDxo taskDxo) {
        this.taskDxo_ = taskDxo;
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskTagService#findByTaskId(int, int)
     */
    public TaskTagDto[] findByTaskId(long taskId, long userId) {
        TaskTag[] tags = 
            this.taskTagDao_.findByTaskIdAndUserId(taskId, userId);
        
        return this.taskDxo_.convertTaskTagDtos(tags);
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskTagService#findByUserId(int)
     */
    public TaskTagDto[] findByUserId(long userId) {
        TaskTag[] tags = this.taskTagDao_.findByUserId(userId);
        return this.taskDxo_.convertTaskTagDtos(tags);
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskTagService#update(
     *   int, java.lang.String[], int)
     */
    public TaskTagDto[] update(long taskId, String[] tags, long userId) {
        // delete tags.
        this.taskTagDao_.delete(taskId);
        if (tags == null || tags.length == 0) {
            return null;
        }
        
        // add task tags.
        List<Long> tagIds = new ArrayList<Long>();
        
        for (int i = 0; i < tags.length; i++) {
            Tag tag = this.tagsDao_.findByValue(tags[i]);
            if (tag == null) {
                this.tagsDao_.insert(tags[i]);
                tag = this.tagsDao_.findByValue(tags[i]);
            }
            TaskTag taskTag = new TaskTag();
            taskTag.setTaskId(taskId);
            taskTag.setUserId(userId);
            taskTag.setTagId(tag.getId());
            taskTag.setPostedAt(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
            
            tagIds.add(tag.getId());
            
            this.taskTagDao_.insert(taskTag);
        }
        
        // share tags.
        TaskRelation task = 
            this.taskRelationsDao_.findByTaskId(taskId, userId);
        
        ActivityMember[] members = 
            this.activityMembersDao_.selectByActivityId(task.getActivityId());
        
        for (ActivityMember m : members) {
            if (m.getUserId() == userId) continue;
            
            for (Long id : tagIds) {
                TaskTag taskTag = new TaskTag();
                taskTag.setTaskId(taskId);
                taskTag.setUserId(m.getUserId());
                taskTag.setTagId(id);
                taskTag.setPostedAt(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
                this.taskTagDao_.insert(taskTag);
            }
        }
        
        // get after updated tags.
        TaskTag[] taskTags = 
            this.taskTagDao_.findByTaskIdAndUserId(taskId, userId);
        
        return this.taskDxo_.convertTaskTagDtos(taskTags);
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskTagService#update(int[], int, int)
     */
    public TaskTagDto[] update(long[] taskIds, long tagId, long userId) {
        for (long id : taskIds) {
            TaskTag[]  tags = 
                this.taskTagDao_.findByTaskIdAndUserId(id, userId);
            
            boolean isExists = false;
            for (TaskTag tag : tags) {
                if (tagId == tag.getTagId()) {
                    isExists = true;
                    break;
                }
            }
            if (! isExists) {
                TaskTag entity = new TaskTag();
                entity.setTaskId(id);
                entity.setTagId(tagId);
                entity.setUserId(userId);
                entity.setPostedAt(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
                
                this.taskTagDao_.insert(entity);
            }
        }
        TaskTag[] taskTags = 
            this.taskTagDao_.findByTaskIdsAndUserId(taskIds, userId);
        
        return this.taskDxo_.convertTaskTagDtos(taskTags);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskTagService#findTasksByHasTasks(int, int, int)
     */
    public TaskRelationDto[] findTasksByHasTasks(long userId, int offset, int limit) {
        return this.findTasksByHasTasks(null, userId, offset, limit);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskTagService#findTasksByHasTasks(java.lang.Integer[], int, int, int)
     */
    public TaskRelationDto[] findTasksByHasTasks(Long[] tagIds, long userId, int offset, int limit) {
        Long[] taskIds = this.getTaskIdsByUserId(tagIds, userId);
        if (taskIds == null || taskIds.length == 0) return null;
        
        TaskBrowsePageCondition condition = new TaskBrowsePageCondition();
        condition.setTagIds(tagIds);
        condition.setUserId(userId);
        condition.setOffset(offset);
        condition.setLimit(limit);
        
        TaskRelation[] targetTaskIds = 
            this.taskRelationsDao_.findTaskIdsByHasTags(condition);
        
        List<Long> taskIdList = new ArrayList<Long>();
        if (targetTaskIds != null && targetTaskIds.length > 0) {
            for (TaskRelation r : targetTaskIds) {
                taskIdList.add(r.getTaskId());
            }
        }
        Long[] ids = taskIdList.toArray(new Long[taskIdList.size()]);
        
        TaskRelation[] tasks = null;
        if (ids == null || ids.length == 0) {
        } else {
            tasks =this.taskRelationsDao_.findByHasTags(ids, userId);
        }
        return this.taskDxo_.convertTaskDtosIncludeTags(tasks);
    }
    
    /**
     * get task ids by user id.
     * 
     * @param tagIds
     * @param userId
     * @return
     */
    Long[] getTaskIdsByUserId(Long[] tagIds, long userId) {
        TaskTag[] taskTags = 
            tagIds == null 
            ? this.taskTagDao_.getTaskIdsByUserId(userId)
            : this.taskTagDao_.getTaskIdsByTagIdsAndUserId(tagIds, userId);
        
        if (taskTags == null || taskTags.length == 0) {
            return null;
        }
        List<Long> list = new ArrayList<Long>();
        for (TaskTag t : taskTags) {
            list.add(t.getTaskId());
        }
        Long[] taskIds = list.toArray(new Long[list.size()]);
        return taskIds;
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskTagService#getRowsByHasTasks(int)
     */
    public int getRowsByHasTasks(long userId) {
        return this.getRowsByHasTasks(null, userId);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskTagService#getRowsByHasTasks(java.lang.Integer[], int)
     */
    public int getRowsByHasTasks(Long[] tagIds, long userId) {
        Long[] taskIds = this.getTaskIdsByUserId(tagIds, userId);
        
        TaskBrowsePageCondition condition = new TaskBrowsePageCondition();
        condition.setTaskIds(taskIds);
        condition.setUserId(userId);
        return this.taskRelationsDao_.getRowsByHasTags(condition);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskTagService#findTagsByKeyword(java.lang.String)
     */
    public TagDto[] findTagsByKeyword(String keyword) {
        Tag[] tags = this.tagsDao_.findByKeyword("%" + keyword + "%");
        return this.taskDxo_.convertTagDtos(tags);
    }

}
