/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package org.seasar.tugboat.gtd.contents.timeline.entity.xml;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

/**
 * <b>Event</b> is generated from xmlTimeline.rng by Relaxer. This class is
 * derived from:
 * 
 * <!-- for programmer <element name="event"> <optional> <attribute
 * name="color"> <data type="token"/> </attribute> </optional> <attribute
 * name="end"> <data type="token"/> </attribute> <optional> <attribute
 * name="image"> <data type="token"/> </attribute> </optional> <optional>
 * <attribute name="isDuration"> <data type="boolean"/> </attribute> </optional>
 * <attribute name="start"> <data type="token"/> </attribute> <attribute
 * name="title"> <data type="token"/> </attribute> <optional> <attribute
 * name="link"> <data type="token"/> </attribute> </optional> <optional>
 * <attribute name="icon"> <data type="token"/> </attribute> </optional> <text/>
 * </element>--> <!-- for javadoc -->
 * 
 * <pre>
 *  &lt;element name=&quot;event&quot;&gt;
 *       &lt;optional&gt;
 *         &lt;attribute name=&quot;color&quot;&gt;
 *           &lt;data type=&quot;token&quot;/&gt;
 *         &lt;/attribute&gt;
 *       &lt;/optional&gt;
 *       &lt;attribute name=&quot;end&quot;&gt;
 *         &lt;data type=&quot;token&quot;/&gt;
 *       &lt;/attribute&gt;
 *       &lt;optional&gt;
 *         &lt;attribute name=&quot;image&quot;&gt;
 *           &lt;data type=&quot;token&quot;/&gt;
 *         &lt;/attribute&gt;
 *       &lt;/optional&gt;
 *       &lt;optional&gt;
 *         &lt;attribute name=&quot;isDuration&quot;&gt;
 *           &lt;data type=&quot;boolean&quot;/&gt;
 *         &lt;/attribute&gt;
 *       &lt;/optional&gt;
 *       &lt;attribute name=&quot;start&quot;&gt;
 *         &lt;data type=&quot;token&quot;/&gt;
 *       &lt;/attribute&gt;
 *       &lt;attribute name=&quot;title&quot;&gt;
 *         &lt;data type=&quot;token&quot;/&gt;
 *       &lt;/attribute&gt;
 *       &lt;optional&gt;
 *         &lt;attribute name=&quot;link&quot;&gt;
 *           &lt;data type=&quot;token&quot;/&gt;
 *         &lt;/attribute&gt;
 *       &lt;/optional&gt;
 *       &lt;optional&gt;
 *         &lt;attribute name=&quot;icon&quot;&gt;
 *           &lt;data type=&quot;token&quot;/&gt;
 *         &lt;/attribute&gt;
 *       &lt;/optional&gt;
 *       &lt;text/&gt;
 *     &lt;/element&gt;
 * </pre>
 * 
 * @version xmlTimeline.rng (Mon Dec 18 18:35:27 JST 2006)
 * @author Relaxer 1.1b (http://www.relaxer.org)
 */
public class Event implements java.io.Serializable, Cloneable, IRContentClassHandler, IRNode {
    private static final long serialVersionUID = 1L;
    
    private String content_;

    private String color_;

    private String end_;

    private String image_;

    private Boolean isDuration_;

    private String start_;

    private String title_;

    private String link_;

    private String icon_;

    private StringBuffer saxBuffer_;

    private IRNode parentRNode_;

    /**
     * Creates a <code>Event</code>.
     * 
     */
    public Event() {
        end_ = "";
        start_ = "";
        title_ = "";
    }

    /**
     * Creates a <code>Event</code>.
     * 
     * @param source
     */
    public Event(Event source) {
        setup(source);
    }

    /**
     * Creates a <code>Event</code> by the Stack <code>stack</code> that
     * contains Elements. This constructor is supposed to be used internally by
     * the Relaxer system.
     * 
     * @param stack
     */
    public Event(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>Event</code> by the Document <code>doc</code>.
     * 
     * @param doc
     */
    public Event(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>Event</code> by the Element <code>element</code>.
     * 
     * @param element
     */
    public Event(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>Event</code> by the File <code>file</code>.
     * 
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Event(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>Event</code> by the String representation of URI
     * <code>uri</code>.
     * 
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Event(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>Event</code> by the URL <code>url</code>.
     * 
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Event(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>Event</code> by the InputStream <code>in</code>.
     * 
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Event(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>Event</code> by the InputSource <code>is</code>.
     * 
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Event(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>Event</code> by the Reader <code>reader</code>.
     * 
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Event(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>Event</code> by the Event <code>source</code>.
     * 
     * @param source
     */
    public void setup(Event source) {
//        int size;
        content_ = source.content_;
        color_ = source.color_;
        end_ = source.end_;
        image_ = source.image_;
        isDuration_ = source.isDuration_;
        start_ = source.start_;
        title_ = source.title_;
        link_ = source.link_;
        icon_ = source.icon_;
    }

    /**
     * Initializes the <code>Event</code> by the Document <code>doc</code>.
     * 
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>Event</code> by the Element <code>element</code>.
     * 
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>Event</code> by the Stack <code>stack</code>
     * that contains Elements. This constructor is supposed to be used
     * internally by the Relaxer system.
     * 
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
//        RStack stack = new RStack(element);
        content_ = URelaxer.getElementPropertyAsString(element);
        color_ = URelaxer.getAttributePropertyAsString(element, "color");
        end_ = URelaxer.getAttributePropertyAsString(element, "end");
        image_ = URelaxer.getAttributePropertyAsString(element, "image");
        isDuration_ = URelaxer.getAttributePropertyAsBooleanObject(element, "isDuration");
        start_ = URelaxer.getAttributePropertyAsString(element, "start");
        title_ = URelaxer.getAttributePropertyAsString(element, "title");
        link_ = URelaxer.getAttributePropertyAsString(element, "link");
        icon_ = URelaxer.getAttributePropertyAsString(element, "icon");
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new Event((Event) this));
    }

    /**
     * Creates a DOM representation of the object. Result is appended to the
     * Node <code>parent</code>.
     * 
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document) parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElement("event");
        URelaxer.setElementPropertyByString(element, this.content_);
//        int size;
        if (this.color_ != null) {
            URelaxer.setAttributePropertyByString(element, "color", this.color_);
        }
        if (this.end_ != null) {
            URelaxer.setAttributePropertyByString(element, "end", this.end_);
        }
        if (this.image_ != null) {
            URelaxer.setAttributePropertyByString(element, "image", this.image_);
        }
        if (this.isDuration_ != null) {
            URelaxer.setAttributePropertyByBoolean(element, "isDuration", this.isDuration_);
        }
        if (this.start_ != null) {
            URelaxer.setAttributePropertyByString(element, "start", this.start_);
        }
        if (this.title_ != null) {
            URelaxer.setAttributePropertyByString(element, "title", this.title_);
        }
        if (this.link_ != null) {
            URelaxer.setAttributePropertyByString(element, "link", this.link_);
        }
        if (this.icon_ != null) {
            URelaxer.setAttributePropertyByString(element, "icon", this.icon_);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>Event</code> by the File <code>file</code>.
     * 
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>Event</code> by the String representation of URI
     * <code>uri</code>.
     * 
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Event</code> by the URL <code>url</code>.
     * 
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Event</code> by the InputStream <code>in</code>.
     * 
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException,
            ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Event</code> by the InputSource <code>is</code>.
     * 
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException,
            ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Event</code> by the Reader <code>reader</code>.
     * 
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
     * Creates a DOM document representation of the object.
     * 
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the String property <b>content</b>.
     * 
     * @return String
     */
    public String getContent() {
        return (content_);
    }

    /**
     * Sets the String property <b>content</b>.
     * 
     * @param content
     */
    public void setContent(String content) {
        this.content_ = content;
    }

    /**
     * Gets the String property <b>color</b>.
     * 
     * @return String
     */
    public String getColor() {
        return (color_);
    }

    /**
     * Sets the String property <b>color</b>.
     * 
     * @param color
     */
    public void setColor(String color) {
        this.color_ = color;
    }

    /**
     * Gets the String property <b>end</b>.
     * 
     * @return String
     */
    public String getEnd() {
        return (end_);
    }

    /**
     * Sets the String property <b>end</b>.
     * 
     * @param end
     */
    public void setEnd(String end) {
        this.end_ = end;
    }

    /**
     * Gets the String property <b>image</b>.
     * 
     * @return String
     */
    public String getImage() {
        return (image_);
    }

    /**
     * Sets the String property <b>image</b>.
     * 
     * @param image
     */
    public void setImage(String image) {
        this.image_ = image;
    }

    /**
     * Gets the boolean property <b>isDuration</b>.
     * 
     * @return boolean
     */
    public boolean getIsDuration() {
        if (isDuration_ == null) {
            return (false);
        }
        return (isDuration_.booleanValue());
    }

    /**
     * Gets the boolean property <b>isDuration</b>.
     * 
     * @param isDuration
     * @return boolean
     */
    public boolean getIsDuration(boolean isDuration) {
        if (isDuration_ == null) {
            return (isDuration);
        }
        return (this.isDuration_.booleanValue());
    }

    /**
     * Gets the boolean property <b>isDuration</b>.
     * 
     * @return Boolean
     */
    public Boolean getIsDurationAsBoolean() {
        return (isDuration_);
    }

    /**
     * Check the boolean property <b>isDuration</b>.
     * 
     * @return boolean
     */
    public boolean checkIsDuration() {
        return (isDuration_ != null);
    }

    /**
     * Sets the boolean property <b>isDuration</b>.
     * 
     * @param isDuration
     */
    public void setIsDuration(boolean isDuration) {
        this.isDuration_ = new Boolean(isDuration);
    }

    /**
     * Sets the boolean property <b>isDuration</b>.
     * 
     * @param isDuration
     */
    public void setIsDuration(Boolean isDuration) {
        this.isDuration_ = isDuration;
    }

    /**
     * Gets the String property <b>start</b>.
     * 
     * @return String
     */
    public String getStart() {
        return (start_);
    }

    /**
     * Sets the String property <b>start</b>.
     * 
     * @param start
     */
    public void setStart(String start) {
        this.start_ = start;
    }

    /**
     * Gets the String property <b>title</b>.
     * 
     * @return String
     */
    public String getTitle() {
        return (title_);
    }

    /**
     * Sets the String property <b>title</b>.
     * 
     * @param title
     */
    public void setTitle(String title) {
        this.title_ = title;
    }

    /**
     * Gets the String property <b>link</b>.
     * 
     * @return String
     */
    public String getLink() {
        return (link_);
    }

    /**
     * Sets the String property <b>link</b>.
     * 
     * @param link
     */
    public void setLink(String link) {
        this.link_ = link;
    }

    /**
     * Gets the String property <b>icon</b>.
     * 
     * @return String
     */
    public String getIcon() {
        return (icon_);
    }

    /**
     * Sets the String property <b>icon</b>.
     * 
     * @param icon
     */
    public void setIcon(String icon) {
        this.icon_ = icon;
    }

    /**
     * Makes an XML text representation.
     * 
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     * 
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
//        int size;
        buffer.append("<event");
        if (color_ != null) {
            buffer.append(" color=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getColor())));
            buffer.append("\"");
        }
        if (end_ != null) {
            buffer.append(" end=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getEnd())));
            buffer.append("\"");
        }
        if (image_ != null) {
            buffer.append(" image=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getImage())));
            buffer.append("\"");
        }
        if (isDuration_ != null) {
            buffer.append(" isDuration=\"");
            buffer.append(URelaxer.getString(getIsDuration()));
            buffer.append("\"");
        }
        if (start_ != null) {
            buffer.append(" start=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getStart())));
            buffer.append("\"");
        }
        if (title_ != null) {
            buffer.append(" title=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getTitle())));
            buffer.append("\"");
        }
        if (link_ != null) {
            buffer.append(" link=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getLink())));
            buffer.append("\"");
        }
        if (icon_ != null) {
            buffer.append(" icon=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getIcon())));
            buffer.append("\"");
        }
        buffer.append(">");
        buffer.append(URelaxer.escapeCharData(URelaxer.getString(getContent())));
        buffer.append("</event>");
    }

    /**
     * Makes an XML text representation.
     * 
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
//        int size;
        buffer.write("<event");
        if (color_ != null) {
            buffer.write(" color=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getColor())));
            buffer.write("\"");
        }
        if (end_ != null) {
            buffer.write(" end=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getEnd())));
            buffer.write("\"");
        }
        if (image_ != null) {
            buffer.write(" image=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getImage())));
            buffer.write("\"");
        }
        if (isDuration_ != null) {
            buffer.write(" isDuration=\"");
            buffer.write(URelaxer.getString(getIsDuration()));
            buffer.write("\"");
        }
        if (start_ != null) {
            buffer.write(" start=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getStart())));
            buffer.write("\"");
        }
        if (title_ != null) {
            buffer.write(" title=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getTitle())));
            buffer.write("\"");
        }
        if (link_ != null) {
            buffer.write(" link=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getLink())));
            buffer.write("\"");
        }
        if (icon_ != null) {
            buffer.write(" icon=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getIcon())));
            buffer.write("\"");
        }
        buffer.write(">");
        buffer.write(URelaxer.escapeCharData(URelaxer.getString(getContent())));
        buffer.write("</event>");
    }

    /**
     * Makes an XML text representation.
     * 
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
//        int size;
        buffer.print("<event");
        if (color_ != null) {
            buffer.print(" color=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getColor())));
            buffer.print("\"");
        }
        if (end_ != null) {
            buffer.print(" end=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getEnd())));
            buffer.print("\"");
        }
        if (image_ != null) {
            buffer.print(" image=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getImage())));
            buffer.print("\"");
        }
        if (isDuration_ != null) {
            buffer.print(" isDuration=\"");
            buffer.print(URelaxer.getString(getIsDuration()));
            buffer.print("\"");
        }
        if (start_ != null) {
            buffer.print(" start=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getStart())));
            buffer.print("\"");
        }
        if (title_ != null) {
            buffer.print(" title=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getTitle())));
            buffer.print("\"");
        }
        if (link_ != null) {
            buffer.print(" link=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getLink())));
            buffer.print("\"");
        }
        if (icon_ != null) {
            buffer.print(" icon=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getIcon())));
            buffer.print("\"");
        }
        buffer.print(">");
        buffer.print(URelaxer.escapeCharData(URelaxer.getString(getContent())));
        buffer.print("</event>");
    }

    /**
     * Makes an XML text representation.
     * 
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     * 
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     * 
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     * 
     * @return String
     */
    public String getContentAsString() {
        return (URelaxer.getString(getContent()));
    }

    /**
     * Gets the property value as String.
     * 
     * @return String
     */
    public String getColorAsString() {
        return (URelaxer.getString(getColor()));
    }

    /**
     * Gets the property value as String.
     * 
     * @return String
     */
    public String getEndAsString() {
        return (URelaxer.getString(getEnd()));
    }

    /**
     * Gets the property value as String.
     * 
     * @return String
     */
    public String getImageAsString() {
        return (URelaxer.getString(getImage()));
    }

    /**
     * Gets the property value as String.
     * 
     * @return String
     */
    public String getIsDurationAsString() {
        return (URelaxer.getString(getIsDuration()));
    }

    /**
     * Gets the property value as String.
     * 
     * @return String
     */
    public String getStartAsString() {
        return (URelaxer.getString(getStart()));
    }

    /**
     * Gets the property value as String.
     * 
     * @return String
     */
    public String getTitleAsString() {
        return (URelaxer.getString(getTitle()));
    }

    /**
     * Gets the property value as String.
     * 
     * @return String
     */
    public String getLinkAsString() {
        return (URelaxer.getString(getLink()));
    }

    /**
     * Gets the property value as String.
     * 
     * @return String
     */
    public String getIconAsString() {
        return (URelaxer.getString(getIcon()));
    }

    /**
     * Sets the property value by String.
     * 
     * @param string
     */
    public void setContentByString(String string) {
        setContent(string);
    }

    /**
     * Sets the property value by String.
     * 
     * @param string
     */
    public void setColorByString(String string) {
        setColor(string);
    }

    /**
     * Sets the property value by String.
     * 
     * @param string
     */
    public void setEndByString(String string) {
        setEnd(string);
    }

    /**
     * Sets the property value by String.
     * 
     * @param string
     */
    public void setImageByString(String string) {
        setImage(string);
    }

    /**
     * Sets the property value by String.
     * 
     * @param string
     */
    public void setIsDurationByString(String string) {
        setIsDuration(new Boolean(string).booleanValue());
    }

    /**
     * Sets the property value by String.
     * 
     * @param string
     */
    public void setStartByString(String string) {
        setStart(string);
    }

    /**
     * Sets the property value by String.
     * 
     * @param string
     */
    public void setTitleByString(String string) {
        setTitle(string);
    }

    /**
     * Sets the property value by String.
     * 
     * @param string
     */
    public void setLinkByString(String string) {
        setLink(string);
    }

    /**
     * Sets the property value by String.
     * 
     * @param string
     */
    public void setIconByString(String string) {
        setIcon(string);
    }

    /**
     * Returns a String representation of this object. While this method informs
     * as XML format representaion, it's purpose is just information, not making
     * a rigid XML documentation.
     * 
     * @return String
     */
    @Override
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Generates SAX events for ContentHandler.
     * 
     * @param handler
     * @exception SAXException
     */
    public void makeDocument(ContentHandler handler) throws SAXException {
        handler.startDocument();
        makeElement(handler);
        handler.endDocument();
    }

    /**
     * Generates SAX events for ContentHandler.
     * 
     * @param handler
     * @exception SAXException
     */
    public void makeElement(ContentHandler handler) throws SAXException {
//        int size;
        String string;
//        String slotQName;
        AttributesImpl attrs = new AttributesImpl();
        if (color_ != null) {
            attrs.addAttribute("", "color", "color", "CDATA", URelaxer.getString(getColor()));
        }
        attrs.addAttribute("", "end", "end", "CDATA", URelaxer.getString(getEnd()));
        if (image_ != null) {
            attrs.addAttribute("", "image", "image", "CDATA", URelaxer.getString(getImage()));
        }
        if (isDuration_ != null) {
            attrs.addAttribute("", "isDuration", "isDuration", "CDATA", URelaxer
                    .getString(getIsDuration()));
        }
        attrs.addAttribute("", "start", "start", "CDATA", URelaxer.getString(getStart()));
        attrs.addAttribute("", "title", "title", "CDATA", URelaxer.getString(getTitle()));
        if (link_ != null) {
            attrs.addAttribute("", "link", "link", "CDATA", URelaxer.getString(getLink()));
        }
        if (icon_ != null) {
            attrs.addAttribute("", "icon", "icon", "CDATA", URelaxer.getString(getIcon()));
        }
        handler.startElement("", "event", "event", attrs);
        string = URelaxer.getString(getContent());
        handler.characters(string.toCharArray(), 0, string.length());
        handler.endElement("", "event", "event");
    }

    /**
     * Generates SAX events for ContentHandler.
     * 
     * @param attrs
     * @exception SAXException
     */
    public void makeAttribute(AttributesImpl attrs) throws SAXException {
    }

    /**
     * Event handler for InitElement during object construction.
     * 
     * @param attrs
     */
    public void initElement(Attributes attrs) {
        String string;
        saxBuffer_ = new StringBuffer();
        string = attrs.getValue("color");
        if (string != null) {
            setColor(string);
        }
        string = attrs.getValue("end");
        setEnd(string);
        string = attrs.getValue("image");
        if (string != null) {
            setImage(string);
        }
        string = attrs.getValue("isDuration");
        if (string != null) {
            setIsDuration(new Boolean(string).booleanValue());
        }
        string = attrs.getValue("start");
        setStart(string);
        string = attrs.getValue("title");
        setTitle(string);
        string = attrs.getValue("link");
        if (string != null) {
            setLink(string);
        }
        string = attrs.getValue("icon");
        if (string != null) {
            setIcon(string);
        }
    }

    /**
     * Event handler for declarePrefixMapping during object construction.
     * 
     * @param prefix
     * @param uri
     */
    public void declarePrefixMapping(String prefix, String uri) {
    }

    /**
     * Event handler for startElement during object construction.
     * 
     * @param namespaceURI
     * @param localName
     * @param qName
     * @param attrs
     * @return IRContentClassHandler
     */
    public IRContentClassHandler startElement(String namespaceURI, String localName, String qName,
            Attributes attrs) {
        return (this);
    }

    /**
     * Event handler for endElement during object construction.
     * 
     * @param namespaceURI
     * @param localName
     * @param qName
     * @return IRContentClassHandler
     */
    public IRContentClassHandler endElement(String namespaceURI, String localName, String qName) {
        String string;
        if ("event".equals(qName)) {
            string = new String(saxBuffer_);
            setContent(string);
        }
        return ((IRContentClassHandler) rGetParentRNode());
    }

    /**
     * Event handler for characters during object construction.
     * 
     * @param ch
     * @param start
     * @param length
     */
    public void characters(char[] ch, int start, int length) {
        if (saxBuffer_ != null) {
            saxBuffer_.append(ch, start, length);
        }
    }

    /**
     * Event handler for processngInstruction during object construction.
     * 
     * @param target
     * @param value
     */
    public void processingInstruction(String target, String value) {
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     * 
     * @return IRNode
     */
    public IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     * 
     * @param parentRNode
     */
    public void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     * 
     * @return IRNode[]
     */
    @SuppressWarnings("unchecked")
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[]) classNodes.toArray(nodes));
    }

    /**
     * Tests if a Element <code>element</code> is valid for the
     * <code>Event</code>.
     * 
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "event")) {
            return (false);
        }
        RStack target = new RStack(element);
//        boolean $match$ = false;
//        Element child;
        if (!URelaxer.hasAttributeHungry(target, "end")) {
            return (false);
        }
//        $match$ = true;
        if (!URelaxer.hasAttributeHungry(target, "start")) {
            return (false);
        }
//        $match$ = true;
        if (!URelaxer.hasAttributeHungry(target, "title")) {
            return (false);
        }
//        $match$ = true;
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code> is valid for
     * the <code>Event</code>. This mehtod is supposed to be used internally
     * by the Relaxer system.
     * 
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code> is valid for
     * the <code>Event</code>. This method consumes the stack contents during
     * matching operation. This mehtod is supposed to be used internally by the
     * Relaxer system.
     * 
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
