/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package org.seasar.tugboat.gtd.contents.timeline.entity.xml;

import java.util.*;
import org.xml.sax.*;

/**
 * @version xmlTimeline.rng 1.0 (Mon Dec 18 18:35:28 JST 2006)
 * @author Relaxer 1.1b (http://www.relaxer.org)
 */
public class XmlTimelineContentHandler extends org.xml.sax.helpers.DefaultHandler {
    private XmlTimelineBaseHandler root_;

    private IRContentClassHandler handler_;

    @SuppressWarnings("unchecked")
    private List mappings_ = new ArrayList();

    /**
     * Gets a object.
     * 
     * @return Object
     */
    public Object get() {
        Object object;
        object = root_.getData();
        if (object != null) {
            return (object);
        }
        return (null);
    }

    /**
     * Gets a Data.
     * 
     * @return Data
     */
    public Data getData() {
        return (root_.getData());
    }

    /**
     * Event handler for setDocumentLocator of ContentHandler.
     * 
     * @param locator
     */
    public void setDocumentLocator(Locator locator) {
    }

    /**
     * Event handler for start of ContentHandler.
     * 
     */
    public void startDocument() {
        handler_ = root_ = new XmlTimelineBaseHandler();
    }

    /**
     * Event handler for endDocument of ContentHandler.
     * 
     */
    public void endDocument() {
    }

    /**
     * Event handler for startPrefixMapping of ContentHandler.
     * 
     * @param prefix
     * @param uri
     */
    @SuppressWarnings("unchecked")
    public void startPrefixMapping(String prefix, String uri) {
        mappings_.add(new String[] { prefix, uri });
    }

    /**
     * Event handler for endPrefixMapping of ContentHandler.
     * 
     * @param prefix
     */
    public void endPrefixMapping(String prefix) {
    }

    /**
     * Event handler for startElement of ContentHandler.
     * 
     * @param namespaceURI
     * @param localName
     * @param qName
     * @param attrs
     */
    public void startElement(String namespaceURI, String localName, String qName, Attributes attrs) {
        handler_ = handler_.startElement(namespaceURI, localName, qName, attrs);
        int size = mappings_.size();
        for (int i = 0; i < size; i++) {
            String[] mapping = (String[]) mappings_.get(i);
            handler_.declarePrefixMapping(mapping[0], mapping[1]);
        }
        mappings_.clear();
    }

    /**
     * Event handler for endElement of ContentHandler.
     * 
     * @param namespaceURI
     * @param localName
     * @param qName
     */
    public void endElement(String namespaceURI, String localName, String qName) {
        handler_ = handler_.endElement(namespaceURI, localName, qName);
    }

    /**
     * Event handler for characters of ContentHandler.
     * 
     * @param ch
     * @param start
     * @param length
     */
    public void characters(char[] ch, int start, int length) {
        handler_.characters(ch, start, length);
    }

    /**
     * Event handler for ignorableWhitespace of ContentHandler.
     * 
     * @param ch
     * @param start
     * @param length
     */
    public void ignorableWhitespace(char[] ch, int start, int length) {
    }

    /**
     * Event handler for processingInstruction of ContentHandler.
     * 
     * @param target
     * @param value
     */
    public void processingInstruction(String target, String value) {
        handler_.processingInstruction(target, value);
    }

    /**
     * Event handler for skippedEntity of ContentHandler.
     * 
     * @param name
     */
    public void skippedEntity(String name) {
    }
}
