/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.timeline.page.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.factory.SingletonS2ContainerFactory;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.contents.timeline.entity.xml.Data;
import org.seasar.tugboat.gtd.contents.timeline.helper.TimelineHelper;
import org.seasar.tugboat.gtd.contents.timeline.page.TimelineAjax;
import org.seasar.tugboat.gtd.contents.timeline.service.TimelineService;

/**
 * Timeline Ajax implement
 * 
 * @author yousuke.hara
 * 
 */
@FormAnnotation(
    formId = "timelineAjax", isAuthorizationCheck = true, isDirectAccessCheck = true)
public class TimelineAjaxImpl implements TimelineAjax {
    /* LOGGER */
    Logger logger_ = Logger.getLogger(TimelineAjaxImpl.class);

    public TimelineAjaxImpl() {
        super();
    }

    /* timeline helper */
    TimelineHelper timelineHelper_;

    public void setTimelineHelper(TimelineHelper timelineHelper) {
        this.timelineHelper_ = timelineHelper;
    }

    /* timeline service */
    TimelineService timelineService_;

    public void setTimelineService(TimelineService timelineService) {
        this.timelineService_ = timelineService;
    }

    /* select functions */
    String selectFunctions_;

    public String getSelectFunctions() {
        return selectFunctions_;
    }

    public void setSelectFunctions(String selectFunctions) {
        this.selectFunctions_ = selectFunctions;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.timeline.ajax.TimelineAjax#getXmlData()
     */
    public String getXmlData() {
        S2Container container = SingletonS2ContainerFactory.getContainer();
        UserDto sessionLoginUser = 
            (UserDto) container.getComponent(UserDto.LOGIN_USER_SESSION_ATTRIBUTE_NAME);

        Data xmlData = null;
        if (this.selectFunctions_ == null || "".equals(this.selectFunctions_)) {
            xmlData = 
                this.timelineService_.getTimelineDataByUserId(sessionLoginUser.getUserId());
        } else {
            List<Integer> list = new ArrayList<Integer>();
            StringTokenizer token = 
                new StringTokenizer(this.selectFunctions_, ",");
            String activityId;
            while (token.hasMoreTokens()) {
                activityId = token.nextToken();

                if (!"0".equals(activityId)) {
                    list.add(new Integer(activityId));
                }
            }
            xmlData = 
                this.timelineService_.getTimelineDataByUserId(
                    sessionLoginUser.getUserId(), list.toArray(
                        new Integer[list.size()]));
        }
        StringBuilder strBuf = new StringBuilder();
        strBuf.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
        strBuf.append(xmlData.makeTextDocument());

        logger_.debug(strBuf.toString());

        return strBuf.toString();
    }
}
