/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.timeline.service.impl;

import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingAgendaDiscussionsDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingAgendasDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingBasicsDao;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgenda;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussion;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasic;
import org.seasar.tugboat.gtd.contents.task.dao.TaskCommentsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskRelationsDao;
import org.seasar.tugboat.gtd.contents.task.entity.TaskComment;
import org.seasar.tugboat.gtd.contents.task.entity.TaskRelation;
import org.seasar.tugboat.gtd.contents.timeline.dxo.TimelineDxo;
import org.seasar.tugboat.gtd.contents.timeline.entity.xml.Data;
import org.seasar.tugboat.gtd.contents.timeline.service.TimelineService;

/**
 * timeline service implement.
 * 
 * @author yousuke.hara
 * 
 */
public class TimelineServiceImpl implements TimelineService {

    /* task relations dao */
    TaskRelationsDao taskRelationsDao_;

    public void setTaskRelationsDao(TaskRelationsDao taskRelationsDao) {
        this.taskRelationsDao_ = taskRelationsDao;
    }

    /* task comment dao */
    TaskCommentsDao taskCommentsDao_;

    public void setTaskCommentsDao(TaskCommentsDao taskCommentsDao) {
        this.taskCommentsDao_ = taskCommentsDao;
    }

    /* meeting basic dao */
    MeetingBasicsDao meetingBasicsDao_;

    public void setMeetingBasicsDao(MeetingBasicsDao meetingBasicsDao) {
        this.meetingBasicsDao_ = meetingBasicsDao;
    }

    /* meeting agenda dao */
    MeetingAgendasDao meetingAgendasDao_;

    public void setMeetingAgendasDao(MeetingAgendasDao meetingAgendasDao) {
        this.meetingAgendasDao_ = meetingAgendasDao;
    }

    /* meeting agenda discussions dao */
    MeetingAgendaDiscussionsDao meetingAgendaDiscussionsDao_;

    public void setMeetingAgendaDiscussionsDao(
            MeetingAgendaDiscussionsDao meetingAgendaDiscussionsDao) {
        this.meetingAgendaDiscussionsDao_ = meetingAgendaDiscussionsDao;
    }

    /* timeline dxo */
    TimelineDxo timelineDxo_;

    public void setTimelineDxo(TimelineDxo timelineDxo) {
        this.timelineDxo_ = timelineDxo;
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.timeline.logic.TimelineService#getTimelineDataByUserId(int)
     */
    public Data getTimelineDataByUserId(long userId) {
        return this.getTimelineDataByUserId(userId, null);
    }

    /*
     * @see org.seasar.tugboat.gtd.contents.timeline.service.TimelineService#getTimelineDataByUserId(int,
     *      java.lang.Integer[])
     */
    public Data getTimelineDataByUserId(long userId, Integer[] selectFunctions) {
        String start = CalendarUtil.addAmount(-60, "yyyyMMdd") + "000000";
        String end = CalendarUtil.now("yyyyMMdd") + "999999";
        
        TaskRelation[] tasks = 
            this.canFilter(selectFunctions, FUNC_TASK) 
            ? this.taskRelationsDao_.getRelationsWithHistoriesByUserId(userId, start, end)
            : null;

        TaskComment[] taskComments = 
            this.canFilter(selectFunctions, FUNC_TASK_COMMENT) 
            ? this.taskCommentsDao_.findByUserId(userId)
            : null;

        //-- update
        MeetingBasic[] meetingBasics = 
            this.canFilter(selectFunctions, FUNC_MEETING) 
            ? this.meetingBasicsDao_.findByUserIdForTimeline(userId, start, end)
            : null;

        //-- update
        MeetingAgenda[] agendas = 
            this.canFilter(selectFunctions, FUNC_MEETING_AGENDA) 
            ? this.meetingAgendasDao_.findByUserIdForTimeline(userId, start, end)
            : null;

        //-- update
        MeetingAgendaDiscussion[] discussions = 
            this.canFilter(selectFunctions, FUNC_MEETING_COMMENT) 
            ? meetingAgendaDiscussionsDao_.findByUserIdForTimeline(userId, start, end)
            : null;

        return timelineDxo_.convertTimelineDatas(tasks, taskComments, meetingBasics, agendas, discussions);
    }

    /**
     * can filter?
     * 
     * @param selectFunctions
     * @return
     */
    private boolean canFilter(Integer[] selectFunctions, int type) {
        if (selectFunctions == null) return true;

        boolean result = false;
        for (Integer f : selectFunctions) {
            if (type == f.intValue()) {
                result = true;
                break;
            }
        }
        return result;
    }

}
