/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.user.dao;

import org.seasar.dao.annotation.tiger.Arguments;
import org.seasar.dao.annotation.tiger.S2Dao;
import org.seasar.tugboat.gtd.contents.common.entity.Role;
import org.seasar.tugboat.gtd.contents.user.entity.User;

/**
 * user role dao.
 * 
 * @author yosuke
 * 
 */
@S2Dao(bean = Role.class)
public interface UserRolesDao {

    /**
     * find 'user role' by userId
     * 
     * @param userId
     *            user id
     * @return role
     */
    @Arguments("userId")
    Role findUserRoleByUserId(long userId);

    /**
     * add user role
     * 
     * @param user
     *            user
     * @param roleId
     *            role id
     */
    @Arguments( { "user", "roleId" })
    void add(User user, int roleId);

    /**
     * update user role
     * 
     * @param user
     *            user
     * @param roleId
     *            role id
     */
    @Arguments( { "user", "roleId" })
    void update(User user, int roleId);
}
