/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.user.entity;

import java.io.Serializable;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.dao.annotation.tiger.Bean;
import org.seasar.dao.annotation.tiger.Column;

/**
 * User Entity
 * 
 * @author yousuke.hara
 * 
 */
@Bean(table = "users")
public class User implements Serializable {
    private static final long serialVersionUID = 1166005311852230265L;

    public User() {
        super();
    }

    /* User Id */
    private long userId_;

    @Column("ID")
    public long getUserId() {
        return userId_;
    }

    public void setUserId(long userId) {
        this.userId_ = userId;
    }

    /* Login Id */
    private String loginId_;

    @Column("LOGIN_ID")
    public String getLoginId() {
        return loginId_;
    }

    public void setLoginId(String loginId) {
        this.loginId_ = loginId;
    }

    /* Email */
    private String email_;

    @Column("EMAIL")
    public String getEmail() {
        return email_;
    }

    public void setEmail(String email) {
        this.email_ = email;
    }

    /* First Name */
    private String firstName_;

    @Column("FIRST_NAME")
    public String getFirstName() {
        return firstName_;
    }

    public void setFirstName(String firstName) {
        this.firstName_ = firstName;
    }

    /* Last Name */
    private String lastName_;

    @Column("LAST_NAME")
    public String getLastName() {
        return lastName_;
    }

    public void setLastName(String lastName) {
        this.lastName_ = lastName;
    }

    /* Password */
    private String password_;

    @Column("PASSWORD")
    public String getPassword() {
        return password_;
    }

    public void setPassword(String password) {
        this.password_ = password;
    }

    /* Photo Image URL */
    private String photoImageUrl_;

    @Column("PHOTO_IMG_URL")
    public String getPhotoImageUrl() {
        return photoImageUrl_;
    }

    public void setPhotoImageUrl(String photoImageUrl) {
        this.photoImageUrl_ = photoImageUrl;
    }

    /* Modified User ID */
    private long modifiedUserId_;

    @Column("MODIFIED_USER_ID")
    public long getModifiedUserId() {
        return modifiedUserId_;
    }

    public void setModifiedUserId(long modifiedUserId) {
        this.modifiedUserId_ = modifiedUserId;
    }

    /* role id */
    private int roleId_;

    @Column("ROLE_ID")
    public int getRoleId() {
        return roleId_;
    }

    public void setRoleId(int roleId) {
        this.roleId_ = roleId;
    }

    /* role name */
    private String roleName_;

    @Column("ROLE_NAME")
    public String getRoleName() {
        return roleName_;
    }

    public void setRoleName(String roleName) {
        this.roleName_ = roleName;
    }
    
    /* registered */
    private String registered_;
    
    @Column("REGISTERED")
    public String getRegistered() {
        return registered_;
    }
    public void setRegistered(String registered) {
        this.registered_ = registered;
    }
    
    /* modified */
    private String modified_;
    
    @Column("MODIFIED")
    public String getModified() {
        return modified_;
    }
    public void setModified(String modified) {
        this.modified_ = modified;
    }

    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this, ToStringStyle.MULTI_LINE_STYLE);
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((email_ == null) ? 0 : email_.hashCode());
        result = prime * result
                + ((firstName_ == null) ? 0 : firstName_.hashCode());
        result = prime * result
                + ((lastName_ == null) ? 0 : lastName_.hashCode());
        result = prime * result
                + ((loginId_ == null) ? 0 : loginId_.hashCode());
        result = prime * result
                + (int) (modifiedUserId_ ^ (modifiedUserId_ >>> 32));
        result = prime * result
                + ((modified_ == null) ? 0 : modified_.hashCode());
        result = prime * result
                + ((password_ == null) ? 0 : password_.hashCode());
        result = prime * result
                + ((photoImageUrl_ == null) ? 0 : photoImageUrl_.hashCode());
        result = prime * result
                + ((registered_ == null) ? 0 : registered_.hashCode());
        result = prime * result + roleId_;
        result = prime * result
                + ((roleName_ == null) ? 0 : roleName_.hashCode());
        result = prime * result + (int) (userId_ ^ (userId_ >>> 32));
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final User other = (User) obj;
        if (email_ == null) {
            if (other.email_ != null)
                return false;
        } else if (!email_.equals(other.email_))
            return false;
        if (firstName_ == null) {
            if (other.firstName_ != null)
                return false;
        } else if (!firstName_.equals(other.firstName_))
            return false;
        if (lastName_ == null) {
            if (other.lastName_ != null)
                return false;
        } else if (!lastName_.equals(other.lastName_))
            return false;
        if (loginId_ == null) {
            if (other.loginId_ != null)
                return false;
        } else if (!loginId_.equals(other.loginId_))
            return false;
        if (modifiedUserId_ != other.modifiedUserId_)
            return false;
        if (modified_ == null) {
            if (other.modified_ != null)
                return false;
        } else if (!modified_.equals(other.modified_))
            return false;
        if (password_ == null) {
            if (other.password_ != null)
                return false;
        } else if (!password_.equals(other.password_))
            return false;
        if (photoImageUrl_ == null) {
            if (other.photoImageUrl_ != null)
                return false;
        } else if (!photoImageUrl_.equals(other.photoImageUrl_))
            return false;
        if (registered_ == null) {
            if (other.registered_ != null)
                return false;
        } else if (!registered_.equals(other.registered_))
            return false;
        if (roleId_ != other.roleId_)
            return false;
        if (roleName_ == null) {
            if (other.roleName_ != null)
                return false;
        } else if (!roleName_.equals(other.roleName_))
            return false;
        if (userId_ != other.userId_)
            return false;
        return true;
    }

}
