/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.user.page.impl;

import java.io.IOException;
import java.io.PrintWriter;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.apache.myfaces.custom.fileupload.UploadedFile;
import org.seasar.framework.container.annotation.tiger.Binding;
import org.seasar.framework.container.annotation.tiger.BindingType;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.user.dto.UserAdminMessageDto;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.contents.user.page.UserAdminAction;
import org.seasar.tugboat.gtd.contents.user.service.UserService;

/**
 * User admin action impl.
 * 
 */
@FormAnnotation(
    formId = "userAdminAction", isAuthorizationCheck = true, isDirectAccessCheck = false, isAdminTool = true, errorPage = "admin_error")
public class UserAdminActionImpl implements UserAdminAction {

    /* message */
    String message_ = "";

    public String getMessage() {
        return message_;
    }

    /* user dto */
    UserDto[] userDtos_;

    public UserDto[] getUserDtos() {
        return userDtos_;
    }

    public void setUserDtos(UserDto[] userDtos) {
        this.userDtos_ = userDtos;
    }

    /* login id */
    String loginId_;

    public String getLoginId() {
        return loginId_;
    }

    public void setLoginId(String loginId) {
        this.loginId_ = loginId;
    }

    /* upload file */
    private UploadedFile uploadedFile_;

    public UploadedFile getUploadedFile() {
        return uploadedFile_;
    }

    @Binding(bindingType = BindingType.NONE)
    public void setUploadedFile(UploadedFile uploadedFile) {
        this.uploadedFile_ = uploadedFile;
    }

    /* user service */
    UserService userService_;

    public void setUserService(UserService userService) {
        this.userService_ = userService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.user.page.UserAdminAction#addFromXls()
     */
    public String addFromXls() {
        UserAdminMessageDto[] arrMessage;
        FacesContext context = FacesContext.getCurrentInstance();
        ExternalContext exContext = context.getExternalContext();
        HttpServletResponse response = 
            (HttpServletResponse) exContext.getResponse();

        try {
            try {
                UserDto sessionLoginAdminUser = SessionUtil.getLoginAdminUser();
                arrMessage = 
                    userService_.addFromExcelFile(
                        this.uploadedFile_.getInputStream(), sessionLoginAdminUser);

                if (arrMessage.length == 0) {
                    message_ = "all user added!";
                } else {
                    StringBuilder strb = new StringBuilder();
                    for (int i = 0; i < arrMessage.length; i++) {
                        strb.append(arrMessage[i]);
                        if (i > arrMessage.length - 2) break;
                        
                        strb.append("<br>");
                    }
                    message_ = strb.toString();
                }
            } catch (Exception e) {
                message_ = "[error]";
            }

            PrintWriter writer = response.getWriter();
            writer.print(message_);
            writer.close();
        } catch (IOException e) {
            e.printStackTrace();
            throw new SystemUnexpectedRuntimeException(e);
        }
        context.responseComplete();
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.user.page.UserAdminAction#logout()
     */
    public String logout() {
        return "login";
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.user.page.UserAdminAction#initialize()
     */
    public String initialize() {
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }
}
