/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.user.page.impl;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.commons.util.ValidatorUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.contents.user.page.UserAdminAjax;
import org.seasar.tugboat.gtd.contents.user.service.UserService;

/**
 * user admin ajax implement.
 *
 */
@FormAnnotation(
    formId = "userAdminAjax", isAuthorizationCheck = true, isDirectAccessCheck = false, isAdminTool = true, errorPage = "admin_error")
public class UserAdminAjaxImpl implements UserAdminAjax {

    /* user id */
    long userId_;
    
    public void setUserId(long userId) {
        this.userId_ = userId;
    }

    /* role id */
    int roleId_;
    
    public void setRoleId(int roleId) {
        this.roleId_ = roleId;
    }
    
    /* email */
    String email_;
    
    public void setEmail(String email) {
        this.email_ = email;
    }

    /* first name */
    String firstName_;
    
    public void setFirstName(String first_name) {
        this.firstName_ = first_name;
    }

    /* last name */
    String lastName_;
    
    public void setLastName(String last_name) {
        this.lastName_ = last_name;
    }

    /* login id */
    String loginId_;
    
    public void setLoginId(String login_id) {
        this.loginId_ = login_id;
    }

    /* middle name */
    String middleName_;
    
    public void setMiddleName(String middle_name) {
        this.middleName_ = middle_name;
    }

    /* password */
    String password_;
    
    public void setPassword(String password) {
        this.password_ = password;
    }
    
    /* offset */
    int offset_;
    
    public void setOffset(int offset) {
        this.offset_ = offset;
    }
    
    /* limit */
    int limit_;
    
    public void setLimit(int limit) {
        this.limit_ = limit;
    }

    /* user service */
    UserService userService_;
    
    public void setUserService(UserService userService) {
        this.userService_ = userService;
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.page.UserAdminAjax#add()
     */
    public String add() {
        UserDto userDto = getUserDto();
        
        if(!ValidatorUtil.isEmail(userDto.getEmail())) {
            return "email";
        }
        UserDto dto = userService_.addUser(userDto);
        if (dto == null) {
            return "";
        }
        JSONObject json = JSONObject.fromObject(dto);
        return json.toString();
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.page.UserAdminAjax#delete()
     */
    public String delete() {
        this.userService_.delete(getUserDto());
        return "";
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.page.UserAdminAjax#list()
     */
    public String list() {
        UserDto[] userDtos = this.userService_.getUsers(this.offset_, this.limit_);
        JSONArray jsonArr = JSONArray.fromObject(userDtos);
        return jsonArr.toString();
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.page.UserAdminAjax#update()
     */
    public String update() {
        UserDto userDto = this.getUserDto();

        if(! ValidatorUtil.isEmail(userDto.getEmail())) {
            return "email";
        }
        this.userService_.update(userDto);
        userDto = this.userService_.getUserById(userDto.getUserId());
        
        return JSONObject.fromObject(userDto).toString();
    }

    /**
     * get user dto.
     * 
     * @return
     */
    UserDto getUserDto() {
        UserDto userDto = null;
        try {
            userDto = new UserDto();
            userDto.setUserId(this.userId_);
            userDto.setLoginId(this.loginId_);
            userDto.setPassword(this.password_);
            userDto.setEmail(this.email_);
            userDto.setLastName(this.lastName_);
            userDto.setFirstName(this.firstName_);
            userDto.setRoleId(this.roleId_);
            UserDto sessionLoginUser = SessionUtil.getLoginAdminUser();
            userDto.setModifiedUserId(sessionLoginUser.getUserId());

        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
        return userDto;
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.page.UserAdminAjax#getCount()
     */
    public String getCount() {
        return Integer.toString(this.userService_.getCount());
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }

}
