/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.user.page.impl;

import net.sf.json.JSONObject;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.contents.user.page.UserAjax;
import org.seasar.tugboat.gtd.contents.user.service.UserService;

/**
 * user ajax implement.
 *
 */
@FormAnnotation(
    formId = "userAjax", description = "userAjax", 
    isAuthorizationCheck = true, isDirectAccessCheck = false, isAdminTool = false, errorPage = "error")
public class UserAjaxImpl implements UserAjax {
	
	/* LOGGER */
    Logger logger_ = Logger.getLogger(UserAjaxImpl.class);
	
    /* user service */
    UserService userService_;

    public void setUserService(UserService userService) {
        this.userService_ = userService;
    }

    /* login id */
    String loginId_;
    
    public String getLoginId() {
    	return loginId_;
    }
    public void setLoginId(String loginId) {
    	this.loginId_ = loginId;
    }
    
    /* password */
    String password_;

    public String getPassword() {
        return password_;
    }
    public void setPassword(String password) {
        this.password_ = password;
    }
    
    /* email */
    String email_;

    public String getEmail() {
        return email_;
    }
    public void setEmail(String email) {
        this.email_ = email;
    }

    /* firstName */
    String firstName_;

    public String getFirstName() {
        return firstName_;
    }
    public void setFirstName(String firstName) {
        this.firstName_ = firstName;
    }

    /* lastName */
    String lastName_;

    public String getLastName() {
        return lastName_;
    }
    public void setLastName(String lastName) {
        this.lastName_ = lastName;
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.page.UserAjax#get()
     */
    public String getUserInfo() {
        JSONObject userJson = JSONObject.fromObject(SessionUtil.getLoginUser());
        return userJson.toString();
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.page.UserAjax#update()
     */
    public String update() {
        UserDto user = SessionUtil.getLoginUser();
        user.setPassword(this.password_);
        user.setFirstName(this.firstName_);
        user.setLastName(this.lastName_);
        user.setEmail(this.email_);
        userService_.updateUserInfo(user);
        
        return getUserInfo();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }

}
