/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package org.seasar.tugboat.gtd.mailservices.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Properties;

import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.seasar.tugboat.gtd.commons.util.SystemConfiguration;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskCommentDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.mailservices.dao.MeetingMailsDao;
import org.seasar.tugboat.gtd.mailservices.dao.TaskMailsDao;
import org.seasar.tugboat.gtd.mailservices.entity.MeetingMail;
import org.seasar.tugboat.gtd.mailservices.entity.TaskMail;
import org.seasar.tugboat.gtd.mailservices.service.MailService;

import com.ozacc.mail.Mail;
import com.ozacc.mail.SendMail;
import com.ozacc.mail.VelocityMailBuilder;

/**
 * mail service implement.
 * 
 * @author kacchi
 * 
 */
public class MailServiceImpl implements MailService {
    /* ozacc sendMail */
    private SendMail sendMail_;

    public void setSendMail(SendMail sendMail) {
        this.sendMail_ = sendMail;
    }

    /* velocity mail builder */
    private VelocityMailBuilder velocityMailBuilder_;

    public void setVelocityMailBuilder(VelocityMailBuilder velocityMailBuilder) {
        this.velocityMailBuilder_ = velocityMailBuilder;
    }

    /* task mail dao */
    private TaskMailsDao taskMailsDao_;

    public void setTaskMailsDao(TaskMailsDao taskMailsDao) {
        this.taskMailsDao_ = taskMailsDao;
    }

    /* meeting mail dao */
    private MeetingMailsDao meetingMailsDao_;

    public void setMeetingMailsDao(MeetingMailsDao meetingMailsDao) {
        this.meetingMailsDao_ = meetingMailsDao;
    }

    /*
     * @see org.seasar.tugboat.gtd.mailservices.service.MailService#sendMail(org.seasar.tugboat.gtd.mailservices.dto.MailTaskDto,
     *      org.seasar.tugboat.gtd.contents.user.dto.UserDto)
     */
    public void sendMail(TaskRelationDto taskDto, TaskCommentDto[] comments,
            ActivityDto activityDto, long userId) {

        if (!isSMTPConfigured(activityDto) || "0".equals(activityDto.getIsSendMail())) {
            return;
        }

        List<ActivityMemberDto> members = activityDto.getMembers();
        Properties p = new Properties();
        try {
            p.load(this.getClass().getResourceAsStream("/velocity.properties"));
            Velocity.init(p);
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
        VelocityContext vc = new VelocityContext();

        vc.put("url", SystemConfiguration.getValue("tugboat.gtd.context_path") + "/browse/tasks/"
                + taskDto.getTaskId());
        vc.put("members", members);
        vc.put("task", taskDto);
        vc.put("comments", comments);
        vc.put("commentSize", comments != null ? comments.length : 0);
        vc.put("from", SystemConfiguration.getValue("tugboat.gtd.mail.from"));
        String returnPath = SystemConfiguration.getValue("tugboat.gtd.mail.return_path");
        vc.put("returnPath", returnPath);

        Mail mail = this.velocityMailBuilder_.buildMail(
                "/org/seasar/tugboat/gtd/mailservices/service/TaskMail.xml", vc);
        try {
            TaskMail taskMail = this.taskMailsDao_.findLatestByTaskId(taskDto.getTaskId());
            if (taskMail != null) {
                String inReplyTo = taskMail.getMessageId();
                mail.addHeader("In-Reply-To", inReplyTo);
            }
            this.sendMail_.send(mail);

            // sent mail's Message-ID
            String messageIdSent = (String) mail.getHeaders().get("Message-ID");

            taskMail = new TaskMail();
            taskMail.setMessageId(messageIdSent);
            taskMail.setSentBy(userId);
            taskMail.setTaskId(taskDto.getTaskId());
            this.taskMailsDao_.insert(taskMail);

        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.mailservices.service.MailService#send(org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto,
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto[],
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto)
     */
    public void send(MeetingBasicDto basicDto, MeetingAgendaDto[] agendaDtos,
            ActivityDto activityDto, MeetingToTaskDto[] meetingToTaskDtos, long userId) {

        if (!isSMTPConfigured(activityDto)) {
            return;
        }
        List<ActivityMemberDto> members = activityDto.getMembers();

        HashMap<Long, String> discussionIdToTaskURL = new HashMap<Long, String>();
        for (MeetingToTaskDto meetingToTaskDto : meetingToTaskDtos) {
            discussionIdToTaskURL.put(meetingToTaskDto.getDiscussionId(), SystemConfiguration
                    .getValue("tugboat.gtd.context_path")
                    + "/browse/tasks/" + meetingToTaskDto.getTaskId());
        }

        Properties p = new Properties();
        try {
            p.load(this.getClass().getResourceAsStream("/velocity.properties"));
            Velocity.init(p);
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
        VelocityContext vc = new VelocityContext();

        vc.put("url", SystemConfiguration.getValue("tugboat.gtd.context_path")
                + "/browse/meetings/" + basicDto.getId());
        vc.put("members", members);
        vc.put("header", basicDto);
        vc.put("agendas", agendaDtos);
        vc.put("mapTaskURL", discussionIdToTaskURL);
        vc.put("from", SystemConfiguration.getValue("tugboat.gtd.mail.from"));
        String returnPath = SystemConfiguration.getValue("tugboat.gtd.mail.return_path");
        vc.put("returnPath", returnPath);

        Mail mail = this.velocityMailBuilder_.buildMail(
                "/org/seasar/tugboat/gtd/mailservices/service/MeetingMail.xml", vc);

        try {
            MeetingMail meetingMail = this.meetingMailsDao_.findLatestByBasicId(basicDto.getId());
            if (meetingMail != null) {
                String inReplyTo = meetingMail.getMessageId();
                mail.addHeader("In-Reply-To", inReplyTo);
            }
            this.sendMail_.send(mail);

            String messageIdSent = (String) mail.getHeaders().get("Message-ID");
            meetingMail = new MeetingMail();
            meetingMail.setMessageId(messageIdSent);
            meetingMail.setSentBy(userId);
            meetingMail.setBasicId(basicDto.getId());
            this.meetingMailsDao_.insert(meetingMail);

        } catch (Exception e) {
            e.printStackTrace();
            throw new SystemUnexpectedRuntimeException(e);
        }

    }

    /**
     * smtp configuration check
     * 
     * @param activityDto
     * @return if smtp server is configured , return true.
     */
    boolean isSMTPConfigured(ActivityDto activityDto) {
        String smtp = SystemConfiguration.getValue("tugboat.gtd.mail.smtp");
        return (smtp != null && !"".equals(smtp));
    }

}
