/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.webservices.dao;

import org.seasar.dao.annotation.tiger.Arguments;
import org.seasar.dao.annotation.tiger.S2Dao;
import org.seasar.tugboat.gtd.webservices.entity.FeedToken;

/**
 * feed tokens dao.
 * 
 * @author yosuke
 * 
 */
@S2Dao(bean = FeedToken.class)
public interface FeedTokensDao {

    /**
     * insert.
     * 
     * @param feedToken
     * @return
     */
    @Arguments("feedToken")
    int insert(FeedToken feedToken);

    /**
     * update delete flag.
     * 
     * @param feedToken
     * @return
     */
    @Arguments("feedToken")
    int updateDeleteFlagByActivityIdAndUserId(FeedToken feedToken);

    /**
     * update delete flag by activity id.
     * 
     * @param activityId
     * @return
     */
    @Arguments("activityId")
    int updateDeleteFlagByActivityId(long activityId);

    /**
     * find by activity id and user id.
     * 
     * @param userId
     * @return
     */
    @Arguments( { "activityId", "userId" })
    FeedToken findByActivityAndUserId(long activityId, long userId);
    
    /**
     * find by user id.
     * @param userId
     * @return
     */
    @Arguments("userId")
    FeedToken findByUserId(long userId);

    /**
     * find by token.
     * 
     * @param token
     * @return
     */
    @Arguments( { "activityId", "token" })
    FeedToken findByActivityIdAndToken(long activityId, String token);
    
    /**
     * find by token.
     * 
     * @param token
     * @return
     */
    @Arguments("token")
    FeedToken findByToken(String token);
}
