/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.webservices.feeds.ical;

import java.net.URI;
import java.net.URISyntaxException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import net.fortuna.ical4j.model.Date;
import net.fortuna.ical4j.model.component.VToDo;
import net.fortuna.ical4j.model.property.CalScale;
import net.fortuna.ical4j.model.property.Method;
import net.fortuna.ical4j.model.property.Priority;
import net.fortuna.ical4j.model.property.ProdId;
import net.fortuna.ical4j.model.property.Url;
import net.fortuna.ical4j.model.property.Version;
import net.fortuna.ical4j.model.property.XProperty;

import org.seasar.tugboat.gtd.commons.util.SystemConfiguration;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityService;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.service.TaskService;

/**
 * task ical publish service impl.
 * 
 * @author yousuke
 *
 */
public class TaskICalPublishServiceImpl implements TaskICalPublishService {

    /* task service */
    private TaskService taskService_;

    public void setTaskService(TaskService taskService) {
        this.taskService_ = taskService;
    }

    /* activity service */
    private ActivityService activityService_;

    public void setActivityService(ActivityService activityService) {
        this.activityService_ = activityService;
    }
    
    /*
     * @see org.seasar.tugboat.gtd.webservices.feeds.ical.TaskICalPublishService#publish(
     *      long, long, int)
     */
    public String publish(long activityId, long userId, int limit) {
        TaskRelationDto[] tasks = 
            this.taskService_.getTasksByActivityAndUserId(activityId, userId, limit);

        ActivityDto activity =
            this.activityService_.getActivityById(activityId);
        
        return this.getICalendar(activity.getName(), tasks);
    }
    
    /**
     * get icalender.
     * 
     * @param activityName
     * @param tasks
     * @return
     */
    private String getICalendar(String activityName, TaskRelationDto[] tasks) {
        net.fortuna.ical4j.model.Calendar calendar = new net.fortuna.ical4j.model.Calendar();
        calendar.getProperties().add(
            new ProdId("-//Tangerine//Tugboat.GTD 0.9.3//EN"));
        calendar.getProperties().add(Version.VERSION_2_0);
        calendar.getProperties().add(CalScale.GREGORIAN);
        calendar.getProperties().add(Method.PUBLISH);
        calendar.getProperties().add(new XProperty("X-WR-CALNAME", activityName));
        calendar.getProperties().add(new XProperty("X-WR-TIMEZONE", "Asia/Tokyo"));
        calendar.getProperties().add(new XProperty("X-WR-CALDESC", ""));
        
        if (tasks == null || tasks.length == 0) {
            return calendar.toString();
        }
        DateFormat formatter = new SimpleDateFormat("yyyyMMdd");
        for (TaskRelationDto task : tasks) {
            Date taskStartDate = null;
            Date taskDeadline = null;
            
            try {
                java.util.Date startDate = formatter.parse(task.getStartDate());
                taskStartDate = new Date(startDate);

                if (task.getDeadline() != null
                 && !"".equals(task.getDeadline()) && !"99999999".equals(task.getDeadline())) {
                    java.util.Date deadline = formatter.parse(task.getDeadline());
                    taskDeadline = new Date(deadline);
                }
            } catch (ParseException e) {
                throw new SystemUnexpectedRuntimeException(e);
            }
            
            VToDo toDo = 
                taskDeadline == null
                ? new VToDo(taskStartDate, task.getTitle())
                : new VToDo(taskDeadline, taskDeadline, task.getTitle());
            
            String url =
                SystemConfiguration.getValue(
                    SystemConfiguration.CONTEXT_PATH) + "/browse/tasks/" + task.getTaskId();
            try {
                toDo.getProperties().add(new Url(new URI(url)));
            } catch (URISyntaxException e) {
                new SystemUnexpectedRuntimeException(e);
            }
            toDo.getProperties().add(new Priority(task.getClassificationId()));
            calendar.getComponents().add(toDo);
        }
        return calendar.toString();
    }

}
