/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.webservices.feeds.rss;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.apache.commons.lang.StringUtils;
import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.commons.util.SystemConfiguration;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityService;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.entity.TaskCollectionOfRecord;
import org.seasar.tugboat.gtd.contents.task.service.TaskService;

import com.sun.syndication.feed.rss.Channel;
import com.sun.syndication.feed.rss.Description;
import com.sun.syndication.feed.rss.Guid;
import com.sun.syndication.feed.rss.Item;
import com.sun.syndication.io.WireFeedOutput;

/**
 * task rss publish service implements.
 * 
 * @author yosuke
 * 
 */
public class TaskRssPublishServiceImpl implements TaskRssPublishService {
    private static final String TITLE1 = "Tugboat - ";
    private static final String TITLE2 = "Task Feeds";
    private static final String FEED_TYPE = "rss_2.0";
    private static final String BODY_TYPE = "text/html";
    
    /* task service */
    private TaskService taskService_;

    public void setTaskService(TaskService taskService) {
        this.taskService_ = taskService;
    }

    /* activity service */
    private ActivityService activityService_;

    public void setActivityService(ActivityService activityService) {
        this.activityService_ = activityService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.rss.service.TaskRssPublishService#publish(int,
     *      int, limit)
     */
    public String publish(long activityId, long userId, int limit) {
        TaskRelationDto[] tasks = 
            this.taskService_.getTasksByActivityAndUserId(activityId, userId, limit);

        ActivityDto activity =
            this.activityService_.getActivityById(activityId);

        return this.getRSS(activity.getName(), tasks);
    }

    /**
     * get rss.
     * 
     * @param activityName
     * @param tasks
     * @param todoRecs
     * @param doneRecs
     * @return
     */
    private String getRSS(
        String activityName, TaskRelationDto[] tasks,
        Map<String, TaskRelationDto[]> todoRecs, Map<String, TaskRelationDto[]> doneRecs) {

        String lang = SystemConfiguration.getValue(SystemConfiguration.RSS_LANG);
        String rootURL = SystemConfiguration.getValue(SystemConfiguration.CONTEXT_PATH);

        Channel channel = new Channel();
        channel.setFeedType(FEED_TYPE);
        channel.setLanguage(lang);
        
        if (activityName == null) {
            channel.setTitle(TITLE1 + TITLE2);
            channel.setDescription(TITLE1 + TITLE2);
        }  else {
            channel.setTitle(TITLE1 + "[" + activityName + "] " + TITLE2);
            channel.setDescription(TITLE1 + "[" + activityName + "] " + TITLE2);
        }
        channel.setLink(rootURL);

        List<Item> items = new ArrayList<Item>();
        if (tasks == null || tasks.length == 0) {
        } else {
            for (TaskRelationDto t : tasks) {
                this.addItem(items, t);
            }
            if (todoRecs != null) {
                buildRssItems(TaskCollectionOfRecord.COLLECT_TYPE_TODAY_TODO, items, todoRecs);
            }
            if (doneRecs != null) {
                buildRssItems(TaskCollectionOfRecord.COLLECT_TYPE_YESTERDAY_DONE, items, doneRecs);
            } 
        }
        channel.setItems(items);
        
        try {
            WireFeedOutput out = new WireFeedOutput();

            StringWriter bw = new StringWriter();
            out.output(channel, bw);
            String result = bw.toString();
            return result;

        } catch (Exception e) {
            e.printStackTrace();
            throw new SystemUnexpectedRuntimeException(e);
        }
    }
    
    /**
     * build rss items
     * 
     * @param items
     * @param collectTasks
     */
    private void buildRssItems(String collectType, List<Item> items, Map<String, TaskRelationDto[]> collectTasks) {
        String rootURL = SystemConfiguration.getValue(SystemConfiguration.CONTEXT_PATH);
        
        for (Iterator<String> i = collectTasks.keySet().iterator(); i.hasNext();) {
            String fulldete = i.next().toString();
            TaskRelationDto[] todos = collectTasks.get(fulldete);
            
            String fullDateString =
                CalendarUtil.getDateString(
                    CalendarUtil.buildCalendar(fulldete, "yyyyMMdd"), "yyyy-MM-dd");
            Calendar fullDateCalendar = 
                CalendarUtil.buildCalendar(
                    fulldete
                    + SystemConfiguration.getValue(SystemConfiguration.BATCH_SCHEDULE_HOUR)
                    + SystemConfiguration.getValue(SystemConfiguration.BATCH_SCHEDULE_MIN), "yyyyMMddHm");
            
            StringBuilder buf = new StringBuilder();
            buf.append("<div><span style=\"font-weight:bold;\">")
               .append(fullDateString)
               .append("</span></div>\n");
            
            int prevStatus = 0;
            int prevClass = 0;
            for (TaskRelationDto t : todos) {
                if ((prevStatus > 0 && prevStatus != t.getStatusId())
                 || (prevClass  > 0 && prevClass  != t.getClassificationId())) {
                    buf.append("<div><br/></div>");
                }
                buf.append("<div style=\"font-family:monospace; font-size:12px;\">")
                   .append("[")
                   .append(t.getActivityValue())
                   .append("][")
                   .append(t.getClassificationId() == 1 ? "" : t.getClassificationValue())
                   .append(
                       t.getClassificationId() == 1
                       ? t.getStatusId() == 1 ? "ToDo"  :
                         t.getStatusId() == 2 ? "Doing" :
                                                ""
                       : "")
                   .append("]")
                   .append("[")
                   .append("".equals(t.getDeadlineFormated()) ? "N/A" : t.getDeadlineFormated())
                   .append("] ")
                   .append("<a href=\"" + rootURL + "/browse/tasks/" + t.getTaskId() + "\" target=\"_blank\">" + t.getTitle() + "</a>")
                   .append("</div>\n");
             
                prevStatus = t.getStatusId();
                prevClass = t.getClassificationId();
            }
            this.addItem(
                collectType, items, fullDateString, buf.toString(), fullDateCalendar);
        }
    }
    
    
    /**
     * add rss items.
     * 
     * @param items
     * @param rssItems
     */
    private void addItem(String mode, List<Item> items, String dateString, String description, Calendar modified) {
        String rootURL = SystemConfiguration.getValue(SystemConfiguration.CONTEXT_PATH);
        Item item = new Item();
        item.setTitle(
            "["
          + dateString
          + "] "
          + (mode.equals(TaskCollectionOfRecord.COLLECT_TYPE_TODAY_TODO) ? "Today Tasks" : "Yesterday Done Tasks")
          );

        String permalink =
            rootURL + "/task-collection-of-records/" 
          + (mode.equals(TaskCollectionOfRecord.COLLECT_TYPE_TODAY_TODO) ? "today" : "yesterday")
          + "/" + dateString;
        item.setLink(permalink);

        Guid guid = new Guid();
        guid.setPermaLink(true);
        guid.setValue(permalink);
        item.setGuid(guid);

        item.setUri(permalink);

        item.setPubDate(modified.getTime());
        item.setAuthor("Tugboat.GTD");

        Description desc = new Description();
        desc.setType(BODY_TYPE);
        desc.setValue(description);

        item.setDescription(desc);
        items.add(item);
    }
    
    /**
     * add rss items.
     * 
     * @param items
     * @param task
     */
    private void addItem(List<Item> items, TaskRelationDto task) {
        String rootURL = SystemConfiguration.getValue(SystemConfiguration.CONTEXT_PATH);
        Item item = new Item();
        item.setTitle(
            "[" + this.getFolderName(task.getStatusId(),
            task.getClassificationId()) + "]" + task.getTitle());

        String permalink = rootURL + "/browse/tasks/" + task.getTaskId();
        item.setLink(permalink);

        Guid guid = new Guid();
        guid.setPermaLink(true);
        guid.setValue(permalink);
        item.setGuid(guid);

        item.setUri(permalink);

        String modified = task.getModified();
        Calendar calendar = new GregorianCalendar(
            Integer.parseInt(modified.substring(0, 4)),
            Integer.parseInt(modified.substring(4, 6)) - 1,
            Integer.parseInt(modified.substring(6, 8)),
            Integer.parseInt(modified.substring(8, 10)),
            Integer.parseInt(modified.substring(10, 12)));
        item.setPubDate(calendar.getTime());
        item.setAuthor(task.getUserLastName() + "," + task.getUserFirstName());

        Description desc = new Description();
        desc.setType(BODY_TYPE);
        desc.setValue(StringUtils.replace(task.getDescription(), "\n", "<br/>"));

        item.setDescription(desc);
        items.add(item);
    }
    
    /**
     * get rss.
     * 
     * @param activityName
     * @param tasks
     * @return
     */
    private String getRSS(String activityName, TaskRelationDto[] tasks) {
        return this.getRSS(activityName, tasks, null, null);
    }

    /*
     * @see org.seasar.tugboat.gtd.webservices.feeds.rss.TaskRssPublishService#publish(long, int)
     */
    public String publish(long userId, int limit) {
        TaskRelationDto[] tasks = 
            this.taskService_.getTasksByUserId(userId, limit);
        
        int retroactiveDate = 
            Integer.parseInt(
                SystemConfiguration.getValue(SystemConfiguration.RSS_RETROACTIVE_DATE));
        
        TaskRelationDto[] collectTasks1 =
            this.taskService_.findTaskCollectionOfRecordsByTerm(
                TaskCollectionOfRecord.COLLECT_TYPE_TODAY_TODO,
                CalendarUtil.addAmount((-1 * retroactiveDate), "yyyyMMdd"),
                CalendarUtil.now("yyyyMMdd"),
                userId);
        
        Map<String, TaskRelationDto[]> collectTasksByFullDate1 = 
            this.getCollectionOfRecordsByFullDate(collectTasks1);
        
        TaskRelationDto[] collectTasks2 = 
            this.taskService_.findTaskCollectionOfRecordsByTerm(
                TaskCollectionOfRecord.COLLECT_TYPE_YESTERDAY_DONE,
                CalendarUtil.addAmount((-1 * retroactiveDate), "yyyyMMdd"),
                CalendarUtil.now("yyyyMMdd"),
                userId);
        
        Map<String, TaskRelationDto[]> collectTasksByFullDate2 =
            this.getCollectionOfRecordsByFullDate(collectTasks2);
        
        return this.getRSS(null, tasks, collectTasksByFullDate1, collectTasksByFullDate2);
    }
    
    /**
     * get collection of records by full date.
     * 
     * @param tasks
     * @return
     */
    private Map<String, TaskRelationDto[]> getCollectionOfRecordsByFullDate(TaskRelationDto[] tasks) {
        if (tasks == null || tasks.length == 0) return null;
        
        Map<String, TaskRelationDto[]> recordsByDate = new TreeMap<String, TaskRelationDto[]>();
        String prevFullDate = "";
        List<TaskRelationDto> list = new ArrayList<TaskRelationDto>();
        
        
        Iterator<TaskRelationDto> itTasks = Arrays.asList(tasks).iterator();
        int counter = 0;
        do {
            TaskRelationDto t = itTasks.hasNext() ? itTasks.next() : null;
            if (counter == 0) {
                 list.add(t);
                 
            } else if (tasks.length == counter) {
                recordsByDate.put(
                    prevFullDate, list.toArray(new TaskRelationDto[list.size()]));
                
            } else if (prevFullDate.equals(t.getFullDate())) {
                list.add(t);
                
            } else if (! prevFullDate.equals(t.getFullDate())) {
                recordsByDate.put(
                    prevFullDate, list.toArray(new TaskRelationDto[list.size()]));
                
                list = new ArrayList<TaskRelationDto>();
                list.add(t);
            }
            if (t != null) prevFullDate = t.getFullDate();
            counter++;
            
        } while (counter <= tasks.length);
        return recordsByDate;
    }
    
    /**
     * get folder name.
     * 
     * @param statusId
     * @param classId
     * @return
     */
    private String getFolderName(int statusId, int classId) {
        String result =
            classId == 1 && statusId == 1 ? "To Do" : 
            classId == 1 && statusId == 2 ? "Doing" : 
            classId == 1 && statusId == 3 ? "Done"  :
            classId == 2                  ? "Week"  :
            classId == 3                  ? "Month" :
            classId == 4 && statusId == 1 ? "Hold"  :
            classId == 4 && statusId == 3 ? "Hold.Done" :
            classId == 5                  ? "Someday"   :
            classId == 9                  ? "Inbox" :
                                            "";
        return result;
    }

}
