/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.webservices.helper.impl;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.webservices.feeds.atom.TaskDetailItemModule;
import org.seasar.tugboat.gtd.webservices.feeds.atom.TaskDetailItemModuleImpl;
import org.seasar.tugboat.gtd.webservices.feeds.atom.TaskHeaderFeedModule;
import org.seasar.tugboat.gtd.webservices.feeds.atom.TaskHeaderFeedModuleImpl;
import org.seasar.tugboat.gtd.webservices.helper.WebServiceHelper;
import org.seasar.tugboat.gtd.webservices.service.DefaultRESTfulWebService;

import com.sun.syndication.feed.atom.Content;
import com.sun.syndication.feed.atom.Entry;
import com.sun.syndication.feed.atom.Feed;
import com.sun.syndication.feed.atom.Link;
import com.sun.syndication.feed.atom.Person;
import com.sun.syndication.io.WireFeedOutput;

/**
 * task web service helper.
 * 
 * @author yosuke
 * 
 */
public class WebServiceHelperImpl implements WebServiceHelper {

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.webservices.helper.WebServiceHelper#createSecureInfo(
     *      java.lang.String, java.lang.String, java.lang.String)
     */
    public String createSecureInfo(String acceptType, String nonce,
            String createdAt) {
        String result = "";
        if (DefaultRESTfulWebService.ACCEPT_ATOM.equals(acceptType)) {
            result = this.createAtomForSecureInfo(nonce, createdAt);
        } else if (DefaultRESTfulWebService.ACCEPT_JSON.equals(acceptType)) {
            result = this.createJsonForSecureInfo(nonce, createdAt);
        }
        return result;
    }

    /**
     * create atom for secure info.
     * 
     * @param nonce
     * @param createdAt
     * @return
     */
    @SuppressWarnings("unchecked")
    protected String createAtomForSecureInfo(String nonce, String createdAt) {
        Feed feed = new Feed();
        feed.setFeedType("atom_1.0");
        feed.setId(nonce);
        feed.setUpdated(new Date());
        feed.setTitle("tugbaot.GTD");

        // added TaskSecurity module.
        TaskHeaderFeedModule exUserElement = new TaskHeaderFeedModuleImpl();
        exUserElement.setNonce(nonce);
        exUserElement.setCreated(createdAt);
        feed.getModules().add(exUserElement);

        return this.generateFeed(feed);
    }

    /**
     * generate feed.
     * 
     * @param feed
     * @return
     */
    protected String generateFeed(Feed feed) {
        try {
            WireFeedOutput out = new WireFeedOutput();
            StringWriter bw = new StringWriter();
            out.output(feed, bw);

            String result = bw.toString();
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /**
     * create json for secure info.
     * 
     * @param nonce
     * @param createdAt
     * @return
     */
    protected String createJsonForSecureInfo(String nonce, String createdAt) {
        Map<String, String> map = new HashMap<String, String>();
        map.put("nonce", nonce);
        map.put("createdAt", createdAt);
        JSONObject json = JSONObject.fromObject(map);
        return json.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskWebServiceHelper#createContent(java.lang.String,
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto)
     */
    public String createContentForTask(String acceptType,
            TaskRelationDto taskRelaiton) {
        String result = "";
        if (DefaultRESTfulWebService.ACCEPT_ATOM.equals(acceptType)) {
            result = this.createAtomForTask(taskRelaiton);
        } else if (DefaultRESTfulWebService.ACCEPT_JSON.equals(acceptType)) {
            result = this.createJsonForTask(taskRelaiton);
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskWebServiceHelper#createContent(
     *      java.lang.String,
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto[])
     */
    public String createContentForTask(String acceptType,
            TaskRelationDto[] taskRelaitons) {
        String result = "";
        if (DefaultRESTfulWebService.ACCEPT_ATOM.equals(acceptType)) {
        } else if (DefaultRESTfulWebService.ACCEPT_JSON.equals(acceptType)) {
            result = this.createJsonForTasks(taskRelaitons);
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.webservices.helper.WebServiceHelper#createFolderRecords(java.lang.String,
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto)
     */
    public String createFolderRecords(String acceptType,
            TaskRelationDto taskRelation) {
        String result = "";
        if (DefaultRESTfulWebService.ACCEPT_ATOM.equals(acceptType)) {
        } else if (DefaultRESTfulWebService.ACCEPT_JSON.equals(acceptType)) {
            result = this.createJsonForFolderRecords(taskRelation);
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.webservices.helper.WebServiceHelper#createActities(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto[])
     */
    public String createActities(String acceptType, ActivityDto[] activities) {
        String result = "";
        if (DefaultRESTfulWebService.ACCEPT_ATOM.equals(acceptType)) {
        } else if (DefaultRESTfulWebService.ACCEPT_JSON.equals(acceptType)) {
            result = this.createJsonForActivities(activities);
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.webservices.helper.WebServiceHelper#createActivityMembers(java.lang.String,
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto[])
     */
    public String createActivityMembers(String acceptType,
            ActivityMemberDto[] members) {
        String result = "";
        if (DefaultRESTfulWebService.ACCEPT_ATOM.equals(acceptType)) {
        } else if (DefaultRESTfulWebService.ACCEPT_JSON.equals(acceptType)) {
            result = this.createJsonForActivityMembers(members);
        }
        return result;
    }

    /**
     * create ATOM.
     * 
     * @param dto
     * @return
     */
    @SuppressWarnings("unchecked")
    protected String createAtomForTask(TaskRelationDto dto) {
        Feed feed = new Feed();
        feed.setFeedType("atom_1.0");
        feed.setId("http://tugboat-gtd.sandbox.seasar.org/users/"
                + dto.getUserId());
        feed.setUpdated(new Date());
        feed.setTitle("tugbaot.GTD - Task feeds");

        Content subtitle = new Content();
        subtitle.setValue("tugbaot.GTD - Task feeds");
        feed.setSubtitle(subtitle);

        Link selfLink = new Link();
        selfLink.setHref("http://tugboat-gtd.sandbox.seasar.org/users/"
                + dto.getUserId());
        selfLink.setRel("self");
        feed.setOtherLinks(Collections.singletonList(selfLink));

        //
        // Entries.
        //
        List<Entry> entries = new ArrayList<Entry>();

        Entry entry1 = new Entry();
        entry1.setTitle(dto.getTitle());
        entry1.setId("http://tugboat-gtd.sandbox.saesar.org/browse/tasks/"
                + dto.getTaskId());

        Link altLink = new Link();
        altLink.setHref("http://tugboat-gtd.sandbox.saesar.org/browse/tasks/"
                + dto.getTaskId());
        entry1.setAlternateLinks(Collections.singletonList(altLink));

        entry1.setPublished(new Date());
        entry1.setUpdated(new Date());
        Person author = new Person();
        author.setName(dto.getUserLastName() + "," + dto.getUserLastName());
        entry1.setAuthors(Collections.singletonList(author));

        Content content1 = new Content();
        content1.setValue(dto.getDescription());
        entry1.setContents(Collections.singletonList(content1));

        // tgb
        TaskDetailItemModule exDetailElement = new TaskDetailItemModuleImpl();
        exDetailElement.setStatusId(dto.getStatusId());
        exDetailElement.setActivityId(dto.getActivityId());
        exDetailElement.setActivityName(dto.getActivityValue());
        exDetailElement.setActivityColor("#" + dto.getColorValue());
        exDetailElement.setClassificationId(dto.getClassificationId());
        exDetailElement.setPersonInChargeId(dto.getPersonInChargeId());
        exDetailElement.setPersonInChargeFirstName(dto
                .getPersonInChargeFirstName());
        exDetailElement.setPersonInChargeLastName(dto
                .getPersonInChargeLastName());
        if (dto.getTaskTagDtos() != null && dto.getTaskTagDtos().length > 0) {
            exDetailElement.setTags(dto.getTaskTagDtos());
        }
        entry1.getModules().add(exDetailElement);

        entries.add(entry1);
        feed.setEntries(entries);

        return this.generateFeed(feed);
    }

    /**
     * create Json for task.
     * 
     * @param dto
     * @return
     */
    protected String createJsonForTask(TaskRelationDto dto) {
        Map<String, String> map = new HashMap<String, String>();
        map.put("id", String.valueOf(dto.getTaskId()));
        map.put("title", dto.getTitle());
        map.put("description", dto.getDescription());
        map.put("activityId", String.valueOf(dto.getActivityId()));
        map.put("activityName", dto.getActivityValue());
        map.put("activityColor", String.valueOf(dto.getColorValue()));
        map.put("classId", String.valueOf(dto.getClassificationId()));
        map.put("classValue", dto.getClassificationValue());
        map.put("statusId", String.valueOf(dto.getStatusId()));
        map.put("statusValue", dto.getStatusValue());
        map.put("personId", String.valueOf(dto.getPersonInChargeId()));
        map.put("personFirstName", dto.getPersonInChargeFirstName());
        map.put("personLastName", dto.getPersonInChargeLastName());
        map.put("due", dto.getDeadlineFormated());

        JSONObject json = JSONObject.fromObject(map);
        return json.toString();
    }

    /**
     * create json for tasks.
     * 
     * @param dtos
     * @return
     */
    protected String createJsonForTasks(TaskRelationDto[] dtos) {
        if (dtos == null || dtos.length == 0)
            return null;
        List<Map<String, String>> list = new ArrayList<Map<String, String>>();
        for (TaskRelationDto dto : dtos) {
            Map<String, String> map = new HashMap<String, String>();
            map.put("id", String.valueOf(dto.getTaskId()));
            map.put("title", dto.getTitle());
            map.put("description", dto.getDescription());
            map.put("activityId", String.valueOf(dto.getActivityId()));
            map.put("activityName", dto.getActivityValue());
            map.put("activityColor", String.valueOf(dto.getColorValue()));
            map.put("classId", String.valueOf(dto.getClassificationId()));
            map.put("classValue", dto.getClassificationValue());
            map.put("statusId", String.valueOf(dto.getStatusId()));
            map.put("statusValue", dto.getStatusValue());
            map.put("personId", String.valueOf(dto.getPersonInChargeId()));
            map.put("personFirstName", dto.getPersonInChargeFirstName());
            map.put("personLastName", dto.getPersonInChargeLastName());
            map.put("due", dto.getDeadlineFormated());

            list.add(map);
        }
        JSONArray jsonArray = JSONArray.fromObject(list.toArray(new Map[list
                .size()]));
        return jsonArray.toString();
    }

    /**
     * create json for folder records.
     * 
     * @param dto
     * @return
     */
    protected String createJsonForFolderRecords(TaskRelationDto dto) {
        Map<String, Integer> map = new HashMap<String, Integer>();
        map.put("inbox", dto.getInboxTasks());
        map.put("tasks", dto.getTodayTasks() + dto.getWeekTasks()
                + dto.getMonthTasks() + dto.getHoldTasks()
                + dto.getSomedayTasks());
        map.put("today", dto.getTodayTasks());
        map.put("todo", dto.getTodayTodoTasks());
        map.put("doing", dto.getTodayDoingTasks());
        map.put("done", dto.getTodayDoneTasks());
        map.put("week", dto.getWeekTasks());
        map.put("month", dto.getMonthTasks());
        map.put("hold", dto.getHoldTasks());
        map.put("someday", dto.getSomedayTasks());
        map.put("archive", dto.getArchiveTasks());
        map.put("trash", dto.getTrashBoxTasks());

        JSONObject json = JSONObject.fromObject(map);
        return json.toString();
    }

    /**
     * create json for activities.
     * 
     * @param activities
     * @return
     */
    protected String createJsonForActivities(ActivityDto[] activities) {
        if (activities == null || activities.length == 0)
            return null;

        List<Map<String, String>> list = new ArrayList<Map<String, String>>();
        for (ActivityDto a : activities) {
            Map<String, String> map = new HashMap<String, String>();
            map.put("id", String.valueOf(a.getId()));
            map.put("value", a.getName());
            map.put("color", a.getColorValue());
            map.put("description", a.getDescription());
            list.add(map);
        }
        JSONArray jsonArray = JSONArray.fromObject(list.toArray(new Map[list
                .size()]));
        return jsonArray.toString();
    }

    /**
     * create json for activity members.
     * 
     * @param members
     * @return
     */
    protected String createJsonForActivityMembers(ActivityMemberDto[] members) {
        if (members == null || members.length == 0)
            return null;

        List<Map<String, String>> list = new ArrayList<Map<String, String>>();
        for (ActivityMemberDto a : members) {
            Map<String, String> map = new HashMap<String, String>();
            map.put("id", String.valueOf(a.getId()));
            map.put("userId", String.valueOf(a.getUserId()));
            map.put("firstName", a.getFirstName());
            map.put("lastName", a.getLastName());

            map.put("roleId", String.valueOf(a.getRoleId()));
            map.put("join", a.getRegisteredFormated());

            list.add(map);
        }
        JSONArray jsonArray = JSONArray.fromObject(list.toArray(new Map[list
                .size()]));
        return jsonArray.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.webservices.helper.WebServiceHelper#createActivityMember(
     *      java.lang.String, org.seasar.tugboat.gtd.contents.user.dto.UserDto)
     */
    public String createActivityMember(String acceptType,
            ActivityMemberDto member) {
        String result = "";
        if (DefaultRESTfulWebService.ACCEPT_ATOM.equals(acceptType)) {
        } else if (DefaultRESTfulWebService.ACCEPT_JSON.equals(acceptType)) {
            result = this.createActivityMemberJSON(member);
        }
        return result;
    }

    /**
     * create activity member json.
     * 
     * @param member
     * @return
     */
    protected String createActivityMemberJSON(ActivityMemberDto member) {
        if (member == null)
            return null;

        Map<String, String> map = new HashMap<String, String>();
        map.put("id", String.valueOf(member.getUserId()));
        map.put("firstName", member.getFirstName());
        map.put("lastName", member.getLastName());

        JSONObject json = JSONObject.fromObject(map);
        return json.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.webservices.helper.WebServiceHelper#createSuggestedList(java.lang.String,
     *      org.seasar.tugboat.gtd.contents.user.dto.UserDto[])
     */
    public String createSuggestedList(String acceptType, UserDto[] members) {
        String result = "";
        if (DefaultRESTfulWebService.ACCEPT_ATOM.equals(acceptType)) {
        } else if (DefaultRESTfulWebService.ACCEPT_JSON.equals(acceptType)) {
            result = this.createSuggestedListJSON(members);
        }
        return result;
    }

    /**
     * create suggested list json.
     * 
     * @param members
     * @return
     */
    protected String createSuggestedListJSON(UserDto[] members) {
        if (members == null || members.length == 0)
            return null;

        List<Map<String, String>> list = new ArrayList<Map<String, String>>();
        for (UserDto a : members) {
            Map<String, String> map = new HashMap<String, String>();
            map.put("id", String.valueOf(a.getUserId()));
            map.put("firstName", a.getFirstName());
            map.put("lastName", a.getLastName());
            list.add(map);
        }
        JSONArray jsonArray = JSONArray.fromObject(list.toArray(new Map[list
                .size()]));
        return jsonArray.toString();
    }
}
