/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.webservices.service.impl;

import org.seasar.tugboat.gtd.container.annotation.RestMethod;
import org.seasar.tugboat.gtd.container.constant.HttpMethod;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.webservices.helper.WebServiceHelper;
import org.seasar.tugboat.gtd.webservices.service.ActivityMemberWebService;
import org.seasar.tugboat.gtd.webservices.service.DefaultRESTfulWebService;

/**
 * activity member web service implements.
 * 
 * @author yosuke
 * 
 */
public class ActivityMemberRESTWebServiceImpl extends DefaultRESTfulWebService
		implements ActivityMemberWebService {

	/* id */
	private long id_;

	public void setId(long id) {
		this.id_ = id;
	}

	/* activity id */
	private long activityId_;

	public void setActivityId(long activityId) {
		this.activityId_ = activityId;
	}

	/* activity ids */
	private Long[] activityIds_;

	public void setActivityIds(Long[] activityIds) {
		this.activityIds_ = activityIds;
	}

	/* keyword */
	private String keyword_;

	public void setKeyword(String keyword) {
		this.keyword_ = keyword;
	}

	/* rold id */
	private int roleId_;

	public void setRoleId(int roleId) {
		this.roleId_ = roleId;
	}

	/* user id */
	private long userId_;

	public void setUserId(long userId) {
		this.userId_ = userId;
	}

	/* activity member service */
	ActivityMembersService activityMemberService_;

	public void setActivityMemberService(
			ActivityMembersService activityMemberService) {
		this.activityMemberService_ = activityMemberService;
	}

	/* web service helper */
	WebServiceHelper webServiceHelper_;

	public void setWebServiceHelper(WebServiceHelper webServiceHelper) {
		this.webServiceHelper_ = webServiceHelper;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.seasar.tugboat.gtd.webservices.service.ActivityMemberWebService#findById()
	 */
	@RestMethod(methods = HttpMethod.GET, parameters = { "activityId", "userId" })
	public String findByActivityAndUser() {
		ActivityMemberDto member = this.activityMemberService_
				.getMemberByActivityIdAndUserId(this.activityId_, this.userId_);

		return this.webServiceHelper_.createActivityMember(this.acceptType_,
				member);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.seasar.tugboat.gtd.webservices.service.ActivityMemberWebService#findByActivity()
	 */
	@RestMethod(methods = HttpMethod.GET, parameters = "activityId")
	public String findByActivity() {
		ActivityMemberDto[] members = this.activityMemberService_
				.getActivityMembersByActivities(new Long[] { this.activityId_ });
		return this.webServiceHelper_.createActivityMembers(this.acceptType_,
				members);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.seasar.tugboat.gtd.webservices.service.ActivityMemberWebService#findByActivities()
	 */
	@RestMethod(methods = HttpMethod.GET, parameters = "activityIds")
	public String findByActivities() {
		ActivityMemberDto[] members = this.activityMemberService_
				.getActivityMembersByActivities(this.activityIds_);
		return this.webServiceHelper_.createActivityMembers(this.acceptType_,
				members);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.seasar.tugboat.gtd.webservices.service.ActivityMemberWebService#findByBelongActivities()
	 */
	@RestMethod(methods = HttpMethod.GET)
	public String findByBelongActivities() {
		ActivityMemberDto[] members = this.activityMemberService_
				.getMembersListRelatedToUser(this.userId_);

		return this.webServiceHelper_.createActivityMembers(this.acceptType_,
				members);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.seasar.tugboat.gtd.webservices.service.ActivityMemberWebService#findAllMembers()
	 */
	@RestMethod(methods = HttpMethod.GET)
	public String findAllMembers() {
		ActivityMemberDto[] members = this.activityMemberService_
				.getAllMembers();

		return this.webServiceHelper_.createActivityMembers(this.acceptType_,
				members);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.seasar.tugboat.gtd.webservices.service.ActivityMemberWebService#suggest()
	 */
	@RestMethod(methods = HttpMethod.GET, parameters = "keyword")
	public String suggest() {
		UserDto[] members = this.activityMemberService_
				.getActivityMembers(this.keyword_);

		return this.webServiceHelper_.createSuggestedList(this.acceptType_,
				members);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.seasar.tugboat.gtd.webservices.service.ActivityMemberWebService#create()
	 */
	@RestMethod(methods = HttpMethod.POST, parameters = { "id", "roleId",
			"activityId" })
	public String create() {
		ActivityMemberDto memberDto = new ActivityMemberDto();
		memberDto.setUserId(this.id_);
		memberDto.setActivityId(this.activityId_);
		memberDto.setRoleId(this.roleId_);
		memberDto.setModifiedUserId(this.userId_);

		this.activityMemberService_.add(memberDto);
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.seasar.tugboat.gtd.webservices.service.ActivityMemberWebService#update()
	 */
	@RestMethod(methods = HttpMethod.PUT, parameters = { "id", "roleId",
			"activityId" })
	public String update() {
		ActivityMemberDto memberDto = new ActivityMemberDto();
		memberDto.setUserId(this.id_);
		memberDto.setActivityId(this.activityId_);
		memberDto.setRoleId(this.roleId_);
		memberDto.setModifiedUserId(this.userId_);

		this.activityMemberService_.update(memberDto);
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.seasar.tugboat.gtd.webservices.service.ActivityMemberWebService#delete()
	 */
	@RestMethod(methods = HttpMethod.DELETE, parameters = { "id", "activityId" })
	public String delete() {

		return null;
	}
}
