/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.webservices.service.impl;

import org.seasar.tugboat.gtd.container.annotation.RestMethod;
import org.seasar.tugboat.gtd.container.constant.HttpMethod;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityRelationDto;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityService;
import org.seasar.tugboat.gtd.webservices.helper.WebServiceHelper;
import org.seasar.tugboat.gtd.webservices.service.ActivityWebService;
import org.seasar.tugboat.gtd.webservices.service.DefaultRESTfulWebService;

/**
 * activity rest web service implement.
 * 
 * @author yosuke
 * 
 */
public class ActivityRESTWebServiceImpl extends DefaultRESTfulWebService
        implements ActivityWebService {

    /* id */
    private int id_;

    public void setId(int id) {
        this.id_ = id;
    }

    /* name */
    private String name_;

    public void setName(String name) {
        this.name_ = name;
    }

    /* description */
    private String description_;

    public void setDescription(String description) {
        this.description_ = description;
    }

    /* color */
    private int color_;

    public void setColor(int color) {
        this.color_ = color;
    }

    /* activity service */
    private ActivityService activityService_;

    public void setActivityService(ActivityService activityService) {
        this.activityService_ = activityService;
    }

    /* activity member service 
    private ActivityMembersService activityMemberService_;

    public void setActivityMembersService(
            ActivityMembersService activityMemberService) {
        this.activityMemberService_ = activityMemberService;
    }
    */

    /* web service helper */
    private WebServiceHelper webServiceHelper_;

    public void setWebServiceHelper(WebServiceHelper webServiceHelper) {
        this.webServiceHelper_ = webServiceHelper;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.webservices.service.ActivityWebService#findById()
     */
    @RestMethod(methods = HttpMethod.GET, parameters = "id")
    public String findById() {
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.webservices.service.ActivityWebService#getOwnActivities()
     */
    @RestMethod(methods = HttpMethod.GET)
    public String getOwnActivities() {
        ActivityDto[] activities = this.activityService_
                .getOwnActivities(this.userId_);

        return webServiceHelper_.createActities(this.acceptType_, activities);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.webservices.service.ActivityWebService#create()
     */
    @RestMethod(methods = HttpMethod.POST, parameters = { "name",
            "description", "color" })
    public String create() {
        ActivityDto activity = new ActivityDto();
        activity.setId(this.id_);
        activity.setName(this.name_);
        activity.setDescription(this.description_);
        activity.setModifiedUserId(this.userId_);

        ActivityRelationDto relationsDto = new ActivityRelationDto();
        relationsDto.setColorId(this.color_);
        relationsDto.setUserId(this.userId_);

        ActivityMemberDto member = new ActivityMemberDto();
        member.setUserId(this.userId_);
        member.setRoleId(1);
        member.setModifiedUserId(this.userId_);

        this.activityService_.addWithMember(activity, relationsDto, member);
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.webservices.service.ActivityWebService#update()
     */
    @RestMethod(methods = HttpMethod.PUT, parameters = { "id", "name",
            "description", "color" })
    public String update() {
        ActivityDto activity = new ActivityDto();
        activity.setId(this.id_);
        activity.setName(this.name_);
        activity.setDescription(this.description_);
        activity.setModifiedUserId(this.userId_);

        ActivityRelationDto relation = new ActivityRelationDto();
        relation.setActivityId(this.id_);
        relation.setUserId(this.userId_);
        relation.setColorId(this.color_);

        this.activityService_.updateWithRelations(activity, relation);
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.webservices.service.ActivityWebService#delete()
     */
    @RestMethod(methods = HttpMethod.DELETE, parameters = "id")
    public String delete() {
        ActivityDto dto = new ActivityDto();
        dto.setId(this.id_);
        dto.setModifiedUserId(this.userId_);

        this.activityService_.remove(dto);
        return null;
    }
}
