/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.webservices.service.impl;

import java.util.Date;

import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang.time.DateFormatUtils;
import org.seasar.tugboat.gtd.webservices.dao.FeedTokensDao;
import org.seasar.tugboat.gtd.webservices.entity.FeedToken;
import org.seasar.tugboat.gtd.webservices.service.FeedTokenService;

/**
 * feed token service implement.
 * 
 * @author yosuke
 * 
 */
public class FeedTokenServiceImpl implements FeedTokenService {
    public static final int TOKEN_LENGTH = 36;
    
    /* feed tokens dao */
    private FeedTokensDao feedTokensDao_;

    public void setFeedTokensDao(FeedTokensDao feedTokensDao) {
        this.feedTokensDao_ = feedTokensDao;
    }

    /*
     * @see org.seasar.tugboat.gtd.feeds.common.service.FeedTokenService#create(int, int)
     */
    public String create(long activityId, long userId) {
        String token = this.createToken();
        FeedToken result = this.feedTokensDao_.findByActivityIdAndToken(activityId, token);
        
        if (result != null && !"".equals(result.getToken())) {
            this.create(activityId, userId);
        }
        FeedToken feedToken = new FeedToken();
        feedToken.setUserId(userId);
        feedToken.setActivityId(activityId);
        feedToken.setToken(token);
        feedToken.setCreatedAt(DateFormatUtils.format(new Date(), "yyyyMMddHHmmss"));
        this.feedTokensDao_.insert(feedToken);
        return token;
    }
    
    /*
     * @see org.seasar.tugboat.gtd.webservices.service.FeedTokenService#create(long)
     */
    public String create(long userId) {
        FeedToken result = this.feedTokensDao_.findByUserId(userId);
        if (result != null && result.getToken() != null && result.getToken().length() == TOKEN_LENGTH) {
            return result.getToken();
        }
        
        String token = this.createToken();
        FeedToken feedToken = new FeedToken();
        feedToken.setUserId(userId);
        feedToken.setToken(token);
        feedToken.setCreatedAt(DateFormatUtils.format(new Date(), "yyyyMMddHHmmss"));
        this.feedTokensDao_.insert(feedToken);
        return token;
    }
    
    /**
     * create token.
     * 
     * @return
     */
    private String createToken() {
        // ${8}-${4}-${4}-${4}-${12}
        StringBuilder token = new StringBuilder(TOKEN_LENGTH);
        token.append(RandomStringUtils.randomAlphanumeric(8).toUpperCase())
             .append("-")
             .append(RandomStringUtils.randomAlphanumeric(4).toUpperCase())
             .append("-")
             .append(RandomStringUtils.randomAlphanumeric(4).toUpperCase())
             .append("-")
             .append(RandomStringUtils.randomAlphanumeric(4).toUpperCase())
             .append("-")
             .append(RandomStringUtils.randomAlphanumeric(12).toUpperCase());
        return token.toString();
    }

    /*
     * @see org.seasar.tugboat.gtd.feeds.common.service.FeedTokenService#reset(int,
     *      int)
     */
    public String reset(long activityId, long userId) {
        FeedToken feedToken = new FeedToken();
        feedToken.setUserId(userId);
        feedToken.setActivityId(activityId);
        this.feedTokensDao_.updateDeleteFlagByActivityIdAndUserId(feedToken);

        return this.create(activityId, userId);
    }

    /*
     * @see org.seasar.tugboat.gtd.feeds.common.service.FeedTokenService#delete(int,
     *      int)
     */
    public void deleteByActivityIdAndUserId(long activityId, long userId) {
        FeedToken feedToken = new FeedToken();
        feedToken.setActivityId(activityId);
        feedToken.setUserId(userId);
        this.feedTokensDao_.updateDeleteFlagByActivityIdAndUserId(feedToken);
    }

    /*
     * @see org.seasar.tugboat.gtd.feeds.common.service.FeedTokenService#deleteByActivityId(int)
     */
    public void deleteByActivityId(long activityId) {
        this.feedTokensDao_.updateDeleteFlagByActivityId(activityId);
    }

    /*
     * @see org.seasar.tugboat.gtd.feeds.common.service.FeedTokenService#findByActivityIdAndToken(int,
     *      java.lang.String)
     */
    public FeedToken findByActivityIdAndToken(long activityId, String token) {
        return this.feedTokensDao_.findByActivityIdAndToken(activityId, token);
    }
    
    /*
     * @see org.seasar.tugboat.gtd.webservices.service.FeedTokenService#findByToken(java.lang.String)
     */
    public FeedToken findByToken(String token) {
        return this.feedTokensDao_.findByToken(token);
    }
    
}
