package org.seasar.tugboat.gtd.contents.activity.dao;

import org.seasar.dao.unit.S2DaoTestCase;
import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.contents.activity.entity.Activity;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityPageCondition;

/**
 * tets ActivitiesDao.
 * 
 * @author yosuke
 * 
 */
public class TestActivitiesDao extends S2DaoTestCase {
    ActivitiesDao activitiesDao_;

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    public void setUp() {
        super.include("dicon/contents/activity/activityDao.dicon");
    }

    /**
     * test ActivitiesDao#selectActivitiesLimit.
     * 
     */
    public void testSelectActivitiesLimitTx() {
        super.readXlsWriteDb("activities.xls");
        
        ActivityPageCondition condition = new ActivityPageCondition();
        condition.setLimit(20);
        condition.setOffset(0);
        condition.setUserId(9990);
        Activity[] activities = 
            this.activitiesDao_.selectActivitiesLimit(condition);

        assertEquals("[OK]", 9, activities.length);
    }

    /**
     * test ActivitiesDao#selectActivitiesSearch.
     * 
     */
    public void testSelectActivitiesSearchTx() {
        super.readXlsWriteDb("activities.xls");
        Activity[] activities = 
            this.activitiesDao_.selectActivitiesSearch("%-5%");

        assertEquals("[OK]", 1, activities.length);
    }

    /**
     * test ActivitiesDao#selectNewActivityId.
     * 
     */
    public void testSelectNewActivityIdTx() {
        super.readXlsWriteDb("activities.xls");
        Activity result = this.activitiesDao_.selectNewId();

        assertEquals("[OK]", 10000, result.getId());
    }

    /**
     * test ActivitiesDao#selectActivityById.
     * 
     */
    public void testSelectActivityByIdTx() {
        super.readXlsWriteDb("activities.xls");
        Activity result = this.activitiesDao_.selectActivityById(9998);

        assertEquals("[OK]", 9998, result.getId());
        assertEquals("[OK]", true, "activity-8".equals(result.getName()));
        assertEquals("[OK]", true, "activity 0008".equals(result.getDescription()));
        assertEquals("[OK]", true, "20070509".equals(result.getStartDate()));
        assertEquals("[OK]", true, "99999999".equals(result.getEndDate()));
    }

    /**
     * test ActivitiesDao#selectOwnActivities
     * 
     */
    public void testSelectOwnActivitiesTx() {
        super.readXlsWriteDb("activities.xls");
        Activity[] activities1 = this.activitiesDao_.selectOwnActivities(9990);
        assertEquals("[OK]", 5, activities1.length);

        Activity[] activities2 = this.activitiesDao_.selectOwnActivities(9991);
        assertEquals("[OK]", 3, activities2.length);
    }

    /**
     * test ActivitiesDao#insert.
     * 
     */
    public void testInsertTx() {
        Activity activity = new Activity();
        activity.setId(9991);
        activity.setName("Test-1");
        activity.setAbbreviation("T1");
        activity.setDescription("Description Test-1");
        activity.setModifiedUserId(1);
        activity.setIsSendMail("1");
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        activity.setStartDate(nowString.substring(0, 8));
        activity.setRegistered(nowString);
        activity.setModified(nowString);
        
        this.activitiesDao_.insert(activity);

        Activity result = this.activitiesDao_.selectActivityById(9991);
        assertEquals("[OK]", activity.getId(), result.getId());
        assertEquals("[OK]", true, activity.getName().equals(result.getName()));
        assertEquals("[OK]", true, activity.getDescription().equals(result.getDescription()));
    }

    /**
     * test ActivitiesDao#update.
     * 
     */
    public void testUpdateTx() {
        Activity activity = new Activity();
        activity.setId(9991);
        activity.setName("Test-1");
        activity.setAbbreviation("T1");
        activity.setDescription("Description Test-1");
        activity.setModifiedUserId(1);
        activity.setIsSendMail("0");
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        activity.setStartDate(nowString.substring(0, 8));
        activity.setRegistered(nowString);
        activity.setModified(nowString);
        
        this.activitiesDao_.insert(activity);

        activity.setName("Test-123");
        activity.setDescription("Description Test-123");
        activity.setIsSendMail("1");
        this.activitiesDao_.update(activity);

        Activity result = this.activitiesDao_.selectActivityById(9991);
        assertEquals("[OK]", activity.getId(), result.getId());
        assertEquals("[OK]", true, activity.getName().equals(result.getName()));
        assertEquals("[OK]", true, activity.getDescription().equals(result.getDescription()));
    }
    
    /**
     * test ActivitiesDao#delete.
     *
     */
    public void testUpdateDeleteFlagByIdTx() {
        Activity activity = new Activity();
        activity.setId(9991);
        activity.setName("Test-1");
        activity.setAbbreviation("T1");
        activity.setDescription("Description Test-1");
        activity.setModifiedUserId(1);
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        activity.setStartDate(nowString.substring(0, 8));
        activity.setRegistered(nowString);
        activity.setModified(nowString);
        
        this.activitiesDao_.insert(activity);
        
        int result = this.activitiesDao_.updateDeleteFlagById(9991);
        assertEquals("[OK]", 1, result);
    }
    
}
