package org.seasar.tugboat.gtd.contents.activity.dao;

import org.seasar.dao.unit.S2DaoTestCase;
import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityMember;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityPageCondition;

/**
 * test ActivityMembersDao.
 * 
 * @author yosuke
 * 
 */
public class TestActivityMembersDao extends S2DaoTestCase {
    ActivityMembersDao activityMembersDao_;
    
    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    public void setUp() {
        super.include("dicon/contents/activity/activityDao.dicon");
    }
    
    /**
     * test ActivityMembersDao#selectByActivityId.
     *
     */
    public void testSelectByActivityIdTx() {
        super.readXlsWriteDb("activities.xls");
        ActivityMember[] members = this.activityMembersDao_.selectByActivityId(9990);
        super.assertEquals("[OK]", 6, members.length);
    }
    
    /**
     * test ActivityMembersDao#selectByActivityIdPerPage.
     *
     */
    public void testSelectByActivityIdPerPageTx() {
        super.readXlsWriteDb("activities.xls");
        
        ActivityPageCondition condition = new ActivityPageCondition();
        condition.setActivityId(9990);
        condition.setLimit(5);
        condition.setOffset(0);
        
        ActivityMember[] members = 
            this.activityMembersDao_.selectByActivityIdPerPage(condition);
        super.assertEquals("[OK]", 5, members.length);
        
        condition.setLimit(5);
        condition.setOffset(5);
        members = 
            this.activityMembersDao_.selectByActivityIdPerPage(condition);
        super.assertEquals("[OK]", 1, members.length);
    }
    
    /**
     * test ActivityMembersDao#findByUserId.
     *
     */
    public void testFindByUserIdTx() {
        super.readXlsWriteDb("activities.xls");
        ActivityMember[] members = this.activityMembersDao_.findByUserId(9990);
        super.assertEquals("[OK]", 6, members.length);
    }
    
    /**
     * test ActivityMembersDao#findByActivityIdAndUserId.
     *
     */
    public void testFindByActivityIdAndUserIdTx() {
        super.readXlsWriteDb("activities.xls");
        ActivityMember member = 
            this.activityMembersDao_.findByActivityIdAndUserId(9990, 9990);
        super.assertNotNull(member);
        super.assertEquals("[OK]", 9990, member.getActivityId());
        super.assertEquals("[OK]", 9990, member.getUserId());
    }
    
    /**
     * test ActivityMembersDao#findActivityMembersSearch.
     *
     */
    public void testFindActivityMembersSearchTx() {
        super.readXlsWriteDb("activities.xls");
        ActivityMember[] members = 
            this.activityMembersDao_.findActivityMembersSearch("%activity%");
        super.assertEquals("[OK]", 12, members.length);
        
        members = this.activityMembersDao_.findActivityMembersSearch("%-5%");
        super.assertEquals("[OK]", 1, members.length);
        super.assertEquals("[OK]", 9995, members[0].getActivityId());
        super.assertEquals("[OK]", 9990, members[0].getUserId());
        super.assertEquals("[OK]", true, "t1".equals(members[0].getFirstName()));
        super.assertEquals("[OK]", true, "test1".equals(members[0].getLastName()));
    }
    
    /**
     * test ActivityMembersDao#findActivityMembersByActivityId.
     *
     */
    public void testFindActivityMembersByActivityIdTx() {
        super.readXlsWriteDb("activities.xls");
        ActivityMember[] members = 
            this.activityMembersDao_.findActivityMembersByActivityId(9990);
        super.assertEquals("[OK]", 6, members.length);
    }
    
    /**
     * test ActivityMembersDao#findActivityMembersByActivityIds.
     *
     */
    public void testFindActivityMembersByActivityIdsTx() {
        super.readXlsWriteDb("activities.xls");
        ActivityMember[] members = 
            this.activityMembersDao_.findActivityMembersByActivityIds(new Long[]{9990L,9991L,9992L});
        
        super.assertEquals("[OK]", 10, members.length);
    }
    
    /**
     * test ActivityMembersDao#findActivityMembersByIdAndKeyword.
     *
     */
    public void testFindActivityMembersByIdAndKeywordTx() {
        super.readXlsWriteDb("activities.xls");
        ActivityMember[] members = 
            this.activityMembersDao_.findActivityMembersByIdAndKeyword(9990, "%1%");
        
        super.assertEquals("[OK]", 1, members.length);
        super.assertEquals("[OK]", 9990, members[0].getUserId());
        super.assertEquals("[OK]", true, "t1".equals(members[0].getFirstName()));
        super.assertEquals("[OK]", true, "test1".equals(members[0].getLastName()));
    }
    
    /**
     * test ActivityMembersDao#getActivityRecordCountByUserId
     *
     */
    public void testGetActivityRecordCountByUserIdTx() {
        super.readXlsWriteDb("activities.xls");
        int rows = this.activityMembersDao_.getActivityRecordCountByUserId(9990);
        super.assertEquals("[OK]", 6, rows);
    }
    
    /**
     * test ActivityMembersDao#insert.
     *
     */
    public void testInsertTx() {
        super.readXlsWriteDb("activities.xls");
        
        ActivityMember activityMembers = new ActivityMember();
        activityMembers.setActivityId(9995);
        activityMembers.setUserId(9999);
        activityMembers.setRoleId(1);
        activityMembers.setModifiedUserId(9990);
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        activityMembers.setStartDate(nowString.substring(0, 8));
        activityMembers.setRegistered(nowString);
        activityMembers.setModified(nowString);
        this.activityMembersDao_.insert(activityMembers);
        
        ActivityMember result = 
            this.activityMembersDao_.findByActivityIdAndUserId(9995, 9999);
        
        super.assertEquals("[OK]", activityMembers.getActivityId(), result.getActivityId());
        super.assertEquals("[OK]", activityMembers.getUserId(), result.getUserId());
    }
    
    /**
     * test ActivityMembersDao#update.
     *
     */
    public void testUpdateTx() {
        super.readXlsWriteDb("activities.xls");
        
        ActivityMember activityMembers = new ActivityMember();
        activityMembers.setActivityId(9995);
        activityMembers.setUserId(9999);
        activityMembers.setRoleId(1);
        activityMembers.setModifiedUserId(9990);
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        activityMembers.setStartDate(nowString.substring(0, 8));
        activityMembers.setRegistered(nowString);
        activityMembers.setModified(nowString);
        this.activityMembersDao_.insert(activityMembers);
        
        activityMembers.setRoleId(2);
        activityMembers.setModifiedUserId(9991);
        activityMembers.setModified(nowString);
        this.activityMembersDao_.update(activityMembers);
        
        activityMembers.setEndDate("20070531");
        activityMembers.setDeleteFlag("1");
        activityMembers.setModified(nowString);
        this.activityMembersDao_.update(activityMembers);
    }
    
    /**
     * test ActivityMembersDao#updateDeleteFlagByActivityIdAndUserId.
     *
     */
    public void testUpdateDeleteFlagByActivityIdAndUserIdTx() {
        super.readXlsWriteDb("activities.xls");
        int result = this.activityMembersDao_.updateDeleteFlagByActivityIdAndUserId(9990, 9990);
        super.assertEquals("[OK]", 1, result);
    }
    
}
