/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dao;

import org.seasar.dao.unit.S2DaoTestCase;
import org.seasar.tugboat.gtd.contents.task.entity.TaskCommentBody;

/**
 * test TaskCommentBodiesDao.
 * 
 * @author yosuke
 * 
 */
public class TestTaskCommentBodiesDao extends S2DaoTestCase {
    TaskCommentBodiesDao taskCommentBodiesDao_;

    @Override
    public void setUp() {
        this.include("dicon/contents/task/taskDao.dicon");
    }

    /**
     * test #insert.
     * 
     */
    public void testInsertTx() {
        super.readXlsWriteDb("task_comments.xls");

        TaskCommentBody commentBody = new TaskCommentBody();
        commentBody.setTaskId(9990);
        commentBody.setCommentId(9990);
        commentBody.setBody("comment:nitidezza di scrittura.");
        this.taskCommentBodiesDao_.insert(commentBody);

        TaskCommentBody[] entities = this.taskCommentBodiesDao_
                .findByTaskId(9990);
        super.assertNotNull(entities);
        super.assertTrue(entities.length > 0);
        super.assertEquals(9990, entities[0].getTaskId());
        super.assertEquals("comment:nitidezza di scrittura.", entities[0]
                .getBody());
    }

    /**
     * test #update.
     * 
     */
    public void testUpdateTx() {
        super.readXlsWriteDb("task_comments.xls");

        TaskCommentBody commentBody = new TaskCommentBody();
        commentBody.setTaskId(9990);
        commentBody.setCommentId(9990);
        commentBody.setBody("comment:nitidezza di scrittura.");
        this.taskCommentBodiesDao_.insert(commentBody);

        commentBody.setBody("comment:Lexikon.");
        this.taskCommentBodiesDao_.update(commentBody);

        TaskCommentBody[] entities = this.taskCommentBodiesDao_
                .findByTaskId(9990);
        super.assertNotNull(entities);
        super.assertTrue(entities.length > 0);
        super.assertEquals(9990, entities[0].getTaskId());
        super.assertEquals("comment:Lexikon.", entities[0].getBody());
    }

    /**
     * test #delete.
     * 
     */
    public void testDeleteTx() {
        super.readXlsWriteDb("task_comments.xls");

        TaskCommentBody commentBody = new TaskCommentBody();
        commentBody.setTaskId(9990);
        commentBody.setCommentId(9990);
        commentBody.setBody("comment:nitidezza di scrittura.");
        this.taskCommentBodiesDao_.insert(commentBody);
        this.taskCommentBodiesDao_.delete(commentBody);

        TaskCommentBody[] entities = this.taskCommentBodiesDao_
                .findByTaskId(9990);
        super.assertTrue(entities.length == 0);
    }

}
