/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dao;

import org.seasar.dao.unit.S2DaoTestCase;
import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.contents.task.entity.TaskComment;

/**
 * test TaskCommentsDao.
 * 
 * @author yosuke
 * 
 */
public class TestTaskCommentsDao extends S2DaoTestCase {
    TaskCommentsDao taskCommentsDao_;

    @Override
    public void setUp() {
        this.include("dicon/contents/task/taskDao.dicon");
    }

    /**
     * test TaskCommentsDao#selectNewCommentId.
     * 
     */
    public void testSelectNewCommentIdTx() {
        super.readXlsWriteDb("task_comments.xls");

        int id = this.taskCommentsDao_.selectNewCommentId();
        super.assertEquals("[OK]", 10000, id);
    }

    /**
     * test TaskCommentsDao#insert.
     * 
     */
    public void testInsertTx() {
        super.readXlsWriteDb("tasks.xls");

        TaskComment comment = new TaskComment();
        comment.setId(9990);
        comment.setTaskId(9991);
        comment.setUserId(9990);
        comment.setRegistered(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
        this.taskCommentsDao_.insert(comment);
    }

    /**
     * test TaskCommentsDao#findById.
     * 
     */
    public void testFindByIdTx() {
        super.readXlsWriteDb("task_comments.xls");
        TaskComment comment = this.taskCommentsDao_.findById(9990);
        super.assertNotNull(comment);
        super.assertEquals("[OK]", 9990, comment.getId());
        super.assertEquals("[OK]", 9990, comment.getTaskId());
        super.assertEquals("[OK]", 9990, comment.getUserId());
        super.assertEquals("[OK]", true, "test1".equals(comment
                .getUserLastName()));
        super.assertEquals("[OK]", true, "t1"
                .equals(comment.getUserFirstName()));
    }

    /**
     * test TaskCommentsDao#findByTaskId.
     * 
     */
    public void testFindByTaskIdTx() {
        super.readXlsWriteDb("task_comment_bodies.xls");
        TaskComment[] comments = this.taskCommentsDao_.findByTaskId(9990);

        super.assertNotNull(comments);
        super.assertEquals("[OK]", true, 0 < comments.length);
    }

    /**
     * test TaskCommentsDao#findByUserId.
     * 
     */
    public void testFindByUserIdTx() {
        super.readXlsWriteDb("task_comment_bodies.xls");
        TaskComment[] comments = this.taskCommentsDao_.findByUserId(9990);

        super.assertNotNull(comments);
        super.assertEquals("[OK]", true, 0 < comments.length);
    }
}
