/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dao;

import org.seasar.dao.unit.S2DaoTestCase;
import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.contents.task.entity.TaskGroup;
import org.seasar.tugboat.gtd.contents.task.entity.TaskGroupActivity;

/**
 * test task group activities dao.
 * 
 * @author yousuke
 * 
 */
public class TestTaskGroupActivitiesDao extends S2DaoTestCase {
    TaskGroupsDao taskGroupsDao_;
    TaskGroupActivitiesDao taskGroupActivitiesDao_;

    @Override
    public void setUp() {
        this.include("dicon/contents/task/taskDao.dicon");
    }

    /**
     * test insert.
     */
    public void testInsertTx() {
        super.readXlsWriteDb("task_groups.xls");

        // get new id.
        long id = this.taskGroupsDao_.getId();
        System.out.println("::: id:[" + id + "]");

        // insert task groups.
        TaskGroup taskGroup = new TaskGroup();
        taskGroup.setId(id);
        taskGroup.setName("New Task Group");
        taskGroup.setDescription("New task group description.");
        taskGroup.setStartDate("20080401");
        taskGroup.setDeadline("20080410");
        taskGroup.setUserId(9990);
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        taskGroup.setRegistered(nowString);
        taskGroup.setModified(nowString);
        this.taskGroupsDao_.insert(taskGroup);

        // insert task group activity.
        TaskGroupActivity taskGroupActivity = new TaskGroupActivity();
        taskGroupActivity.setTaskGroupId(id);
        taskGroupActivity.setActivityId(9990);
        this.taskGroupActivitiesDao_.insert(taskGroupActivity);

        // find by task group id.
        TaskGroupActivity entity = this.taskGroupActivitiesDao_
                .findByTaskGroupId(id);

        super.assertNotNull(entity);
        super.assertEquals(id, entity.getTaskGroupId());
        super.assertEquals(9990, entity.getActivityId());
    }

    /**
     * test update.
     */
    public void testUpdateTx() {
        super.readXlsWriteDb("task_groups.xls");

        // get new id.
        long id = this.taskGroupsDao_.getId();
        System.out.println("::: id:[" + id + "]");

        // insert task groups.
        TaskGroup taskGroup = new TaskGroup();
        taskGroup.setId(id);
        taskGroup.setName("New Task Group");
        taskGroup.setDescription("New task group description.");
        taskGroup.setStartDate("20080401");
        taskGroup.setDeadline("20080410");
        taskGroup.setUserId(9990);
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        taskGroup.setRegistered(nowString);
        taskGroup.setModified(nowString);
        this.taskGroupsDao_.insert(taskGroup);

        // insert task group activity.
        TaskGroupActivity taskGroupActivity = new TaskGroupActivity();
        taskGroupActivity.setTaskGroupId(id);
        taskGroupActivity.setActivityId(9990);
        this.taskGroupActivitiesDao_.insert(taskGroupActivity);

        // update task group activity.
        taskGroupActivity.setActivityId(9991);
        this.taskGroupActivitiesDao_.update(taskGroupActivity);

        // find by task group id.
        TaskGroupActivity entity = this.taskGroupActivitiesDao_
                .findByTaskGroupId(id);

        super.assertNotNull(entity);
        super.assertEquals(id, entity.getTaskGroupId());
        super.assertEquals(9991, entity.getActivityId());
    }

    /**
     * test find by task group id.
     */
    public void testFindByTaskGroupIdTx() {
        super.readXlsWriteDb("task_groups.xls");

        TaskGroupActivity entity = this.taskGroupActivitiesDao_
                .findByTaskGroupId(9995);
        
        super.assertNotNull(entity);
        super.assertEquals(9995, entity.getTaskGroupId());
        super.assertEquals(9997, entity.getActivityId());
    }

}
