/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dao;

import org.seasar.dao.unit.S2DaoTestCase;
import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.contents.task.entity.TaskGroup;
import org.seasar.tugboat.gtd.contents.task.entity.TaskGroupActivity;
import org.seasar.tugboat.gtd.contents.task.entity.TaskGroupRelation;

/**
 * test task group relations dao.
 * 
 * @author yousuke
 * 
 */
public class TestTaskGroupRelationsDao extends S2DaoTestCase {
    TaskGroupsDao taskGroupsDao_;
    TaskGroupActivitiesDao taskGroupActivitiesDao_;
    TaskGroupRelationsDao taskGroupRelationsDao_;

    @Override
    public void setUp() {
        this.include("dicon/contents/task/taskDao.dicon");
    }

    /**
     * test insert.
     */
    public void testInsertTx() {
        super.readXlsWriteDb("task_groups.xls");

        // get new id.
        long id = this.taskGroupsDao_.getId();
        System.out.println("::: id:[" + id + "]");

        // insert task group.
        TaskGroup taskGroup = new TaskGroup();
        taskGroup.setId(id);
        taskGroup.setName("New Task Group");
        taskGroup.setDescription("New task group description.");
        taskGroup.setStartDate("20080401");
        taskGroup.setDeadline("20080410");
        taskGroup.setUserId(9990);
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        taskGroup.setRegistered(nowString);
        taskGroup.setModified(nowString);
        this.taskGroupsDao_.insert(taskGroup);

        // insert task group activity.
        TaskGroupActivity taskGroupActivity = new TaskGroupActivity();
        taskGroupActivity.setTaskGroupId(id);
        taskGroupActivity.setActivityId(9990);
        this.taskGroupActivitiesDao_.insert(taskGroupActivity);

        // insert task group relations.
        TaskGroupRelation taskGroupRelation = new TaskGroupRelation();
        taskGroupRelation.setTaskGroupId(id);
        taskGroupRelation.setActivityId(9990);
        taskGroupRelation.setTaskId(9990);
        this.taskGroupRelationsDao_.insert(taskGroupRelation);

        taskGroupRelation.setTaskId(9991);
        this.taskGroupRelationsDao_.insert(taskGroupRelation);

        taskGroupRelation.setTaskId(9992);
        this.taskGroupRelationsDao_.insert(taskGroupRelation);

        // find by task group id.
        TaskGroupRelation entity = this.taskGroupRelationsDao_
                .findByTaskGroupId(id, 9990);

        super.assertNotNull(entity);
        super.assertEquals(id, entity.getTaskGroupId());
        super.assertEquals(9990, entity.getActivityId());
        super.assertEquals(3, entity.getAllTasks());
        super.assertEquals(1, entity.getDoneTasks());
    }

    /**
     * test delete.
     */
    public void testDeleteTx() {
        super.readXlsWriteDb("task_groups.xls");
        this.taskGroupRelationsDao_.delete(9990);

        TaskGroupRelation entity = this.taskGroupRelationsDao_
                .findByTaskGroupId(9990, 9990);

        System.out.println(entity);
        super.assertEquals(0, entity.getTaskId());
    }

    /**
     * test find by task group id.
     */
    public void testFindByTaskGroupIdTx() {
        super.readXlsWriteDb("task_groups.xls");
        TaskGroupRelation entity = this.taskGroupRelationsDao_
                .findByTaskGroupId(9998, 9990);

        super.assertNotNull(entity);
        super.assertEquals(9998, entity.getTaskGroupId());
        super.assertEquals(9999, entity.getActivityId());
        super.assertEquals(5, entity.getAllTasks());
        super.assertEquals(2, entity.getDoneTasks());
        System.out.println(entity);
    }

    /**
     * test getTasksByTaskGroupId.
     */
    public void testGetTasksByTaskGroupIdTx() {
        super.readXlsWriteDb("task_groups.xls");

        TaskGroupRelation[] entities = this.taskGroupRelationsDao_
                .getTasksByTaskGroupId(9990, 9990);

        super.assertNotNull(entities);
        super.assertTrue(entities.length == 3);

        for (int i = 0; i < entities.length; i++) {
            super.assertEquals(9990, entities[i].getTaskGroupId());
            super.assertEquals(9990, entities[i].getActivityId());

            if (i == 0) {
                super.assertEquals(9990, entities[i].getTaskId());
                super.assertEquals(1, entities[i].getTaskClassificationId());
                super.assertEquals(1, entities[i].getTaskStatusId());
                super.assertEquals("Test Task 1", entities[i].getTaskTitle());
                super.assertNull(entities[i].getTaskDeadline());

            } else if (i == 1) {
                super.assertEquals(9991, entities[i].getTaskId());
                super.assertEquals(2, entities[i].getTaskClassificationId());
                super.assertEquals(2, entities[i].getTaskStatusId());
                super.assertEquals("Test Task 2", entities[i].getTaskTitle());
                super.assertNull(entities[i].getTaskDeadline());

            } else if (i == 2) {
                super.assertEquals(9992, entities[i].getTaskId());
                super.assertEquals(3, entities[i].getTaskClassificationId());
                super.assertEquals(3, entities[i].getTaskStatusId());
                super.assertEquals("Test Task 3", entities[i].getTaskTitle());
                super.assertNull(entities[i].getTaskDeadline());
            } else {
                super.fail();
            }
        }
    }

    /**
     * test find by user id.
     */
    public void testFindByUserIdTx() {
        super.readXlsWriteDb("task_groups.xls");

        TaskGroupRelation[] taskGroupRelations = this.taskGroupRelationsDao_
                .findActiveGroupsByUserId(9990);

        for (TaskGroupRelation t : taskGroupRelations) {
            if (t.getTaskGroupId() == 9990) {
                super.assertEquals(9990, t.getActivityId());
                super.assertEquals("TG-1", t.getName());
                super.assertEquals("20080301", t.getStartDate());
                super.assertEquals("20080401", t.getDeadline());
                super.assertEquals(1, t.getDoneTasks());
                super.assertEquals(3, t.getAllTasks());

            } else if (t.getTaskGroupId() == 9991) {
                super.assertEquals(9990, t.getActivityId());
                super.assertEquals("TG-2", t.getName());
                super.assertEquals("20080302", t.getStartDate());
                super.assertEquals("20080402", t.getDeadline());
                super.assertEquals(0, t.getDoneTasks());
                super.assertEquals(0, t.getAllTasks());
                
            } else if (t.getTaskGroupId() == 9992) {
                super.assertEquals(9990, t.getActivityId());
                super.assertEquals("TG-3", t.getName());
                super.assertEquals("20080303", t.getStartDate());
                super.assertEquals("20080403", t.getDeadline());
                super.assertEquals(1, t.getDoneTasks());
                super.assertEquals(3, t.getAllTasks());

            } else if (t.getTaskGroupId() == 9998) {
                super.assertEquals(9999, t.getActivityId());
                super.assertEquals("TG-9", t.getName());
                super.assertEquals("20080309", t.getStartDate());
                super.assertEquals("20080409", t.getDeadline());
                super.assertEquals(2, t.getDoneTasks());
                super.assertEquals(5, t.getAllTasks());
            } else {
                super.fail();
            }
        }
    }

    /**
     * test find by activity id.
     */
    public void testFindByActivityIdTx() {
        super.readXlsWriteDb("task_groups.xls");

        //
        // posted by me.
        //
        TaskGroupRelation[] taskGroupRelations = this.taskGroupRelationsDao_
                .findByActivityId(9997, 9995);

        super.assertNotNull(taskGroupRelations);
        super.assertFalse(taskGroupRelations.length == 0);
        
        for (TaskGroupRelation t : taskGroupRelations) {
            super.assertEquals(9997, t.getActivityId());
            System.out.println(t.toString());
            
            if (t.getTaskGroupId() == 9995) {
                super.assertEquals("TG-6", t.getName());
                super.assertEquals("20080306", t.getStartDate());
                super.assertEquals("20080406", t.getDeadline());
                super.assertEquals(2, t.getAllTasks());
                super.assertEquals(1, t.getDoneTasks());

            } else if (t.getTaskGroupId() == 9996) {
                super.assertEquals("TG-7", t.getName());
                super.assertEquals("20080307", t.getStartDate());
                super.assertEquals("20080407", t.getDeadline());
                super.assertEquals(3, t.getAllTasks());
                super.assertEquals(1, t.getDoneTasks());

            } else if (t.getTaskGroupId() == 9997) {
                super.assertEquals("TG-8", t.getName());
                super.assertEquals("20080308", t.getStartDate());
                super.assertEquals("20080408", t.getDeadline());
                super.assertEquals(3, t.getAllTasks());
                super.assertEquals(1, t.getDoneTasks());

            } else {
                super.fail();
            }
        }

        //
        // posted by another.
        //
        taskGroupRelations = this.taskGroupRelationsDao_
                .findByActivityId(9990, 9994);

        super.assertNotNull(taskGroupRelations);
        super.assertFalse(taskGroupRelations.length == 0);
        
        for (TaskGroupRelation t : taskGroupRelations) {
            super.assertEquals(9990, t.getActivityId());

            if (t.getTaskGroupId() == 9990) {
                super.assertEquals("TG-1", t.getName());
                super.assertEquals("20080301", t.getStartDate());
                super.assertEquals("20080401", t.getDeadline());
                super.assertEquals(3, t.getAllTasks());
                super.assertEquals(1, t.getDoneTasks());

            } else if (t.getTaskGroupId() == 9991) {
                super.assertEquals("TG-2", t.getName());
                super.assertEquals("20080302", t.getStartDate());
                super.assertEquals("20080402", t.getDeadline());
                super.assertEquals(0, t.getAllTasks());
                super.assertEquals(0, t.getDoneTasks());
                
            } else if (t.getTaskGroupId() == 9992) {
                super.assertEquals("TG-3", t.getName());
                super.assertEquals("20080303", t.getStartDate());
                super.assertEquals("20080403", t.getDeadline());
                super.assertEquals(3, t.getAllTasks());
                super.assertEquals(1, t.getDoneTasks());
                
            } else {
                super.fail();
            }
        }
    }

}
