/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dao;

import org.seasar.dao.unit.S2DaoTestCase;
import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.contents.task.entity.TaskGroup;

/**
 * test task groups dao.
 * 
 * @author yousuke
 * 
 */
public class TestTaskGroupsDao extends S2DaoTestCase {
    TaskGroupsDao taskGroupsDao_;

    @Override
    public void setUp() {
        this.include("dicon/contents/task/taskDao.dicon");
    }

    /**
     * test insert.
     */
    public void testInsertTx() {
        super.readXlsWriteDb("task_groups.xls");

        // get new id.
        long id = this.taskGroupsDao_.getId();
        System.out.println("::: id:[" + id + "]");

        // insert.
        TaskGroup taskGroup = new TaskGroup();
        taskGroup.setId(id);
        taskGroup.setName("New Task Group");
        taskGroup.setDescription("New task group description.");
        taskGroup.setStartDate("20080401");
        taskGroup.setDeadline("20080410");
        taskGroup.setUserId(9990);
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        taskGroup.setRegistered(nowString);
        taskGroup.setModified(nowString);
        this.taskGroupsDao_.insert(taskGroup);

        // find by id.
        TaskGroup entity = this.taskGroupsDao_.findById(id);
        System.out.println(entity);
        super.assertNotNull(entity);
        super.assertEquals(id, entity.getId());
        super.assertEquals("New Task Group", entity.getName());
        super.assertEquals("New task group description.", entity
                .getDescription());
        super.assertEquals("20080401", entity.getStartDate());
        super.assertEquals("20080410", entity.getDeadline());
        super.assertEquals(9990, entity.getUserId());
    }

    /**
     * test update.
     */
    public void testUpdateTx() {
        super.readXlsWriteDb("task_groups.xls");

        long id = this.taskGroupsDao_.getId();
        System.out.println("::: id:[" + id + "]");

        // insert.
        TaskGroup taskGroup = new TaskGroup();
        taskGroup.setId(id);
        taskGroup.setName("New Task Group");
        taskGroup.setDescription("New task group description.");
        taskGroup.setStartDate("20080401");
        taskGroup.setDeadline("20080410");
        taskGroup.setUserId(9990);
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        taskGroup.setRegistered(nowString);
        taskGroup.setModified(nowString);
        this.taskGroupsDao_.insert(taskGroup);

        // update.
        taskGroup.setName("Update Task Group");
        taskGroup.setDescription("Update task group description.");
        taskGroup.setStartDate("20080501");
        taskGroup.setDeadline("20080510");
        taskGroup.setUserId(9991);
        this.taskGroupsDao_.update(taskGroup);

        // find by id.
        TaskGroup entity = this.taskGroupsDao_.findById(id);
        System.out.println(entity);
        super.assertNotNull(entity);
        super.assertEquals(id, entity.getId());
        super.assertEquals("Update Task Group", entity.getName());
        super.assertEquals("Update task group description.", entity
                .getDescription());
        super.assertEquals("20080501", entity.getStartDate());
        super.assertEquals("20080510", entity.getDeadline());
        super.assertEquals(9991, entity.getUserId());
    }

    /**
     * test update delete flag.
     */
    public void testUpdateDelFlagTx() {
        super.readXlsWriteDb("task_groups.xls");

        // get new id.
        long id = this.taskGroupsDao_.getId();
        System.out.println("::: id:[" + id + "]");

        // insert.
        TaskGroup taskGroup = new TaskGroup();
        taskGroup.setId(id);
        taskGroup.setName("New Task Group");
        taskGroup.setDescription("New task group description.");
        taskGroup.setStartDate("20080401");
        taskGroup.setDeadline("20080410");
        taskGroup.setUserId(9990);
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        taskGroup.setRegistered(nowString);
        taskGroup.setModified(nowString);
        this.taskGroupsDao_.insert(taskGroup);

        // update del flag.
        taskGroup.setDelFlag(1);
        this.taskGroupsDao_.updateDelFlag(taskGroup);

        // find by id.
        TaskGroup entity = this.taskGroupsDao_.findById(id);
        System.out.println(entity);
        super.assertNotNull(entity);
        super.assertEquals(id, entity.getId());
        super.assertEquals(1, entity.getDelFlag());
    }

    /**
     * test find by user id.
     */
    public void testFindByUserIdTx() {
        super.readXlsWriteDb("task_groups.xls");

        TaskGroup[] entities = this.taskGroupsDao_.findByUserId(9990);
        super.assertNotNull(entities);
        super.assertTrue(entities.length > 0);
        super.assertTrue(entities.length == 5);

        for (TaskGroup t : entities) {
            if (t.getId() == 9990) {
                super.assertEquals("TG-1", t.getName());
                super.assertEquals("task group #1", t.getDescription());
                super.assertEquals("20080301", t.getStartDate());
                super.assertEquals("20080401", t.getDeadline());
                super.assertEquals(0, t.getDelFlag());
                super.assertEquals(9990, t.getUserId());
                
            } else if (t.getId() == 9991) {
                super.assertEquals("TG-2", t.getName());
                super.assertEquals("task group #2", t.getDescription());
                super.assertEquals("20080302", t.getStartDate());
                super.assertEquals("20080402", t.getDeadline());
                super.assertEquals(0, t.getDelFlag());
                super.assertEquals(9990, t.getUserId());

            } else if (t.getId() == 9992) {
                super.assertEquals("TG-3", t.getName());
                super.assertEquals("task group #3", t.getDescription());
                super.assertEquals("20080303", t.getStartDate());
                super.assertEquals("20080403", t.getDeadline());
                super.assertEquals(0, t.getDelFlag());
                super.assertEquals(9990, t.getUserId());

            } else if (t.getId() == 9993) {
                super.assertEquals("TG-4", t.getName());
                super.assertEquals("task group #4", t.getDescription());
                super.assertEquals("20080304", t.getStartDate());
                super.assertEquals("20080404", t.getDeadline());
                super.assertEquals(0, t.getDelFlag());
                super.assertEquals(9990, t.getUserId());

            } else if (t.getId() == 9994) {
                super.assertEquals("TG-5", t.getName());
                super.assertEquals("task group #5", t.getDescription());
                super.assertEquals("20080305", t.getStartDate());
                super.assertEquals("20080405", t.getDeadline());
                super.assertEquals(0, t.getDelFlag());
                super.assertEquals(9990, t.getUserId());
            }
        }
    }

    /**
     * test find by id.
     */
    public void testFindByIdTx() {
        super.readXlsWriteDb("task_groups.xls");
        
        TaskGroup entity = this.taskGroupsDao_.findById(9990);
        super.assertNotNull(entity);
        super.assertEquals("TG-1", entity.getName());
        super.assertEquals("task group #1", entity.getDescription());
        super.assertEquals("20080301", entity.getStartDate());
        super.assertEquals("20080401", entity.getDeadline());
        super.assertEquals(0, entity.getDelFlag());
        super.assertEquals(9990, entity.getUserId());
    }

    /**
     * test get id.
     */
    public void testGetIdTx() {
        super.readXlsWriteDb("task_groups.xls");

        long id = this.taskGroupsDao_.getId();
        super.assertEquals(10000, id);
    }
}
