/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dao;

import java.util.ArrayList;
import java.util.List;

import org.seasar.dao.unit.S2DaoTestCase;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBrowsePageCondition;
import org.seasar.tugboat.gtd.contents.task.entity.TaskRelation;

/**
 * test TaskRelationsDao.
 * 
 * @author yosuke
 * 
 */
public class TestTaskRelationsDao extends S2DaoTestCase {
    TaskRelationsDao taskRelationsDao_;

    @Override
    public void setUp() {
        this.include("dicon/contents/task/taskDao.dicon");
    }

    /**
     * test TaskRelationsDao#findByTaskId.
     * 
     */
    public void testFindByTaskIdTx() {
        super.readXlsWriteDb("task_bodies.xls");
        TaskRelation relation = this.taskRelationsDao_.findByTaskId(9990, 9990);
        super.assertNotNull(relation);
        super.assertEquals("[OK]", 9990, relation.getTaskId());
        super.assertEquals("[OK]", true, "Test Task 1".equals(relation
                .getTitle()));
        super.assertEquals("[OK]", true, "task comment-1".equals(relation
                .getDescription()));
        super.assertEquals("[OK]", true, "20070501".equals(relation
                .getStartDate().trim()));
        super.assertNull("[OK]", relation.getDeadline());
        super.assertEquals("[OK]", 1, relation.getStatusId());
        super.assertEquals("[OK]", 9990, relation.getPersonInChargeId());
    }

    /**
     * test TaskRelationsDao#findByUserId.
     * 
     */
    public void testFindByUserIdTx() {
        super.readXlsWriteDb("task_bodies.xls");
        TaskRelation[] relations = this.taskRelationsDao_.findByUserId(9990,
                true);
        super.assertNotNull(relations);
        super.assertEquals("[OK]", 4, relations.length);
    }

    /**
     * test TaskRelationsDao#getRelationsWithHistoriesByUserId.
     * 
     */
    public void testGetRelationsWithHistoriesByUserIdTx() {
        super.readXlsWriteDb("task_bodies.xls");
        TaskRelation[] relations = this.taskRelationsDao_
                .getRelationsWithHistoriesByUserId(9990, "00000000000000", "99999999999999");

        super.assertNotNull(relations);
        super.assertEquals("[OK]", 4, relations.length);
    }

    /**
     * test TaskRekationsDao#findTaskIdsByQueryParams.
     * 
     */
    public void testFindTaskIdsByQueryParamsTx() {
        super.readXlsWriteDb("task_bodies.xls");

        TaskBrowsePageCondition condition = new TaskBrowsePageCondition();
        // #1
        condition.setLimit(20);
        condition.setOffset(0);
        condition.setUserId(9990);

        TaskRelation[] results = this.taskRelationsDao_
                .findTaskIdsByQueryParams(condition);

        super.assertNotNull(results);
        super.assertEquals("[OK]", 5, results.length);

        // #2
        condition.setQueryPersonIds(new Long[] { 9990L });
        results = this.taskRelationsDao_.findTaskIdsByQueryParams(condition);

        super.assertNotNull(results);
        super.assertEquals("[OK]", 4, results.length);

        // #3
        condition.setQueryActivityIds(new Long[] { 9990L });
        results = this.taskRelationsDao_.findTaskIdsByQueryParams(condition);
        super.assertNotNull(results);
        super.assertEquals("[OK]", 3, results.length);

        // #4
        condition.setQueryStatusIds(new Integer[] { 1, 2 });
        results = this.taskRelationsDao_.findTaskIdsByQueryParams(condition);
        super.assertNotNull(results);
        super.assertEquals("[OK]", 2, results.length);

        // #5
        condition.setQueryClassificationIds(new Integer[] { 1, 9 });
        results = this.taskRelationsDao_.findTaskIdsByQueryParams(condition);
        super.assertNotNull(results);
        super.assertEquals("[OK]", 1, results.length);
    }

    /**
     * test TaskRelationsDao#findByQueryParams.
     * 
     */
    public void testFindByQueryParams_1_Tx() {
        super.readXlsWriteDb("task_bodies.xls");

        TaskBrowsePageCondition condition = new TaskBrowsePageCondition();
        condition.setLimit(20);
        condition.setOffset(0);
        condition.setUserId(9990);

        TaskRelation[] results = this.taskRelationsDao_.findTaskIdsByQueryParams(condition);

        List<Long> taskIdList = new ArrayList<Long>();
        for (TaskRelation r : results) {
            taskIdList.add(r.getTaskId());
        }
        Long[] targetTaskIds = taskIdList.toArray(new Long[taskIdList.size()]);

        TaskRelation[] relations = this.taskRelationsDao_.findByQueryParams(targetTaskIds, 1, 9990);
        super.assertNotNull(relations);
        super.assertEquals("[OK]", 11, relations.length);
    }

    /**
     * test TaskRelationsDao#findByQueryParams#2.
     */
    public void testFindByQueryParams_2_Tx() {
        super.readXlsWriteDb("task_bodies.xls");

        TaskBrowsePageCondition condition = new TaskBrowsePageCondition();
        condition.setLimit(20);
        condition.setOffset(0);
        condition.setUserId(9990);

        condition.setQueryPersonIds(new Long[] { 9990L });

        TaskRelation[] results = this.taskRelationsDao_
                .findTaskIdsByQueryParams(condition);

        List<Long> taskIdList = new ArrayList<Long>();
        for (TaskRelation r : results) {
            taskIdList.add(r.getTaskId());
        }
        Long[] targetTaskIds = taskIdList.toArray(new Long[taskIdList.size()]);

        TaskRelation[] relations = this.taskRelationsDao_.findByQueryParams(targetTaskIds, 1, 9990);

        relations = this.taskRelationsDao_.findByQueryParams(targetTaskIds, 1, 9990);
        super.assertNotNull(relations);
        super.assertEquals("[OK]", 10, relations.length);
    }

    /**
     * test TaskRelationsDao#findByQueryParams#3.
     */
    public void testFindByQueryParams_3_Tx() {
        super.readXlsWriteDb("task_bodies.xls");

        TaskBrowsePageCondition condition = new TaskBrowsePageCondition();
        condition.setLimit(20);
        condition.setOffset(0);
        condition.setUserId(9990);
        condition.setQueryPersonIds(new Long[] { 9990L });
        condition.setQueryActivityIds(new Long[] { 9990L });

        TaskRelation[] results = this.taskRelationsDao_
                .findTaskIdsByQueryParams(condition);

        List<Long> taskIdList = new ArrayList<Long>();
        for (TaskRelation r : results) {
            taskIdList.add(r.getTaskId());
        }
        Long[] targetTaskIds = taskIdList.toArray(new Long[taskIdList.size()]);

        TaskRelation[] relations = this.taskRelationsDao_.findByQueryParams(targetTaskIds, 1, 9990);

        relations = this.taskRelationsDao_.findByQueryParams(targetTaskIds, 1, 9990);
        super.assertNotNull(relations);
        super.assertEquals("[OK]", 6, relations.length);
    }

    /**
     * test TaskRelationsDao#findByQueryParams#4.
     */
    public void testFindByQueryParams_4_Tx() {
        super.readXlsWriteDb("task_bodies.xls");

        TaskBrowsePageCondition condition = new TaskBrowsePageCondition();
        condition.setLimit(20);
        condition.setOffset(0);
        condition.setUserId(9990);
        condition.setQueryPersonIds(new Long[] { 9990L });
        condition.setQueryActivityIds(new Long[] { 9990L });
        condition.setQueryStatusIds(new Integer[] { 1, 2 });

        TaskRelation[] results = this.taskRelationsDao_
                .findTaskIdsByQueryParams(condition);

        List<Long> taskIdList = new ArrayList<Long>();
        for (TaskRelation r : results) {
            taskIdList.add(r.getTaskId());
        }
        Long[] targetTaskIds = taskIdList.toArray(new Long[taskIdList.size()]);

        TaskRelation[] relations = this.taskRelationsDao_.findByQueryParams(targetTaskIds, 1, 9990);
        super.assertNotNull(relations);
        super.assertEquals("[OK]", 4, relations.length);
    }

    /**
     * test TaskRelationsDao#findByQueryParams#5.
     */
    public void testFindByQueryParams_5_Tx() {
        super.readXlsWriteDb("task_bodies.xls");

        TaskBrowsePageCondition condition = new TaskBrowsePageCondition();
        condition.setLimit(20);
        condition.setOffset(0);
        condition.setUserId(9990);
        condition.setQueryPersonIds(new Long[] { 9990L });
        condition.setQueryActivityIds(new Long[] { 9990L });
        condition.setQueryStatusIds(new Integer[] { 1, 2 });

        condition.setQueryClassificationIds(new Integer[] { 1, 9 });
        TaskRelation[] results = this.taskRelationsDao_
                .findTaskIdsByQueryParams(condition);

        List<Long> taskIdList = new ArrayList<Long>();
        for (TaskRelation r : results) {
            taskIdList.add(r.getTaskId());
        }
        Long[] targetTaskIds = taskIdList.toArray(new Long[taskIdList.size()]);

        TaskRelation[] relations = this.taskRelationsDao_.findByQueryParams(targetTaskIds, 1, 9990);
        super.assertNotNull(relations);
        super.assertEquals("[OK]", 3, relations.length);
    }

    /**
     * test TaskRelationsDao#getRowsByQueryParams.
     * 
     */
    public void testGetRowsByQueryParamsTx() {
        super.readXlsWriteDb("task_bodies.xls");

        TaskBrowsePageCondition condition = new TaskBrowsePageCondition();
        condition.setLimit(20);
        condition.setOffset(0);
        condition.setUserId(9990);
        TaskRelation result = this.taskRelationsDao_
                .getRowsByQueryParams(condition);
        super.assertNotNull(result);
        super.assertEquals("[OK]", 5, result.getRows());

        condition.setQueryPersonIds(new Long[] { 9990L });
        result = this.taskRelationsDao_.getRowsByQueryParams(condition);
        super.assertNotNull(result);
        super.assertEquals("[OK]", 4, result.getRows());

        condition.setQueryActivityIds(new Long[] { 9990L });
        result = this.taskRelationsDao_.getRowsByQueryParams(condition);
        super.assertNotNull(result);
        super.assertEquals("[OK]", 3, result.getRows());

        condition.setQueryStatusIds(new Integer[] { 1, 2 });
        result = this.taskRelationsDao_.getRowsByQueryParams(condition);
        super.assertNotNull(result);
        super.assertEquals("[OK]", 2, result.getRows());

        condition.setQueryClassificationIds(new Integer[] { 1, 9 });
        result = this.taskRelationsDao_.getRowsByQueryParams(condition);
        super.assertNotNull(result);
        super.assertEquals("[OK]", 1, result.getRows());
    }

    /**
     * test TaskRelationsDao#getDefaultFolderTasks.
     * 
     */
    public void testGetDefaultFolderTasksTx() {
        super.readXlsWriteDb("tasks.xls");
        TaskRelation result = this.taskRelationsDao_.getDefaultFolderTasks(
                9990L, new long[] { 9990L, 9991L });

        super.assertNotNull(result);
        super.assertEquals("[OK]", 0, result.getInboxTasks());
        super.assertEquals("[OK]", 1, result.getTodayTasks());
        super.assertEquals("[OK]", 1, result.getWeekTasks());
        super.assertEquals("[OK]", 1, result.getHoldTasks());
        super.assertEquals("[OK]", 1, result.getSomedayTasks());
    }

    /**
     * test TaskRelationsDao#getPersonInChargeByActivityIds.
     * 
     */
    public void testGetPersonInChargeByActivityIdsTx() {
        super.readXlsWriteDb("tasks.xls");
        TaskRelation[] relations = this.taskRelationsDao_
                .getPersonInChargeByActivityIds(new Integer[] { 9990, 9991 });

        super.assertNotNull(relations);
        super.assertEquals("[OK]", true, 0 < relations.length);
        super.assertEquals("[OK]", 2, relations.length);
    }

    /**
     * test TaskRelationsDao#getPersonInChargeByOwnActivities.
     * 
     */
    public void testGetPersonInChargeByOwnActivitiesTx() {
        super.readXlsWriteDb("task_bodies.xls");
        TaskRelation[] relations = this.taskRelationsDao_
                .getPersonInChargeByOwnActivities(9990);

        super.assertNotNull(relations);
        super.assertEquals("[OK]", true, 0 < relations.length);
        super.assertEquals("[OK]", 2, relations.length);
    }

    /**
     * test TaskRelationsDao#insert.
     * 
     */
    public void testInsertTx() {
        super.readXlsWriteDb("tasks.xls");
        TaskRelation taskRelations = new TaskRelation();
        taskRelations.setTaskId(9990);
        taskRelations.setActivityId(9990);
        taskRelations.setStatusId(1);
        taskRelations.setClassificationId(9);
        taskRelations.setPersonInChargeId(9990);
        taskRelations.setUserId(9991);
        this.taskRelationsDao_.insert(taskRelations);
    }

    /**
     * test TaskRelationsDao#update.
     * 
     */
    public void testUpdateTx() {
        super.readXlsWriteDb("tasks.xls");
        TaskRelation taskRelations = new TaskRelation();
        taskRelations.setTaskId(9990);
        taskRelations.setActivityId(9990);
        taskRelations.setStatusId(1);
        taskRelations.setClassificationId(9);
        taskRelations.setPersonInChargeId(9990);
        taskRelations.setUserId(9991);
        this.taskRelationsDao_.insert(taskRelations);

        taskRelations.setStatusId(2);
        taskRelations.setClassificationId(1);
        taskRelations.setPersonInChargeId(9991);
        this.taskRelationsDao_.update(taskRelations);
    }

    /**
     * test TaskRelationsDao#updateClassification.
     * 
     */
    public void testUpdateClassificationTx() {
        super.readXlsWriteDb("tasks.xls");
        this.taskRelationsDao_.updateClassification(9990, 1, 0);
        this.taskRelationsDao_.updateClassification(9990, 1, 2);
    }

    /**
     * test TaskRelationsDao#updateClassificationsForTransferTasks.
     * 
     */
    public void testUpdateClassificationsForTransferTasksTx() {
        super.readXlsWriteDb("tasks.xls");
        this.taskRelationsDao_
                .updateClassificationsForTransferTasks(new Long[] { 9990L,
                        9991L });
    }

    /**
     * test TaskRelationsDao#getUpdateTargetByDateForMoveToArchive.
     * 
     */
    public void testGetUpdateTargetByDateForMoveToArchiveTx() {
        super.readXlsWriteDb("tasks.xls");
        TaskRelation[] relations = this.taskRelationsDao_
                .getUpdateTargetByDateForMoveToArchive("20070505000000");

        super.assertNotNull(relations);
        super.assertEquals("[OK]", true, 0 < relations.length);
        super.assertEquals("[OK]", 1, relations.length);
    }

    /**
     * test TaskRelationsDao#updateClassificationsMoveToArchive.
     * 
     */
    public void testUpdateClassificationsMoveToArchiveTx() {
        super.readXlsWriteDb("tasks.xls");
        int result = this.taskRelationsDao_
                .updateClassificationsMoveToArchive(new Long[] { 9990L, 9991L });
        super.assertEquals("[OK]", 2, result);
    }

    /**
     * test TaskRelationsDao#findTaskByDiscussionId.
     * 
     */
    public void testFindTaskByDiscussionIdTx() {
        super.readXlsWriteDb("tasks.xls");

        int discussionId = 9990;
        TaskRelation actual = this.taskRelationsDao_
                .findTaskByDiscussionId(discussionId);

        super.assertEquals(9990, actual.getTaskId());
        super.assertEquals(1, actual.getStatusId());
        super.assertEquals(9990, actual.getActivityId());
        super.assertEquals(1, actual.getClassificationId());
        super.assertEquals(0, actual.getAlertId());
        super.assertEquals(9990, actual.getPersonInChargeId());
        super.assertEquals(9990, actual.getUserId());
        super.assertEquals("Test Task 1", actual.getTitle());
        super.assertEquals(0, actual.getShare());
        super.assertEquals("20070501", actual.getStartDate());
        super.assertEquals(2007, actual.getStartYear());
        super.assertEquals(5, actual.getStartMonth());
        super.assertEquals(1, actual.getStartDayOfMonth());
        super.assertTrue(actual.getDeadline() == null);
        super.assertEquals(0, actual.getDeadlineYear());
        super.assertEquals(0, actual.getDeadlineMonth());
        super.assertEquals(0, actual.getDeadlineDayOfMonth());
        super.assertEquals(0, actual.getPrivacy());
        super.assertEquals(1, actual.getPriorityOrder());
        super.assertEquals(0, actual.getDeleteFlag());
        super.assertEquals("20070501000000", actual.getRegistered());
        super.assertEquals("task_body_9990", actual.getDescription());

        discussionId = 9991;
        actual = this.taskRelationsDao_.findTaskByDiscussionId(discussionId);

        super.assertEquals(9991, actual.getTaskId());
        super.assertEquals(2, actual.getStatusId());
        super.assertEquals(9990, actual.getActivityId());
        super.assertEquals(2, actual.getClassificationId());
        super.assertEquals(0, actual.getAlertId());
        super.assertEquals(9990, actual.getPersonInChargeId());
        super.assertEquals(9990, actual.getUserId());
        super.assertEquals("Test Task 2", actual.getTitle());
        super.assertEquals(0, actual.getShare());
        super.assertEquals("20070502", actual.getStartDate());
        super.assertEquals(2007, actual.getStartYear());
        super.assertEquals(5, actual.getStartMonth());
        super.assertEquals(2, actual.getStartDayOfMonth());
        super.assertTrue(actual.getDeadline() == null);
        super.assertEquals(0, actual.getDeadlineYear());
        super.assertEquals(0, actual.getDeadlineMonth());
        super.assertEquals(0, actual.getDeadlineDayOfMonth());
        super.assertEquals(0, actual.getPrivacy());
        super.assertEquals(1, actual.getPriorityOrder());
        super.assertEquals(0, actual.getDeleteFlag());
        super.assertEquals("20070502000000", actual.getRegistered());
        super.assertEquals("task_body_9991", actual.getDescription());

        discussionId = -1;
        actual = this.taskRelationsDao_.findTaskByDiscussionId(discussionId);

        super.assertTrue(actual == null);
    }

    /**
     * test TaskRelationsDao#getCountByActivityId.
     * 
     */
    public void testGetCountByActivityIdTx() {
        super.readXlsWriteDb("tasks.xls");
        int tasks = this.taskRelationsDao_.getCountByActivityId(9990);
        super.assertEquals("[OK]", 3, tasks);
    }

    /**
     * test TaskRelationsDao#getCountByActivityIdAndPersonId.
     * 
     */
    public void testGetCountByActivityIdAndPersonIdTx() {
        super.readXlsWriteDb("tasks.xls");
        int tasks = this.taskRelationsDao_.getCountByActivityIdAndPersonId(
                9990, 9990);
        super.assertEquals("[OK]", 3, tasks);
    }

    /**
     * test TaskRelationsDao#findTaskIdsByHasTags.
     * 
     */
    public void testFindTaskIdsByHasTagsTx() {
        super.readXlsWriteDb("task_bodies.xls");
        TaskBrowsePageCondition condition = new TaskBrowsePageCondition();
        condition.setTagIds(new Long[] { 9990L });
        condition.setUserId(9990);
        condition.setOffset(0);
        condition.setLimit(50);

        TaskRelation[] result = this.taskRelationsDao_
                .findTaskIdsByHasTags(condition);
        super.assertEquals(1, result.length);
    }

    /**
     * test TaskRelationsDao#findByHasTags.
     * 
     */
    public void testFindByHasTagsTx() {
        super.readXlsWriteDb("task_bodies.xls");
        TaskBrowsePageCondition condition = new TaskBrowsePageCondition();
        condition.setTagIds(new Long[] { 9990L });
        condition.setUserId(9990);
        condition.setOffset(0);
        condition.setLimit(50);

        TaskRelation[] result = this.taskRelationsDao_
                .findTaskIdsByHasTags(condition);

        List<Long> taskIdList = new ArrayList<Long>();
        for (TaskRelation r : result) {
            taskIdList.add(r.getTaskId());
        }
        Long[] ids = taskIdList.toArray(new Long[taskIdList.size()]);

        TaskRelation[] tasks = this.taskRelationsDao_.findByHasTags(ids, 9990);
        super.assertEquals(3, tasks.length);
    }
}
