/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dao;

import org.seasar.dao.unit.S2DaoTestCase;
import org.seasar.tugboat.gtd.contents.common.entity.Classification;
import org.seasar.tugboat.gtd.contents.task.entity.TaskSelectView;

/**
 * test task select views dao.
 * 
 * @author yosuke
 * 
 */
public class TestTaskSelectViewsDao extends S2DaoTestCase {

    TaskSelectViewsDao taskSelectViewsDao_;

    @Override
    public void setUp() {
        this.include("dicon/contents/task/taskDao.dicon");
    }

    /**
     * test insert.
     * 
     */
    public void testInsert1Tx() {
        super.readXlsWriteDb("taskBrowseRelated.xls");
        TaskSelectView entity = new TaskSelectView();
        entity.setClassId(1);
        entity.setUserId(9990);

        this.taskSelectViewsDao_.insert(entity);

        TaskSelectView view = this.taskSelectViewsDao_.findByUserId(9990);
        System.out.println(view);
        super.assertNotNull(view);
        super.assertEquals(9990, view.getUserId());
        super.assertEquals(1, view.getClassId());
        super.assertEquals(0, view.getModeId());
        super.assertEquals(0, view.getIsVisibleBrowser());
    }

    /**
     * test insert.
     * 
     */
    public void testInsert2Tx() {
        super.readXlsWriteDb("taskBrowseRelated.xls");

        TaskSelectView entity = new TaskSelectView();
        entity.setModeId(1);
        entity.setUserId(9990);

        this.taskSelectViewsDao_.insert(entity);

        TaskSelectView view = this.taskSelectViewsDao_.findByUserId(9990);
        System.out.println(view);
        super.assertNotNull(view);
        super.assertEquals(9990, view.getUserId());
        super.assertEquals(Classification.INBOX, view.getClassId());
        super.assertEquals(1, view.getModeId());
        super.assertEquals(0, view.getIsVisibleBrowser());
    }

    /**
     * test insert.
     * 
     */
    public void testInsert3Tx() {
        super.readXlsWriteDb("taskBrowseRelated.xls");

        TaskSelectView entity = new TaskSelectView();
        entity.setIsVisibleBrowser(1);
        entity.setUserId(9990);

        this.taskSelectViewsDao_.insert(entity);

        TaskSelectView view = this.taskSelectViewsDao_.findByUserId(9990);
        System.out.println(view);
        super.assertNotNull(view);
        super.assertEquals(9990, view.getUserId());
        super.assertEquals(Classification.INBOX, view.getClassId());
        super.assertEquals(0, view.getModeId());
        super.assertEquals(1, view.getIsVisibleBrowser());
    }

    /**
     * test update.
     * 
     */
    public void testUpdateTx() {
        super.readXlsWriteDb("taskBrowseRelated.xls");

        TaskSelectView entity = new TaskSelectView();
        entity.setIsVisibleBrowser(1);
        entity.setUserId(9990);

        this.taskSelectViewsDao_.insert(entity);

        entity.setIsVisibleBrowser(0);
        this.taskSelectViewsDao_.update(entity);

        TaskSelectView view = this.taskSelectViewsDao_.findByUserId(9990);
        System.out.println(view);
        super.assertNotNull(view);
        super.assertEquals(9990, view.getUserId());
        super.assertEquals(Classification.INBOX, view.getClassId());
        super.assertEquals(0, view.getModeId());
        super.assertEquals(0, view.getIsVisibleBrowser());
    }

    /**
     * test find by user id.
     * 
     */
    public void testFindByUserIdTx() {
        super.readXlsWriteDb("taskBrowseRelated.xls");

        TaskSelectView entity = new TaskSelectView();
        entity.setIsVisibleBrowser(1);
        entity.setUserId(9990);
        this.taskSelectViewsDao_.insert(entity);

        TaskSelectView selectView = this.taskSelectViewsDao_.findByUserId(9990);
        System.out.println(selectView.toString());

        TaskSelectView view = this.taskSelectViewsDao_.findByUserId(9990);
        System.out.println(view);
        super.assertNotNull(view);
        super.assertEquals(9990, view.getUserId());
        super.assertEquals(Classification.INBOX, view.getClassId());
        super.assertEquals(0, view.getModeId());
        super.assertEquals(1, view.getIsVisibleBrowser());
    }

}
