/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dao;

import org.seasar.dao.unit.S2DaoTestCase;
import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.contents.task.entity.Task;

/**
 * test TasksDao.
 * 
 * @author yosuke
 * 
 */
public class TestTasksDao extends S2DaoTestCase {
    TasksDao taskDao_;

    @Override
    public void setUp() {
        this.include("dicon/contents/task/taskDao.dicon");
    }

    /**
     * test TestTasksDao#getNewTaskId.
     * 
     */
    public void testGetNewTaskIdTx() {
        super.readXlsWriteDb("tasks.xls");

        long id = this.taskDao_.getId();
        super.assertEquals("[OK]", 10000, id);
    }

    /**
     * test TestTasksDao#insert.
     * 
     */
    public void testInsertTx() {
        super.readXlsWriteDb("tasks.xls");

        Task task = new Task();
        task.setId(10000);
        task.setTitle("Task-10000-title");
        task.setPriorityOrder(0);
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        task.setStartDate(nowString.substring(0, 8));
        task.setStartYear(Integer.parseInt(nowString.substring(0, 4)));
        task.setStartMonth(Integer.parseInt(nowString.substring(4, 6)));
        task.setStartDayOfMonth(Integer.parseInt(nowString.substring(6, 8)));
        task.setRegistered(nowString);
        this.taskDao_.insert(task);

        task.setId(10001);
        task.setTitle("Task-10001-title");
        task.setDeadline("20070531");
        task.setDeadlineYear(2007);
        task.setDeadlineMonth(5);
        task.setDeadlineDayOfMonth(31);
        this.taskDao_.insert(task);
    }

    /**
     * test TestTasksDao#update.
     * 
     */
    public void testUpdateTx() {
        super.readXlsWriteDb("tasks.xls");

        Task task = new Task();
        task.setId(10000);
        task.setTitle("Task-10000-title");
        task.setShare(0);
        task.setPriorityOrder(0);
        task.setPrivacy(0);
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        task.setStartDate(nowString.substring(0, 8));
        task.setStartYear(Integer.parseInt(nowString.substring(0, 4)));
        task.setStartMonth(Integer.parseInt(nowString.substring(4, 6)));
        task.setStartDayOfMonth(Integer.parseInt(nowString.substring(6, 8)));
        task.setRegistered(nowString);
        this.taskDao_.insert(task);

        task.setTitle("Task-10000-title-update");
        this.taskDao_.update(task);

        task.setDeadline("20070531");
        task.setDeadlineYear(2007);
        task.setDeadlineMonth(5);
        task.setDeadlineDayOfMonth(31);
        this.taskDao_.update(task);
    }

    /**
     * test TestTasksDao#updateDeleteFlag.
     * 
     */
    public void testUpdateDeleteFlagTx() {
        super.readXlsWriteDb("tasks.xls");

        Task task = new Task();
        task.setId(10000);
        task.setTitle("Task-10000-title");
        task.setShare(0);
        task.setPriorityOrder(0);
        task.setPrivacy(0);
        
        String nowString = CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB);
        task.setStartDate(nowString.substring(0, 8));
        task.setStartYear(Integer.parseInt(nowString.substring(0, 4)));
        task.setStartMonth(Integer.parseInt(nowString.substring(4, 6)));
        task.setStartDayOfMonth(Integer.parseInt(nowString.substring(6, 8)));
        task.setRegistered(nowString);
        this.taskDao_.insert(task);

        this.taskDao_.updateDeleteFlag(task);

        task.setDeleteFlag("1");
        this.taskDao_.updateDeleteFlag(task);
    }

    /**
     * test #getUpdateTargetByDate.
     * 
     */
    public void testGetUpdateTargetByDateTx() {
        super.readXlsWriteDb("tasks.xls");

        Task[] tasks = this.taskDao_.getUpdateTargetByDate("20070505");

        super.assertNotNull(tasks);
        super.assertEquals("[OK]", true, 0 < tasks.length);
    }
}
