/*
 * Copyright 2004-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.service;

import org.seasar.dao.unit.S2DaoTestCase;
import org.seasar.tugboat.gtd.contents.task.dto.TaskGroupDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskGroupRelationDto;

/**
 * test task group service.
 * 
 * @author yousuke
 * 
 */
public class TestTaskGroupService extends S2DaoTestCase {
    TaskGroupService taskGroupService_;

    @Override
    public void setUp() {
        this.include("dicon/contents/task/taskService.dicon");
    }

    /**
     * test add.
     */
    public void testAddTx() {
        // read test data.
        super.readXlsWriteDb("task_groups.xls");

        // new task group.
        TaskGroupDto taskGroupDto = new TaskGroupDto();
        taskGroupDto.setName("test task group name #1");
        taskGroupDto.setDescription("test task group description #1");
        taskGroupDto.setStartDate("20080401");
        taskGroupDto.setDeadline("20080430");
        taskGroupDto.setUserId(9990);
        this.taskGroupService_.add(9990, taskGroupDto);

        //
        // check no tasks.
        //
        // -- 1.find by id.
        TaskGroupRelationDto dto = this.taskGroupService_.findById(10000, 9990);
        super.assertNotNull(dto);
        super.assertEquals(10000, dto.getTaskGroupId());
        super.assertEquals(9990, dto.getActivityId());
        super.assertEquals("test task group name #1", dto.getName());
        super.assertEquals("20080401", dto.getStartDate());
        super.assertEquals("20080430", dto.getDeadline());
        super.assertEquals(0, dto.getAllTasks());
        super.assertEquals(0, dto.getDoneTasks());

        // -- 2.find by user id.
        TaskGroupRelationDto[] dtos = this.taskGroupService_.findActiveGroupsByUserId(9990);
        super.assertNotNull(dtos);
        super.assertTrue(dtos.length == 5);

        for (TaskGroupRelationDto t : dtos) {
            if (t.getTaskGroupId() == 9990) {
                super.assertEquals("TG-1", t.getName());
                super.assertEquals("20080301", t.getStartDate());
                super.assertEquals("20080401", t.getDeadline());
                super.assertEquals(3, t.getAllTasks());
                super.assertEquals(1, t.getDoneTasks());
            } else if (t.getTaskGroupId() == 9991) {
                super.assertEquals("TG-2", t.getName());
                super.assertEquals("20080302", t.getStartDate());
                super.assertEquals("20080402", t.getDeadline());
                super.assertEquals(0, t.getAllTasks());
                super.assertEquals(0, t.getDoneTasks());
            } else if (t.getTaskGroupId() == 9992) {
                super.assertEquals("TG-3", t.getName());
                super.assertEquals("20080303", t.getStartDate());
                super.assertEquals("20080403", t.getDeadline());
                super.assertEquals(3, t.getAllTasks());
                super.assertEquals(1, t.getDoneTasks());
            } else if (t.getTaskGroupId() == 9998) {
                super.assertEquals("TG-9", t.getName());
                super.assertEquals("20080309", t.getStartDate());
                super.assertEquals("20080409", t.getDeadline());
                super.assertEquals(5, t.getAllTasks());
                super.assertEquals(2, t.getDoneTasks());
            } else if (t.getTaskGroupId() == 10000) {
                super.assertEquals("test task group name #1", t.getName());
                super.assertEquals("20080401", t.getStartDate());
                super.assertEquals("20080430", t.getDeadline());
                super.assertEquals(0, t.getAllTasks());
                super.assertEquals(0, t.getDoneTasks());
            } else {
                super.fail();
            }
        }

        // -- 3. find by activity id.
        dtos = this.taskGroupService_.findByActivityId(9990, 9990);
        super.assertNotNull(dtos);
        super.assertTrue(dtos.length == 4);
        
        for (TaskGroupRelationDto t : dtos) {
            if (t.getTaskGroupId() == 9990) {
                super.assertEquals("TG-1", t.getName());
                super.assertEquals("20080301", t.getStartDate());
                super.assertEquals("20080401", t.getDeadline());
                super.assertEquals(3, t.getAllTasks());
                super.assertEquals(1, t.getDoneTasks());
            } else if (t.getTaskGroupId() == 9991) {
                super.assertEquals("TG-2", t.getName());
                super.assertEquals("20080302", t.getStartDate());
                super.assertEquals("20080402", t.getDeadline());
                super.assertEquals(0, t.getAllTasks());
                super.assertEquals(0, t.getDoneTasks());
            } else if (t.getTaskGroupId() == 9992) {
                super.assertEquals("TG-3", t.getName());
                super.assertEquals("20080303", t.getStartDate());
                super.assertEquals("20080403", t.getDeadline());
                super.assertEquals(3, t.getAllTasks());
                super.assertEquals(1, t.getDoneTasks());
            } else if (t.getTaskGroupId() == 10000) {
                super.assertEquals("test task group name #1", t.getName());
                super.assertEquals("20080401", t.getStartDate());
                super.assertEquals("20080430", t.getDeadline());
                super.assertEquals(0, t.getAllTasks());
                super.assertEquals(0, t.getDoneTasks());
            } else {
                super.fail();
            }
        }
    }

    /**
     * test update.
     */
    public void testUpdateTx() {
        // read test data.
        super.readXlsWriteDb("task_groups.xls");

        // new task group.
        TaskGroupDto taskGroupDto = new TaskGroupDto();
        taskGroupDto.setName("test task group name #1");
        taskGroupDto.setDescription("test task group description #1");
        taskGroupDto.setStartDate("20080401");
        taskGroupDto.setDeadline("20080430");
        taskGroupDto.setUserId(9990);
        this.taskGroupService_.add(9990, taskGroupDto);

        // update task group.
        taskGroupDto.setName("update test task group name #1");
        taskGroupDto.setDescription("update test task group description #1");
        taskGroupDto.setStartDate("20080410");
        taskGroupDto.setDeadline("20080509");
        this.taskGroupService_.update(9990, taskGroupDto);

        // find by id.
        TaskGroupRelationDto dto = this.taskGroupService_.findById(10000, 9990);
        System.out.println(dto.toString());

        super.assertNotNull(dto);
        super.assertEquals(10000, dto.getTaskGroupId());
        super.assertEquals(9990, dto.getActivityId());
        super.assertEquals("update test task group name #1", dto.getName());
        super.assertEquals("20080410", dto.getStartDate());
        super.assertEquals("20080509", dto.getDeadline());
        super.assertEquals(0, dto.getAllTasks());
        super.assertEquals(0, dto.getDoneTasks());
    }

    /**
     * test delete.
     */
    public void testDeleteTx() {
        // read test data.
        super.readXlsWriteDb("task_groups.xls");

        // new task group.
        TaskGroupDto taskGroupDto = new TaskGroupDto();
        taskGroupDto.setName("test task group name #1");
        taskGroupDto.setDescription("test task group description #1");
        taskGroupDto.setStartDate("20080401");
        taskGroupDto.setDeadline("20080430");
        taskGroupDto.setUserId(9990);
        this.taskGroupService_.add(9990, taskGroupDto);

        // delete
        this.taskGroupService_.delete(taskGroupDto);

        // find by id.
        TaskGroupRelationDto dto = this.taskGroupService_.findById(10000, 9990);
        super.assertNull(dto);
    }

    /**
     * test find by id.
     */
    public void testFindByIdTx() {
        // read test data.
        super.readXlsWriteDb("task_groups.xls");

        // new task group.
        TaskGroupDto taskGroupDto = new TaskGroupDto();
        taskGroupDto.setName("test task group name #1");
        taskGroupDto.setDescription("test task group description #1");
        taskGroupDto.setStartDate("20080401");
        taskGroupDto.setDeadline("20080430");
        taskGroupDto.setUserId(9990);
        this.taskGroupService_.add(9990, taskGroupDto);

        // find by id.
        TaskGroupRelationDto dto = this.taskGroupService_.findById(9990, 9990);

        super.assertNotNull(dto);
        super.assertEquals(9990, dto.getTaskGroupId());
        super.assertEquals(9990, dto.getActivityId());
        super.assertEquals("TG-1", dto.getName());
        super.assertEquals("20080301", dto.getStartDate());
        super.assertEquals("20080401", dto.getDeadline());
        super.assertEquals(3, dto.getAllTasks());
        super.assertEquals(1, dto.getDoneTasks());
    }

    /**
     * test find by user id.
     */
    public void testFindByUserIdTx() {
        // read test data.
        super.readXlsWriteDb("task_groups.xls");

        // find by user id.
        TaskGroupRelationDto[] taskGroupRelations = this.taskGroupService_
                .findActiveGroupsByUserId(9995);

        super.assertNotNull(taskGroupRelations);

        for (TaskGroupRelationDto t : taskGroupRelations) {
            if (t.getTaskGroupId() == 9995) {
                super.assertEquals(9997, t.getActivityId());
                super.assertEquals("TG-6", t.getName());
                super.assertEquals("20080306", t.getStartDate());
                super.assertEquals("20080406", t.getDeadline());
                super.assertEquals(2, t.getAllTasks());
                super.assertEquals(1, t.getDoneTasks());

            } else if (t.getTaskGroupId() == 9996) {
                super.assertEquals(9997, t.getActivityId());
                super.assertEquals("TG-7", t.getName());
                super.assertEquals("20080307", t.getStartDate());
                super.assertEquals("20080407", t.getDeadline());
                super.assertEquals(3, t.getAllTasks());
                super.assertEquals(1, t.getDoneTasks());

            } else if (t.getTaskGroupId() == 9997) {
                super.assertEquals(9997, t.getActivityId());
                super.assertEquals("TG-8", t.getName());
                super.assertEquals("20080308", t.getStartDate());
                super.assertEquals("20080408", t.getDeadline());
                super.assertEquals(3, t.getAllTasks());
                super.assertEquals(1, t.getDoneTasks());

            } else if (t.getTaskGroupId() == 9998) {
                super.assertEquals(9999, t.getActivityId());
                super.assertEquals("TG-9", t.getName());
                super.assertEquals("20080309", t.getStartDate());
                super.assertEquals("20080409", t.getDeadline());
                super.assertEquals(5, t.getAllTasks());
                super.assertEquals(2, t.getDoneTasks());

            } else {
                super.fail();
            }
        }

    }

    /**
     * test find by activity id.
     */
    public void testFindByActivityIdTx() {
        // read test data.
        super.readXlsWriteDb("task_groups.xls");

        // find by activity id.
        this.taskGroupService_.findByActivityId(9995, 9991);
    }

}
