package org.seasar.tugboat.gtd.contents.task.service;

import org.seasar.dao.unit.S2DaoTestCase;
import org.seasar.tugboat.gtd.contents.common.entity.Classification;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBodyDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto;

/**
 * test TaskService.
 * 
 * @author yosuke
 * 
 */
public class TestTaskService extends S2DaoTestCase {
    TaskService taskService_;
    
    /*
     * (non-Javadoc)
     * @see junit.framework.TestCase#setUp()
     */
    public void setUp() {
       this.include("dicon/contents/task/task.dicon"); 
    }
    
    /**
     * test #addTask.
     *
     */
    public void testAddTaskTx() {
        super.readXlsWriteDb("task_bodies.xls");
        
        TaskDto task = new TaskDto();
        task.setUserId(9990);
        task.setTitle("Test-1");
        task.setDeadline("20070701");
        task.setDeadlineYear(2007);
        task.setDeadlineMonth(7);
        task.setDeadlineDayOfMonth(1);
        
        TaskBodyDto body = new TaskBodyDto();
        body.setBody("Test bosy-1");
        
        TaskRelationDto relations = new TaskRelationDto();
        relations.setActivityId(9990);
        relations.setStatusId(TaskRelationDto.STATUS_TODO);
        relations.setClassificationId(Classification.INBOX);
        relations.setPersonInChargeId(9990);
        relations.setUserId(9990);
        
        long result = 
            this.taskService_.addTask(task, body, relations);
        
        super.assertEquals(10000, result);
        System.out.println(result);
    }
    
    /**
     * test #updateTask.
     *
     */
    public void testUpdateTaskTx() {
        super.readXlsWriteDb("task_bodies.xls");
        
        TaskDto task = new TaskDto();
        task.setUserId(9990);
        task.setTitle("Test-1");
        task.setDeadline("20070701");
        task.setDeadlineYear(2007);
        task.setDeadlineMonth(7);
        task.setDeadlineDayOfMonth(1);
        
        TaskBodyDto body = new TaskBodyDto();
        body.setBody("Test bosy-1");
        
        TaskRelationDto relations = new TaskRelationDto();
        relations.setActivityId(9990);
        relations.setStatusId(TaskRelationDto.STATUS_TODO);
        relations.setClassificationId(Classification.INBOX);
        relations.setPersonInChargeId(9990);
        relations.setUserId(9990);
        
        long result = 
            this.taskService_.addTask(task, body, relations);
        
        task.setId(result);
        body.setTaskId(result);
        relations.setTaskId(result);
        
        TaskRelationDto dto = 
            this.taskService_.updateTask(task, body, relations, null);
        
        super.assertEquals(result, dto.getTaskId());
        super.assertEquals(1, dto.getStatusId());
        super.assertEquals(9, dto.getClassificationId());
        super.assertEquals(9990, dto.getActivityId());
        super.assertEquals(9990, dto.getPersonInChargeId());
        super.assertEquals(9990, dto.getUserId());
        super.assertEquals("20070701", dto.getDeadline());
        
        System.out.println("********\n" + dto.toString());
    }
    
    /**
     * test #getDefaultFoloderTasks.
     *
     */
    public void testGetDefaultFoloderTasksTx() {
        
    }
    
    /**
     * test #getTaskById.
     *
     */
    public void testGetTaskByIdTx() {
        
    }
    
    /**
     * test #getTasksByQueryParams.
     *
     */
    public void testGetTasksByQueryParams_1Tx() {
        super.readXlsWriteDb("task_bodies.xls");
        
        // #1. simple
        TaskDto taskDto = new TaskDto();
        taskDto.setUserId(9990);
        taskDto.setQueryStatusIds(new Integer[]{1});
        taskDto.setQueryActivityIds(new Long[]{9990L});
        taskDto.setQueryClassificationIds(new Integer[]{1});
        taskDto.setQueryPersonIds(new Long[]{9990L});
        taskDto.setOffset(0);
        taskDto.setLimit(50);
        int selectFolder = 1;
        
        TaskRelationDto[] relation = 
            this.taskService_.getTasksByQueryParams(taskDto, selectFolder, false);
        
        super.assertEquals(1, relation.length);
        
        // #2. complex conditions.
        taskDto.setQueryStatusIds(new Integer[]{1});
        taskDto.setQueryActivityIds(new Long[]{9990L,9991L,9992L});
        taskDto.setQueryClassificationIds(new Integer[]{1,2,3,4,5});
        taskDto.setQueryPersonIds(new Long[]{9990L,9991L,9992L});
        taskDto.setOffset(0);
        taskDto.setLimit(50);
        selectFolder = 1;
        
        relation = 
            this.taskService_.getTasksByQueryParams(taskDto, selectFolder, false);
        
        super.assertEquals(2, relation.length);
        
        // #3. complex conditions.
        taskDto.setQueryStatusIds(new Integer[]{1,2,3});
        taskDto.setQueryActivityIds(new Long[]{9990L});
        taskDto.setQueryClassificationIds(new Integer[]{1,2,3,4,5});
        taskDto.setQueryPersonIds(new Long[]{9990L});
        taskDto.setOffset(0);
        taskDto.setLimit(50);
        selectFolder = 1;
        
        relation = 
            this.taskService_.getTasksByQueryParams(taskDto, selectFolder, false);
        
        super.assertEquals(3, relation.length);
    }
    
    /**
     * test #getTasksByQueryParams. [check task tags]
     *
     */
    public void testGetTasksByQueryParams_2Tx() {
        super.readXlsWriteDb("task_bodies.xls");
        
        TaskDto taskDto = new TaskDto();
        taskDto.setUserId(9990);
        taskDto.setQueryStatusIds(new Integer[]{1,2,3});
        taskDto.setQueryActivityIds(new Long[]{9990L,9991L,9992L});
        taskDto.setQueryClassificationIds(new Integer[]{1,2});
        taskDto.setQueryPersonIds(new Long[]{9990L,9991L});
        taskDto.setOffset(0);
        taskDto.setLimit(50);
        
        int selectFolder = 1;
        
        TaskRelationDto[] relation = 
            this.taskService_.getTasksByQueryParams(taskDto, selectFolder, false);
        
        super.assertEquals(2, relation.length);
        
        System.out.println("----- ST -----");
        for (TaskRelationDto dto : relation) {
            System.out.println("[" + dto.getTaskId() + "]");
            
            if (dto.getTaskTagDtos().length > 0) {
                for(TaskTagDto tag : dto.getTaskTagDtos()) {
                    System.out.println(tag.getTagId() + ":" + tag.getTagValue());
                }
            }
            
            if (dto.getTaskId() == 9990) {
                super.assertEquals(3, dto.getTaskTagDtos().length);
            }
            if (dto.getTaskId() == 9991) {
                super.assertEquals(1, dto.getTaskTagDtos().length);
            }
            
            System.out.println("-----");
        }
    }
    
    /**
     * test #getTaskCountByQueryParams.
     *
     */
    public void testGetTaskCountByQueryParamsTx() {
        super.readXlsWriteDb("task_bodies.xls");
        
        TaskDto taskDto = new TaskDto();
        taskDto.setUserId(9990);
        taskDto.setQueryStatusIds(new Integer[]{1,2,3});
        taskDto.setQueryActivityIds(new Long[]{9990L,9991L,9992L});
        taskDto.setQueryClassificationIds(new Integer[]{1,2});
        taskDto.setQueryPersonIds(new Long[]{9990L,9991L});
        taskDto.setOffset(0);
        taskDto.setLimit(50);
        
        int result = this.taskService_.getTaskCountByQueryParams(taskDto);
        super.assertEquals(2, result);
    }
    
    /**
     * test #updateClassificationId.
     *
     */
    public void testUpdateClassificationIdTx() {
        
    }
    
    /**
     * test #getWeekStartDateAndWeekEndDate.
     *
     */
    public void testGetWeekStartDateAndWeekEndDateTx() {
        
    }
    
    /** 
     * test #emptyTrashBox.
     *
     */
    public void testEmptyTrashBoxTx() {
        
    }
    
    /**
     * test #updateSelectedMode.
     *
     */
    public void testUpdateSelectedModeTx() {
        
    }
    
    /**
     * test #updateIsVisibleBrowser.
     *
     */
    public void testUpdateIsVisibleBrowserTx() {
        
    }
    
    /**
     * test #getViewStateRelations.
     *
     */
    public void testGetViewStateRelationsTx() {
        
    }
    
    /**
     * test #findByTaskGroupId.
     */
    public void testfindByTaskGroupIdTx() {
        
    }
    
}
