package org.seasar.tugboat.gtd.contents.user.dao;

import org.seasar.dao.unit.S2DaoTestCase;
import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.contents.user.entity.User;
import org.seasar.tugboat.gtd.contents.user.entity.UserBrowsePageCondition;

public class TestUsersDao extends S2DaoTestCase {
    UsersDao usersDao;
    
    
    protected void setUp() throws Exception {
        this.include("dicon/contents/user/userDao.dicon"); 
    }

    public void testGetUserIdTx() {
        super.readXlsWriteDb("users.xls");
        User user =usersDao.getUserId();
        assertNotNull(user);
        user = usersDao.findUserById(user.getUserId());
        assertNull(user);
    }

    public void testInsertTx() {
        super.readXlsWriteDb("users.xls");
        User user =usersDao.getUserId();
        user.setFirstName("First");
        user.setLastName("Last");
        user.setEmail("a@a.com");
        user.setPassword("password");
        user.setLoginId("loginId");
        user.setModifiedUserId(9990);
    }

    public void testUpdateTx() {
        super.readXlsWriteDb("users.xls");
        User user = usersDao.findUserById(9990);
        user.setFirstName("F");
        user.setLastName("L");
        user.setEmail("____@____");
        user.setPassword("P");
        user.setModified(CalendarUtil.now(CalendarUtil.DATE_LONG_FORMAT_FOR_RDB));
        int count = usersDao.update(user);
        assertEquals(1, count);
        
        user = usersDao.findUserById(9990);
        
        assertEquals(user.getFirstName(),"F");
        assertEquals(user.getLastName(),"L");
        assertEquals(user.getEmail(),"____@____");
        assertEquals(user.getPassword(),"P");

        user = usersDao.findUserById(9991);
        
        assertEquals(user.getFirstName(),"t2");
    }

    public void testFindUserByIdTx() {
        super.readXlsWriteDb("users.xls");
        User user = usersDao.findUserById(9990);
        assertEquals(user.getUserId(),9990);
        assertEquals(user.getLoginId(),"test-1");
        assertEquals(user.getPassword(),"password");
        assertEquals(user.getEmail(),"test-1@tester");
        assertEquals(user.getLastName(),"test1");
        assertEquals(user.getFirstName(),"t1");
        assertEquals(user.getPhotoImageUrl(),"t1.jpg");
        assertEquals(user.getModifiedUserId(),1);
    }

    public void testFindUsersByKeywordTx() {
        super.readXlsWriteDb("users.xls");
        User[] users = usersDao.findUsersByKeyword("%1%");
        assertEquals(users.length, 2);
    }

    public void testFindUsersByActivityIdAndKeywordTx() {
        super.readXlsWriteDb("activities.xls");
        User[] users = usersDao.findUsersByActivityIdAndKeyword(9990, "%1%");
        assertEquals(users.length , 1);
        users = usersDao.findUsersByActivityIdAndKeyword(10000, "%2%");
        assertEquals(users.length , 0);
    }

    public void testFindByLoginIdTx() {
        super.readXlsWriteDb("users.xls");
        User user = usersDao.findByLoginId("test-1");
        assertEquals(user.getUserId(),9990);
        assertEquals(user.getLoginId(),"test-1");
        assertEquals(user.getPassword(),"password");
        assertEquals(user.getEmail(),"test-1@tester");
        assertEquals(user.getLastName(),"test1");
        assertEquals(user.getFirstName(),"t1");
        assertEquals(user.getPhotoImageUrl(),"t1.jpg");
        assertEquals(user.getModifiedUserId(),1);
    }

    public void testGetAllUserTx() {
        super.readXlsWriteDb("users.xls");
        UserBrowsePageCondition condition = new UserBrowsePageCondition();
        condition.setLimit(5);
        condition.setOffset(0);
        User[] users = usersDao.getAllUser(condition);
        assertEquals(users.length, 5);
        
        condition.setLimit(999999);
        users = usersDao.getAllUser(condition);
        assertEquals(users.length, 11);// 10(xls) + 1(admin_user)
        
    }

    public void testUpdateDeleteFlagByUserIdTx() {
        super.readXlsWriteDb("users.xls");
        int count = usersDao.updateDeleteFlagByUserId(9990);
        assertEquals(count, 1);
        User user = usersDao.findUserById(9990);
        assertNull(user);
    }

    public void testUpdateDeleteFlagByUserTx() {
        super.readXlsWriteDb("users.xls");
        User user = new User();
        user.setUserId(9990);
        int count = usersDao.updateDeleteFlagByUser(user);
        assertEquals(count, 1);
        user = usersDao.findUserById(9990);
        assertNull(user);
    }

    public void testFindUserByLoginIdAndPasswordTx() {
        super.readXlsWriteDb("users.xls");
        User user = usersDao.findUserByLoginIdAndPassword("test-1", "password");
        assertEquals(user.getFirstName(),"t1");
        user = usersDao.findUserByLoginIdAndPassword("test-1", "pass");
        assertNull(user);
        user = usersDao.findUserByLoginIdAndPassword("test", "password");
        assertNull(user);
        user = usersDao.findUserByLoginIdAndPassword("delete-1", "password");
        assertNull(user);
    }

    public void testFindAdminUserByLoginIdAndPasswordTx() {
        super.readXlsWriteDb("users.xls"); 
        User user = usersDao.findAdminUserByLoginIdAndPassword("test-1", "password");
        assertEquals(user.getFirstName(),"t1");
        user = usersDao.findAdminUserByLoginIdAndPassword("test-2", "password");
        assertNull(user);
        user = usersDao.findAdminUserByLoginIdAndPassword("delete-1", "password");
        assertNull(user);
    }

}
